package Modules::BannerAd;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Input;
#use ExSite::Module;
use Modules::BaseDCD;
use MIME::Base64;
use ExSite::ReportBuilder;

use vars qw(@ISA %format);
push(@ISA,"Modules::BaseDCD");

#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "Advertising";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "The Advertising module serves banner ads to websites, 
tracks advertising clicks, and monitors balances on pay-per-view or 
pay-per-click campaigns.";
	}elsif (/isRestricted/){
		return 1;
	}
	elsif (/DynContentParameter/) {
	    $this->fetchFormats();
	    my $out = "<p><b>Format:</b>
<select name=\"param\">
<option value=\"\">(default)</option>\n";
	    foreach my $format (values %format) {
		$out .= "<option value=\"format=$format->{name}\">$format->{name}</option>\n";
	    }
	    $out .= "</select></p>\n";
	    return $out;
	}
	elsif (/Publish/){
                return \&publish;
	}
	return;
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action);
	%opt = &DecodeString($options);
	$action = $this->{input}{action} || $opt{action};
	# Process click through
	if ($this->{input}{bannerad_id}){
		return $this->processBannerClick($this->{input}{bannerad_id});
	}
	# Display banners
	else{
		return $this->displayBanner($opt{format},$opt{count});
	}
	return undef;
}
#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	my $out;	
	my $action = $this->{input}{action};
	my $sectionid = exists $this->{input}{section_id} ? $this->{input}{section_id} : 0;
	my $banneradid = $this->{input}{bannerad_id};
	my $formatid = $this->{input}{format_id} || 0;
	if ($action eq "publish" && $sectionid){
		$out = $this->publish($sectionid);
	}
	# Add a new banner ad
	elsif($action eq "new_bannerad"){
	    $out = $this->newBanner($sectionid);
	}
	# Delete a banner ad
	elsif($action eq "delete_bannerad" && $banneradid){
	    $out = $this->deleteBanner($banneradid);
	}
	# Edit a banner ad
	elsif($action eq "edit_bannerad" && $banneradid){
	    $out = $this->editBanner($banneradid,$sectionid);
	}
	# View banner ad details
	elsif($action eq "detail_bannerad" && $banneradid){
	    $out = $this->detailBanner($banneradid);
	}
	# Copy banner ad
	elsif ($action eq "copy_bannerad" && $banneradid){
	    $out = $this->copyBanner($banneradid);
	}
	# Export banner click through log to excel
	elsif($action eq "export_log" && $banneradid){
	    $out = $this->exportLog($banneradid);
	}
	# Add a new format
	elsif($action eq "new_format") {
	    $out = $this->newFormat($sectionid);
	}
	# Delete a banner ad
	elsif($action eq "delete_format") {
	    $out = $this->deleteFormat($formatid);
	}
	# Edit a banner ad
	elsif($action eq "edit_format") {
	    $out = $this->editFormat($formatid,$sectionid);
	}
	# View summary reports
	else{
	    $out = $this->viewSummary($sectionid);
	}
	return $out;
}
#----------------------------------------------------------------------
# Publish banner
sub publish{
	my ($this) = @_;
	my $sectionid = $this->{input}{section_id} || 0;	
	my @log;
	my @banners;
	if ($sectionid > 0){
		@banners = $this->fetchBannerad($sectionid);
	}else{
		# Publish global banners
		@banners = $this->{db}->custom_query("select * from bannerad where section_id = 0 || section_id is null or section_id = ''");
	}
	my $disk_path = $this->getBannerDiskPath($sectionid);
	if (!$disk_path){
		return "<p>Unable to create directory for image publishing</p>";
	}
	foreach my $banner(@banners){
		my $filename = $this->getBannerFilename($banner);
		if (ExSite::Misc::is_tainted($filename)){
			$filename =~ /^([a-zA-Z0-9_-]+\.[a-zA-Z0-9_-]+)$/;
			$filename = $1;
		}
		my $write_file = $disk_path."/".$filename;
		if (open(OFILE,">$write_file")){
			my $file_content = $this->getBannerData($banner);
			print OFILE $file_content;
			close OFILE;
			push(@log,"Write File... $filename");
		}else{
			push(@log,"<span class=\"error\">FAILED: $!</span>");
		}
	}
	return "\n<p>".join("</p>\n<p>",@log)."</p>";
}
# Decode banner data
sub getBannerData{
	my ($this,$banner_rec) = @_;
	my $fdata = $banner_rec->{fdata};
	if (!$fdata){
		return "";
	}
	my (undef,$fcontent) = split(/#/,$fdata,2);
	return decode_base64($fcontent);
}
# Get banner html path
sub getBannerWebPath{
	my ($this,$sectionid) = @_;
	$sectionid = $sectionid || 0;
	return "$config{server}{server}$config{server}{HTMLpath}/_Modules/BannerAd/img/$sectionid";
}
# Check and create directory for publishing
sub checkDiskDir{
	my ($this,$dir) = @_;
	if (!$dir){
		return 0;
	}
	my $checked = $dir;
	if (ExSite::Misc::is_tainted($dir)){
		$dir =~ /^((\/{1}[a-zA-Z0-9_-]+)+)$/;
		$checked = $1;
	}
	if (! -e $checked){
		if (!mkdir($checked)){
			return 0;
		}
	}
	return 1;
}
# Get banner publishing disk path
sub getBannerDiskPath{
	my ($this,$sectionid) = @_;
	$sectionid = $sectionid || 0;
	my $module_path = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/BannerAd/img";
	if (!$this->checkDiskDir($module_path)){
		return "";
	}
	my $disk_path = "$module_path/$sectionid";
	if (!$this->checkDiskDir($disk_path)){
		return "";
	}
	# Tainted check
	my $checked;
	$disk_path =~ /^((\/{1}[a-zA-Z0-9_-]+)+)$/;
	my $checked = $1;
	return $checked;
}
# Get banner filename
sub getBannerFilename{
	my ($this,$banner_rec) = @_;
	my $bannerid = $banner_rec->{bannerad_id};
	return "banner_${bannerid}.gif";
}
# Get banner url
sub getBannerUrl{
	my ($this,$banner_rec) = @_;
	my $sectionid = $banner_rec->{section_id};
	my $filename = $this->getBannerFilename($banner_rec);
	my $path = $this->getBannerDiskPath($sectionid);
	my $diskpath = $path."/".$filename;
	if (-e $diskpath){
		my $serverpath = $this->getBannerWebPath($sectionid);
		return $serverpath."/".$filename;
	}
	# Banner isn't published
	return "";
}
# Get banner img html tag
sub getBannerHTML{
	my ($this,$banner_rec) = @_;
	my $banner_url = $this->getBannerUrl($banner_rec);
	if ($banner_url){
		return "<img src=\"$banner_url\" border=\"0\">";
	}
	# Banner isn't published. Use getdata.cgi
	return $this->{db}->show_data("bannerad","fdata",$banner_rec->{fdata},$banner_rec->{bannerad_id});
}
# Get publish url for bannerad control panel
sub publish_url{
	my ($this,$sectionid) = @_;
	return "$config{server}{server}$config{server}{CGIpath}/publish.cgi?module=BannerAd&section_id=$sectionid";
}
#----------------------------------------------------------------------
# Summary report
sub viewSummary{
	my ($this,$sectionid) = @_;
	my @sites = $this->{db}->my_real_sites();
	my @licensed_sites;
	foreach my $site (@sites) {
	    if ($this->site_license($site->{section_id})) {
		push @licensed_sites,$site;
	    }
	}
	if (@licensed_sites == 0) {
	    return $this->warn("None of your sites are licensed to use the Advertising plug-in.");
	}

	my $sid = @licensed_sites==1 ? 
	    $licensed_sites[0]{section_id} : 
	    $this->{input}{section_id};
	my $out;
	if (@licensed_sites > 1) {
	    $out .= "<p><form method=\"GET\">Select website: <select name=\"section_id\" onchange=\"document.forms[0].submit()\">\n<option value=\"0\">== choose a site ==</option>\n";
	    foreach my $s (@licensed_sites) {
		$out .= "<option value=\"$s->{section_id}\"";
		$out .= " selected" if ($s->{section_id} == $sid);
		$out .= ">$s->{title}</option>\n";
	    }
	    $out .= "</select></form>\n";
	}

	$out .= $this->viewBannerReport({section_id=>0})."<br>\n";

	if ($sid) {
	    my %s = &keywise("section_id",\@licensed_sites);
	    $out .= $this->viewBannerReport($s{$sid})."<br>\n";
	}

	$out .= $this->viewFormatReport($sectionid)."<br>\n";
	return $out;
}
# Add a new banner ad
sub newBanner{
	my ($this,$sectionid) = @_;
	return $this->form_bannerad({label=>"New Banner",section_id=>$sectionid});	
}
# Edit a banner ad
sub editBanner{
	my ($this,$banneradid,$sectionid) = @_;
	return $this->form_bannerad({label=>"Edit Banner",section_id=>$sectionid,bannerad_id=>$banneradid});	
}
# Banner form
sub form_bannerad{
	my ($this,$opt) = @_;
	my $label = $opt->{label};
	my $sectionid = $opt->{section_id};
	my $banneradid = $opt->{bannerad_id};
	# Set options to generate the form
	my %form_option = (
		table=>"bannerad",
		hide=>["section_id"],
		extra=>{
			reply=>$this->link(action=>undef,bannerad_id=>undef,section_id=>$sectionid),
		},
	);	
	if ($banneradid){
		# Add record id for editing a banner
		$form_option{record} = $banneradid;
	}else{
		# Add section id for inserting a new banner
		$form_option{data}{section_id} = $sectionid || 0;
	}
	my $form = $this->{db}->make(%form_option);
	return <<END;
<h1>$label</h1>
$form
END
}
# Add a new banner ad
sub newFormat{
	my ($this,$sectionid) = @_;
	return $this->form_format({label=>"New Format",section_id=>$sectionid});
}

# Edit a banner ad
sub editFormat{
	my ($this,$formatid,$sectionid) = @_;
	return $this->form_format({label=>"Edit Format",section_id=>$sectionid,format_id=>$formatid});
}
# Banner format form
sub form_format{
	my ($this,$opt) = @_;
	my $label = $opt->{label};
	my $sectionid = $opt->{section_id} || 0;
	my $formatid = $opt->{format_id} || 0;
	# Set options to generate the form
	my %form_option = (
		table=>"bannerad_format",
		extra=>{
			reply=>$this->link(action=>undef,format_id=>undef,section_id=>$sectionid),
		},
	);
	if ($formatid){
		# Add record id for editing a banner
		$form_option{record} = $formatid;
	}
	my $form = $this->{db}->make(%form_option);
	return <<END;
<h1>$label</h1>
$form
END
}
# Export banner ad click through log to excel
sub exportLog{
	my ($this,$banneradid) = @_;
	# Report headings
	my %heading = (time=>"Timestamp",viewer_ip=>"Banner viewer's ip",referrer=>"Referrer");
	# Name of the columns that we are going to export
	my @column = keys(%heading);
	my @logs = $this->{db}->fetch_match("bannerad_log",{bannerad_id=>$banneradid},"bannerad_log_id asc");
	for(my $i = 0; $i < @logs; $i++){
		foreach my $col(@column){
			# Format export data
			$logs[$i]{$col} = $this->{db}->show_data("bannerad_log",$col,$logs[$i]{$col},$logs[$i]{bannerad_log_id});
		}
	}
	my $filename = "banner_log.xls";
	print "content-type: application/vnd.ms-excel\ncontent-disposition: attachment; filename=$filename\n\n";
	print $this->{db}->export({table=>"bannerad_log",format=>"excel",data=>\@logs,column=>\@column,heading=>\%heading});
        return;
}
# View banner details
sub detailBanner{
	my ($this,$banneradid) = @_;
	# Click through report
	#
	# Fetch the logs
	my @logs = $this->{db}->fetch_match("bannerad_log",{bannerad_id=>$banneradid},"bannerad_log_id asc");
	# Report heading
	my @lheader = ("Timestamp","Banner viewer's ip","Referrer");
	my @ldata;
	foreach my $log(@logs){
		# Gather data
		push(@ldata,[
			$this->{db}->show_data("bannerad_log","time",$log->{time},$log->{bannerad_log_id}),
			$this->{db}->show_data("bannerad_log","viewer_ip",$log->{viewer_ip},$log->{bannerad_log_id}),
			$this->{db}->show_data("bannerad_log","referrer",$log->{referrer},$log->{bannerad_log_id}),
		]);
	}
	my $excel_url = $this->link(bannerad_id=>$banneradid,action=>"export_log");
	# Rxport to excel link
	my $tool = <<END;
<a href="$excel_url"><img src="$this->{html}/_Modules/BannerAd/excel16.gif" border="0"></a>
END
	my %lreport = (
		title=>"Click Log for this Ad",
		headers=>\@lheader,
		data=>\@ldata,
		tools=>$tool,
	);
	# Place the click through report in the footer section of the banner details report
	my $foot = $this->reportbuilder(\%lreport);
	# Banner details report
	#
	# Fetch the banner ad
	my %ad = $this->{db}->fetch("bannerad",$banneradid);
	# Define column heading
	my @header = ("Ad link","Image","Link text","Description text","Type",
		"Cost per view/click","Enable logging","Total remaining value of views/clicks","Priority to be given to this ad",
		"Date to stop showing timed ads","Status");
	my @header_col = ("url","fdata","adata","ddata","bannerad_type","rate","logging","balance","weight","expiry_date","status");
	my @data;
	# Only display the columns defined in @header_col
	foreach my $col(@header_col){
		# format data
		push(@data,$this->{db}->show_data("bannerad",$col,$ad{$col},$ad{bannerad_id}));
	}
	# Define options link info
	my @options = (		
		{
			id=>"edit",label=>"Edit",
			url=>$this->link(action=>"edit_bannerad",banner_id=>$ad{bannerad_id}),
			img=>"$this->{html}/_Modules/BannerAd/edit12.gif"
		},
		{
			id=>"delete",label=>"Delete",
			url=>"javascript:deleteConfirm($ad{bannerad_id})",
			img=>"$this->{html}/_Modules/BannerAd/rm12.gif"
		},
	);
	# Options links(edit and delete commands)
	my $option;
	foreach my $opt(@options){
		$option .= "<a href=\"$opt->{url}\">";
		$option .= "<img src=\"$opt->{img}\" hspace=\"2\" border=\"0\" align=\"middle\" title=\"$opt->{label}\" alt=\"$opt->{label}\">";
		$option .= "</a>";
	}
	my %report = (
		title=>"Banner Ad",
		headers=>\@header,
		data=>\@data,
		tools=>$option,
		foot=>$foot,
	);
	my $out = $this->javascript();
	$out .= "\n<h1>View Details</h1>\n";
	$out .= $this->reportbuilder(\%report);
	return $out;
}
# Delete a banner ad
sub deleteBanner{
	my ($this,$banneradid) = @_;
	my %banner = $this->{db}->fetch("bannerad",$banneradid);
	$this->{db}->trash_r("bannerad",$banneradid);
	$this->redirect($this->link(section_id=>($banner{section_id} || 0),bannerad_id=>undef,action=>undef));
	return;
}
# Delete a banner ad
sub deleteFormat{
	my ($this,$formatid) = @_;
	$this->{db}->trash_r("bannerad_format",$formatid);
	$this->redirect($this->link(action=>undef,format_id=>undef));
	return;
}
# Copy a banner ad
sub copyBanner{
	my ($this,$banneradid) = @_;
	my %bannerad = $this->{db}->fetch("bannerad",$banneradid);
	# Remove bannerad_id(primary key)
	delete $bannerad{bannerad_id};
	# Set the balance to 0
	$bannerad{balance} = 0;
	$this->{db}->insert("bannerad",\%bannerad);
	$this->redirect($this->link(action=>undef,section_id=>$bannerad{section_id},bannerad_id=>undef));
	return;
}
# Fetch banner ads
sub fetchBannerad{
	my ($this,$sectionid) = @_;
	if (! $sectionid) { $sectionid = 0; }
	return $this->{db}->fetch_child("bannerad","section",$sectionid);
}
# fetch formats
sub fetchFormats {
    my ($this) = @_;
    if (scalar keys %format == 0) {
	my @formats = $this->{db}->fetch_all("bannerad_format");
	%format = &keywise("bannerad_format_id",\@formats);
    }
}

# Format report
sub viewFormatReport{
	my ($this,$sectionid) = @_;
	# Fetch formats
	$this->fetchFormats;
	# Report heading
	my @header = ("Format","Width","Height","Options");
	my @data;
	foreach my $f (values %format) {
	        next if (scalar keys %$f == 0);
		# Create option links
		my $edurl = $this->link(action=>"edit_format",format_id=>$f->{bannerad_format_id},section_id=>$sectionid);
		my $delurl = $this->link(action=>"delete_format",format_id=>$f->{bannerad_format_id},section_id=>$sectionid);
		my $option .= "<a href=\"$edurl\"><img src=\"$this->{html}/_Modules/BannerAd/edit12.gif\" height=12 width=12 border=0></a>
<a href=\"$delurl\"><img src=\"$this->{html}/_Modules/BannerAd/rm12.gif\" height=12 width=12 border=0></a>";
		# Gather and format data
		push(@data,[
			    $f->{name},
			    $f->{width},
			    $f->{height},
			    $option,
			    ]);
	}
	# Tool links
	my $tool = "<a href=\"".$this->link(action=>"new_format")."\">";
	$tool .= "<img src=\"$this->{html}/_Modules/BannerAd/new12.gif\" border=\"0\" title=\"New Format\" alt=\"New Format\"></a>";
	my %report = (
		title=>"Ad Formats",
		headers=>\@header,
		data=>\@data,
		tools=>$tool,
	);
	my $out = $this->javascript();
	$out .= $this->reportbuilder(\%report);
	return $out;
}

# Summary report
sub viewBannerReport{
	my ($this,$section) = @_;
	my $sectionid = $section->{section_id};
	# Fetch banner ads
	my @banners = $this->fetchBannerad($sectionid);
	$this->fetchFormats;
	# Report heading
	my @header = ("Format","Status","Type","Weight","Expiry Date","Ad link","Style","Logging","Rate","Balance","Options");
	my @data;
	foreach my $banner(@banners){
		my $bannerid = $banner->{bannerad_id};
		# Define option link info
		my @options = (
		{
			id=>"detail",label=>"View Details",
			url=>$this->link(action=>"detail_bannerad",bannerad_id=>$bannerid,section_id=>$sectionid),
			img=>"$this->{html}/_Modules/BannerAd/view12.gif"
		},
		{
			id=>"edit",label=>"Edit",
			url=>$this->link(action=>"edit_bannerad",bannerad_id=>$bannerid,section_id=>$sectionid),
			img=>"$this->{html}/_Modules/BannerAd/edit12.gif"
		},
#		{
#			id=>"copy",label=>"Copy",
#			url=>$this->link(action=>"copy_bannerad",bannerad_id=>$bannerid,section_id=>$sectionid),
#			img=>"$this->{html}/_Modules/BannerAd/copy12.gif"
#		},
		{
			id=>"delete",label=>"Delete",
			url=>"javascript:deleteConfirm($bannerid)",
			img=>"$this->{html}/_Modules/BannerAd/rm12.gif"
		},
		);
		my $option;
		# Create option links
		foreach my $opt(@options){
			$option .= "<a href=\"$opt->{url}\">";
			$option .= "<img src=\"$opt->{img}\" hspace=\"2\" border=\"0\" align=\"middle\" title=\"$opt->{label}\" alt=\"$opt->{label}\">";
			$option .= "</a>";
		}
		# Gather and format data
		push(@data,[
			    $format{$banner->{format}}{name},
			    $this->displayStatus($banner),
			    $banner->{bannerad_type},
			    $banner->{weight},
			    $this->{db}->show_data("bannerad","expiry_date",$banner->{expiry_date},$banner->{bannerad_id}),
			    $this->{db}->show_data("bannerad","url",$banner->{url},$banner->{bannerad_id}),			
			    $banner->{fdata} ? "image" : "text",
			    $banner->{logging},			
			    $this->{db}->show_data("bannerad","rate",$banner->{rate},$banner->{bannerad_id}),
			    $this->{db}->show_data("bannerad","balance",$banner->{balance},$banner->{bannerad_id}),
			    $option,
		]);
	}
	my $url_banner_publish = $this->publish_url($sectionid);
	my $url_new_banner = $this->link(action=>"new_bannerad",section_id=>$sectionid);
	# Tool links
	my $tool = <<END;
<a href="$url_banner_publish" target="publish">Publish</a>
<a href="$url_new_banner"><img src="$this->{html}/_Modules/BannerAd/new12.gif" border="0" title="New banner" alt="New banner"></a>
END
	my %report = (
		title=>$section->{title} || "Ads for all sites",
		headers=>\@header,
		data=>\@data,
		tools=>$tool,
	);
	my $out = $this->javascript();
	$out .= $this->reportbuilder(\%report);
	return $out;
}
# Display the status column on the summary report
sub displayStatus{
	my ($this,$bannerad) = @_;
	my @status;
	# Default status(active or inactive)
	push(@status,$bannerad->{status});
	if ($this->isExpired($bannerad)){
		# expired banner ad
		push(@status,"expired");
	}
	if ($this->isExhausted($bannerad)){
		# exhausted banner ad
		push(@status,"exhausted");
	}
	return join(", ",@status);
}
# Is the banner ad exhausted?
sub isExhausted{
	my ($this,$bannerad) = @_;
	# Only check the pay-per-view or pay-per-click ads
	return 0 if $bannerad->{bannerad_type} !~ /pay/i;
	# Funds available
	return 0 if $bannerad->{balance} > 0;
	return 1;
}
sub getTimeObject{
	my ($this) = @_;
	if (!$this->{time_object}){
		require ExSite::Time;
		$this->{time_object} = new ExSite::Time;
	}
	return $this->{time_object};
}
sub CompareToNow{
	my ($this,$t1,$t2) = @_;
	my $tObj = $this->getTimeObject();
	return $tObj->CompareToNow($t1,$t2);	
}
# Is the banner ad expired?
sub isExpired{
	my ($this,$bannerad) = @_;
	# Only check the timed ads
	return 0 if $bannerad->{bannerad_type} !~ /timed/i;
	# The ad isn't expired
	return 0 if ($this->CompareToNow($bannerad->{expiry_date}) >= 0);
	return 1;
}
# Is the ad active?
sub isActive{
	my ($this,$bannerad) = @_;
	# The ad is considered active when it is not:
	# - expired
	# - exhausted
	return 1 if (!$this->isExpired($bannerad) && !$this->isExhausted($bannerad));
	return 0;
}
# Filter viewable ads by status
sub filterByStatus{
	my ($this,$bannerads) = @_;
	my @newbanners;
	# Construct a list consist of active ads
	foreach my $ad(@$bannerads){
		if ($this->isActive($ad)){
			push(@newbanners,$ad);
		}
	}
	return \@newbanners;
}
# Filter viewable ads by format
sub filterByFormat{
	my ($this,$bannerads,$format) = @_;
	if (! $format) { return $bannerads; }
	$this->fetchFormats;
	my @newbanners;
	# Construct a list consisting of ads of the given format
	foreach my $ad(@$bannerads){
	    if (! $ad->{format} || 
		($format{$ad->{format}}{name} eq $format)) {
		push(@newbanners,$ad);
	    }
	}
	return \@newbanners;
}
# Filter ads by weights
sub filterByWeight{
	my ($this,$bannerads) = @_;
	# Separate ads into:
	# - display_always: weight >= 10
	# - display_random: weight < 10
	my (@display_always,@display_random);
	foreach my $ad(@$bannerads){
		if ($ad->{weight} >= 10){
			push(@display_always,$ad);
		}else{
			# if no weight is specified, the weight is presumed to be 1
			$ad->{weight} = $ad->{weight} || 1;
			push(@display_random,$ad);
		}
	}
	return (\@display_always,\@display_random);
}
# Filter displayed ads
sub filterByDisplayedID{
	my ($this,$bannerads) = @_;
	# Skip the check if no ads are displayed on the page
	return $bannerads if !exists($this->{displayed_id});
	# Get the list of displayed ads from $this->{displayed_id}
	my $idlist = join("|",keys(%{$this->{displayed_id}}));
	my @newlist;
	# Construct a list of ads that has not being displayed on the page
	foreach my $ad(@$bannerads){
		if ($ad->{bannerad_id} !~ /$idlist/){
			push(@newlist,$ad);
		}
	}
	return \@newlist;
}
# Adjust ad's balance
sub adjustBalance{
	# $type: banner ad type
	my ($this,$bannerad,$type) = @_;
	# Only adjust the balance on the ad if it matches the $type
	return if $bannerad->{bannerad_type} !~ /$type/i;
	my $cost = $bannerad->{balance} - $bannerad->{rate};
	$this->{db}->update("bannerad",{balance=>$cost},{bannerad_id=>$bannerad->{bannerad_id}});
	return;
}
# Adjust balance for the pay per view ad
sub adjustBalancePayPerViewAd{
	my ($this,$bannerad) = @_;
	return $this->adjustBalance($bannerad,"pay-per-view");
}
# Adjust balance for the pay per click ad
sub adjustBalancePayPerClickAd{
	my ($this,$bannerad) = @_;
	return $this->adjustBalance($bannerad,"pay-per-click");
}
# Show banners on the page
sub showBanner{
	my ($this,$bannerad) = @_;
	# Adjust balance for pay per view ads
	$this->adjustBalancePayPerViewAd($bannerad);	
	my $link = "";
	# Click through link
	if ($bannerad->{url}){
		$link = "$this->{cgi}/bannerad.cgi?bannerad_id=$bannerad->{bannerad_id}";
	}
	# get format dimensions
	my $format_css;
	if ($bannerad->{format}) {
	    $this->fetchFormats;
	    my $x = $format{$bannerad->{format}}{width};
	    my $y = $format{$bannerad->{format}}{height};
	    $format_css = " style=\"width:$x; height:$y\"";
	}
	my $out .= "<div class=\"BannerAd\"$format_css>";
	# Use banner image
	if ($bannerad->{fdata}){
		$out .= $link ? "<a href=\"$link\" target=\"bannerad\">" : "";
		#$out .= $this->{db}->show_data("bannerad","fdata",$bannerad->{fdata},$bannerad->{bannerad_id});
		$out .= $this->getBannerHTML($bannerad);
		$out .= $link ? "</a>" : "";
	# Use text
	}
	else{
		$out .= "<div class=\"BannerAdText\">\n";
		$out .= $link ? "<a href=\"$link\" target=\"bannerad\">" : "";
		$out .= $bannerad->{adata};
		$out .= $link ? "</a>" : "";
		$out .= $bannerad->{ddata} ?  "<br>$bannerad->{ddata}" : "";
		$out .= "</div>\n";
	}
	$out .= "</div>";
	return $out;
}
# Process banner click through
sub processBannerClick{
	my ($this,$banneradid) = @_;
	# Fetch the ad
	my %ad = $this->{db}->fetch("bannerad",$banneradid);
	# Adjust balance for pay per click ad
	$this->adjustBalancePayPerClickAd(\%ad);
	# log the click through info if the ad has the logging option enabled
	if ($ad{logging} =~ /on/i){
		my %insert = (
			bannerad_id=>$ad{bannerad_id},
			viewer_ip=>$ENV{REMOTE_ADDR},
			referrer=>$ENV{HTTP_REFERER},
		);
		$this->{db}->insert("bannerad_log",\%insert);
	}
	# Fix the ad url
	my $url = $ad{url} =~ /^http/i ? "" : "http://";
	$url .= $ad{url};
	# Use meta refresh for redirection
	my $out = "content-type: text/html\n\n
<html>
<head>
<meta http-equiv=\"refresh\" content=\"0;url=$url\">
</head>
<body>
</body>
</html>";
	return $out;
}
# Display banners on the screen
sub displayBanner{
	my ($this,$format,$count) = @_;
	# Skip if the page object doesn't exist
	return if !exists($share{Page});
	if ($count < 1) { $count = 1; }
	# Get section_id
	my $sectionid = $share{Page}->id("section");
	# Fetch ads
	#my $sql = "select * from bannerad where status = 'active' and (section_id = $sectionid or section_id = 0 or section_id is null)";
	my $sql = "select * from bannerad where status = 'active' and (section_id = ? or section_id=0)";
	my @bannerads = $this->{db}->custom_query($sql,$sectionid);
	# Remove incompatible formats
	@bannerads = @{$this->filterByFormat(\@bannerads,$format)};
	# Remove displayed ads
	@bannerads = @{$this->filterByDisplayedID(\@bannerads)};
	# Remove inactive ads
	@bannerads = @{$this->filterByStatus(\@bannerads)};
	# Separate adds into always displayed ads and randomly displayed ads
	my ($display_always,$display_random) = $this->filterByWeight(\@bannerads);
	my %display_random;
	# Weight total for the random ads
	my $weight_total = 0;
	# Calculate the weight total
	foreach my $ad (@$display_random){
		$display_random{$ad->{bannerad_id}} = $ad;
		$weight_total += $ad->{weight};
	}
	# Put selected banners into @out_banner
	my @out_banner;
	if ($count > @bannerads) { $count = @bannerads; }
	for(my $i = 0; $i < $count; $i++){
		# Always selected ads(weight >= 10)
		if (($i + 1) <= @$display_always){
			# Sort ads by weight then id
			@$display_always = sort{($b->{weight} <=> $a->{weight}) || ($a->{bannerad_id} <=> $b->{bannerad_id})} @$display_always;
			# Select the ads one by one
			for(my $i = 0; $i < $count; $i++){
				# Put the ad into @out_banner
				push(@out_banner,$this->showBanner($display_always->[$i]));
				# Flag the ad as displayed
				$this->{displayed_id}{$display_always->[$i]{bannerad_id}} = 1;
			}
		# Randomly selected ads(weight < 10)
		# 
		# Ad selection is done by choosing the position($ad_position) that 
		# the ad locates on the list($weight_total: total number of positions).
		# The number of positions that an ad may take on the list is determined by its weight.
		#
		}else{
			# Generate a random number
			my $random = int(scalar(@bannerads) * rand(10000));
			# Position of the ad that we are going to select
			my $ad_position = ($random % $weight_total) + 1;
			# Track the ad position.
			my $count = 0;
			# Loop through the hash %display_random
			foreach my $id(keys(%display_random)){
				# Accumulate the counter
				$count  = $count + $display_random{$id}{weight};
				# Once the counter reaches the position($count >= $ad_position), the ad is selected.
				if ($count >= $ad_position){
					push(@out_banner,$this->showBanner($display_random{$id}));
					# Remove the weight of the selected ad from $weight_total
					$weight_total = $weight_total - $display_random{$id}{weight};
					# Remove the ad from the hash
					delete $display_random{$id};
					# Flag the ad as displayed
					$this->{displayed_id}{$id} = 1;
					# Exit the loop
					last;
				}
			}
		}
	}
	return join($config{BannerAd}{join_ads}."\n",@out_banner);
}
#----------------------------------------------------------------------
# Delete confirmation popup
sub javascript{
	my ($this) = @_;
	my $delete_url = $this->link(action=>"delete_bannerad");
	return <<END;
<script language="javascript">
function deleteConfirm(id){
	if (confirm('Are you sure want to delete this record?')){
		location.href = '${delete_url}&bannerad_id=' + id;
		}
	}
</script>
END
}
# Build reports using ExSite::ReportBuilder
sub reportbuilder{
	my ($this,$opt) = @_;
	require ExSite::ReportBuilder;
	$this->{rb} = new ExSite::ReportBuilder() if !$this->{rb};
	my $rb = $this->{rb};
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
 	return $rb->make();
}
# Page redirection
sub redirect{
        my ($this,$reply) = @_;
        if (exists ($share{Page})){
                $share{Page}->redirect($reply);
        }
        print "location: $reply\n\n";
        return;
}
#----------------------------------------------------------------------

sub site_license {
    my ($this,$site) = @_;
    return 0 if (! $share{DB});
    if ($this->ioctl("isRestricted")) {
	# this service requires licenses
	if (!$site && $share{Page}) {
	    $site = $share{Page}->id("section");
	}
	if ($site) {
	    my $licenses = $this->{db}->fetch_match("service",
						    {section_id => $site,
						     name => "BannerAd" }
						    );
	    return (scalar @$licenses > 0);
	}
	# hmm, that's strange, we're not on a page
	return 0;
    }
    # this modules is not a service - anyone can use it
    return 1;
}

1;
