package Modules::Calendar;

use strict;
use ExSite::Config;
use ExSite::Input;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::Image;
use ExSite::Module qw(&site_license);
use ExSite::Time;
# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD ExSite::ContentData);

use vars qw($ml @wdy @month @dayinmonth $notification);

@wdy = qw(Sun Mon Tue Wed Thu Fri Sat);
@month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
@dayinmonth = qw(31 28 31 30 31 30 31 31 30 31 30 31);

$notification = "Name:       [[name]]
Type:       [[type]]
Begins:     [[start]] [[tstart]]
Ends:       [[end]] [[tend]]
Place:      [[place]]
Host:       [[host]]
More Info:  [[url]]
Register:   [[reg_url]]
Access:     [[access]]
Status:     [[status]]

Brief Description: [[description]]

Full Text:
----------
[[body]]
";

sub write {
    my ($this,$options) = @_;
	
    my $in = new ExSite::Input;
    $ml = new ExSite::ML;
    my $args = $in->combine();
    my %args = %$args;
    $this->{event_table} = $config{$this->module_name}{event_table} || "event";

    $this->setup_querylib();

    # see if we are showing events from someone else's calendar
    if ($options =~ /section_id=(\d+)/) {
	$this->{share_section} = $config{Calendar}{share_calendars} ? $1 : 0;
    }

    # predefined action
    if ($options =~ /upcoming/) {
	return $this->wrap($this->list_upcoming);
    }
    elsif ($options =~ /past/) {
    #if ($options =~ /past/) {
	return $this->wrap($this->list_past);
    }	
    elsif ($options =~ /next/) {
	return $this->wrap($this->next_event($options));
    }
    elsif ($options =~ /thismonthfull/) {
    	return $this->wrap($this->thismonth_fullscreen($options));
    }
    elsif ($options =~ /newevent/) {
	# new event action hard-coded in oyster
	if ($this->admin) {
	    if ($in->{caladm} eq "doform") {
		return $this->doform;
	    }
	    return $this->newevent;
	}
	return $msg{"New Event: Permission Denied."};
    }
    elsif ($args{sid} && $args{calendar} eq "reg" && $args{evtid}){
    	return $this->wrap($this->registration_form($args{sid},$args{evtid}));
    }
    
    my $db = $share{DB}; 
    my $in = $this->{input};

    # adminstrator action
    if ($this->admin) {
	# new event action selected from control panel
	if ($in->{caladm} eq "new") {
	    return $this->newevent;
	}
	elsif ($in->{caladm} eq "doform") {
	    return $this->doform;
	}
    }

    # fetch selected event
    if ($in->{evtid}) {
	$this->fetch($in->{evtid});
    }

    # select date to show
    if (! $args{date} && $options =~ /thismonth/) {
	my $today = $this->today;
	$today =~ s/-\d{1,2}$//;
	$args{date} = $today;
    }
    my ($y,$m,$d);
    if ($args{date}) {
	# explicit
	($y,$m,$d) = split /-/,$args{date};
	if (! $d) { $d = 0; }
    }
    elsif ($this->{event}) {
	# use event date
	($y,$m,$d) = split /-/,$this->{event}{start};
    }
    else {
	# no date - use today
	($y,$m,$d) = $this->today;
    }
    # save date info for other methods
    $this->{date} = [$y,$m,$d];

    # fetch events for the selected month
    my @events = $this->{share_section} ?
	$this->fetch_events_by_date($y,$m,undef,undef,$this->{share_section}) : 
	$this->fetch_events_by_date($y,$m);

    # show the month

    my $out = $this->show_month(\@events);

    # list events

    if ($in->{evtid}) {
	# show selected event
	$out .= $this->show;
    }
    elsif ($args{date}) {
	$out .= $this->list($args{date});
    }
    else {
	$out .= $this->list_upcoming;
    }

    return $this->wrap($out);
}

sub admin {
    my $this = shift;
    return ($config{Calendar}{new_event_access} =~ /public/) ||
	($config{Calendar}{new_event_access} =~ /users|members/ && 
	 $share{DB}->authorize);
}

sub wrap {
    my ($this,$out) = @_;
    return $ml->div($out, { class => "Calendar" });
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ModuleName/) {
	return "Event Calendar";
    }
    elsif (/isRestricted/) {
	return defined $config{Calendar}{isRestricted} ?
	    $config{Calendar}{isRestricted} : 1;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/Publish/) {
	return \&publish;
    }
    elsif (/DynContentParameter/) {
	$ml = new ExSite::ML;
	my $selectOptions = $ml->select([
			      		        ["option", "(default)", { value => "" }],
					        ["option", "Upcoming Events", { value => "upcoming" }],
					        ["option", "Next Event", { value => "next" }],
					        ["option", "What's Happening This Month", { value => "thismonth" }],
					        ["option", "What's Happening This Month (Full Screen)", { value => "thismonthfull" }],
					        ["option", "New Event Form", { value => "newevent" }],
			      		      ],
			      		      { name => "param" });
	
	return $ml->Element("p", $ml->b("Option:") . $selectOptions);
    }
    elsif (/ModuleInfo/) {
	return "The Calendar tool manages and displays an event calendar.
Users can navigate the calendar by month, or visit particular days that
contain events.  Individual events may be visited, to view event details,
view event web pages, or download brochures or other info.";
    }
    return;
}

sub show {
    my $this = shift;
    my $options = shift;
    my $e = $this->{event};
    
    return if (! $e or $e->{status} eq "hidden");
    
    
    my ($out,$img);
    if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Calendar/$e->{type}.gif") {
	my $image = $ml->img(undef, { 
					    src => $config{server}{HTMLpath} . "/_Modules/Calendar/" . $e->{type} . ".gif",
					    align => "middle",
			    		  }
			    	  );
	my $span = $ml->span($e->{type}, { style => "font-size:8pt;" });
	
   	$img = $ml->table(
			$ml->tr(
				$ml->td($image . $ml->br() . $span, { align => "center" })
			),
		{ align => "right", style => "padding: 12pt;" });
    }
    $out .= $ml->h2($img . $this->show_data("name"));
    if ($e->{status} eq "canceled") {
    	$out .= $ml->h3($msg{"THIS EVENT HAS BEEN CANCELED"}, { class => "error" });
    }
    
    my $eventDetails;
    if ($e->{tstart} !~ /00:00:00/) {
	my $time = $this->show_time($e->{tstart},$e->{tend});
    	$eventDetails .= $ml->br() . $msg{'Time:'} . " " . $ml->i($time);
    }
    if ($e->{place}) {
    	$eventDetails .= $ml->br() . $msg{'Place:'} ." " . $ml->i($this->show_data("place"));
    }
    if ($e->{host}) {
    	$eventDetails .= $ml->br() . $msg{'Host:'} . " " . $ml->i($this->show_data("host"));
    }
    
    $out .= $ml->h4($msg{"Date:"} . " " . $ml->i($this->show_date($e->{start},$e->{end})) . $eventDetails);
    
    $out .= $ml->p($this->show_data("description"));
    $out .= $ml->p($this->show_data("body"));
    if ($e->{upload}) {
    	$out .= $ml->p($msg{"Brochure:"} . " " . $this->show_data("upload"));
    }
    if ($e->{url}) {
    	$out .= $ml->p($msg{"More Information:"} . " " . $this->show_data("url"));
    }
    if ($e->{photo}) {
	my $img = new ExSite::Image($e->{photo});
	my $filename = $img->name;
	my $imgurl = "$config{server}{HTMLpath}/_Modules/Calendar/$e->{event_id}/$filename";
	if (! -e "$config{server}{HTMLroot}$imgurl") {
	    $imgurl = "$config{server}{CGIpath}/getdata.cgi/$filename?tab=$this->{event_table}&col=photo&id=$e->{event_id}";
	}
    	$out .= $ml->p($ml->img(undef, { src => $imgurl }));
    }
    # don't show the 'register...' link for canceled and past events,
    # or for admin view
    if ($this->{admin}) {
    	$out .= $ml->p($ml->i("[registration link will be placed here]"));
    }
    else {
	if ($e->{status} ne "canceled" || !$this->past_event_check($e->{start})){
	    if ($e->{survey_id}){
	    	$out .= $ml->p($ml->a($msg{"Click here to register"}, { href => $this->link(sid=>$e->{survey_id},calendar=>"reg",evtid=>$e->{event_id}) }));
	    }elsif ($e->{reg_url}) {
	    	$out .= $ml->p($ml->a($msg{"Click here to register"}, { href => $e->{reg_url} }));	
	    }
	}
    }
    if( $options =~ /thismonthfull/ )
    {
    	my $returnUrl = $this->link( date => join("-", @{$this->{date}}), evtid => undef, viewtype => undef );
    	$out .= $ml->br() . $ml->br() . $ml->a($msg{"Back to Calendar"}, { href => $returnUrl }) . $ml->br() . $ml->br();
    }
    return $out;
}

# past event check
sub past_event_check{
    my ($this,$startdate) = @_;
    return 0 if !$startdate;
    return 0 if $startdate gt $this->today();
    return 1;
}

sub show_data {
    my ($this,$col) = @_;
    
    
    return undef if (! $this->{event});
    if ($col eq "upload" && $this->{event}{$col}) {
	my $img = new ExSite::Image($this->{event}{upload});
	my $filename = $img->name;
	my $pubpath = "$config{server}{HTMLpath}/_Modules/Calendar/$this->{event}{event_id}/$filename";
	if (-e "$config{server}{HTMLroot}$pubpath") {
	    #return "<a href=\"$pubpath\">$filename</a>";
	    return $ml->a($filename, { href => $pubpath });
	}
    }
    elsif ($col eq "photo" && $this->{event}{$col}) {
	my $img = new ExSite::Image($this->{event}{photo});
	my $filename = $img->name;
	my $pubpath = "$config{server}{HTMLpath}/_Modules/Calendar/$this->{event}{event_id}/$filename";
	if (-e "$config{server}{HTMLroot}$pubpath") {
	    #return "<p><img src=\"$pubpath\"></p>\n";
	    return $ml->p($ml->img(undef, { src => "$pubpath" }));
	}
    }
    return $share{DB}->show_data($this->{event_table},$col,$this->{event}{$col},
				 $this->{event}{event_id});
}

sub list {
    my $this = shift;
    my $date = shift;
    
    my (@events,$eventspan);
    my $db = $share{DB}; 
    if (! $date) {
	# no date requested - select whole month
	@events = $this->fetch_events_by_date($this->{date}[0],$this->{date}[1],undef,undef,$this->{share_section});
	$eventspan = "month";
    }
    elsif ($date =~ /^\d+-\d+$/) {
	# a whole month was requested
	@events = $this->fetch_events_by_date($this->{date}[0],$this->{date}[1],undef,undef,$this->{share_section});
	$eventspan = "month";
    }
    else {
	# specific day was selected
	@events = $this->fetch_events_by_date($this->{date}[0],$this->{date}[1],$this->{date}[2],undef,undef,$this->{share_section});
	$eventspan = "date";
    }

    my $out .= $ml->h1($msg{$config{Calendar}{heading}{calendar}});
    
    if (@events) {
	$out .= $ml->p($msg{"For more details, click on the event link."});
	my $eventSummaries;
	foreach my $evt (@events) {
	    #$out .= $this->summarize_event($evt);
	    $eventSummaries .= $this->summarize_event($evt);
	}
	$out .= $ml->dl($eventSummaries);
    }
    else {
    	$out .= $ml->p(&substitute($msg{"No events were found for this [[evtspan]]"},{evtspan=>$eventspan}));
    }
    return $out;
}

sub list_upcoming {
    my $this = shift;
    my $db = $share{DB}; 
    my $date = $this->today;
    my $limit = $config{Calendar}{max_upcoming} || 10;
    

    my $out .= $ml->h1($msg{$config{Calendar}{heading}{upcoming}});

    my @events = $this->fetch_events($date,$limit,$this->{share_section});

    if (@events) {
	$out .= $ml->p($msg{"For more details, click on the event link."});
	my $eventSummaries;
	foreach my $evt (@events) {
	    #$out .= $this->summarize_event($evt);
	    $eventSummaries .= $this->summarize_event($evt);
	}
	$out .= $ml->dl($eventSummaries);
    }
    else {
    	$out .= $ml->p($msg{"No upcoming events."});
    }
    return $out;
}

sub thismonth_fullscreen {
    my ($this, $options) = @_;
	
    my $in = new ExSite::Input;
    
    my $args = $in->combine();
    my %args = %$args;
    $this->{event_table} = $config{$this->module_name}{event_table} || "event";
   
    my $db = $share{DB}; 
    my $in = $this->{input};

    # fetch selected event
    if ($in->{evtid}) {
	$this->fetch($in->{evtid});
    }

    my ($y,$m,$d);
    if ($args{date}) {
	# explicit
	($y,$m,$d) = split /-/,$args{date};
	if (! $d) { $d = 0; }
    }
    elsif ($this->{event}) {
	# use event date
	($y,$m,$d) = split /-/,$this->{event}{start};
    }
    else {
    	# no date - use today
	($y,$m,$d) = split /-/,$this->today;
    }
    # save date info for other methods
    $this->{date} = [$y,$m,$d];

    # fetch events for the selected month
    my @events = $this->fetch_events_by_date($y,$m,undef,undef,$this->{share_section});

    if ($in->{evtid}) {
	# show selected event
	return $this->show($options);
    }
    elsif ( $args{viewtype} =~ /view_more/ ) {
    	my $returnUrl = $this->link( date => "$y-$m-$d", evtid => undef, viewtype => undef );
    	return $this->list($args{date}) . $ml->br() . $ml->br() . 
    		$ml->a($msg{"Back to Calendar"}, { href => $returnUrl, class => "BackToCalendar" }) . 
    		$ml->br() . $ml->br();
    }
    else {
    	return $this->show_month(\@events, $options);
    }
}

sub list_past {
    my $this = shift;
    my $db = $share{DB}; 
    my $date = $this->today;
    my $limit = $config{Calendar}{max_upcoming} || 10;
    

    my $out .= $ml->h1($msg{$config{Calendar}{heading}{upcoming}});

    my @events = $this->fetch_past_events($date,$limit,$this->{share_section});

    if (@events) {
	$out .= $ml->p($msg{"For more details, click on the event link."});
	my $eventSummaries;
	foreach my $evt (@events) {
	    $eventSummaries .= $this->summarize_event($evt);
	}
	$out .= $ml->dl($eventSummaries);
    }
    else {
    	$out .= $ml->p($msg{"No upcoming events."});
    }
    return $out;
}

sub next_event {
    my ($this,$options) = @_;
    my $db = $share{DB}; 
    my $date = $this->today;
    my @events = $this->fetch_events($date,1,$this->{share_section});
    
    if (@events) {
	$this->{event} = $events[0];
	my $out;
	if ($options !~ /noheader/) {
		$out .= $ml->h3($msg{$config{Calendar}{heading}{next}});
	}
	$out .= $this->show;
	return $out;
    }
    return $ml->p($msg{"No upcoming events."});
}

sub summarize_event {
    my ($this,$evt) = @_;
    my ($out,$img,$url);
    $this->{event} = $evt;
    my $edate = $this->show_date($evt->{start},$evt->{end});
    
    $url = $this->link(evtid=>$evt->{event_id});
    $out .= $ml->dt($ml->b($edate));
    
    my $eventInfo;
    $eventInfo .= $ml->a($this->show_data("name"), { href => $url }) . $ml->br();
    if ($evt->{host}) {
	my $hostmsg = $config{Calendar}{message}{hosted_by} || "Hosted by [[host]].";
	$eventInfo .= &substitute($msg{$hostmsg},{host=>$this->show_data("host")}) . $ml->br();
    }
    $eventInfo .= $this->show_data("description") . $ml->p();
    
    $out .= $ml->dd($img . $eventInfo);
    
    return $out;
}

# show_month : display calendar month, with links to events

sub show_month {
    my ($this,$events,$options) = @_;
    my ($sec,$min,$hr,$mday,$mon,$year,$wday,$yday,$isdst);
    my ($time,$today);
    my @today = $this->today;
    my @events = @$events;
    

    # pick the date that the calendar should reflect

    if ($this->{date}) {
	# explicit date
	my ($y,$m,$d) = @{$this->{date}};
	if ($y < 1950 || $y > 2050 || $m < 1 || $m > 12) {
	    return $msg{"(invalid date)"};
	}
	use Time::Local;
	my $day = $d =~ /^0+$/ ? 1 : $d;
	$m--;
	$time = timelocal(0,0,12,$day,$m,$y);
    }
    else {
	# deprecated
	$today = 1;
	$time = time();
    }

    # convert date to localtime() values

    ($sec,$min,$hr,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($time);
    # convert to YYYY-MM-DD format
    $year += 1900;
    my $realmonth = $mon + 1;
    if ($realmonth < 10) { $realmonth = "0$realmonth"; }
    my $realmday = $mday;
    if ($realmday < 10) { $realmday = "0$realmday"; }
    my $leap = ((($year % 4) == 0) && (($year % 100) != 0)) ||
	(($year % 400) == 0);
    if ($leap) { $dayinmonth[1] += 1; }

    # flag the days with events
    my @dayflags;
    foreach my $evt (@events) {
	my ($sy,$sm,$sd) = split /-/, $evt->{start};
	my ($ey,$em,$ed) = split /-/, $evt->{end};
	
	#next if ($sm != $realmonth && $em != $realmonth);
	next unless ($sy <= $year && $sm <= $realmonth && $ey >= $year && $em >= $realmonth) ||
		    ($sy < $year && $ey > $year);
	
	my $beginday = ($sm == $realmonth ? $sd : 1);
	my $endday = ($em == $realmonth ? $ed : $dayinmonth[$mon]);

	if ( $ey > $year )
	{
		$endday = $dayinmonth[$mon];
	}
	
	foreach my $eventday ($beginday..$endday) {
	    $dayflags[$eventday] = 1;
	}
    }

    # build the calendar HTML

    my $out;
    # setup links to last month and next month
    my ($lastmonthurl, $nextmonthurl);
    my ($lastyear,$lastmonth) = my ($nextyear,$nextmonth) = ($year,$mon);
    # lastmonth already good, since $mon goes from 0..11
    # must add 2 to next month to get the real next month, tho
    $nextmonth += 2;
    if (! $lastmonth) {
	$lastyear--;
	$lastmonth = 12;
    }
    if ($nextmonth > 12) {
	$nextyear++;
	$nextmonth = 1;
    }
    if ($lastmonth < 10) { $lastmonth = "0$lastmonth"; }
    if ($nextmonth < 10) { $nextmonth = "0$nextmonth"; }
    
    my $lastmonthurl = $this->link(date=>"$lastyear-$lastmonth",evtid=>undef);
    my $nextmonthurl = $this->link(date=>"$nextyear-$nextmonth",evtid=>undef);
    
    my $url = $this->link(evtid=>undef);
    my $monthOptions;
    my $date_span = $config{Calendar}{date_selector_range} || 3;
    my $frommonth = $mon - $date_span;
    my $tomonth = $mon + $date_span;
    for (my $imon = $frommonth; $imon <= $tomonth; $imon++) {
	my $y = $year;
	my $dmon = $imon + 1;
	if ($dmon < 1) { $y--; $dmon += 12; }
	if ($dmon > 12) { $y++; $dmon -= 12; }
	my $tmpmon = $dmon - 1;
	my $selected;
	if ($imon == $mon) { $selected .= " selected"; }
	
	$monthOptions .= $ml->option($msg{$month[$tmpmon]} . " " . $y, { value => $y . "-" . $dmon, $selected => undef });
    }
    
    my $formmethod = $config{Calendar}{change_month_method} || "POST";
    my $formcontents = $ml->select($monthOptions, { name => "date", class => "Calendar", onchange => "document.calform.submit()" });
    $formcontents .= $config{page}{dyn_url_type} eq "id" ? 
	$ml->input(undef,{type=>"hidden",name=>"_id",value=>$this->{input}{_id}}) : "";
    my $monthselector = $ml->form($formcontents,{ method => $formmethod, action => $url, name => "calform" });

    my $tdDays;
    my $currentTdAttributes;
    my $currentWeeksTds;
    my $tdDay;
    my $first_wday = $wday - ($mday % 7);
    if ($first_wday < 0) { $first_wday += 7; }
    my $weekday = 0;
    if ($first_wday < 6) {  ## kludge
	foreach (0..$first_wday) {
	    $currentWeeksTds .= $ml->td();
	    $weekday++;
	}
    }
    foreach my $d (1..$dayinmonth[$mon]) {
	if ($weekday == 7) {
	    $tdDays .= $ml->tr($currentWeeksTds, { bgcolor => "#C0C0C0" });
	    $weekday = 0;
	}
	my $day = $d < 10 ? "0$d" : $d;
	if ($weekday == 0) {
	    $currentWeeksTds = "";
	}
	if ($dayflags[$d]) {
	    $currentTdAttributes = { class => "CalendarEvent", bgcolor => "#FFCCCC" };
	}
	elsif ($d == $mday && $today[1]==$this->{date}[1] && $today[2]==$this->{date}[2]) {
	    # today is yellow
	    $currentTdAttributes = { class => "CalendarToday", bgcolor => "#FFFFCC" };
	}
	else {
	    # everything else is grey
	    $currentTdAttributes = {};
	}
	if ($dayflags[$d]) {
	    if( $options =~ /thismonthfull/ )
	    {
	        my $limit = $config{Calendar}{max_fullview_list_size} || 3;
	        my @todaysEvents = $this->fetch_events_by_date($year, $realmonth, $day, $limit);
	        my @eventCount = $this->fetch_events_by_date($year, $realmonth, $day);
	        my $eventLinks;
		
		my $viewAllUrl = $this->link( date => "$year-$realmonth-$day", evtid => undef, viewtype => "view_more");
	        my $currentDay = $ml->a($d, { href => $viewAllUrl, class => "CalendarDayLink" });
	        
	        foreach my $evt ( @todaysEvents )
	        {
	    	    $this->{event} = $evt;
	    	    my $url = $this->link( date => "$year-$realmonth-$day", evtid => $evt->{event_id} );
	    	    $eventLinks .= $ml->a($this->show_data("name"), { href => $url }) . $ml->br();
	        }
	    
	        if ( @eventCount > @todaysEvents )
	        {
	    	    my $url = $this->link( date => "$year-$realmonth-$day", evtid => undef, viewtype => "view_more" );
	            $eventLinks .= $ml->br() . $ml->a("View all events ...", { href => $url, class => "CalendarViewMoreLink" });
	        }
	        
	        $tdDay = $ml->table($ml->tr($ml->td($currentDay, { class => "CalendarCurrentDay" })) .
	        		    $ml->tr($ml->td($eventLinks, { class => "CalendarCurrentEvents" })), { class => "CalendarCurrentDayTable" });
	    }
	    else
	    {
	        my $url = $this->link(date=>"$year-$realmonth-$day",evtid=>undef);
	        $tdDay = $ml->a($d, { href => $url });
	    }
	}
	else {
	    $tdDay = $d;
	}
	$currentWeeksTds .= $ml->td($tdDay, $currentTdAttributes);

	$weekday++;
    }
    if ($weekday != 7) { 
	foreach my $d ($weekday+1 .. 7) { $currentWeeksTds .= $ml->td(); } #$out .= "<td></td>"; }
    }
    $tdDays .= $ml->tr($currentWeeksTds, { bgcolor => "#C0C0C0" });
    
    $out .= $ml->table($ml->tr($ml->th($monthselector, { colspan => "7", bgcolor => "#669999" })) .
    		       $ml->tr($ml->th($msg{"Su"}) . $ml->th($msg{"Mo"}) . $ml->th($msg{"Tu"}) . $ml->th($msg{"We"}) . $ml->th($msg{"Th"}) . $ml->th($msg{"Fr"}) . $ml->th($msg{"Sa"}), { bgcolor => "#CCCCFF" }) .
    		       $tdDays, 
    	    	       { cellspacing => "1", cellpadding => "0", border => "0", class => "Calendar", align => "right" });
    return $out;
}

sub today {
     my $t = new ExSite::Time;
     
     my $today = $t->write("sql_date");
     
     if (wantarray) {
	return split("-", $today);
     }
     
     return $today;
}

# compare_date:
#   returns 1 if date1 > date2
#           0 if date1 == date2
#          -1 if date1 < date2

sub compare_date {
      my ($this, $date1, $date2) = @_;
      my $t = new ExSite::Time;
      return $t->compare2($date1, $date2, "sql_date");
}

# show date/time : display a datetime, but with more logic than the default
# method in Report::show_data

sub show_date {
     my ($this, $date1, $date2) = @_;
     return "" if (!$date1);
     
     my $t1 = new ExSite::Time($date1, "sql_date");
     my $t2 = new ExSite::Time($date2, "sql_date");
    
     if ($t2->write("sql_date") && $t2->write("sql_date") ne $t1->write("sql_date")) {
 	if ($t1->write("year") != $t2->write("year") || $t1->write("raw_month") != $t2->write("raw_month")) {
 	    return $t1->write("date") . " - " . $t2->write("date");
 	}
 	else {
 	    return $t1->write("month") . " " . $t1->write("raw_day") . " - " . $t2->write("raw_day") . ", " . $t1->write("year");
 	}
     }
     else {
        return $t1->write("date");
     }     
}

sub show_time {
    my ($this,$time1,$time2) = @_;
    
    my $t1 = new ExSite::Time($time1, "time");
    my $t2 = new ExSite::Time($time2, "time");
    
    my $out;
    
    if ($t1->write("time") != /^[\s0]0:00/) {
	$out = $t1->write("time");
    }
    
    if ($t2->write("time") ne $t1->write("time") && $t2->write("time") != /^[\s0]0:00/) {
	$out .= " - " . $t2->write("time");
    }
    
    return $out;
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    my $db = $share{DB};
    $ml = new ExSite::ML;
    $this->setup_querylib();

    $this->{admin} = 1;  # set admin flag to add admin tools

    my $url = "$config{server}{CGIpath}/ctrl-panel.cgi/Calendar";
    $_ = $in->{caladm};
	$this->{event_table} = $config{$this->module_name}{event_table} || "event";

	if ($in->{evtid}) {
	$this->fetch($in->{evtid});
    	return $ml->p("Invalid event ID.", { class => "error" }) if (! $this->{event});
    }

    if (/new/) {
	return $this->newevent;
    }
    elsif (/edit/) {
	my $action = $this->link(caladm=>"doform");
	$out .= $ml->h1($config{Calendar}{heading}{edit});
	$this->calendar_handler();
	$out .= $db->make(table=>$this->{event_table},
			  record=>$in->{evtid},
			  action=>$action,
			  extra=>{caladm=>"doform"},
			  );
	return $out;
    }
    elsif (/doform/) {
	return $this->doform;
    }
    elsif (/delete/) {
	$share{DB}->trash_key($this->{event_table},$in->{evtid});
	# errorcheck?
    }
    elsif (/show/) {
	if ($in->{evtid}) {
	    $this->fetch($in->{evtid});
	    if ($this->{event}) {
		$out .= $this->show;
		my $edit = $this->link(caladm=>"edit");
		my $delete = $this->link(caladm=>"delete");

		$out .= $ml->p($ml->a($ml->img(undef, { src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{edit}{img}, border => "0" }), { href => $edit }) .
			       $ml->a($ml->img(undef, { src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{delete}{img}, border => "0" }), { href => $delete }));
	    }
	}
	else {
	    $out .= $ml->p("No event selected.", { class => "error" });
	}
    }
    $out .= $ml->h1($config{Calendar}{heading}{myevents});
    $out .= $this->set_section_id;
    $this->{section_id} = $this->get_section_id;
    $out .= $this->report;
    return $out;
}

sub newevent {
    my $this = shift;
    my $in = $this->{input};
    my $db = $share{DB};
    my $sid = $in->{section_id} || ($share{Page} ? $share{Page}->id("section") : 0);
    my $action = $this->link(caladm=>"doform");
    
    my $out .= $ml->h1($msg{$config{Calendar}{heading}{new}});
    $this->calendar_handler();
    $out .= $db->make(table=>$this->{event_table},
		      action=>$action,
		      data=>{section_id=>$sid,member_id=>$db->my_uid},
		      extra=>{caladm=>"doform"},
		      );
    return $out;
}

sub doform {
    my $this = shift;
    my $in = $this->{input};
    my $db = $share{DB};
    
    my $out;
    $db->get_data;
    # tweak the form data
    my @evtid = keys %{$db->{form}{$this->{event_table}}};
    my $evtid = $evtid[0] || "_";
    $db->{action} = (exists $db->{form}{$this->{event_table}}{$evtid}{event_id} ? 
		     "update" : "insert");
    if (! $db->{form}{$this->{event_table}}{$evtid}{status}) {
	# default the event status to active
	$db->{form}{$this->{event_table}}{$evtid}{status} = "active";
    }
    if (! $db->{form}{$this->{event_table}}{$evtid}{access}) {
	# default the event access to public
	$db->{form}{$this->{event_table}}{$evtid}{access} = "public";
    }
    # tie event to current site
    my $site = $in->{section_id} || $db->this_site()->{section_id};
    if ($db->{form}{$this->{event_table}}{$evtid}{section_id}) {
	# ownership check
	if (! $db->owns($db->{form}{$this->{event_table}}{$evtid}{section_id})) {
	    return $this->error($msg{"You do not have permission to add events to this website."});
	}
    }
    elsif (! $db->is_admin) {
	$db->{form}{$this->{event_table}}{$evtid}{section_id} = $site;
	$this->warn($msg{"Event changed to a local event - you do not have permission to add global events."});
    }

    # tie event to member
    if ($db->{form}{$this->{event_table}}{$evtid}{member_id}) {
	if (! $db->is_admin &&
	    $db->{form}{$this->{event_table}}{$evtid}{member_id} != $db->my_uid) {
	    $db->{form}{$this->{event_table}}{$evtid}{member_id} = $db->my_uid;
	}
    }
    elsif (! $this->admin) {
	$db->{form}{$this->{event_table}}{$evtid}{member_id} = $db->my_uid;
    }
    
    # scale photo to a reasonable size
    if ($db->{form}{$this->{event_table}}{$evtid}{photo}) {
	my $img = new ExSite::Image($db->{form}{$this->{event_table}}{$evtid}{photo});
	$img->scale(400,400,-1);
	$db->{form}{$this->{event_table}}{$evtid}{photo} = $img->encode;
    }
    
    # process the form
    $out = $db->do;

    # notification
    if ($config{Calendar}{notification}) {
	my $evt = $db->{form}{$this->{event_table}}{$evtid};
	my $whoami = $db->my_name;
	my $message = &substitute($notification,$evt);
	my $s = $db->fetch("section",$site);
	my $calendar_mgr = $db->owner_email($s);
	&SendMail($calendar_mgr,$calendar_mgr,
		  "$config{site}{name} - Calendar Update: $evt->{name}",
		  $message);
    }

    if ($out =~ /Done\!/) {
	my $url = $this->link(caladm=>undef,evtid=>undef);
	$out .= $ml->h3($msg{"Calendar Updated Successfully"}) . 
		$ml->p($ml->a($msg{"Click here to return to calendar view"}, { href => $url }) . ".");
    }
    return $out;
}

sub report {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    my $r = new ExSite::ReportBuilder();
    

    # shared events
    if ($db->is_admin) {
	$r->init(title=>"Shared Events",
		 headers=>[ "Date","Event","Type","Status","Options" ],
		 nodata=>"No events found.",
		 );

	my @e = $db->fetch_match($this->{event_table},
				 {section_id=>0},
				 {start=>"desc"} );
	    
	if (@e > 0) {
	    foreach my $e (@e) {
		$r->push($this->report_row($e));
	    }
	    my $new = $this->link(caladm=>"new",section_id=>0);
	    my $imageAttributes = { 
		    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{new}{img}, 
	    	    border => "0",
	    	    alt => "new event",
	    	    title => "new event",
	    };
	    $r->tools($ml->a($ml->img(undef, $imageAttributes), { href => $new }));
	    $out .= $r->make;
	    $out .= $ml->p();
	}
    }

    # site-specific events
    my $site = $db->fetch("section",$this->{section_id});
    $r->init(title=>$site->{title},
	     headers=>[ "Date","Event","Type","Status","Options" ],
	     nodata=>"No events found.",
	     );

    if (&site_license("Calendar",$site->{section_id})) {
	my @e = $db->fetch_child($this->{event_table},
				 "section",
				 $site->{section_id},
				 {start=>"desc"}
				 );
	
	if (@e > 0) {
	    foreach my $e (@e) {
		$r->push($this->report_row($e));
	    }
	}
	my $new = $this->link(caladm=>"new",section_id=>$site->{section_id});
	my $imageAttributes = {
		src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{new}{img},
		border => "0",
		alt => "new event",
		title => "new event",
	};
	$r->tools($ml->a($ml->img(undef, $imageAttributes), { href => $new }));
	$out .= $r->make;
	$out .= $ml->p();
    }
    return $out;
}

sub report_row {
    my ($this,$e) = @_;
    my @data;
    
    $this->{event} = $e;
    my $edit = $this->link(caladm=>"edit",evtid=>$e->{event_id});
    my $view = $this->link(caladm=>"show",evtid=>$e->{event_id});
    my $delete = $this->link(caladm=>"delete",evtid=>$e->{event_id});
    
    my $viewAttributes = { 
	    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{view}{img},
	    border => "0",
	    alt => "info",
	    title => "info"
    };
    my $editAttributes = { 
	    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{edit}{img},
	    border => "0",
	    alt => "edit",
	    title => "edit"
    };
    my $deleteAttributes = {
	    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{delete}{img},
	    border => "0",
	    alt => "delete",
	    title => "delete"
    };
    my $imageLinks = $ml->a($ml->img(undef, $viewAttributes), { href => $view }) .
    		     $ml->a($ml->img(undef, $editAttributes), { href => $edit }) .
    		     $ml->a($ml->img(undef, $deleteAttributes), { href => $delete });
    
    push @data, $this->show_date($e->{start},$e->{end})." ".$this->show_time($e->{tstart},$e->{tend});
    push @data, ($this->show_data("name"),
		 $this->show_data("type"),
		 $this->show_data("status"),
		 $imageLinks);
    return @data;
}

# calendar handler
sub calendar_handler{
	my $this = shift;
	my $db = $share{DB};
	$db->handler("select_foreign_key",\&handler_select_foreign_key);
	return;
}
# select foreign key handler
sub handler_select_foreign_key{
	my $this = shift;
	my $table = shift;
	my $rec = shift;
	my $col = shift;
	my $ftable = shift;
	my $ok = $this->authorize();
	if ($ok == 2){
		my $sql = "select * from survey where type = 'registration' ";
		my @sites = $this->my_real_sites();
		my @siteid;
		foreach my $site(@sites){
			push(@siteid,$site->{section_id});
		}
		$sql .= " and (section_id = ".join(" or section_id = ",@siteid).")";
		# fetch registration forms
		return $this->custom_query($sql);
	}
	return undef;
}
# SimpleRegistration form
sub registration_form{
	my $this = shift;
	my $surveyid = shift;
	my $eventid = shift;
	return "<!--&SimpleRegistration(action=do&sid=$surveyid&evtid=$eventid)-->";
}

### FETCHING EVENTS

sub setup_querylib {
    my ($this) = @_;
    my $db = $share{DB};
    $db->set_query("events after date",
		   ( 
		     sql => "select * from $this->{event_table} where start >= ? and status = \"active\" and (section_id=? or section_id=0) order by start limit ?",
		     nparam => 3,
		     mode => "r",
		     keys => $this->{event_table} )
		   );
    $db->set_query("events before date",
		   ( 
		     sql => "select * from $this->{event_table} where start < ? and status = \"active\" and (section_id=? or section_id=0) order by start desc limit ?",
		     nparam => 3,
		     mode => "r",
		     keys => $this->{event_table} )
		   );
    $db->set_query("my events",
		   ( 
		     sql => "select * from $this->{event_table} where start >= ? and status = \"active\" and (section_id=? or section_id=0) and member_id=? order by start limit ?",
		     nparam => 4,
		     mode => "r",
		     keys => $this->{event_table} )
		   );
}

sub fetch {
    my ($this,$id) = @_;
    my $db = $share{DB};
    my $e = $db->fetch($this->{event_table},$id);
    # validate that event belongs to this site
    if ($e->{section_id}) {
	if (! $this->{admin} && 
	    ! $config{Calendar}{share_calendars} &&
	    ($e->{section_id} != $share{DB}->this_site()->{section_id})) {
	    return undef; 
	}
    }
    $this->{event} = $e;
    return $e;
}

# fetch a number of events after a certain date

sub fetch_events {
    my ($this,$date,$limit,$section_id) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    if (! $limit) { $limit = $config{Calendar}{max_list_size} || 10; }
    $section_id or $section_id = $db->this_site()->{section_id};
    my @events = $db->get_query("events after date",$date,$section_id,$limit);
    return @events;
}

# fetch a number of events before a certain date

sub fetch_past_events {
    my ($this,$date,$limit,$section_id) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    if (! $limit) { $limit = $config{Calendar}{max_list_size} || 10; }
    $section_id or $section_id = $db->this_site()->{section_id};
    my @events = $db->get_query("events before date",$date,$section_id,$limit);
    return @events;
}

# same as fetch_events, but fetches member-owned events only

sub fetch_my_events {
    my ($this,$date,$limit,$section_id) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    if (! $limit) { $limit = $config{Calendar}{max_list_size} || 10; }
    $section_id or $section_id = $db->this_site()->{section_id};
    my $uid = $db->my_uid;
    my @events = $db->get_query("my events",,$date,$section_id,$uid,$limit);
    return @events;
}

# fetch_events_by_date : year, month required; day and limit are optional

sub fetch_events_by_date {
    my ($this,$y,$m,$d,$limit,$section_id) = @_;
    my (@events,@param,$st,$end,$cond);
    if ($d) { 
	$st = $end = "$y-$m-$d";
	$cond = "start <= ? and end >= ?";
	push @param, $st,$end;
    }
    else {
	$st = "$y-$m-01";
	$end = "$y-$m-".$ExSite::Report::month_days[$m];
	$cond = "((start >= ? and start <= ?) or (end >= ? and end <= ?) or (start <= ? and end >= ?))";
	push @param,$st,$end,$st,$end,$st,$end;
    }
    $section_id or $section_id = $share{DB}->this_site()->{section_id};
    $cond .= " and status=\"active\" and (section_id=? or section_id=0)";
    push @param, $section_id;
    my $limitCond;
    if ($limit) 
    {
    	$limitCond .= " limit ?"; 
	push @param, $limit;
    }
    
    @events = $share{DB}->custom_query("select * from $this->{event_table} where $cond order by start$limitCond",@param);
    
    return @events;
}

### PUBLISHING

sub publish {
    my ($this) = @_;
    $this->{event_table} = $config{$this->module_name}{event_table} || "event";
    my $in = new ExSite::Input;
    my $data = $in->query_or_post;
    $this->setup_querylib();
    if ($data->{evt_id}) {
	$this->publish_event($data->{evt_id});
    }
    else {
	my $today = $this->today;
	my @events = $share{DB}->custom_query("select event_id,upload,photo from $this->{event_table} where start >= \"$today\"");
	foreach my $evt (@events) {
	    $this->publish_event($evt);
	}
    }
}

sub unpublish {
    my ($this) = @_;
    my $in = new ExSite::Input;
    my $data = $in->query_or_post;
    if ($data->{evt_id}) {
	$this->rmdir("$config{server}{HTMLpath}/_Modules/Calendar/$data->{evt_id}");
    }
    else {
	# don't bother fetching events - just delete every subdir in the
	# calendar directory
	opendir D, "$config{server}{HTMLpath}/_Modules/Calendar";
	my @files = <D>;
	closedir D;
	foreach my $f (@files) {
	    if ($f =~ /^\d+$/ && -d $f) {
		$this->rmdir("$config{server}{HTMLpath}/_Modules/Calendar/$f");
	    }
	}
    }
}

sub publish_event {
    my ($this,$evt) = @_;
    my $db = $share{DB};
    my $event = ref $evt eq "HASH" ? $evt : $this->fetch($evt);
    my $out;
    my $path = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Calendar/$event->{event_id}";
    if ($event->{upload} || $event->{photo}) {
	my $mkdir = $this->mkdir($path);
	if (! $mkdir) {
	    return $this->show_diagnostics("error","html");
	}
	if ($event->{upload}) {
	    my $img = new ExSite::Image($event->{upload});
	    my $filename = $img->name;
	    my $pubpath = "$path/$filename";
	    $out .= $this->publish_file($pubpath,$img->get);
	}
	if ($event->{photo}) {
	    my $img = new ExSite::Image($event->{photo});
	    my $filename = $img->name;
	    my $pubpath = "$path/$filename";
	    $out .= $this->publish_file($pubpath,$img->get);
	}
    }
    return $out;
}

sub publish_file {
    my ($this,$path,$imgdata) = @_;
    my $out;

    $ml = new ExSite::ML;
    
    $path =~ /^(((\/[\w_-]+)+)\/([\w_-]+\.[\w]+))$/;
    my $secure_filename = $1;
    if ($secure_filename) {
	$out .= "Writing $secure_filename... ";
	if (open OUTFILE, ">$secure_filename") {
	    print OUTFILE $imgdata;
	    close OUTFILE;
	    #$out .= "succeeded.<br>\n";
	    $out .= "succeeded." . $ml->br();
	}
	else {
	    $out .= $this->error("Calendar::publish: publish $secure_filename failed: $!");
	}
    }
    else {
	$out .= $this->error("Calendar::publish: invalid filename $path");
    }
    return $out;
}

1;
