package Modules::FileServer;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::HTML;
use ExSite::Content;
use ExSite::Time;
use ExSite::Tree;
use ExSite::URI;
use Modules::BaseDCD;
use MIME::Base64;
use POSIX qw(ceil floor);

use CGI::Carp qw(fatalsToBrowser);
use File::Find;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD ExSite::ContentData);

# multiple section_id
# more data integrity checking
# recursive delete not allowed? conf maybe
# different row color for permission
# level 1's can't edit folders
# fix css tags and such

my $debug = 0;
if ($debug) {
	print "Content-type: text/html\n\n";
}

sub read {
    my ($this,$opt) = @_;
    # fetch overlayed query and form input
    my $in = new ExSite::Input;

	# set entire path
	$this->{in} = $in;
	$in->path_info();
	$this->{path_info} = $in->path();

	# claim object path 	
	# eventually we only want to claim the parts that are verified to be folders
	$this->{in}->new_path("FileServer",$this->{path_info});

	# set object URI
	$this->{URI} = &get_obj("URI");

	# set object post and query data
    $this->{input} = $in->combine;
}

sub write {
    my ($this,$options) = @_;
    my $in = $this->{input};  # set by BaseDCD::read
    my %opt = &DecodeString($options);
    # Save input parameters
    $this->{write_options} = \%opt;
   	my $uri = $this->{URI};
	my $path = $uri->write_relative();
	my ($css,$menu,$workarea,$action);
	
    my ($css,$selector,$menu,$workarea,$action);
    $this->{site} = $this->get_section_id;
    $this->{folder} = $this->{input}{folder};
    $css = $this->css;
    $action = $this->{input}{cmd};
    if ($action) {
	if ($action =~ /^(new_folder|edit_folder|do_newfolder|do_editfolder|folder_view|do_upload|rmdirect)$/) {
	    $workarea = $this->$action;
	}
    }
    elsif (! $workarea) {
	    $workarea = $this->folder_view;
    }

   $menu = $this->album_menu();
    my $out = "<script language='javascript' src='$config{server}{HTMLpath}/_ExSite/js/misc.js'></script>$css\n";
	my $icon = $this->getIcon("close");
	$out .= <<END;
<div class=\"SiteSelector\">\n$selector\n</div>
<script language='javascript' src='$config{server}{HTMLpath}/_Modules/FileServer/jquery.js'></script>
<table class="FileServerCtrlPanel"><tr><td valign="top" class="AlbumMenu">
<div class="tools"><a class="explorer" onclick="return false;" href="#"><img style="float: right" border="0" src="$config{server}{HTMLpath}/_Modules/FileServer/$icon" title="close" alt="close"></a>	</div>
$menu
</td><td valign="top" class="WorkArea">
$workarea
</td></tr>
</table>
END
	return $out;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "File Server";
    }
    elsif (/isRestricted/) {
	return $config{$this->module_name}{isRestricted};
    }	
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/isService/) {
	return 0;
    }	
    elsif (/DynContentParam/){
#    	return $this->getDynContentParameter(shift);
    }
}

sub ctrl_panel {
    my $this = shift;
    my ($css,$selector,$menu,$workarea,$action);
	# set admin flag to add admin tools
	$this->{admin} = 1;
	
    if (! $share{DB}->is_admin) {
	return "<p class=error>Permission denied.</p>\n";
    }
	
    $this->{site} = $this->get_section_id() || 0;
    $this->{folder} = $this->{input}{folder};

    $css = $this->css;
	$selector = $this->site_selector();
    $action = $this->{input}{cmd};

    if ($action) {
	if ($action =~ /^(new_folder|edit_folder|do_newfolder|do_editfolder|folder_view|do_upload|rmdirect)$/) {
	    $workarea = $this->$action;
	}
    }
    elsif (! $workarea) {
	    $workarea = $this->folder_view;
    }
	
    $menu = $this->album_menu();
    my $out = "<script language='javascript' src='$config{server}{HTMLpath}/_ExSite/js/misc.js'></script>\n$css\n<div class=\"SiteSelector\">\n$selector\n</div>\n";
	my $icon = $this->getIcon("close");
	if ($menu) {
	$out .= <<END;
<script language='javascript' src='$config{server}{HTMLpath}/_Modules/FileServer/jquery.js'></script>
<table class="FileServerCtrlPanel"><tr><td valign="top" class="AlbumMenu">
<div class="tools"><a class="explorer" onclick="return false;" href="#"><img style="float: right" border="0" src="$config{server}{HTMLpath}/_Modules/FileServer/$icon" title="close" alt="close"></a>	</div>
$menu
</td><td valign="top" class="WorkArea">
$workarea
</td></tr>
</table>
END
	}
	return $out;
}

sub getIconByExtension {
	my ($this,$name) = @_;
	my $icon;
	if ($name =~ /\.xls$/i) 					{ $icon = "page_white_excel.png"; }
	elsif ($name =~ /\.mp3$|\.mp4?$/i)				{ $icon = "sound.png"; }
	elsif ($name =~ /\.divx$/i) 					{ $icon = "sound.png"; }	
	elsif ($name =~ /\.swf$|\.fla?$/i) 			{ $icon = "page_white_flash.png"; }
	elsif ($name =~ /\.zip$|\.rar?$|\.gzip?$/i)	{ $icon = "page_white_flash.png"; }			
	else										{ $icon = "application.png"; }
	return $icon;
}

sub getIcon {
	my ($this,$type) = @_;
	my $icon;
	if    ($type =~ /text\/plain/i)	{ $icon = "page_white_text.png"; }
	elsif ($type =~ /text\/html/i)	{ $icon = "html.png"; }
	elsif ($type =~ /application\/msword/i)	{ $icon = "page_white_word.png"; }	
	elsif ($type =~ /application\/powerpoint/i)	{ $icon = "page_white_powerpoint.png"; }		
	elsif ($type =~ /application\/pdf/i)	{ $icon = "page_white_acrobat.png"; }			
	elsif ($type =~ /image\/gif/i)	{ $icon = "picture.png"; }				
	elsif ($type =~ /image\/jpe/i)	{ $icon = "picture.png"; }					
	elsif ($type =~ /image\/png/i)	{ $icon = "picture.png"; }						
	elsif ($type =~ /video\/mpe/i)	{ $icon = "film.png"; }							
	elsif ($type =~ /video\/quicktime/i)	{ $icon = "film.png"; }									
	elsif ($type =~ /video\/x-msvideo/i)	{ $icon = "film.png"; }									
	elsif ($type =~ /video\/x-pn-realvideo/i)	{ $icon = "film.png"; }										
	elsif ($type =~ /audio\/basic/i)	{ $icon = "sound.png"; }												
	elsif ($type =~ /audio\/x-aiff/i)	{ $icon = "sound.png"; }													
	elsif ($type =~ /audio\/x-wav/i)	{ $icon = "sound.png"; }														
	# custom types
	elsif ($type =~ /folder/i)		{ $icon = "folder_page_white.png"; }
	elsif ($type =~ /delete/i)		{ $icon = "del16.gif"; }
	elsif ($type =~ /edit/i)		{ $icon = "conf16.gif"; }
	elsif ($type =~ /explorer/i)		{ $icon = "compress.png"; }	
	elsif ($type =~ /close/i)		{ $icon = "close16.gif"; }		
	else 							{ $icon = "notfound"; }
	return $icon;
}

sub do_upload {
	my ($this) = @_;
	my $out;

	# write access check
	my $path_info = $this->get_path_info;
	my $folder_id = $this->get_folder_id($path_info);
	my $access = $this->checkAccess({type=>"write",folder_id=>$folder_id});
	my $level =  $share{DB}->authorize;
	
	if ($access) {
	my $val = $share{input}{post}{data}{file};
	my $fdata = undef;
	my $size = 0;
	if ($ENV{CONTENT_TYPE} =~ /multipart\/form-data/) {
	    # read file contents
	    my $continue = 1;
	    while ($continue) {
		$continue = read($val,$fdata,1024,$size);
		$size += $continue;
	    }
	}
	
	if ($size > $config{$this->module_name}{max_upload_size}) {
		if ($level < 3) {
			return $this->systemOut("File is too large - $config{$this->module_name}{max_upload_size} bytes max") . $this->folder_view;	
		} 
	}
	
	my $current_diskpath = $this->getDiskPath;
	my $filename = &clean_filename($val);
	my $filepath = $current_diskpath . "/" . $filename;
	
	if (open OUTFILE, ">$filepath") {
		print OUTFILE $fdata;
		close OUTFILE;
		$out .= $this->systemOut("Successfully uploaded $val ($size bytes)");
		if ($size > $config{$this->module_name}{max_upload_size}) {
#			my $delurl = $this->link(cmd=>"rmdirect",cmdpath=>"$path_info/$filename");
			$out .= $this->delete_javascript;
			$out .= $this->systemOut("Warning: file is very large ($size bytes). Click <a href=\"javascript:deleteRecord('$path_info/$filename')\">here</a> to delete.");
		}		
	} else {
		$out .= $this->systemOut("Could not upload file<br>");
	}
		return $out . $this->folder_view;
	} else {
		$out = $this->systemOut("You do not have permission to write to the current folder. If you believe that you should have access please contact the site administrator.");	
		return $out . $this->folder_view;
	}
}

sub folder_view {
    my ($this) = @_;
	
	my $out;
	my $path_info = $this->get_path_info();
#	$this->{in}->new_path("FileServer",$path_info);
#	print "Content-type: text/html\n\n";
#	print &ShowHash($share{input}{path});
#	print "folder_view_after: " . $this->{URI}->write_relative() . "<br>";

	my $db = $share{DB};
	$out .= "<table class=\"FileServer\" cellspacing=\"3px\">";
	$out .= "<tr><th width=\"16\"></th><th width=\"300\">Name</th><th width=\"100\">Size</th><th width=\"100\">Date Modified</th><th>Options</th></tr>\n";
	my $basepath = $config{FileServer}{basepath};

	my $current_diskpath = $this->getDiskPath($path_info);

	my $folder_id;
	if (-d $current_diskpath) {	
		$folder_id = $this->get_folder_id($path_info);
		if (! defined $folder_id) { return $this->systemOut("Data integrity problem encountered, please contact the system administrator."); }		
	} elsif (-f $current_diskpath) {
		$folder_id = $this->getFolderOfFileId($path_info);
		if (! defined $folder_id) { return $this->systemOut("Data integrity problem encountered, please contact the system administrator."); }		
	} else {
		return "Folders referenced in the database could not be found on the server. Please check with your server administrator.";
	}
	my $access = $this->checkAccess({folder_id=>$folder_id,type=>"read"});

	if ($access) {
	if (-d $current_diskpath) {
	opendir(DIRHANDLE, "$current_diskpath") || $this->error("Cannot opendir: $current_diskpath");
	$out .= $this->delete_javascript;
	$out .= $this->topmenu;
	# bad idea to show label name as well as file name since readdir knows nothing about fileserver_folder table
	foreach my $name (sort readdir(DIRHANDLE)) {
		next if $name =~ /^\.\.?$/;
   		my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat("$current_diskpath/$name");
		my $link_path = "$ENV{PATH_INFO}/$name";
		my $url = $this->strip_path_info($link_path);
		my $t = new ExSite::Time($mtime, "unix_timestamp");
		$mtime = $t->write("date");
		my $icon;
		my $options;

		my $delurl = "javascript:deleteRecord('$path_info/$name')";
		my $delicon = $this->getIcon("delete");
		my $editicon = $this->getIcon("edit");
		if (-d "$current_diskpath/$name") {
			$icon = $this->getIcon("folder");
			my $editurl = $this->link(cmd=>"edit_folder",cmdpath=>"$path_info/$name");
			if ($db->is_admin) {
				$options = "<a href=\"$delurl\"><img src=\"$config{server}{HTMLpath}/_Modules/FileServer/$delicon\" border=\"0\"></a>&nbsp;<a href=\"$editurl\"><img src=\"$config{server}{HTMLpath}/_Modules/FileServer/$editicon\" border=\"0\"></a>";
			} elsif ($db->is_member) {
				$options = "<a href=\"$editurl\"><img src=\"$config{server}{HTMLpath}/_Modules/FileServer/$editicon\" border=\"0\"></a>";
			}
		} elsif (-f "$current_diskpath/$name") {
			my $mimetype = &MimeType($name);
			$icon = $this->getIcon($mimetype);			
			# handle file types not found by &MimeType
			if ($icon =~ /notfound/i) { $icon = $this->getIconByExtension($name); }
			if ($db->is_admin) {
				$options = "<a href=\"$delurl\"><img src=\"$config{server}{HTMLpath}/_Modules/FileServer/$delicon\" border=\"0\"></a>";
			}
		} 

		# convert size to KB
		$size = ceil($size / 1024);
		# push directory or file row onto table
		$out .= "<tr><td><img src=\"$config{server}{HTMLpath}/silk_icons/$icon\" border=\"0\"></td><td><a href=\"$url\">$name</a></td><td>$size KB</td><td>$mtime</td><td>$options</td></tr>\n";
	}
	closedir(DIRHANDLE);	
	$out .= "</table>\n";
	$out .= $this->folder_tools;
	
	} elsif (-f $current_diskpath) {
		my $p = $share{Page} || $share{PageBuf};
		if ($p) { $p->cancel };
		my $mimetype = &MimeType($ENV{PATH_INFO});
		print "Content-type: $mimetype\n\n";
		open F, "<$config{FileServer}{basepath}$path_info";
		while (read(F,$_,1024)) { print; }
		close F;
	}
	} else {
		$out = $this->systemOut("You do not have permission to open this file/folder. If you believe that you should have access please contact the site administrator.(fid:$folder_id)");
		$path_info =~ /(\/?.*)\/{1}.*$/;
		my $previous_path = $1;
		my $return_url = $this->strip_path_info($previous_path);
		$out .= "<p>Click <a href=\"$return_url\">here</a> to return to previous folder.</p>"; 
	}
	return $out;
}

sub folder_tools {
	my ($this) = @_;
	my $db = $share{DB};
	my $out;
	if ($db->is_member) {
    # new folder link
	$this->{URI}->query(cmd=>"new_folder");
   
 	my $url = $this->{URI}->write_relative();
    $out .= "<table cellspacing=\"5px\" class=\"tools\"><tr><td><a href=\"$url\"><img src=\"$config{server}{HTMLpath}/silk_icons/folder_add.png\" height=\"16\" width=\"16\" border=\"0\" class=\"ToolIcon\"></a></td>
<td><a href=\"$url\">New Folder</a></td></tr></table>\n";
	
	# quick explorer link
	my $icon = $this->getIcon("explorer");
	if ($config{FileServer}{enable_quick_explorer}) {
	$out .= "<table cellspacing=\"5px\" class=\"tools\"><tr><td><a class=\"explorer\" onclick=\"return false;\" href=\"#\"><img border=\"0\" src=\"$config{server}{HTMLpath}/silk_icons/$icon\" title=\"Quick Explorer\" alt=\"Quick Explorer\"></a></td>
	<td><a class=\"explorer\" onclick=\"return false;\" href=\"#\">Quick Explorer</a></td></tr></table>\n";
	}

	# getting around bug in URI class
	my $parsed_uri = $this->{URI}->info();
	$this->{URI}->query(cmd=>"do_upload");
	$this->{URI}->path_info($parsed_uri->{path_info});
	my $upload_path = $this->{URI}->write_relative();

	$out .= "<form action=\"$upload_path\" METHOD=\"post\" enctype=\"multipart/form-data\">\n";
 	$out .= "<p>Upload file:<br>
<input name=\"file\" type=\"file\" size=\"20\"></p>\n";
	$out .= "<p><input type=\"submit\" value=\"Submit\"></p></form>\n";
	}
	return $out;
}

sub getDiskPath {
	my ($this,$path_info) = @_;
	if (!$path_info) {
		$path_info = $this->get_path_info;
	}
	my $path = "$config{FileServer}{basepath}$path_info";
	return $path;
}		

sub delete_javascript {
	my $this = shift;
	my $out = "
<script language=\"javascript\">
function deleteRecord(path){
	if (confirm('Are you sure you want to delete this file?')){
		location = '".$this->link(cmd=>"rmdirect",cmdpath=>undef)."&cmdpath=' + path;
	}else{
		return;
	}
}
</script>";
}

sub rmdirect {
	my ($this,$path) = @_;
	my $out;
	my $db = $share{DB};
	return $this->systemOut("Only site administrators have permission to delete.") . $this->folder_view if (!$db->is_admin);
	
	if (!defined $path) {
		$path = $this->{input}{cmdpath};
	} else {
		$out .= $this->systemOut("Nothing to delete!");
	}
	my $diskpath = $this->getDiskPath($path);
	if (-d $diskpath) {
		my $rmdir = $this->rmdir($diskpath);
	    if (! $rmdir) {
			my $error .= $this->show_diagnostics("error","html");
			$out .= $this->systemOut($error);
	    } else {
			$out .= $rmdir;
		}
	} elsif (-f $diskpath) {
		$out .= $this->rm($diskpath);
	} else {
		$out .= $this->systemOut("Not a file or directory ($diskpath).");
	}
	$diskpath =~ /(\/.*)\/(.+)$/;
	my $filename = $+;
	if ($out =~ /Removed/) {
	if (-d $diskpath) {
		$out = $this->systemOut("Deleted $filename."); 
		my $tree = $this->get_folder_tree;
		# get fileserver_folder_id of the folder to be deleted
		my $folder_id = $this->get_folder_id($path);
		if (! defined $folder_id) { return $this->systemOut("Data integrity problem encountered, please contact the system administrator."); }		
		$db->delete_key("fileserver_folder",$folder_id);
	} else {
		$out = $this->systemOut("Deleted $filename."); 
	}
	} else {
		my $error .= $this->show_diagnostics("error","html");
		if ($error =~ /Is a directory/) {
			$out .= $this->systemOut("Unable to delete $filename because there are subdirectories in that folder.");
		}
		else {
			$out .= $this->systemOut($error);
		}
	}
	$out .= $this->folder_view;
	return $out;
}

sub rmdir {
    my ($this,$dir) = @_;
    my $out = " ";  # dummy return value
    # can only remove directories in the HTMLpath
    #$dir =~ /^($config{server}{HTMLroot}$config{server}{HTMLpath}\/[\w-\/]+)$/;
    #my $secure_dir = $1;
    my $secure_dir = $this->safe_path($dir,"$config{server}{HTMLroot}$config{server}{HTMLpath}");
    if (! $secure_dir) { 
	$this->error("rmdir: invalid directory: $dir");
	return undef;
    }
    elsif (-e $secure_dir) {
	opendir D,$secure_dir;
	my @files = readdir D;
	closedir D;
	foreach my $f (@files) {
	    next if ($f =~ /^\.\.?$/);
	    $f =~ /^(.+)$/;
	    my $secure_file = $secure_dir."/".$1;
	    if (-d "$dir/$f") {
		# subdir; better remove it, too
		$out .= $this->rmdir("$dir/$f");
	    }
	    else {
		# must move unlink to its own if statement, or it causes
		# an insecure dependency error 
		if (unlink $secure_file) {
		    $out .= "Removed $f.<br>\n";
		}
		else {
		    $this->error("rmdir: $secure_file failed to remove $f: $!");
		    return undef;
		}
	    }
	}
	if (rmdir $secure_dir) {
	    $out .= "Removed $secure_dir.<br>\n";
	}
	else {
	    $this->error("rmdir: failed to remove $secure_dir: $!");
	    return undef;
	}
    }
    return $out;
}


# get id of fileserver_folder where path is to a file
# verify that the path is really to a file with -f before using this function
sub getFolderOfFileId {
	my ($this,$path) = @_;
	$path =~ /(\/?.*)\/{1}.*$/;
	my $previous_path = $1;
	return $this->get_folder_id($previous_path);
}

# get id of fileserver_folder by traversing path
sub get_folder_id {
	my ($this,$path) = @_;

	# return 0 if path is empty to signify root folder
	if ($path eq "" || $path eq "/") { return 0; } 
	my $tree = $this->get_folder_tree();
	my @nodes = $tree->get_topnodes;

	# remove empty strings from path 
	$path =~ s/^\///;

	# split path into its components
	my @path = split(/\//,$path);
	my $folder_id = $this->getNode($tree,\@nodes,\@path);
	return $folder_id;
}

# recursive routine 
# looks through an array to find the name of the folder and returns its children and next folder
sub getNode {
	my ($this,$tree,$nodes,$path) = @_;
	my @patharray = @$path;
	my $folder = shift(@patharray);
	foreach my $node (@$nodes) {
		if ($node->{data}{name} eq $folder) {
			# found the node that we are looking for
			# get the children and look up next part of path			
			my $folder_id = $node->{data}{fileserver_folder_id};
			my @childnodes = $tree->get_child_nodes($folder_id);
			my $size_of_childnodes = @childnodes;
			my $size_of_patharray = @patharray;
			# if the size of childnodes is 0 then this node has no children
			# return the node we just matched
			if ($size_of_patharray == 0) { 
				return $folder_id;
			} else {
				# recursive call here - calls itself
				return $this->getNode($tree,\@childnodes,\@patharray);
			}
		}
	}
	# should not get here unless theres a data integrity problem
	return undef;
}

sub get_folder_tree {
	my ($this) = @_;
	my $db = $share{DB};
	my $tree;
	if (defined $share{$this->module_name}{FolderTree}) {
		$tree = $share{$this->module_name}{FolderTree};
	} else {
		my @folders = $db->fetch_all("fileserver_folder");
		$tree = new ExSite::Tree("fileserver_folder_id","parent_id",@folders);
		$share{$this->module_name}{FolderTree} = $tree;
	}
	return $tree;
}

sub systemOut {
	my ($this,$msg) = @_;
	return "<div style=\"border: 1px solid #fad163; background-color: #fff; padding: 5px;\">$msg</div>"; 
}

# return the current url with new path_info
sub strip_path_info {
	my ($this,$new_path_info) = @_;
    $this->{URI}->query(cmd=>undef,cmdpath=>undef);	
	$this->{URI}->path_info($new_path_info);
	my $link = $this->{URI}->write_relative();
	$this->{URI}->reset();
	return $link;
}

sub topmenu {
	my ($this) = @_;
	my $path_info = $this->get_path_info;
	my (@path) = split(/\//,$path_info);
	my $current_url = $this->link;
	my $url = $this->strip_path_info;
	if ($this->{admin}) {
		$url = $this->strip_path_info("/FileServer");
	}
	my $out = <<END;
<script type=text/javascript>
\$(document).ready(function() {
\$("a.explorer").click(function() {
\$("td.AlbumMenu").toggle()
return false;
});
});
</script>
END
	$out .= "<div class=\"tools\"><a href=\"$url\">Top</a>";
	my $link_path = "";
	if ($this->{admin}) {
		$link_path = "/FileServer";
	}
	foreach my $path (@path) {
		if ($path =~ /\w+/) {
		$link_path = $link_path . "/$path";
		$url = $this->strip_path_info($link_path);
		$out .= " &gt; <a href=\"$url\">$path</a>";
		}
	}
	$out .= "</div><br>\n";
	return $out;
}	

# process new folder form
sub do_newfolder {
    my ($this,$table) = @_;
	$table = $table || "fileserver_folder";
    my $in = $this->{input};
	my $db = $share{DB};
	
    my $out;
	$db->get_data;
	my @id = keys %{$db->{form}{$table}};
	my $id = $id[0] || "_";	
	my $filename = &clean_filename($db->{form}{$table}{$id}{label});

	# check if folder of same name and parent_id exists already
	my @folders = $db->fetch_match($table, { parent_id=>$db->{form}{$table}{$id}{parent_id},name=>$filename});
	return $this->systemOut("Error: Folder of same name already exists. Please try again.") . $this->folder_view if (scalar @folders);

	$db->{form}{$table}{$id}{name} = $filename;
	$out = $db->do;

	if ($out =~ /Done\!/) {
		my $sql = "select max(fileserver_folder_id) as fileserver_folder_id from $table";
		my @folder = $db->custom_query($sql);
		
		my $url = $this->link(cmd=>"folder_view",folder=>$folder[0]->{fileserver_folder_id});
	
		my $path_info = $this->get_path_info;
		my $path = "$config{FileServer}{basepath}$path_info/$filename";

		my $mkdir = $this->mkdir($path);
		if (! $mkdir) {
			$out = $this->show_diagnostics("error","html");
			$out = $this->systemOut($out);
		} else {
			$out = $this->show_diagnostics("info","html");
			$out = $this->systemOut($out);
		}
	}
	$out .= $this->folder_view;
	return $out; 
}

# process edit folder form
sub do_editfolder {
    my ($this,$table) = @_;
	$table = $table || "fileserver_folder";
    my $in = $this->{input};
	my $db = $share{DB};
    my $out;
	$db->get_data;
	my @id = keys %{$db->{form}{$table}};
	my $id = $id[0] || "_";		
	my $label = $db->{form}{$table}{$id}{label};
	if ($label) {
		my $name = &clean_filename($label);
		$db->{form}{$table}{$id}{name} = $name;
		my $oldpath = $config{FileServer}{basepath} . $in->{cmdpath};
		my $newpath = $oldpath;
		$newpath =~ s/\/[\w-]*$/\/$name/;
		rename($oldpath, $newpath) or system("mv", $oldpath, $newpath) or $out = $this->systemOut("Couldn't rename folder to $name: $!");
#		rename($oldpath, $newpath) or die "something is wrong";
	}
	$out .= $db->do;
	if ($out =~ /Done\!/) {
		$out = $this->systemOut("Changes to folder saved.");
	}
	$out .= $this->folder_view;
	return $out; 
}

sub get_path_info {
	my ($this) = @_;
	my $path_info = $this->{path_info}; 	
	return $path_info;
}
	
sub mkdir {
    my ($this,$dir) = @_;
    if (! $this->{dircache}) { $this->{dircache} = {}; }
    return $this->{dircache}{$dir} if (exists $this->{dircache}{$dir});
    my $out = " ";  # dummy return value
    $dir =~ /(\/[\w-\/]+)/;
    my $secure_dir = $1;
    if (! $secure_dir) { 
	$this->error("mkdir: invalid directory: $dir");
	return undef;
    }
    my @path = split(/\//,$secure_dir);
    shift @path; # first one is a dummy
    my $path;
    foreach my $dir (@path) {
	$path .= "/$dir";
	next if (-e "$path");
	if (! mkdir("$path",0777)) {
	    $this->error("mkdir $dir: failed: $!");
	    $this->{dircache}{$dir} = undef;
	    return undef;
	}
	else {
	    $this->info("Created $dir folder.");
	    $this->{dircache}{$dir} = " ";
	}
    }
    return $out;
}

# returns an array of flags
# size is 0 or 1 for list type or 0 or more for set type
# $opt->{type} = read | write
# $opt->{folder_id} = int
sub getFolderAccess {
	my ($this,$opt) = @_;
	my $db = $share{DB};
	my $folder_id = $opt->{folder_id};
	my $tree = $this->get_folder_tree;
	my $folder = $tree->getnode_data($folder_id);
#	my $folder = $db->fetch("fileserver_folder",$folder_id);	
	my @access;
	@access = split(/; /,$folder->{access_read}) if ($opt->{type} eq "read"); 
	@access = split(/; /,$folder->{access_write}) if ($opt->{type} eq "write");
	return @access;
}

# returns an array of flags
# size is 0 or 1 for list type or 0 or more for set type
sub getUserAccess {
	my ($this) = @_;
	my $db = $share{DB};
	my $user = $db->my_user_record;
	my $column = $config{$this->module_name}{member_access_column} || "type";
	my @access = split(/; /,$user->{$column});
	return @access;
}

# $opt->{type} = read | write
# $opt->{content_type} = folder | file
# $opt->{folder_id}	= id of current folder
sub checkAccess {
	my ($this,$opt) = @_;
	my $db = $share{DB};
	my $level = $db->authorize;
	# administrators (level 3) can read and write anything
	if ($db->is_admin) {
		return 1;
	}

	if (! defined $opt->{folder_id}) { return 0; }
	# root folder
	# everyone can read but nobody can write unless they are admin
	if (!$opt->{folder_id}) {
		return 1 if ($opt->{type} eq "read");
		return 0;
	}

	my @folder_access = $this->getFolderAccess($opt);
	return 1 if (! scalar @folder_access);
	my @user_access = $this->getUserAccess;
	foreach my $user_access (@user_access) {
		foreach my $folder_access (@folder_access) {
			return 1 if ($user_access eq $folder_access);
		}
	}

	return 0;
}

sub new_folder {
	my ($this) = @_;
    my $in = $this->{input};
	my $db = $share{DB};

	my $form_action;
	my $out = "<h3>New Folder</h3>\n";
	
	# access check
	# must have write access to current folder to add subfolder to it
	my $path_info = $this->get_path_info;
	my $folder_id = $this->get_folder_id($path_info);
	if (! defined $folder_id) { return $this->systemOut("Data integrity problem encountered, please contact the system administrator."); }
	my $access = $this->checkAccess({folder_id=>$folder_id,type=>"write"});
	my $section_id = $this->get_section_id || 1;
	if ($access) {
		my $tree = $this->get_folder_tree;
		my $folder = $tree->getnode_data($folder_id);	
		my $path_info;
		$path_info = $this->get_path_info;
		$form_action = $this->link(cmd=>"do_newfolder");
		$out .= "<ul><li>
			<li>If read or write access permissions are left completely unchecked(nothing checked) it is assumed that all members of the site have access to the folder.</li>
			<li>Checking 1 group of users will allow only members belonging to that group to access the directory.</li>
			<li>Checking multiple groups of users will allow anyone in any of these groups to access the directory.</li></ul>";
		$out .= $share{DB}->make(table=>"fileserver_folder",
			action=>$form_action,
			data=>{
			section_id=>$section_id,
			parent_id=>$folder_id,
			access_write=>$folder->{access_write},
			access_read=>$folder->{access_read},			
		});
	} else {
		$out = $this->systemOut("You do not have permission to add subfolders to the current folder. If you believe that you should have access please contact the site administrator.");
		my $return_url = $this->strip_path_info($path_info);
		$out .= "<p>Click <a href=\"$return_url\">here</a> to return to previous folder.</p>"; 
	}
	return $out;
}

# takes cmdpath from querystring
sub edit_folder {
	my ($this) = @_;
    my $in = $this->{input};
	my $db = $share{DB};

	my $form_action;
	my $out = "<h3>Edit Folder</h3>\n";
	
	# access check
	# must have write access to folder to edit it	
	my $folder_id = $this->get_folder_id($in->{cmdpath});
	if (! defined $folder_id) { return $this->systemOut("Data integrity problem encountered, please contact the system administrator."); }
	my $access = $this->checkAccess({folder_id=>$folder_id,type=>"write"});
	
	if ($access) {
		$out .= "<ul><li>
			<li>If read or write access permissions are left completely unchecked(nothing checked) it is assumed that all members of the site have access to the folder.</li>
			<li>Checking 1 group of users will allow only members belonging to that group to access the directory.</li>
			<li>Checking multiple groups of users will allow anyone in any of these groups to access the directory.</li></ul>";
	$form_action = $this->link(cmd=>"do_editfolder");
	$out .= $share{DB}->make(table=>"fileserver_folder",
				action=>$form_action,
				record=>$folder_id,
			);
	} else {
		my $path_info = $this->get_path_info;
		$out = $this->systemOut("You do not have permission to edit the current folder. If you believe that you should have access please contact the site administrator.");
		my $return_url = $this->strip_path_info($path_info);
		$out .= "<p>Click <a href=\"$return_url\">here</a> to return to previous folder.</p>"; 
	}
	return $out;
}

sub album_menu {
    my $this = shift;
#    return undef if (! $this->{site});
    my $out;
	$out .= <<END;
	<script src="$config{server}{HTMLpath}/_Modules/FileServer/aqtree3clickable.js" type="text/javascript"></script>
	<link rel="stylesheet" href="$config{server}{HTMLpath}/_Modules/FileServer/aqtree3clickable.css">
	<strong>Quick Explorer</strong>	
	<table><tr><td>
	<ul class="aqtree3clickable">
END

	my $rootdir = $config{FileServer}{basepath};
	my $current_url = $this->link(cmd=>undef);
	my $sitepath = $current_url;
	my $opt = { rootdir=> $rootdir, sitepath=>$sitepath };
	if ($config{FileServer}{enable_quick_explorer}) {
 		$out .= $this->createTree($opt);
	} else {
		$out .= "Feature Disabled."
	}
	$out .= "</ul></ul></td></tr></table>\n";
    return $out;
}

sub css {
    my $this = shift;
    return "
<style>
table.FileServerCtrlPanel {
    border:1px solid #666;
    background-color:#fff;
    border-collapse:collapse;
}
table.FileServerView td {
    padding:5px;
    background-color:#fff;
}
td.AlbumMenu {
	border:0px solid #666;
	background-color: #E1ECFE;
    padding:0px;
    width:300px;
	min-height: 500px;
	height: 100%;
 	overflow: auto;
    overflow-x:auto;
    overflow-y:visible;	
	display: none;
}
td.AlbumMenu ul li {
padding: 0 0 0 5px ;
}
td.AlbumMenu {
padding: 10px;
margin: 0px;
}
td.WorkArea {
    border:1px solid #666;
    padding:20px;
}
a.FileServerLink {
    display:block;
    width:100%;
    padding:8px;
    padding-right:20px;
    text-decoration:none;
}
a.FileServerLink:hover {
    background-color:#999999;
}
a#thistab {
    background-color:#666666;
    color:#cccccc;
}
td.AlbumMenu p {
    padding-left:8px;
}
img.ToolIcon {
    padding-top:1px;
}
th {
    background-color:#999;
}
table.FileServer td { padding: 10px; margin: 10px; }
table.FileServer td { height: 50px; background-color: #E1ECFE; }
table.FileServer th { background-color: #bfccf2; color: black; font-weight: normal; }
table.FileServer td { text-align: left; }
div.tools { margin: 5px 0 5px 5px; }
.tools a { font-size: 90%; text-decoration: none; font-weight: normal; }
.tools * { vertical-align: middle; text-align: left; margin: 0; padding: 0;  }
</style>
";
}

sub site_selector {
    my $this = shift;
    my $out;
    if (! $share{Page}) {
	# only output a site selector on control panels
	my @sites = $share{DB}->my_real_sites;
	$out .= "<p><form>Select website: <select name=\"section_id\" onchange=\"document.forms[0].submit()\">\n";
	foreach my $s (@sites) {
	    $out .= "<option value=\"$s->{section_id}\"";
	    if ($this->{site} == $s->{section_id}) { 
		$out .= " selected";
	    }
	    $out .= ">$s->{title}</option>\n";
	}
	$out .= "</select><input type=\"submit\" value=\"Submit\"></form></p>\n";
    }
    return $out;
}

# functions to produce tree representation of a directory for aqtree javascript
sub createTree {
	my ($this,$options) = @_;
	my %opt = %$options;
	# rootdir is the directory in which directories and files are being read
	# sitepath is the http path from which the rootdir can be accessed
	$share{dir}{rootdir} = $opt{rootdir};
	$share{dir}{sitepath} = $opt{sitepath};
	$share{dir}{prevdir} = $share{dir}{rootdir};
	$share{dir}{output} = "";
	$share{FileServerObject} = $this;
	find({ wanted => \&edits, preprocess => \&dirsort }, $share{dir}{rootdir});
	return $share{dir}{output};
}

sub dirsort() {
	return reverse(sort(@_));
}

sub edits() {

#	For example, when examining the file /some/path/foo.ext you will have:
#	$File::Find::dir  = /some/path/
#	$_                = foo.ext
#	$File::Find::name = /some/path/foo.ext
 	my $dir = $File::Find::dir;
	my $name = $File::Find::name;
	my $filename = $_;
    if ($name eq $share{dir}{rootdir}) { return; }

	if ($name eq $share{dir}{rootdir}) { return; }
	
    $name =~ /$share{dir}->{rootdir}\/(.+$)/i;
	my $fileitem = $1;
   	# if the current containing directory is not equal to the last directory
	unless ($share{dir}{prevdir} eq $dir) {
		# check how many directories to close off
		# count the number of / representing subdirectories
		# for every extra / we close off another directory
		my $count = ($share{dir}{prevdir} =~ tr/\///);
		my $count2 = ($dir =~ tr/\///);
		while ($count - $count2 > 0) {
			$share{dir}->{output} .= "</ul>\n";
			$count2++;
		}
	}
	if (-d $name) {
		# if the file is a directory
		# open the files in the directory
		# to check if the directory contains other directories

=pod	
		# specific to FileServer - access check	
		my $path_info = $dir;
		my $basepath = $config{FileServer}{basepath};
		$path_info =~ s/$basepath//;
		my $folder_id = $share{FileServerObject}->get_folder_id($path_info);
		if (! defined $folder_id) { return $share{FileServerObject}->systemOut("Data integrity problem encountered, please contact the system administrator."); }
		my $access = $share{FileServerObject}->checkAccess({folder_id=>$folder_id,type=>"read"});
		if (!$access) {	return; }
=cut
		my $url = &edits_strip_path_info($fileitem);
		
		$share{dir}{output} .=  "<li><a href=\"$url\">$filename</a><div><a href=\"$url\">(open)</a></div>\n";
        opendir( DIR, $name )
            or die "Can't open $name: $!";
        while ( my $entry = readdir( DIR ) ) {
            if ( -d "$name\/$entry" ) {
                # the directory contains other directories
				$share{dir}->{output} .= "<ul>\n";
                $share{dir}{prevdir} = $name;
                last;
            }
        }
        closedir( DIR );        
        return;

    } else {
		my $url = &edits_strip_path_info($fileitem);
		$share{dir}{output} .= "<li><a href=\"$url\">$filename</a></li>\n";
	}
}

# modified version of sub strip_path_info for sub edits
sub edits_strip_path_info {
	my ($new_path_info) = @_;
	my $module_name = "FileServer";
	my $p = $share{FileServerObject} || $share{PageBuf} || $share{Page};
	my $url = $p->link;
	if ($url =~ /ctrl-panel\.cgi/) {
		$url =~ s/ctrl-panel\.cgi.*\?/ctrl-panel.cgi\/$module_name\/$new_path_info\?/;	
	} else {
		$url =~ s/page\.cgi.*\?/page.cgi\/$new_path_info\?/;
	}	
	return $url;
}
	
# end

1;
