package Modules::LocationFinder;

=pod

comments incoming...

=cut

use strict;
use ExSite::Config;
use ExSite::Module qw(&site_license);
# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD ExSite::ContentData);
use CGI::Carp qw(fatalsToBrowser);

my $searchTypeNames = { state_prov => "State/Province", area_code => "Area Code", postal_code => "Zip/Postal Code", keyword => "Keyword" };

sub write {
    my ($this,$options) = @_;
	
    my $in = new ExSite::Input;
    
    my $args = $in->combine();
    my %args = %$args;
    $this->{location_table} = $config{LocationFinder}{table_name} || "location_finder";
	
    my $db = $share{DB}; 
    my $in = $this->{input};

    if ($in->{locid}) 
    {
	$this->fetch($in->{locid});
    }

    my $out;

    if ($options =~ /location_edit/)
    {
    	my $action = $in->{cmd};
    	if ($action =~ /doform/)
    	{
    	    $out .= $this->$action();
    	}
    	else
    	{
    	    $out .= $this->locationEditer();
    	}
    }
    else 
    {	
	if ($in->{search_type} && $in->{search_term})
	{
	    $out = $this->findMatches($in->{search_type}, $in->{search_term}) . $share{ML}->br();
	}
	elsif ($in->{search_type} && !$in->{search_term})
	{
	    $out = "Please enter in a value to search by.";
	}

	$out .= $this->searchform();
    }
    
    return $this->wrap($out);
}

sub searchform {
    my $this = shift;
    
    my $out;
    $out = $share{ML}->h4("Search by...") if ($config{LocationFinder}{"show_header"});
   
    my @searchForms;
    
    foreach my $searchTypeName ( sort keys %$searchTypeNames )
    {
    	my $searchForm;
    	
	if ( $searchTypeName eq "state_prov" )
	{
	    $searchForm = $this->buildStateProvForm();
	}
	else
	{
    	    $searchForm = $this->buildSearchForm($searchTypeName);
    	}
    	
    	push(@searchForms, $searchForm);
    }
    
    if ($config{LocationFinder}{searchFieldLayout} =~ /horizontal/)
    {
    	my $searchTable;
    	
    	foreach my $searchForm ( @searchForms )
    	{
 	    $searchTable .= $share{ML}->td($searchForm);
    	}
    	
	$out .= $share{ML}->table(
			      $share{ML}->tr(
					     $searchTable
					     ),
			      { class => "searchTable" }
			      );    
    }
    else
    {
	#foreach my $searchForm ( @searchForms )
	#{
	#    $out .= $share{ML}->div($searchForm, {class=>"search_form"});
	#}
        my $searchTableGroup;

        foreach my $searchTable ( @searchForms )
        {
                $searchTableGroup .= $searchTable;
        }

	$out .= $share{ML}->div($searchTableGroup, {class => "search_form"});
    }
    
    return $out;
}

sub buildSearchForm {
    my $this = shift;
    my $searchTypeName = shift;
    
    my $searchForm = $share{ML}->form($searchTypeNames->{$searchTypeName} . ": " . 
				      $share{ML}->br() . 
				      $share{ML}->input(
							undef,
							{ type => "input", name => "search_term"}
							) . " " .
				      $share{ML}->input(
							undef, 
							{ type => "submit", value => " Search " }
							) .
				      $share{ML}->input(
							undef,
							{ type => "hidden", name => "search_type", value => $searchTypeName }
							) .
				      $share{ML}->input(
							undef,
							{ type => "hidden", name => "view_all", value => "no" }
							),	
				      { method => "post" }
				      );
    
    return $searchForm;
}

sub buildStateProvForm {
    my $this = shift;
    
    my $searchForm = $share{ML}->form($searchTypeNames->{state_prov} . ": " . 
				      $share{ML}->br() . 
				      $share{ML}->select([
							  ["option", "== select ==", { value => "" }],
							  ["option", "Provinces", { value => "" }],
							  ["option", "---------", { value => "" }],
							  ["option", "AB - Alberta", { value => "AB" }],
							  ["option", "BC - British Columbia", { value => "BC" }],
							  ["option", "MB - Manitoba", { value => "MB" }],
							  ["option", "NB - New Brunswick", { value => "NB" }],
							  ["option", "NL - Newfoundland and Labrador", { value => "NL" }],
							  ["option", "NS - Nova Scotia", { value => "NS" }],
							  ["option", "NT - Northwest Territories", { value => "NT" }],
							  ["option", "NU - Nunavut", { value => "NU" }],
							  ["option", "ON - Ontario", { value => "ON" }],
							  ["option", "PE - Prince Edward Island", { value => "PE" }],
							  ["option", "QC - Quebec", { value => "QC" }],
							  ["option", "SK - Saskatchewan", { value => "SK" }],
							  ["option", "YT - Yukon", { value => "YT" }],
							  ["option", "", { value => "" }],
							  ["option", "States", { value => "" }],
							  ["option", "------", { value => "" }],
							  ["option", "AK - Alaska", { value => "AK" }],
							  ["option", "AL - Alabama", { value => "AL" }],
							  ["option", "AR - Arkansas", { value => "AR" }],
							  ["option", "AZ - Arizona", { value => "AZ" }],
							  ["option", "CA - California", { value => "CA" }],
							  ["option", "CO - Colorado", { value => "CO" }],
							  ["option", "CT - Connecticut", { value => "CT" }],
							  ["option", "DC - District of Columbia", { value => "DC" }],
							  ["option", "DE - Delaware", { value => "DE" }],
							  ["option", "FL - Florida", { value => "FL" }],
							  ["option", "GA - Georgia", { value => "GA" }],
							  ["option", "HI - Hawaii", { value => "HI" }],
							  ["option", "IA - Iowa", { value => "IA" }],
							  ["option", "ID - Idaho", { value => "ID" }],
							  ["option", "IL - Illinois", { value => "IL" }],
							  ["option", "IN - Indiana", { value => "IN" }],
							  ["option", "KS - Kansas", { value => "KS" }],
							  ["option", "KY - Kentucky", { value => "KY" }],
							  ["option", "LA - Louisiana", { value => "LA" }],
							  ["option", "MA - Massachusetts", { value => "MA" }],
							  ["option", "MD - Maryland", { value => "MD" }],
							  ["option", "ME - Maine", { value => "ME" }],
							  ["option", "MI - Michigan", { value => "MI" }],
							  ["option", "MN - Minnesota", { value => "MN" }],
							  ["option", "MO - Missouri", { value => "MO" }],
							  ["option", "MS - Mississippi", { value => "MS" }],
							  ["option", "MT - Montana", { value => "MT" }],
							  ["option", "NC - North Carolina", { value => "NC" }],
							  ["option", "ND - North Dakota", { value => "ND" }],
							  ["option", "NE - Nebraska", { value => "NE" }],
							  ["option", "NH - New Hampshire", { value => "NH" }],
							  ["option", "NJ - New Jersey", { value => "NJ" }],
							  ["option", "NM - New Mexico", { value => "NM" }],
							  ["option", "NV - Nevada", { value => "NV" }],
							  ["option", "NY - New York", { value => "NY" }],
							  ["option", "OH - Ohio", { value => "OH" }],
							  ["option", "OK - Oklahoma", { value => "OK" }],
							  ["option", "OR - Oregon", { value => "OR" }],
							  ["option", "PA - Pennsylvania", { value => "PA" }],
							  ["option", "RI - Rhode Island", { value => "RI" }],
							  ["option", "SC - South Carolina", { value => "SC" }],
							  ["option", "SD - South Dakota", { value => "SD" }],
							  ["option", "TN - Tennessee", { value => "TN" }],
							  ["option", "TX - Texas", { value => "TX" }],
							  ["option", "UT - Utah", { value => "UT" }],
							  ["option", "VA - Virginia", { value => "VA" }],
							  ["option", "VT - Vermont", { value => "VT" }],
							  ["option", "WA - Washington", { value => "WA" }],
							  ["option", "WI - Wisconsin", { value => "WI" }],
							  ["option", "WV - West Virginia", { value => "WV" }],
							  ["option", "WY - Wyoming", { value => "WY" }],
						         ],
							 { name => "search_term" }
							 ) . " " .
				      $share{ML}->input(
							undef, 
							{ type => "submit", value => " Search " }
							) .
				      $share{ML}->input(
							undef,
							{ type => "hidden", name => "search_type", value => "state_prov" }
							) .
				      $share{ML}->input(
							undef,
							{ type => "hidden", name => "view_all", value => "no" }
							),		,
				      { method => "post" }
				      );
    
    return $searchForm;
}

sub findMatches {
    my $this = shift;
    my $searchType = shift;
    my $searchTerm = shift;
       
    my $conditions = "WHERE ";
    my @locations;
    
    if ($searchType =~ /state_prov/)
    {
    	$conditions .= "state_prov LIKE '$searchTerm' ";
    	@locations = $this->searchByCondition($conditions);
    }
    elsif ($searchType =~ /area_code/)
    {
    	@locations = $this->searchByAreaCode($searchTerm);
    }
    elsif ($searchType =~ /postal_code/)
    {
    	@locations = $this->searchByPostalCode($searchTerm);
    }
    elsif ($searchType =~ /keyword/)
    {
    	$conditions .= "name LIKE '%$searchTerm%' OR description LIKE '%$searchTerm%' 
    			OR city LIKE '%$searchTerm%' OR address LIKE '%$searchTerm%' ";
    	@locations = $this->searchByCondition($conditions);
    }
    else
    {
    	$conditions .= "$searchType LIKE '%$searchTerm%'";
    	
    	@locations = $this->searchByCondition($conditions);
    }
    
    @locations = sort {
	lc($a->{name}) cmp lc($b->{name})
    } @locations;    
    
    my $searchName = $searchTypeNames->{$searchType};
    
    return $share{ML}->h3("Matches for \"$searchTerm\" by $searchName") . $this->displayResults(@locations);
}

sub searchByCondition {
    my $this = shift;
    my $conditions = shift;
   
    my $section_id = $share{DB}->this_site()->{section_id};
    
    my @locations = $share{DB}->custom_query("SELECT * 
    					      FROM $this->{location_table} 
    					      $conditions
    					      AND (section_id = 0 OR section_id = $section_id OR section_id IS NULL)");
    
    return @locations;
}

sub searchByAreaCode {
    my $this = shift;
    my $searchTerm = shift;
    
    my $section_id = $share{DB}->this_site()->{section_id};
    
    my @locations;
    my @possibleLocations = $share{DB}->custom_query("SELECT *
    						      FROM $this->{location_table}
    						      WHERE phone LIKE '%$searchTerm%'
    						      OR fax LIKE '%$searchTerm%'
    						      AND (section_id = 0 OR section_id = $section_id OR section_id IS NULL)");

    foreach my $possibleLocation ( @possibleLocations )
    {
    	my $phone = $possibleLocation->{phone};
    	my $fax = $possibleLocation->{fax};
    	
    	# remove any logical non-digit characters from the phone and fax number
    	$phone =~ s/[.()-\s]//g;
    	$fax =~ s/[.()-\s]//g;
    	
    	if( (length($phone) == 10 && $phone =~ /^($searchTerm)\d+$/) || 
    	    (length($fax) == 10 && $fax =~ /^($searchTerm)\d+$/) )
    	{
    	    push(@locations, $possibleLocation);
    	}
    }

    return @locations;
}

sub searchByPostalCode {
    my $this = shift;
    my $searchTerm = shift;
    
    my $section_id = $share{DB}->this_site()->{section_id};
    
    my @locations;
    
    for (my $i = 3; $i > 0; $i--)
    {
    	$searchTerm = substr $searchTerm, 0, $i;
    	
    	@locations = $share{DB}->custom_query("SELECT *
    					       FROM $this->{location_table}
    					       WHERE postalcode LIKE '$searchTerm%'
    					       AND (section_id = 0 OR section_id = $section_id OR section_id IS NULL)");
    	
    	last if ( @locations > 0 );
    }
    
    return @locations;
}

sub displayResults {
    my ($this, @locations) = @_;
    
    my $out;
    my $locationCount = 0;
    my $in = $this->{input};
    
    my $maxLocations = $config{LocationFinder}{max_locations} || 10;
 
    if (@locations > 0)
    {
	foreach my $location ( @locations )
	{
	    $this->{location} = $location;
	    $out .= $this->show();
	    $locationCount++;
	    
	    if ( @locations > $maxLocations && $locationCount == $maxLocations && $in->{view_all} ne "yes" )
	    {
	    	my $action = $this->link(view_all => "yes", search_term => $in->{search_term}, search_type => $in->{search_type});
	    	$out .= $share{ML}->a("View all matches", { href => $action });
	    	last;
	    }
	}
    }
    else
    {
    	$out .= "No matches found.";
    }
    
    $out .= $share{ML}->br();
    
    return $out;
}

sub wrap {
    my ($this,$out) = @_;
    
    my $css = $share{ML}->link(undef, {
		rel=>"stylesheet",
		type=>"text/css",
		href=>"$this->{HTMLpath}/LocationFinder.css",
    });
    
    return $css . $share{ML}->Element("div", "\n$out\n", { class => "LocationFinder" }) . "\n";
}

sub ioctl {
    my $this = shift;
    &get_obj("ML");
    $_ = shift;   # $_ is the ioctl request
    
    my $additionalSearchFields = $config{LocationFinder}{additional_search_fields};
    my @additionalSearchFields = split(/\&/, $additionalSearchFields);
        
    foreach my $searchField (@additionalSearchFields) {
    	my @searchFields = split(/=/, $searchField);
      	$searchTypeNames->{$searchFields[0]} = $searchFields[1];
    }
        
    my $excludeSearchFields = $config{LocationFinder}{exclude_search_fields};
    my @excludeSearchFields = split(/ /, $excludeSearchFields);
        
    foreach my $searchField (@excludeSearchFields) {
      	delete $searchTypeNames->{$searchField};
    }
    
    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ModuleName/) {
	return "Location Finder";
    }
    elsif (/isRestricted/) {
	return defined $config{LocationFinder}{isRestricted} ?
	    $config{LocationFinder}{isRestricted} : 1;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/ModuleInfo/) {
	return "Location finder info coming...";
    }
    elsif (/DynContentParameter/) {
	my $selectOptions = $share{ML}->select([
			      		        ["option", "(default)", { value => "" }],
					        ["option", "Location Finder", { value => "" }],
					        ["option", "Location Editor", { value => "location_edit" }],
			      		      ],
			      		      { name => "param" });
	
	return $share{ML}->Element("p", $share{ML}->b("Option:") . $selectOptions) . "\n";
    }    
    return;
}

sub show {
    my $this = shift;
    my $l = $this->{location};
    
    return if (!$l);
    
    my $out;
    
    my $website;
    if ($this->show_data("website")) {
    	my $http = "http://" if ($this->show_data("website") !~ /http/);
    	$website = $share{ML}->a($this->show_data("website"), { href => $http . $this->show_data("website"), target => "_blank" });	
    }
    
    $out .= $share{ML}->h4(($this->show_data("name") || "N/A"));
    $out .= $share{ML}->b("Description: ") . $share{ML}->br() . ($this->show_data("description") || "N/A") . $share{ML}->br() . $share{ML}->br(); 
    
    $out .= $share{ML}->table(
    			      $share{ML}->tr(
    					     $share{ML}->td($share{ML}->b("Address: ") . ($this->show_data("address") || "N/A"),
    					     		    { colspan => "3" }
    					     		    )
    			      		     ) .
    			       $share{ML}->tr(
    					     $share{ML}->td($share{ML}->b("City: ") . ($this->show_data("city") || "N/A")
					         	    ) .
					     $share{ML}->td($share{ML}->b("Phone: ") . ($this->show_data("phone") || "N/A")
    					     		    ) .
    					     $share{ML}->td($share{ML}->b("Email: ") . ($this->show_data("email") || "N/A")
					         	    )
    			      		     ) .
    			       $share{ML}->tr(
    					     $share{ML}->td($share{ML}->b("State/Province: ") . ($this->show_data("state_prov") || "N/A")
    					     		    ) .
    					     $share{ML}->td($share{ML}->b("Fax: ") . ($this->show_data("fax") || "N/A")
					         	    ) .
    					     $share{ML}->td($share{ML}->b("Website: ") . ($website || "N/A")
					         	    )
    			      		     ) .
    			       $share{ML}->tr(
    					     $share{ML}->td($share{ML}->b("Zip/Postal Code: ") . ($this->show_data("postalcode") || "N/A")
					         	    )
    			      		     ),
    			      { class => "LocationFinderDataTable", border => "0", cellspacing => "0", cellpadding => "0" }
    			      );
    
    $out .= $share{ML}->br();
    
    return $out;
}

sub show_data {
    my ($this,$col) = @_;
    
    return undef if (! $this->{location});

    return $share{DB}->show_data($this->{location_table}, $col, $this->{location}{$col},
				 $this->{location}{location_id});
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    my $db = $share{DB};
    
    $this->{admin} = 1;  # set admin flag to add admin tools

    $this->{location_table} = $config{LocationFinder}{table_name} || "location_finder";

    my $action = $in->{action};

    if ($in->{locid}) {
	$this->fetch($in->{locid});
	
	return $share{ML}->p("Invalid event ID.", { class => "error" }) if (! $this->{location});
    }

    if ($action =~ /new/) {
	return $this->newlocation;
    }
    elsif ($action =~ /edit/) {
	my $action = $this->link(action => "doform");
	$out .= $share{ML}->h1($config{LocationFinder}{heading}{edit}) . "\n" . $share{ML}->dl() . "\n";
	
	$out .= $db->make(table => $this->{location_table},
			  record => $in->{locid},
			  action => $action,
			  extra => { action => "doform"},
			  );
	return $out;
    }
    elsif ($action =~ /doform/) {
	return $this->doform;
    }
    elsif ($action =~ /delete/) {
	$share{DB}->trash_key($this->{location_table},$in->{locid});
	# errorcheck?
    }
    elsif ($action =~ /show/) {
	if ($in->{locid}) {
	    $this->fetch($in->{locid});
	    if ($this->{location}) {
		$out .= $this->show;
		my $edit = $this->link(action => "edit");
		my $delete = $this->link(action => "delete");

		$out .= $share{ML}->p($share{ML}->a($share{ML}->img(undef, { src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{edit}{img}, border => "0" }), { href => $edit }) .
			$share{ML}->a($share{ML}->img(undef, { src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{delete}{img}, border => "0" }), { href => $delete })) . "\n";
	    }
	}
	else {
	    $out .= $share{ML}->p("No location selected.", { class => "error" }) . "\n";
	}
    }
    
    $out .= $share{ML}->h1($config{LocationFinder}{heading}{mylocations}) . "\n";

    #$out .= $this->set_section_id;
    $this->{section_id} = $this->get_section_id;
    
    $out .= $this->report;
    return $out;
}

sub newlocation {
    my $this = shift;
    my $in = $this->{input};        
    my $db = $share{DB};
    my $sid = $in->{section_id} || ($share{Page} ? $share{Page}->id("section") : "0");
    my $action = $this->link(action => "doform");
    my $out .= $share{ML}->h1($config{LocationFinder}{heading}{new}) . "\n" . $share{ML}->dl() . "\n";
    $out .= $db->make(table => $this->{location_table},
		      action => $action,
		      data => { section_id => $sid },
		      extra => { action => "doform" },
    );
    
    return $out;
}

sub doform {
    my $this = shift;
    my $in = $this->{input};        
    my $db = $share{DB};
    my $out;
    
    $db->get_data;
    # tweak the form data
    my @locid = keys %{$db->{form}{$this->{location_table}}};
    my $locid = $locid[0] || "_";    
    
    # tie location to current site
    my $site = $in->{section_id} || $db->this_site()->{section_id};
    #if ($db->{form}{$this->{location_table}}{$locid}{section_id}) {
	# ownership check
	#if (! $db->owns($db->{form}{$this->{location_table}}{$locid}{section_id})) {
	#    return $this->error("You do not have permission to add locations to this website.");
	#}
    #}
    if (! $db->is_admin) {
	$db->{form}{$this->{location_table}}{$locid}{section_id} = $site;
	$this->warn("Location changed to a local location - you do not have permission to add global locations.");
    }    
    
    # process the form
    $out = $db->do;

    if ($out =~ /Done/) {
	my $url = $this->link(action => undef);
	$out .= $share{ML}->h3("Location Updated Successfully") . 
		$share{ML}->p($share{ML}->a("Click here to return to location view", { href => $url }) . ".") . "\n";
    }
    
    return $out;
}

sub report {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    my $r = new ExSite::ReportBuilder();

    my $new = $this->link(action => "new", section_id => 0);
    $out .= $share{ML}->a("Add New Location", { class => "nav", href => $new});
    $out .= $share{ML}->br() . $share{ML}->br() . $share{ML}->br();
    
    $out .= $this->search_form();
	      
    $out .= $this->alphabet();  
    
    # shared events
    if ($db->is_admin) 
    {
	my $in = $this->{input};
	my $char = $in->{char} || "A";
	my $loc_name = $in->{loc_name} || "";
	
	my $searchTerm = $loc_name || $char . "%";
	$searchTerm = "%$searchTerm%" if ($loc_name);

	#my @l = $db->fetch_match($this->{location_table}, { section_id => 0 });
	my @l = $db->fetch_match($this->{location_table}, { name => "$searchTerm" });

	$r->init(title => "Locations",
		headers => [ "Name","Address","City","State/Province","Description", "Options" ],
		nodata => "No locations found.",
		foot => $this->standard_report_rowcount(\@l),
	);
	
	#if (@l > 0)
	#{
	foreach my $l (@l) 
	{
		$r->push($this->report_row($l));
	}

	#my $imageAttributes = { 
	#	src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{new}{img}, 
	#	border => "0",
	#	alt => "new location",
	#	title => "new location",
	#};

	#$r->tools($share{ML}->a($share{ML}->img(undef, $imageAttributes), { href => $new }));

	$out .= $r->make;
	$out .= $share{ML}->p() . "\n";
	#}
    }

    # site-specific events
    #my $site = $db->fetch("section", $this->{section_id});
    #$r->init(title => $site->{title},
    #	     headers => [ "Name","Address","City","State/Province","Description","Options" ],
    #	     nodata => "No locations found.",
    #	     );

    #if (&site_license("LocationFinder", $site->{section_id})) {
    #	my @l = $db->fetch_match($this->{location_table}, { section_id => $site->{section_id} });
	
    #	if (@l > 0) {
    #	    foreach my $l (@l) {
    #		$r->push($this->report_row($l));
    #	    }
    #	}
	
    #	my $new = $this->link(action => "new", section_id => $site->{section_id});
	
    #	my $imageAttributes = {
    #		src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{new}{img},
    #		border => "0",
    #		alt => "new location",
    #		title => "new location",
    #	};
    #	$r->tools($share{ML}->a($share{ML}->img(undef, $imageAttributes), { href => $new }));
    #	$out .= $r->make;
    #	$out .= $share{ML}->p() . "\n";
    #}

    return $out;
}

sub report_row {
    my ($this,$l) = @_;
    my @data;
    $this->{location} = $l;
    my $edit = $this->link(action => "edit", locid => $l->{location_id});
    #my $view = $this->link(action => "show", locid => $l->{location_id});
    my $delete = $this->link(action => "delete", locid => $l->{location_id});
    
    #my $viewAttributes = { 
    #	    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{view}{img},
    #	    border => "0",
    #	    alt => "info",
    #	    title => "info"
    #};
    my $editAttributes = { 
	    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{edit}{img},
	    border => "0",
	    alt => "edit",
	    title => "edit"
    };
    my $deleteAttributes = {
	    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{delete}{img},
	    border => "0",
	    alt => "delete",
	    title => "delete"
    };
    #$share{ML}->a($share{ML}->img(undef, $viewAttributes), { href => $view }) .
    my $imageLinks = $share{ML}->a($share{ML}->img(undef, $editAttributes), { href => $edit }) .
    		     $share{ML}->a($share{ML}->img(undef, $deleteAttributes), { href => $delete }) . "\n";
    
    push @data, ($this->show_data("name"),
    		 $this->show_data("address"),
    		 $this->show_data("city"),
		 $this->show_data("state_prov"),
		 $this->show_data("description"),
		 $imageLinks);
    return @data;
}

sub alphabet {
	my ($this) = @_;
	my @alpha = ("A" .. "Z");
	my $out;

	$share{ML}->Clear;
	foreach my $letter (@alpha) {
		my $url = $this->link(char=>$letter);
		$share{ML}->_a($letter, { href=>$url });
	}
	$share{ML}->__p({class=>"alphabet"});
	return $share{ML}->Write();
}

sub standard_report_rowcount {
	my $this = shift;
	my $objects = shift;
	my $num = scalar(@$objects);

	return "No records found." if (! $num );
	return "$num records found";
}

sub search_form {
	my ($this) = @_;
	
	my @search_columns = split(/ /, $config{LocationFinder}{search_columns});
	my $action = $this->link();
	#my $page_id = $this->{query}->{_id} || undef;
	my $table = $share{ML}->div(
		$share{ML}->form("Name: " . 
				      $share{ML}->br() . 
				      $share{ML}->input(
						undef,
						{ type => "input", name => "loc_name"}
				                ) . " " .
				      $share{ML}->input(
						undef, 
						{ type => "submit", value => " Search " }
						),	
				      { method => "post" })
	, {class => "search_div"});
	
	return $share{ML}->br() . $table . $share{ML}->br();
}

sub locationEditer {
	my ($this) = @_;
	my $db = $share{DB};
	
	my $in = $this->{input};
	my $out;
	my $member_id = $db->my_uid;

	my $level = $db->authorize;
	if (!$level) 
	{
		return $this->error("You do not have permission to edit locations.");
	}
		
	if ($in->{cmd} =~ /edit_location/)
	{		
		my $action = $this->link(cmd => "doform");
		my $reply = $this->link(cmd => undef, loc_id => undef);
	
		$out .= $db->make(
			table => "location_finder",
			record => $in->{loc_id},
			action => $action,
			extra => { reply => $reply }
		);
	}
	elsif ($in->{cmd} =~ /new/)
	{
		my $sid = $in->{section_id} || ($share{Page} ? $share{Page}->id("section") : 0);
		my $action = $this->link(cmd => "doform");
		my $reply = $this->link(cmd => undef);
	
		$out .= $db->make(
			table => "location_finder",
			record => $in->{loc_id},
			action => $action,
			data => { section_id => $sid, member_id => $member_id },
			extra => { reply => $reply }
		);
	}
	else
	{
		my $addNewLocation;
		
		if ($config{LocationFinder}{member_can_add_store} || $db->is_admin())
		{
			my $addNewLink = $this->link(cmd => "new");
			$addNewLocation = $share{ML}->b($share{ML}->a("Add new location", { href => $addNewLink }));
		}
	
		my @l;
		
		if ($db->is_admin())
		{
			@l = $db->fetch_all($this->{location_table}); 	
		}
		else
		{
			@l = $db->fetch_match($this->{location_table}, { member_id => "$member_id" });
		}
		
		@l = sort { lc($a->{name}) cmp lc($b->{name}) } @l;

		my $storeLocations;

		foreach my $location (@l) {
			my $url = $this->link( loc_id => $location->{location_id}, cmd => "edit_location" );
			$storeLocations .= $share{ML}->a($location->{name}, { href => $url }) . $share{ML}->br();
		}
		
		$storeLocations = "No locations found." if (!$storeLocations);
		
		$out .= $share{ML}->table(
			$share{ML}->tr(
				$share{ML}->td($addNewLocation)
			) .
			$share{ML}->tr(
				$share{ML}->td("\&nbsp;")
			) .
			$share{ML}->tr(
				$share{ML}->td($share{ML}->h3("Edit Location") . $storeLocations)
			),
			{ class => "editLocationTable" }
		);
	}
	
	return $out;
}

### FETCHING EVENTS

sub fetch {
    my ($this,$id) = @_;
    my $db = $share{DB};
    my $l = $db->fetch($this->{location_table}, $id);
    
    # validate that event belongs to this site
    if ($l->{location_id}) {
	if (! $this->{admin} && 
	    ($l->{location_id} != $share{DB}->this_site()->{location_id})) {
	    return undef; 
	}
    }
    
    $this->{location} = $l;
    
    return $l;
}

1;
