package Modules::Logs;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::DB;
use ExSite::Misc;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    return undef;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "The Logs tools is used to review your ExSite logs for
events, errors, or other incidents.";
    }
    elsif (/Category/) {
	return "System";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    return;
}

sub ctrl_panel {
    my $this = shift;
    return "<h1 class=\"error\">Permission denied</h1>"
	if (! $share{DB}->is_admin);

    my @logs = ("db", "cms", "general");
    my %query = %{$this->{input}};
    if (! $this->{input}{log}) {
	%query = ( log=>$logs[0], nlines=>25, format=>"formatted" );
    }
    my $out = "<form method=GET>\n";
    $out .= "Logfile: <select name=log onChange=submit()>\n";
    foreach my $log (@logs) {
	$out .= "<option";
	if ($log eq $query{log}) { $out .= " selected"; }
	$out .= ">$log</option>\n";
    }
    $out .= "</select>&nbsp;&nbsp;&nbsp;&#149;&nbsp;&nbsp;&nbsp;\n";
    $out .= "Show last <select name=nlines onChange=submit()>\n";
    foreach my $nl (10,25,100,0) {
	$out .= "<option value=$nl";
	if ($nl == $query{nlines}) { $out .= " selected"; }
	if ($nl) { $out .= ">$nl"; }
	else { $out .= ">all"; }
	$out .= "</option>\n";
    }
    $out .= "</select>\nmessages&nbsp;&nbsp;&nbsp;&#149;&nbsp;&nbsp;&nbsp;\n";
    $out .= "Formatting: <select name=format onChange=submit()><option";
    if ($query{format} eq "formatted") { $out .= " selected"; }
    $out .= ">formatted</option><option";
    if ($query{format} eq "unformatted") { $out .= " selected"; }
    $out .= ">unformatted</option></select>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n";
    #$out .= "<input type=submit>\n</form>\n<hr>\n";
    $out .= "</form>\n";
    
    # show the logs
    
    my @logdata = $share{DB}->showlog($query{log},$query{nlines});
    if ($query{format} eq "unformatted") {
	$out .= "<pre>\n".join("",@logdata)."\n</pre>\n";
    }
    else {
	my $r = new ExSite::ReportBuilder;
	$r->title($query{log});
	$r->headers(["Time","User","Request","Message"]);
	foreach my $row (@logdata) {
	    $r->push(split(/\t/,$row));
	}
	$out .= $r->make;
    }
    return $out;
}

1;

