package Modules::Member;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

# Member : simple toolkit for managing members

### TODO
###
### handler for selecting only those sites with member service
### public membership sign-up

use strict;
use ExSite::Config;
use ExSite::Mail;
use ExSite::Misc;
use ExSite::ML;
use ExSite::HTML;
use ExSite::FormBuilder;
use ExSite::Captcha;
use Modules::BaseDCD;

use vars qw(@ISA $ml %email);
@ISA = qw(Modules::BaseDCD);

# email templates

%email = (
    thanks => "Thank you for registering as a member of [[title]].
Keep this message for your records.

Login ID: [[login]]
Password: [[password]]

You can use this information to access private areas of [[title]].

[[url]]\n",
    notify => "The following person has registered as a member of [[title]]:

Name:  [[name]]
Login: [[login]]
Email: [[email]]",
);

sub write {
    my ($this,$options) = @_;
    my $db = $share{DB};
    $ml = new ExSite::ML;
    my $out;
    my $in = new ExSite::Input;
    my $data = $in->post;
    if ($options eq "name") {
	return $this->whoami;
    }
    elsif ($options eq "newmember") {
	if (keys %$data > 0) {
	    $out .= $this->update($data);
	}
	else {
	    $out .= $this->new_member_form;
	}
    }
    elsif ($db->authorize) {
	$out .= $this->my_acct($options,$data);
    }
    else {
	$out = $ml->p($msg{"You are not logged in"});
    }
    return $out;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    # handle the standard ioctl requests for webtop integration
    $ml = new ExSite::ML;
    if (/ModuleName/) {
	# the user-friendly name for this app
	return "Members";
    }
    elsif (/ControlPanel/) {
	# a reference to the administrator control panel for this app
	return \&ControlPanel;
    }
    elsif (/isRestricted/) {
	return exists $config{Member}{isRestricted} ? $config{Member}{isRestricted} : 0;
    }
    elsif (/Category/) {
	return "System";
    }
    elsif (/ModuleInfo/) {
	return $msg{"This is a member-management tool that lets you create, 
delete, and edit the various users who have permission to log in to the
private members-only areas of your website."};
    }
    elsif (/DynContentParameter/) {
	return $ml->p(
		      $ml->select( [
				    [ "option", $msg{"(default)"}, {value=>""} ],
				    [ "option", $msg{"New Member Sign-up Form"}, { value=>"newmember" } ],
				    [ "option", $msg{"Update password"}, { value=>"password" } ],
				    [ "option", $msg{"Update contact info"}, { value=>"contact" } ],
				    [ "option", $msg{"Update password OR contact info"}, { value=>"" } ],
				    [ "option", $msg{"Show name of current member"}, { value=>"name" } ]
				    ], { name=>"param"})
		      );
    }
    return undef;
}

sub ControlPanel {
    my $this = shift;
    my $db = $share{DB};
    $ml = new ExSite::ML;
    my $out = $this->set_section_id();
    my $cmd = $this->{input}{mbrcmd};

    if ($cmd =~ /myacct|pass|contact/) {
	my $in = new ExSite::Input;
	return $this->my_acct(undef,$in->post());
    }

    my @sites;
    if ($this->{input}{site} && $db->owns($this->{input}{site})) {
	@sites = (scalar $db->fetch("section",$this->{input}{site}));
    }
    else {
	@sites = $db->my_real_sites;
    }

    # administrator options

    if ($db->is_admin) {
	if ($cmd eq "all") {
	    return $db->report_relational({tab=>$config{auth}{user_table}});
	}
	elsif (@sites > 1) {
	    my $url = $this->link(mbrcmd=>"all",member=>undef);
	    $out .= $ml->p(
			   $ml->a("All members",{href=>$url}).
			   $msg{"(system administrator view)"}
			   );
	}
	my $url = $this->link(mbrcmd=>"myacct",member=>undef);
	$out .= $ml->p(
		       $ml->a("My account",{href=>$url})
		       );
    }

    if ($this->{input}{member}) {
	my $member = $db->fetch($config{auth}{user_table},$this->{input}{member});
	if ($member) {
	    if (! $db->owns($member->{section_id})) { 
		return $this->error($msg{"Permission denied."});
	    }
	}
	else {
	    return $this->error($msg{"Permission denied - invalid member ID."});
	}
    }

    # hidden fields in member forms
    my @hide;
    if (! $db->is_admin) { @hide = ("access"); }
    if (@sites == 1) { push @hide,"section_id"; }

    # execute member functions

    if ($cmd eq "edit") {
	my $action = $this->link(caladm=>"submit",member=>undef);
	$out .= $ml->h1("Edit Member");
	$out .= $db->make(table=>$config{auth}{user_table},
			  record=>$this->{input}{member},
			  action=>$action,
			  hide=>\@hide,
			  extra=>{mbrcmd=>"submit"},
			  );
	return $out;
    }
    elsif ($cmd eq "new") {
	my $action = $this->link(caladm=>"submit",member=>undef);
	$out .= $ml->h1("New Member");
	$out .= $db->make(table=>$config{auth}{user_table},
			  action=>$action,
			  hide=>\@hide,
			  data=>{section_id=>$this->{input}{site}},
			  extra=>{mbrcmd=>"submit"},
			  );
	return $out;
    }
    elsif ($cmd eq "submit") {
	$db->get_data;
	if ($db->{submit_action} ne "Cancel") {
	    # tweak the form data
	    my @memid = keys %{$db->{form}{member}};
	    my $memid = $memid[0] || "_";
	    $db->{action} = (exists $db->{form}{member}{$memid}{member_id} ? 
			     "update" : "insert");
	    
	    # validate inputs

	    my @err;
	    if ($db->{form}{member}{$memid}{section_id}) {
		# ownership check
		if (! $db->owns($db->{form}{member}{$memid}{section_id})) {
		    push @err, $msg{"Invalid website - permission denied."};
		}
	    }
	    elsif ($db->{action} ne "update") {
		push @err, $msg{"Member does not belong to any website."};
	    }
	    my @mem;
	    if ($db->{action} eq "insert") {
		@mem = $db->fetch_match($config{auth}{user_table},{login=>$db->{form}{member}{$memid}{login}});
	    }
	    else {
		@mem = $db->custom_query("select * from member where login=\"$db->{form}{member}{$memid}{login}\" and member_id != $memid");
	    }
	    
	    if (@mem > 0) {
		push @err, $msg{"That login name has been taken - please choose another."};
	    }
	    if (@err > 0) {
		$out .= $ml->h3($msg{"Problem:"}).
		    $ml->p(join($ml->br(),@err), {class=>"error"});
		$out .= $ml->a($msg{"Back to form"},{href=>"javascript:history.back()"});
		return $out;
	    }
	    # set the access level to no higher than the current member
	    $db->{form}{member}{$memid}{access} = 
		$db->{form}{member}{$memid}{access} <= $db->level ? 
		($db->{form}{member}{$memid}{access} || 1) : 1;
	    # process the form (more validation done here)
	    $db->do;
	    # check errors
	    if ($db->errorcheck) {
		$out = $ml->h3("Problem:");
		$out .= $ml->p($db->show_diagnostics("error","html"));
		$out .= $ml->p($ml->a($msg{"Back to form"}, {href=>"javascript:history.back()"}));
	    }
	}
    }
    elsif ($cmd eq "del") {
	my $m = $db->fetch($config{auth}{user_table},$this->{input}{member});
	if (! $db->owns($m->{section_id})) {
	    return $this->error($msg{"Permission denied."});
	}
	$db->trash_key($config{auth}{user_table},$this->{input}{member});
	$out .= $ml->p("$msg{Deleting} $m->{name}... $msg{done}!");
    }

    # membership reports

    my $url;
    my $sid = $this->get_section_id();
    return $out if (! $sid);
    my $site = $share{DB}->fetch("section",$sid);
    my %report = ( title => "$site->{title} $msg{Members}",
		   nodata => $msg{"No members found."},
		   headers => [ $msg{Name}, $msg{Login}, $msg{'E-mail'}, $msg{Options} ],
		   data => [] );
    my @members = $db->fetch_child($config{auth}{user_table},"section",$site->{section_id},$config{auth}{user_sort_column});
    if (@members > 0) {
	foreach my $m (@members) {
	    my @data;
	    push @data, ($m->{name},$m->{login},
			 $db->show_data($config{auth}{user_table},"email",$m->{email}));
	    my $url = $this->link(mbrcmd=>"edit",member=>$m->{member_id});
	    my $opt = $ml->a(
			     $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{edit}{img}", height=>"$config{db_ops}{edit}{img_height}", width=>"$config{db_ops}{edit}{img_width}", border=>0, alt=>"edit", title=>"edit"}),
			     { href=>$url }
			     );
	    $url = $this->link(mbrcmd=>"del",member=>$m->{member_id});
	    $opt .= $ml->a(
			   $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{delete}{img}", height=>"$config{db_ops}{delete}{img_height}", width=>"$config{db_ops}{delete}{img_width}", border=>0, alt=>"delete", title=>"delete"}),
			   { href=>$url }
			   );
	    push @data,$opt;
	    push @{$report{data}},\@data;
	}
    }
    $url = $this->link(mbrcmd=>"new",,member=>undef,site=>$site->{section_id});
    $report{tools} = $ml->a(
			    $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{new}{img}", height=>"$config{db_ops}{new}{img_height}", width=>"$config{db_ops}{new}{img_width}", border=>0, alt=>"add new member", title=>"add new member"}),
			    { href=>$url }
			    );
    my $r = new ExSite::ReportBuilder(%report);
    $out .= $r->make;
    return $out;
}

sub my_acct {
    my ($this,$options,$data) = @_;
    my $db = $share{DB};
    my $out;

    if (keys %$data > 0) {
	$out .= $this->update($data);
    }
    elsif ($this->{input}{mbrcmd}) {
	$_ = $this->{input}{mbrcmd};
	if (/pass/) {
	    return $this->passwd_form;
	}
	elsif (/contact/) {
	    return $this->contact_form;
	}
    }
    elsif ($options eq "password") {
	return $this->passwd_form;
    }
    elsif ($options eq "contact") {
	return $this->contact_form;
    }
    my $name = $this->show_data("name");
    my $contact_url = $this->link(mbrcmd=>"contact");
    my $pwd_url = $this->link(mbrcmd=>"pass");
    $out .= $ml->div(
		     $ml->h1($name).
		     $ml->table(
				$ml->tr(
					$ml->td(
						$ml->a(
						       $ml->img(
								undef,
								{src=>"$config{server}{HTMLpath}/_Modules/Member/contact.gif", height=>100, width=>100, border=>0}
								).
						       $ml->br().
						       $msg{"My Contact Info"},
						       { href=>$contact_url }
						       ),
						{ align=>"center" }
						).
					$ml->td(
						$ml->a(
						       $ml->img(
								undef,
								{src=>"$config{server}{HTMLpath}/_Modules/Member/passwd.gif", height=>100, width=>100, border=>0}
								).
						       $ml->br().
						       $msg{"My Password"},
						       { href=>$pwd_url }
						       ),
						{ align=>"center" }
						)
					),
				{ cellpadding=>50 }
				),
		     { align=>"center" }
		     );
    return $out;
}


sub whoami {
    my $this = shift;
    return $share{DB}->my_name;
}

sub show_data { 
    my ($this,$col) = @_;
    my $db = $share{DB};
    return $db->show_data($config{auth}{user_table},$col,$share{identity}{lookup}{$col});
}

sub new_member_form {
    my $this = shift;
    my $url = $this->link;
    my ($pwd_form,$current_user);
    if (! $config{Member}{allow_new_member}) {
	return $this->error($msg{"New member sign-up is not permitted."});
    }
    my $db = $share{DB};
    if ($db->level()) {
	$current_user = $this->warn($msg{"You are already logged in as"}.
				    " \"".$db->my_login."\" (".$db->my_name.").");
    }
    my $c = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
    my $captcha = $c->make();

    my $form = $db->form(action=>$url,method=>"post");
    my @fields = $this->get_contact_fields();
    foreach my $fld (@fields) {
	$db->input_column(
			  table=>$config{auth}{member_table},
			  column=>$fld->{column},
			  name=>$fld->{column},
			  prompt=>$fld->{label},
			  datatype=>$fld->{datatype},
			  required=>($fld->{validate}=~/soft|hard/),
			  );
    }
    # add login info
    $db->input_column(
		      table=>$config{auth}{member_table},
		      column=>$config{auth}{user_login_column},
		      name=>$config{auth}{user_login_column},
		      prompt=>$msg{"Choose a Login ID"},
		      datatype=>"text",
		      required=>1,
		      );
    if (! $config{Member}{mail_password}) {
	$db->input_column(
			  table=>$config{auth}{member_table},
			  column=>$config{auth}{user_pwd_column},
			  name=>"passwd",
			  prompt=>$msg{Password},
			  datatype=>"password",
			  required=>1,
			  );
	$db->input_column(
			  table=>$config{auth}{member_table},
			  column=>$config{auth}{user_pwd_column},
			  name=>"passwd2",
			  prompt=>"$msg{Password} (".$msg{"enter again, to confirm"}.")",
			  datatype=>"password",
			  required=>1,
			  );
    }
    $form->input(name=>"captcha",prompt=>" ",input=>$captcha);
    $form->input(type=>"hidden",name=>"newmember",value=>1);
    my $out = $ml->h1($msg{"New Member Sign-up"});
    $out .= $form->make();
    return $out;
}

sub passwd_form {
    my $this = shift;
    my $url = $this->link;
    my $db = $share{DB};
    my $form = $db->form(action=>$url,method=>"post");
    $form->input(
		 type=>"password",
		 name=>"passwd0",
		 prompt=>$msg{"Old Password"}.":",
		 required=>1,
		 );
    $form->input(
		 type=>"password",
		 name=>"passwd1",
		 prompt=>$msg{"New Password"}.":",
		 required=>1,
		 );
    $form->input(
		 type=>"password",
		 name=>"passwd2",
		 prompt=>$msg{"New Password"}." ".$msg{"(again, to confirm)"}.":",
		 required=>1,
		 );
    my $out = $ml->h1($msg{"Change Password"});
    $out .= $form->make();
    return $out;
}

sub contact_form {
    my $this = shift;
    my $url = $this->link;
    my $db = $share{DB};
    my $m = $db->my_user_record();
    my $form = $db->form(action=>$url,method=>"post");
    my @fields = $this->get_contact_fields();
    foreach my $fld (@fields) {
	$db->input_column(
			  table=>$config{auth}{member_table},
			  column=>$fld->{column},
			  record=>$fld->{auth}{user_uid_column},
			  name=>$fld->{column},
			  prompt=>$fld->{label},
			  value=>$m->{$fld->{column}},
			  data=>$m,
			  datatype=>$fld->{datatype},
			  required=>($fld->{validate}=~/soft|hard/),
			  );
    }
    my $out = $ml->h1($msg{"Update Contact Info"});
    $out .= $form->make();
    return $out;
}

sub get_contact_fields {
    my $this = shift;
    my $map = $share{DB}{map};
    my $level = $share{DB}->level();
    my @col =  $config{Member}{contact_columns} ?
	@{$config{Member}{contact_columns}} : 
	$map->get_columns($config{auth}{user_table});
    my @fields;
    foreach my $col (@col) {
	my %col = $map->get_column($config{auth}{user_table},$col);
	if ($col{write} <= $level) {
	    if ($col{datatype} =~ /^(string|text|list|email|url)/ && 
		$col{column} ne $config{auth}{user_login_column}) {
		# allow simple input fields only
		push @fields,\%col;
	    }
	}
    }
    return wantarray ? @fields : \@fields;
}

sub update {
    my ($this,$in) = @_;
    my $out;
    delete $in->{submit_action};
    my $db = $share{DB};
    if ($in->{passwd0}) {
	my $err = $db->change_password($in->{passwd0},$in->{passwd1},$in->{passwd2});
	if ($err) {
	    $out = $ml->p("ERROR: $err",{class=>"error"});
	}
	else {
	    $out = $this->info($msg{"Password was changed sucessfully."});
	}
    }
    elsif (exists $in->{newmember}) {
	# new member
	return $this->make_member($in);
    }
    elsif (exists $in->{name} || exists $in->{email}) {
	if (! $in->{name}) {
	    return $this->error($msg{"You must provide a name."});
	}
	elsif (! $in->{email}) {
	    $out .= $ml->warn($msg{"without a defined email address, you may miss important updates and notices."});
	}
	my $err = $db->update($config{auth}{user_table},$in,{member_id=>$db->my_uid});
	if ($err) {
	    $out .= $ml->p($err,{class=>"error"});
	}
    }
    return $out;
}

sub make_member {
    my ($this,$in) = @_;
    delete $in->{submit_action};
    if (! $config{Member}{allow_new_member}) {
	return $this->error($msg{"New member sign-up is not permitted."});
    }
    my $db = $share{DB};

    # validate inputs

    my @err;
    if ($in->{captcha}) {
	my $c = new ExSite::Captcha();
        if (! $c->pass()) {
    	    push @err, $msg{"Sorry, you entered the incorrect code."};
        }
    }
    else {
	push @err, $msg{"Error - missing captcha!"};
    }
    if (! $in->{login}) {
	push @err, $msg{"You must provide a login ID."};
    }
    elsif (length $in->{login} < 3) {
	push @err, $msg{"Login ID too short - please choose another."};
    }
    else {
	my @users = $share{DB}->fetch_match($config{auth}{user_table},{login=>$in->{login}});
	if (@users > 0) {
	    push @err, $msg{"That login ID has been taken - please choose another."};
	}
    }
    if (! $in->{name}) {
	push @err, $msg{"You must provide your name."};
    }

    my $email_re = $share{DB}{map}->regexp("email");
    if (! $in->{email}) {
	push @err, $msg{"You must provide a valid email address."};
    }
    elsif ($in->{email} !~ /$email_re/) {
	push @err, $msg{"Invalid email address!"};
    }

    if ($config{Member}{mail_password}) {
	$in->{passwd} = &randtext(6,'ABCDEFGHJKLMNPQRSTUVWXYZ23456789');
    }
    elsif (! $in->{passwd}) {
	push @err, $msg{"You must provide a password."};
    }	    
    elsif (! $in->{passwd2}) {
	push @err, $msg{"You must confirm your password."};
    }
    elsif ($in->{passwd} ne $in->{passwd2}) {
	push @err, $msg{"You failed to confirm your password - please try again."};
    }
    else {
	my $pwd_re = $share{DB}{map}->regexp("password");
	if ($in->{passwd1} =~ /$pwd_re/) {
	    push @err, $msg{"Password too short - please choose a better password."};
	}
    }
    
    # create the member
    
    if (@err > 0) {
	return $this->error(@err);
    }
    else {
	my $site = $share{DB}->this_site();
	my %user = (
		    login=>$in->{login},
		    name=>$in->{name},
		    email=>$in->{email},
		    password=>$in->{passwd},
		    access=>1,
		    section_id=>$site->{section_id}
		    );
	my $uid = $share{DB}->insert($config{auth}{user_table},\%user);
	if ($uid) {
	    my $out = $ml->h1($msg{"Welcome"}).
		$ml->p($msg{"User ID"}." \"$user{login}\" ".$msg{"was created successfully.  You may now visit the restricted areas of this website."});

	    # notifications
	    if ($site) {
		my $s = new ExSite::Section(section=>$site);
		my $title = $s->get_my("title");
		my $url = $s->get_url();
		my %subst = (title=>$title,
			     login=>$user{login},
			     password=>$user{password},
			     url=>$url,
			     email=>$user{email},
			     name=>$user{name});
		&SendMail($user{email},
			  $share{DB}->owner_email($site),
			  "$title ".$msg{"Login Information"},
			  &substitute($msg{$email{thanks}},\%subst) );
		if ($config{Member}{newmember_notify}) {
		    &SendMail($share{DB}->owner_email($site),
			      $share{DB}->owner_email($site),
			      "$title - New Member",
			      &substitute($msg{$email{notify}},\%subst) );
		}
		$out .= $ml->p($msg{"Your login information has been sent to"}." $user{email}.");
	    }

	    if (! $share{DB}->authorize()) {
		# user is NOT logged in yet, so let's log them in
		$share{DB}->do_login(\%user);
	    }
	    return $out;
	}
	else {
	    my $message = $msg{"Failed to create new member.  Please contact the system administrator at [[email]] for further assistance."};
	    my $email = &HTML::MailTo($config{site}{administrator});
	    return $ml->p(&substitute($message,{email=>$email}));
	}
    }
}

1;
