package Modules::MySite;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#   MySite is a simplified content management system that is suitable for
#   newbies and ExSite-Lite users.  It can inline all CMS functions into the
#   website itself, rather than the administrator control panel.  That 
#   makes it a little harder to set up, since a website must preexist to 
#   embed the MySite tools into;  but much easier for casual users to use 
#   once the parent site is running.
#
#   Features of MySite:
#       - creates native websites only
#       - simplified library management ("My Image Library")
#       - simplified page creation assumes common defaults
#       - inline content editor appears right in page
#       - one-step content updates
#
#   For advanced CMS features like template building, version control, 
#   polymorphic content, dynamic page configuration, raw-HTML editing, etc. 
#   you must use the CMS module instead.
#----------------------------------------------------------------------------

use strict;
use MIME::Base64;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Util;
use ExSite::HTML;
use ExSite::Mail;
use ExSite::Input;
use ExSite::Image;
use ExSite::Section;
use ExSite::Tree;
use ExSite::FormBuilder;
use Modules::BaseDCD;

use vars qw(@ISA $ml);
@ISA = qw(Modules::BaseDCD);

my $rowcount;

# write : redirect to user control panel, or prompt for login

sub write {
    my ($this,$options) = @_;
    $ml = &get_obj("ML");
    my %odata = &DecodeString($options);
    my $cmd = $odata{ecmd} || $this->{input}{ecmd};
    # public tools
    if ($cmd eq "remind") {
	return $this->reminder;
    }
    elsif ($cmd eq "menu") {
	return "";  # deprecated function
    }
    # member-only tools
    if ($share{DB}->level() >= 2) {
	return $this->my_switchboard($options);
    }
    # login
    return $this->Authenticate($options);
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "My Website";
    }
    elsif (/ControlPanel/) {
	#my @sites = $share{DB}->my_real_sites;
	#if (@sites > 1) {
	#    # too many sites - redirect to CMS instead
	#    return "ctrl-panel.cgi/CMS";
	#}
	# only one site, so we can use MySite Control Panel
	return \&my_switchboard;
    }
    elsif (/Category/) {
	return "CMS";
    }
    elsif (/ModuleInfo/) {
	return "My Website is a simple, user-friendly website management
tool.  It doesn't support the full range of features of the Website 
Editor, but makes all of the commonly-used features easier to use.
My Website lets you easily create and delete pages, edit your pages,
change your design templates, and upload and manage a library of photos
or other images.";
    }
    return undef;
}

sub mysite {
    my $this = shift;
    if (! $this->{site}) {
	my $section_id;
	my $db = $share{DB};
	if ($this->{input}{sid} && $db->owns($this->{input}{sid})) {
	    # we own the selected site
	    my $site = $db->fetch("section",$this->{input}{sid});
	    $this->{site} = $site if ($site);
	}
	else {
	    # no site was selected
	    my %u = $db->my_user_record;
	    my @sites = $share{DB}->my_real_sites;
	    my $sites = &keywise("section_id",\@sites);
	    if ($u{section_id} && $sites->{$u{section_id}}) {
		# we own our home site
		$this->{site} = $db->fetch("section",$u{section_id});
	    }
	    else {
		# take the first site we own
		my @sites = $share{DB}->my_real_sites;
		$this->{site} = $sites[0];
	    }
	}
    }
    return $this->{site};
}

sub mysite_selector {
    my $this = shift;
    my @sites = $share{DB}->my_real_sites;
    return undef if (@sites <= 1);
    my $pid = $this->{input}{_id};
    my $sid = $this->{site} ? $this->{site}{section_id} : 0;
    my $out = $msg{"Select website"}.": ";
    my $options = $ml->option("== ".$msg{"choose a site"}." ==",{value=>0});
    foreach my $s (@sites) {
	my %optattr = ( value=>$s->{section_id} );
	if ($s->{section_id} == $sid) { $optattr{selected} = undef; }
	$options .= $ml->option($s->{title},\%optattr);
    }
    $out .= $ml->select($options,{name=>"sid",onchange=>"document.forms[0].submit()"});
    $out .= $ml->input(undef,{type=>"hidden",name=>"_id",value=>$pid}) if $pid;
    return $ml->form($out,{method=>"GET"});
}

sub myurl {
    my $this = shift;
    my $site = $this->mysite;
    my $s = new ExSite::Section(section=>$site);
    return $s->get_url;
}

# user's control panel

sub my_switchboard {
    my ($this,$options) = @_;
    $ml = &get_obj("ML");
    my $db = $share{DB};
    my $site = $this->mysite;
    if (! $site) {
	return $msg{"Permission denied - no website found."};
    }
    my $in = new ExSite::Input;
    my $data = $in->combine();
    my %odata = &DecodeString($options);

    if ($data->{submit_action} eq "Cancel") {
	# revert to preview page
	$odata{ecmd} = "edit_content_link";
	delete $data->{ecmd};
    }

    # special handling of inlined editing tools

    if ($odata{ecmd} eq "edit_content_link") {
	if ($data->{ecmd} eq "update") {
	    # replace the edit link with an editor
	    return $this->update_body;
	}
	elsif ($data->{ecmd} eq "doupdate") {
	    return $this->do_update;
	}
	else {
	    # insert a link to edit content inline
	    my $url = $this->link(ecmd=>"update");
	    return $ml->br.
		$ml->a(
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/b_edit.gif", height=>16, width=>64, border=>0, alt=>"Edit this page", style=>"padding:2px;"}),
		       {href=>$url}
		       );
	}
    }

    # finally, user-selected options

    my $cmd = $data->{ecmd} || $odata{ecmd};
    if ($cmd eq "edit") {
	if ($share{Page}) {
	    $share{Page}->redirect("$config{server}{CGIpath}/home.cgi");
	}
	elsif ($share{PageBuf}) {
	    $share{PageBuf}->location("$config{server}{CGIpath}/home.cgi");
	}
    }
    elsif ($cmd eq "newsite" && $config{MySite}{allow_private_newsite}) {
	return $this->newsite;
    }
    elsif ($cmd eq "newpage") {
	return $this->newpage;
    }
    elsif ($cmd eq "pageconf") {
	return $this->pageconf;
    }
    elsif ($cmd eq "updatepage") {
	return $this->update_page;
    }
    elsif ($cmd eq "delpage") {
	return $this->delpage;
    }
    elsif ($cmd eq "rollback") {
	return $this->rollback;
    }
    elsif ($cmd eq "template") {
	return $this->template;
    }
    elsif ($cmd eq "config") {
	return $this->config;
    }
    elsif ($cmd eq "passwd") {
	return $this->passwd;
    }
    elsif ($cmd eq "images") {
	return $this->library;
    }
    elsif ($cmd eq "addimg") {
	return $this->image_add;
    }
    elsif ($cmd eq "delimg") {
	return $this->image_remove;
    }
    elsif ($cmd eq "updateimg") {
	return $this->image_update;
    }
    return $this->my_ctrl($site);
}

# mysite style sheet

sub mysite_css {
    my $this = shift;
    my $out;
    # to override the default CSS, set custom_css = 1 in the conf file
    if ($share{Page} && ! $config{MySite}{custom_css}) {
	$out .= $ml->link(undef,{rel=>"stylesheet",type=>"text/css",href=>"$config{server}{HTMLpath}/_Modules/MySite/MySite.css"});
    }
    # include some JS that we need, while we're at it
    $out .= $ml->script(undef,{type=>"text/javascript", src=>"$config{server}{HTMLpath}/_ExSite/js/misc.js"});
    return $out;
}

# my control panel

sub my_ctrl {
    my $this = shift;
    my $site = shift || $this->mysite;
    my $db = $share{DB};
    my $out = $this->mysite_selector;
    $out .= $ml->h1($site->{title});
    my $mapwid = $config{MySite}{sitemap_width} || 540;
    $out .= $this->mysite_css;
    my $options = $this->options_link();

    my ($sitemap,$other);

    # get all pages

    my %match = (section_id=>$site->{section_id},
		 status=>"active",);
    my @pages = $db->fetch_match("page",\%match,["rank","page_id"]);
    $rowcount = 0;
    my $tree = new ExSite::Tree("page_id","parent_id",@pages);

    # track sitemap info for javascript
    $this->{divs} = [];
    $this->{contents} = [];

    # pages

    $sitemap .= $ml->div($ml->div($options,{id=>"opt",class=>"MySiteOptions"})."Site Map",
		     {class=>"MySiteMapHeader"});
    $rowcount = -1;
    my @p = $tree->get_topnodes_data();
#    $sitemap .= $this->sitemap($tree);
    my $maplist = $this->sitemap($tree,["alternate","page"],@p);
    if ($maplist) {
	$sitemap .= &ExSite::HTML::DynList(@$maplist);
    }

    # libraries

    my @dlist;
    $maplist = $this->sitemap($tree,"library",@p);
    if ($maplist) {
	$sitemap .= $ml->div($msg{"Libraries"},{class=>"MySiteMapHeader"});
	$sitemap .= &ExSite::HTML::DynList(@$maplist);
    }

    # templates

    if ($config{MySite}{show_templates}) {
	@dlist = ();
	my $ttree = new ExSite::Tree("page_id","template_id",@pages);
	my @p = $ttree->get_topnodes_data();
	$maplist = $this->sitemap($ttree,"template",@p);
	if ($maplist) {
	    $sitemap .= $ml->div($msg{"Templates"},{class=>"MySiteMapHeader"});
	    $sitemap .= &ExSite::HTML::DynList(@{$this->sitemap($ttree,"template",@p)});
	}
    }

    # menu

    $other .= $ml->tr($ml->th($msg{"Other Functions"}));
    my $url = $this->page_limit < 0 ?
	"javascript:alert('".$msg{"You must upgrade your site to add pages."}."')" :
	$this->link(ecmd=>"newpage");
    $other .= $this->button($url,"ico_blank.gif",$msg{"New Page"});
    $url = $this->link(ecmd=>"template");
    $other .= $this->button($url,"ico_theme.gif",$msg{"Set Theme"});
    $url = "$config{server}{CGIpath}/publish.cgi?section=$site->{section_id}";
    $other .= $this->button("javascript:popup_large('$url')","ico_pub.gif",$msg{"Publish Site"});
    $url = $this->link(ecmd=>"config");
    $other .= $this->button($url,"ico_conf.gif",$msg{"Configure Site"});
    $url = $this->link(ecmd=>"passwd");
    $other .= $this->button($url,"ico_pass.gif",$msg{"Change Password"});
    if ($this->site_type eq "advanced") {
	$url = "$config{server}{CGIpath}/home.cgi";
	$other .= $this->button($url,"ico_adv.gif",$msg{"Advanced Control Panel"});
    }
    $url = "javascript:popup_medium('$config{server}{CGIpath}/help.cgi/_Modules/MySite/help.html')";
    $other .= $this->button($url,"ico_help.gif",$msg{"Help"});
    $url = "$config{server}{CGIpath}/logout.cgi";
    $other .= $this->button($url,"ico_logout.gif",$msg{"Logout"});

    $other = $ml->table($other,{class=>"MySiteMenu",width=>"90%"});

    my $myurl = $this->myurl;
    my $target = $config{MySite}{public_url_target} ?
	" target=\"$config{MySite}{public_url_target}\"" :
	"";
    $other .= $ml->p($ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/caution20.gif",height=>20,width=>20,align=>"bottom"})." ".
		     $msg{"You must publish your site before your changes will be visible to the public!"});
    $other .= $ml->p($ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/rarr.gif",height=>9,width=>9,align=>"bottom"})." ".
		     $msg{"Click the arrows to expand/shrink your sitemap."});
    $other .= $ml->p($msg{"The public URL to your website is:"}.
		    $ml->br.
		    $ml->a($myurl,{href=>$myurl,target=>$config{MySite}{public_url_target}})
		    );

    $out .= $ml->table(
		       $ml->tr(
			       $ml->td($ml->div($sitemap,{class=>"MySiteMap"}),{valign=>"top",width=>"100%"}).
			       $ml->td($ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/null.gif",height=>10,width=>10})).
			       $ml->td($other,{valign=>"top",width=>"200"})
			       ),
		       {cellspacing=>0,cellpadding=>0,width=>"100%",border=>0}
		       );

    # build javascript for dynamic icon panels

    my $ndiv = scalar @{$this->{divs}};
    my $optlabel = $msg{Options};
    my $js = "\nvar divs = new Array(\n'".
    join("',\n'",@{$this->{divs}}).
    "')
var contents = new Array(\n'".
    join("',\n'",@{$this->{contents}}).
    "')
function HideOpts() {
  var locopt = document.getElementById(\"opt\")
  locopt.innerHTML = '$optlabel <a href=javascript:ShowOpts() style=\"text-decoration:none\">[+]</a>'
  for (i = 0; i < divs.length; i++) {
    var loc = document.getElementById(divs[i])
    loc.innerHTML = ''
  }
  document.cookie='MySiteOptions=0; path=$config{server}{CGIpath}; domain=$config{server}{domain}'
}
function ShowOpts() {
  var locopt = document.getElementById(\"opt\")
  locopt.innerHTML = '$optlabel <a href=javascript:HideOpts() style=\"text-decoration:none\">[-]</a>'
  for (i = 0; i < divs.length; i++) {
    var loc = document.getElementById(divs[i])
    loc.innerHTML = contents[i]
  }
  document.cookie='MySiteOptions=1; path=$config{server}{CGIpath}; domain=$config{server}{domain}'
}\n";
    $js = $ml->script($js,{language=>"javascript"});

    if ($cookie{MySiteOptions}) {
	$out .= $ml->script("\nShowOpts();\n",{language=>"javascript"});
    }
    return "$js$out\n";
}

sub button {
    my ($this,$url,$imgsrc,$label) = @_;
    return $ml->tr(
		   $ml->td(
			   $ml->a(
				  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/$imgsrc",border=>0,align=>"middle"})." ".$label,
				  {href=>$url}
				  )
			   )
		   );
}

sub options_link {
    my $this = shift;
    if ($cookie{MySiteOptions}) {
	return "Options ".$ml->a("[-]",{href=>"javascript:HideOpts()",style=>"text-decoration:none"});
    }
    else {
	return "Options ".$ml->a("[+]",{href=>"javascript:ShowOpts()",style=>"text-decoration:none"});
    }
}

sub sitemap {
    my ($this,$tree,$types,@p) = @_;
    my @dlist;
    my @types = ref $types eq "ARRAY" ? @$types : ( $types );
    foreach my $type (@types) {
	foreach my $p (@p) {
	    next if ($p->{type} ne $type);
	    next if ($config{MySite}{hidden_pages} && $p->{filename} =~ /^_/);
	    $this->sitemap_row($tree,$types,$p,\@dlist);
	}
    }
    return @dlist > 0 ? \@dlist : undef;
}

sub sitemap_row {
    my ($this,$tree,$types,$p,$dlist) = @_;
    $rowcount++;
    my $info = $this->page_info($p);
    my $label;
    if ($p->{version}) {
	$label .= $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/flag.gif",alt=>$p->{version},title=>$p->{version}})." ";
    }
    $label .= ($p->{label} || $p->{title}).
	" ".&ExSite::HTML::ToolTip(pane=>$info, label=>"?", width=>200);
    push @$dlist, {
	prefix=>$this->page_icons($p),
	label=>$label,
	sublist=> defined $tree ? $this->sitemap($tree,$types,$tree->get_child_data($p->{page_id})) : undef,
	open=>$config{MySite}{open_submenus},
    };
}

sub page_info {
    my ($this,$p) = @_;
    my $info;
    if ($p->{type} =~ /page|alternate/) {
	$info = $ml->table( [
			     [$ml->strong($msg{Title}.": "), $p->{title}],
			     [$ml->strong($msg{File}.": "), $p->{filename}],
			     [$ml->strong($msg{Visibility}.": "),$p->{visibility}],
			     [$ml->strong($msg{Mode}.": "),$p->{publish_method}],
			     [$ml->strong($msg{Privacy}.": "),$p->{access}],
			     [$ml->strong($msg{Version}.": "),$p->{version}],
			     ] );
    }
    else {
	$info = $ml->table( [
			     [$ml->strong($msg{Title}.": "), $p->{title}],
			     [$ml->strong($msg{Folder}.": "), $p->{filename}],
			     ] );
    }
    return $info;
}

sub page_icons {
    my ($this,$page) = @_;
    push @{$this->{divs}}, "r".$rowcount;
    my $dyndiv;
    my $db = $share{DB};
    my @attribute;
    my $url;
    my $out;
    if ($page->{type} =~ /page|alternate/) {
	# preview
	$out .= $ml->a(
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_page.gif", border=>0, height=>20, width=>20, alt=>$msg{preview}, title=>$msg{preview}}),
		       { href=>"$config{server}{CGIpath}/page.cgi?_id=$page->{page_id}&_cms=1", target=>"preview"}
		       );
	# edit
	$out .= $ml->a(
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_edit.gif", border=>0, height=>20, width=>20, alt=>$msg{edit}, title=>$msg{edit}}),
		       { href=>"$config{server}{CGIpath}/page.cgi?_id=$page->{page_id}&_cms=1&ecmd=update", target=>"preview"}
		       );
    }
    elsif ($page->{type} eq "library") {
	# edit
	$url = $this->link(ecmd=>"images",lib=>$page->{page_id});
	$out .= $ml->a(
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_edit.gif", border=>0, height=>20, width=>20, alt=>$msg{update}, title=>$msg{update}}),
		       { href=>$url }
		       );
    }
    elsif ($page->{type} =~ "template") {
	# preview
	$out .= $ml->a(
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_page.gif", border=>0, height=>20, width=>20, alt=>$msg{preview}, title=>$msg{preview}}),
		       { href=>"$config{server}{CGIpath}/page.cgi?_id=$page->{page_id}&_cms=1", target=>"preview"}
		       );
    }
    $out .= "<span id=\"r$rowcount\"></span>";
    # publish
    $dyndiv .= $ml->a(
		      $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_pub.gif", border=>0, height=>20, width=>20, alt=>$msg{publish}, title=>$msg{publish}}),
		      { href=>"$config{server}{CGIpath}/publish.cgi?page=$page->{page_id}", target=>"publish"}
		      );
    if ($this->site_type eq "advanced") {
	# advanced CMS shortcut
	$url = "$config{server}{CGIpath}/ctrl-panel.cgi/CMS?path=/$page->{section_id}/$page->{page_id}";
	$dyndiv .= $ml->a(
		      $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_padv.gif", border=>0, height=>20, width=>20, alt=>$msg{"advanced page builder"}, title=>$msg{"advanced page builder"}}),
		      { href=>"javascript:popup_large(\\'$url\\')" }
		      );
    }
    if ($page->{type} =~ /page|alternate/) {
	# config
	$url = $this->link(ecmd=>"pageconf",id=>$page->{page_id});
	$dyndiv .= $ml->a(
			  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_pconf.gif", border=>0, height=>20, width=>20, alt=>$msg{configure}, title=>$msg{configure}}),
			  { href=>$url }
			  );
	# rollback
	$url = $this->link(ecmd=>"rollback",id=>$page->{page_id},"_bare"=>1);
	$dyndiv .= $ml->a(
			  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_rollback.gif", border=>0, height=>20, width=>20, alt=>$msg{rollback}, title=>$msg{rollback}}),
			  { href=>"javascript:popup_small(\\'$url\\')" }
			  );
    }
    # delete
    if ($page->{filename} !~ /^index.(php|asp|html?)$/) {
	$url = $this->link(ecmd=>"delpage",id=>$page->{page_id});
	$dyndiv .= $ml->a(
			  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_del.gif", border=>0, height=>20, width=>20, alt=>$msg{delete}, title=>$msg{delete}}),
			  { href=>$url }
			  );
    }
    # add dyndiv contents to contents list
    push @{$this->{contents}}, $dyndiv;

    return $ml->div($out,{class=>"MySiteMapIcons"});
}

sub Authenticate {
    my ($this,$options) = @_;
    my $out;
    if ($this->{input}->{login}) {
	# attempted login
	# clear last attempt to identity user
	delete $share{identity};
	my $in = $this->{input};
	require ExSite::Auth;
	my $a = new ExSite::Auth;
	if ($a->login($in->{login},$in->{password})) {
	    if ($this->mysite) {
		return $this->my_ctrl($options);
	    }
	    else {
		$out = $ml->h3($msg{"No Website"},{class=>"error"}).
		    $ml->p($msg{"You do not have a website in this system.  Please double-check your Login ID to make sure you are using the correct account."});
	    }
	}
	else {
	    $out = $ml->h3($msg{"Login Failed"},{class=>"error"}).
		$ml->p($msg{"Please double-check your login ID and password."});;
	}
    }
    elsif ($share{DB}->level >= 2) {
	$out = $ml->h3($msg{"No Website"},{class=>"error"}).
	    $ml->p($msg{"You do not have a website in this system.  Please double-check your Login ID to make sure you are using the correct account."});
    }
    elsif ($share{DB}->level) {
	$out = $ml->h3($msg{"Permission Denied"},{class=>"error"}).
	    $ml->p($msg{"You do not have a website in this system.  Please double-check your Login ID to make sure you are using the correct account."});
    }
#   $out .= $this->LoginForm();
    $out .= $share{DB}->login_form(undef,$this->link());
    return $out;
}

# following login methods deprecated??

sub LoginForm {
    my $this = shift;
    my $url = $this->link();
    my $remindurl = $this->link(ecmd=>"remind");
    my $form = new ExSite::FormBuilder(method=>"post",action=>$url);
    $form->template(
		    $ml->table([
				[ "[[login:prompt]]:", "[[login:input]]" ],
				[ "[[password:prompt]]:", "[[password:input]]" ],
				],
			       {class=>"form"}
			       )
		    );
    $form->input(name=>"login",prompt=>$msg{Login},type=>"text",size=>20,required=>1);
    $form->input(name=>"password",prompt=>$msg{Password},type=>"password",size=>20,required=>1);
    $form->submit;
    my $loginform = $form->make;
    my $title = $msg{"My Site"};
    return $ml->div($ml->br.$ml->br.
		    $ml->div(
			     $ml->h1($msg{"My Site"}).
			     $loginform.
			     $ml->br.$ml->br.
			     $ml->p($ml->a($msg{"Please send me my login and password again."},{href=>$remindurl}))
			     ).
		    $ml->br.$ml->br.$ml->br,
		    {align=>"center"});
}

sub reminder {
    my $this = shift;
    my $reminder_sent;
    my $out = "<br><br>\n<h1>My Site (Password Reminder)</h1>\n";
    my $title = $config{site}{name};
    if ($this->{input}{siteid} || $this->{input}{email}) {
	my (@member,%match);
	if ($this->{input}{siteid}) {
	    $match{login} = $this->{input}{siteid};
	}
	if ($this->{input}{email}) {
	    $match{email} = $this->{input}{email};
	}
	@member = $share{DB}->fetch_match("member",\%match);
	if (@member > 0) {
	    my $info;
	    if (@member > 1) {
		$info = "(You have several sites at $title.  Each of the following
Login/Password pairs accesses one site.)\n\n";
	    }
	    foreach my $m (@member) {
		$info .= "Login:    $m->{login}
Password: $m->{password}\n\n";
	    }
	    &SendMail($member[0]->{email},$config{site}{administrator},
		      "$title login reminder",
		      "This message was sent to you because someone requested a login/password 
reminder at $title.  If that person was not you, don't worry 
because only you are receiving this message.

Here is your login info:		      

$info
-- The $title Team\n");
	    my $login = $this->link(ecmd=>undef,action=>undef);
	    $out .= "<p>Password reminder sent.  It should arrive in your inbox soon.</p>\n<p><a href=\"$login\">Retry login</a></p>";
	    $reminder_sent = 1;
	}
	else {
	    $out .= "<p class=\"error\">No matching websites were found.</p>\n";
	}
    }
    if (! $reminder_sent) {
	my $url = $this->link(ecmd=>"remind");
	my $form = new ExSite::FormBuilder(method=>"post",action=>$url);
	$form->template("<table class=\"form\">
<tr><td>[[siteid:prompt]]:</td><td>[[siteid:input]]</td></tr>
<tr><td>[[email:prompt]]:</td><td>[[email:input]]</td></tr>
</table>");
	$form->input(name=>"siteid",prompt=>"Site ID",type=>"text",size=>20);
	$form->input(name=>"email",prompt=>"OR, your email address",type=>"text",size=>20);
	$form->submit;
	my $remindform = $form->make;

	$out .= "<p>Please provide us with either your login ID, or your email address.</p>
$remindform
<br><br>\n";
    }
    return $out;
}

sub newsite {
    my $this = shift;
    return $this->Error("MySite::newsite() is no longer supported.  The webmaster should be using Sitebuilder plug-in, instead.");
}

sub newpage {
    my $this = shift;
    my $site = $this->mysite(); 
    my $p = new ExSite::Page();
    if (! $p->allow_insert("page",{section_id=>$site->{section_id}})) {
	return $this->Error($msg{"Permission denied - please contact your system administrator."}); 
    }
    my %page;
    my $db = $share{DB};
    my $out;
    my @pages = $db->fetch_match("page",{type=>"page",
					 section_id=>$site->{section_id},
				     });
    my $limit = $this->page_limit;
    # assume that page_limit == 0 means no limit
    if ($limit < 0) {
	$out = $ml->h1($msg{"New Pages Not Allowed"}).
	    $ml->p($msg{"You must upgrade your website to enable this feature."});
    }
    elsif ($limit && scalar @pages >= $limit) {
	$out = $ml->h1($msg{"Page Limit Exceeded"}).
	    $ml->p(&substitute($msg{"The present configuration of your site allows for [[limit]] pages.  
You will need to upgrade your site's capacity before you can add more pages.
Alternatively, you could remove other pages that you no longer need."},
			       {limit=>$limit}));
    }
    elsif ($this->{input}{'label'}) {
	# we have input
	$this->insert_page;
    }
    else {
	# no input - throw up a form
	my $action = $this->link();
	my $form = new ExSite::FormBuilder(method=>"post",action=>$action);
	# maximum label length = maximum filename length, minus 5 chars for .html
	my $maxlength = $share{DB}{map}->get_column_attr("page","filename","size")-5;
	$form->input(name=>"label",
		     prompt=>"Menu Label",
		     type=>"text",
		     size=>40,
		     required=>1,
		     maxlength=>$maxlength);
	my $newpageform = $form->make;

	my $out .= $ml->h1($msg{"Make a new web page"});
	$out .= $ml->p($msg{"Your web page will appear in site menus under a
short but descriptive name.  Please specify the menu label to use for
this new page ($maxlength characters max):"});
	$out .= $newpageform;
	$out .= $ml->p($ml->strong($msg{"Note:"})." ".
		       $msg{"To customize the details or position of this page after you have created 
it, use the configure page function.  Click on the [+] link by \"Options\", 
which will reveal the configure page button:"}." ".
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_pconf.gif",height=>20,width=>20,border=>0,align=>"middle"}) );
	return $out;
    }
}

# reconfigure a page

sub pageconf {
    my $this = shift;
    my $pid = shift || $this->{input}{id};
    my $db = $share{DB};
    my %page;
    if (! $pid) { 
	return $this->Error($msg{"No page to configure!"}); 
    } 
    %page = $db->fetch("page",$pid);
    my $p = new ExSite::Page(page=>\%page);
    if (! $p->allow_update("page")) {
	return $this->Error($msg{"Permission denied - please contact your system administrator."}); 
    }
    if ($this->{input}{label}) {
	# we have input
	return $this->update_page;
    }
    my $action = $this->link();
    my $form = new ExSite::FormBuilder(method=>"post",action=>$action);
    $form->input(name=>"label",prompt=>$msg{"Menu Label"},type=>"text",size=>40,required=>1,value=>$page{label});
    $form->input(name=>"title",prompt=>$msg{"Title - defaults to menu label if not set"},type=>"text",size=>60,required=>0,value=>$page{title});
    $form->input(name=>"description",prompt=>$msg{"Description - may be used for search engine optimization"},type=>"text",size=>60,required=>0,value=>$page{description});
    $form->input(name=>"keywords",prompt=>$msg{"Keywords - may be used for search engine optimization"},type=>"text",size=>60,required=>0,value=>$page{keywords});
    $form->input(name=>"visibility",prompt=>$msg{"Page Visibility - controls if the page appears in menus"},type=>"select",options=>["visible","hidden","sitemap-only","menu-only"],nullvalue=>"== ".$msg{"select visibility"}." ==",required=>0,value=>$page{visibility});

    # parent pages
    my @parent_pages;
    my $site = $this->mysite;
    my @pages = $db->fetch_child("page","section",$site->{section_id});
    foreach my $p (@pages) {
	next if ($p->{type} !~ /page|alternate/);
	next if ($p->{page_id} == $pid);
	push @parent_pages, { value=>$p->{page_id}, text=>$p->{label} }
    }
    $form->input(name=>"parent_id",prompt=>$msg{"Parent Page - sets the position in the site map"},type=>"select",options=>\@parent_pages,nullvalue=>"== ".$msg{"select parent page"}." ==",required=>0,value=>$page{parent_id});

    # templates
    my $s = new ExSite::Section(section=>$site);
    my @t = $s->get_templates();
    my @templates = map { { value=>$_->{page_id}, text=>$_->{title} } } @t;
    $form->input(name=>"template_id",prompt=>$msg{"Template - determines the page layout"},type=>"select",options=>\@templates,nullvalue=>"== ".$msg{"select template"}." ==",value=>$page{template_id});

    $form->input(name=>"rank",prompt=>$msg{"Page Rank - a number (typically 1 - 9) used for ordering pages in menus and submenus (lower values appear higher in menus)"},type=>"text",size=>4,value=>$page{rank});
    $form->input(name=>"filename",prompt=>$msg{"File Name - used to build the URL to this page"},type=>"text",size=>40,required=>1,value=>$page{filename});
    my @privacy_options = ( { value=>"public",text=>$msg{"Public page"} },
			    { value=>"members",text=>$msg{"Members only"} }, );
    $form->input(name=>"access",prompt=>$msg{"Privacy"},type=>"select",options=>\@privacy_options,nullvalue=>"== ".$msg{"select privacy"}." ==",value=>$page{access});
    $form->input(name=>"page_id",type=>"hidden",value=>$pid);

    my $out = $ml->h2($msg{"Configure Page"});
    $out .= $form->make;

    return $out;
}

sub update_page {
    my $this = shift;
    my $db = $share{DB};
    my $site = $this->mysite;
    my @err;
    my $oldpage = $share{DB}->fetch("page",$this->{input}{id});
    if ($oldpage->{section_id} ne $site->{section_id}) {
	push @err, $msg{"site mismatch"};
    }
    # we have input - process the form
    my $label = $this->{input}{label};
    my $title = $this->{input}{title} || $label;
    my $template = $this->{input}{template_id} || $site->{template_id};
    my $access = $this->{input}{access} || "public";
    my $publish_method = $access =~ /member/ ? "dynamic" : $oldpage->{publish_method};
    my $pid;
    my %page = (
		page_id=>$this->{input}{page_id},
		label=>$label,
		title=>$title,
		access=>$access,
		rank=>$this->{input}{rank},
		description=>$this->{input}{description},
		keywords=>$this->{input}{keywords},
		visibility=>$this->{input}{visibility},
		parent_id=> ($this->{input}{parent_id} ? $this->{input}{parent_id} : 0),
		section_id=>$site->{section_id},
		template_id=>$template,
		filename=>$this->{input}{filename},
		type=>$oldpage->{type},
		publish_method=>$publish_method,
		);
    my $p = new ExSite::Page(page=>\%page);
    push @err, $p->validate();
    if (@err > 0) {
	my $out = $this->Error(@err);
	$out .= &ExSite::HTML::Button(url=>"javascript:history.back()",label=>"Return to form");
	return $out;
    }
    my $err = $db->update("page",\%page);
    if ($err) { push @err, $err; }
    if (@err > 0) {
	my $out = $this->Error(@err);
	$out .= &ExSite::HTML::Button(url=>"javascript:history.back()",label=>"Return to form");
	return $out;
    }
    return $this->my_ctrl;
}

sub insert_page {
    my $this = shift;
    my $db = $share{DB};
    my $site = $this->mysite;
    # we have input - process the form
    my $label = $this->{input}{'label'};
    my $title = $this->{input}{'title'} || $label;
    my @err;
    my $pid;
    my %page = (
		label=>$label,
		title=>$title,
		type=>"page",
		status=>"active",
		access=>($this->{input}{access} || "public"),
		publish_method=>"static",
		visibility=>"visible",
		rank=>($this->{input}{rank} || "1"),
		description=>$this->{input}{description},
		keywords=>$this->{input}{keywords},
		parent_id=> ($this->{input}{parent_id} ? $this->{input}{parent_id} : 0),
		section_id=>$site->{section_id},
		template_id=>$site->{template_id},
		);

    # check for existing pages
    my $filename = $label.".html";
    $filename = &clean_filename($filename);
    my @exist = $db->fetch_match("page",{section_id=>$site->{section_id},
					 filename=>$filename});
    my $n = 0;
    while (@exist > 0) {
	$n++;
	$filename = $label."$n.html";
	$filename = &clean_filename($filename);
	@exist = $db->fetch_match("page",{section_id=>$site->{section_id},
					  filename=>$filename});
    }
    # filename is now unique
    $page{filename} = $filename;
	
    $pid = $db->insert("page",\%page);
    if (! $pid) {
	push @err, ($msg{"failed to add new page"}.": ".
		    $db->show_diagnostics("error","html"));
    }
    else {
	# now create a body
	my %content = (
		       page_id => $pid,
		       name => "body",
		       type => "editorial",
		       );
	my $id = $db->insert("content",\%content);
	if (! $id) {
	    push @err, ($msg{"page was created, but failed to add page body"}.": ".
			$db->show_diagnostics("error","html"));
	}
    }
	
    my $out;
    if (@err > 0) {
	$out .= $this->Error(@err);
	$out .= &ExSite::HTML::Button(url=>"javascript:history.back()",label=>"&lt; Back");
    }
    else {
	$out .= $this->my_ctrl;
    }
    return $out;
}

sub update_body {
    my $this = shift;
    my $db = $share{DB};
    my ($out, %content, @cdata, $selected, $id, $pid);

    # get the page id and content id from the current page object
    my $id;
    if ($share{Page}) {
	$pid = $share{Page}->id();
	if ($pid) {
	    my @content = $share{Page}->fetch_match("content",{name=>"body",
							       page_id=>$pid});
	    if (@content > 0) {
		%content = %{$content[-1]};
		$id = $content[-1]->{content_id};
	    }
	    else {
		return $this->Error($msg{"no page body found"});
	    }
	}
	else {
	    return $this->Error($msg{"undefined page"});
	}
    }
    else {
	return $this->Error($msg{"MySite::update_body can only be used inlined into a page"});
    }

    my $action = $this->link(ecmd=>"doupdate");
    $out .= $ml->p(
		   $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/caution20.gif",height=>20,width=>20,border=>0,align=>"middle"}).
		   $msg{"You must hit \"Submit\" (at bottom) to save your changes.
If you navigate away from this page using the menu links, your updates will be 
discarded."}
		   );
    my $form = new ExSite::FormBuilder(method=>"POST",formid=>"exsiteform",action=>$action);
    $form->input(type=>"hidden",name=>"action",value=>"insert");
    $form->input(type=>"hidden",name=>"content_data#_2#mime_type",value=>"text/html");
    $form->input(type=>"hidden",name=>"content_data#_2#content_id", value=>$id);

    if ($id =~ /\d+/) {
	@cdata = $db->fetch_child("content_data","content",$id,"content_data_id");
    }
    if (@cdata == 0) {
	$cdata[0] = { content_data_id=>"_2", mime_type => "text/html" };
    }
    my $cid = $cdata[-1]{content_data_id};
    my $name = $content{name};
    $form->input(type=>"hidden",name=>"content_data_id",value=>"$cid");

    my $update_method;
    if ($cdata[-1]->{mime_type} =~ /^text\/html$/o) {
	$update_method = "html";
    }
    else {
	$update_method = "upload";
    }

    if ($db->use_wysiwyg) {
	my $type = $this->site_type;
	my $op_image = $config{MySite}{$type}{allow_addimg};
	my $op_dyncontent = $config{MySite}{$type}{allow_dynamic};
	$form->input(name=>"content_data#_2#data",
		     prompt=>" ",
		     input=>$db->input_htmleditor(name=>"content_data#_2#data",
						  table=>"content_data",
						  record=>$cid,
						  page=>$pid,
						  section=>$share{Page}->id("section"),
						  column=>"data",
						  editor_width=>$config{MySite}{editor_width},
						  editor_height=>$config{MySite}{editor_height},
						  form=>$form,
						  editor=>{
						      op_image=>$op_image,
						      op_link=>1,
						      op_table=>1,
						      op_dyncontent=>$op_dyncontent,
						  }),
		     );
    }
    else {
	# default to text area
	$form->input(type=>"preformatted",
		     name=>"warning",
		     prompt=>" ",
		     input=>$ml->p($msg{"WARNING: Your browser is not compatible
with the Exsite Webware editor.  To use the editor, you should upgrade to
one of the following browsers:  Windows IE 6.0+, Netscape Navigator 7.1+, 
Mozilla 1.4+, Firefox, or Safari.  Otherwise, you are restricted to editing in 
raw HTML mode:"},
				   {class=>"error"}
				   )
		     );
	
	my $content = $cdata[-1]{data};
	$content =~ s/</&\#60;/g;
	$content =~ s/>/&\#62;/g;
	$form->input(type=>"textarea",
		     name=>"content_data#_2#data",
		     prompt=>$msg{"Page Content"},
		     rows=>24,
		     cols=>80,
		     value=>$content);
    }

    # 3) revision control comment

    my $comment = "updated by ".$db->my_name." on ".scalar(localtime);
    $form->input(type=>"hidden",name=>"content_data#_2#note",value=>$comment);
    (tied %msg)->nohighlight();
    $form->set("buttons",
	       $ml->input(undef,{type=>"submit",name=>"submit_action",value=>$msg{Submit}})."&nbsp;&nbsp;".
	       $ml->input(undef,{type=>"submit",name=>"submit_action",value=>$msg{Cancel}})."&nbsp;&nbsp;".
	       $ml->input(undef,{type=>"reset"})
	       );
    (tied %msg)->restore();
    $out .= $form->make;

    return $out;
}

sub do_update {
    my $this = shift;
    my $form = $share{DB};
    $form->get_data;

    # patch up those mangled page links
    $form->{form}->{content_data}->{_2}->{data} =~ s/%7B/{/g;
    $form->{form}->{content_data}->{_2}->{data} =~ s/%7D/}/g;

    $form->do_db("insert");
    if ($form->errorcheck()) {
	return $form->show_diagnostics("error","html");
    }
    elsif ($share{Page}) {
	$share{Page}->location($this->link(__plaintext=>1,ecmd=>undef,_cms=>2));
    }
    else {
	return $msg{"Page updated."};
    }
}

sub delpage {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    if ($this->{input}{id}) {
	# specific page id given in URL
	my $page = $this->{input}{id};
	my $pdata = $db->fetch("page",$page);
	if ($this->{input}{confirm}) {
	    $out .= $msg{"deleting page"}." $page...<br>";
	    my $p = new ExSite::Page(page=>$pdata);
	    if ($p->allow_delete("page",$pdata)) {
		# unpublish
		&run_publisher("page",$page,1);
		# trash the db records
		$db->trash_r("page",$page,1);
		&redirect($this->link(__plaintext=>1,
				      "sid"=>$this->{input}{sid},
				      "ecmd"=>undef,
				      "confirm"=>undef,
				      "id"=>undef));
	    }
	    else {
		$out .= $this->Error($msg{"Delete page: permission denied."});
	    }
	}
	else {
	    my $action = $share{Page} ? "page.cgi" : "ctrl-panel.cgi/MySite";
	    $out .= $ml->h4(&substitute($msg{"Delete page \"[[title]]\""},{title=>$pdata->{title}}));
	    $out .= $ml->p($msg{"This action will remove this page from your website.  Your website should be republished after doing this, so that the menus on other pages will be updated appropriately."});
	    $out .= $ml->p($msg{"If you want to recover a deleted page, they can be restored from the Trash in the advanced control panel."});
	    $out .= $ml->p(
			   $ml->form(
				     $ml->input(undef,{type=>"hidden",name=>"_id",value=>$this->{input}{_id}}).
				     $ml->input(undef,{type=>"hidden",name=>"id",value=>$this->{input}{id}}).
				     $ml->input(undef,{type=>"hidden",name=>"sid",value=>$this->{input}{sid}}).
				     $ml->input(undef,{type=>"hidden",name=>"ecmd",value=>$this->{input}{ecmd}}).
				     $ml->input(undef,{type=>"hidden",name=>"confirm",value=>1}).
				     $ml->input(undef,{type=>"submit",value=>$msg{DELETE}}).
				     $ml->input(undef,{type=>"button",value=>$msg{Cancel},OnClick=>"javascript:history.back()"})
				     )
			   );
	}
    }
    else {
	$out = $this->Error($msg{"Error Deleting Page: no page was specified!"});
    }
    return $out;
}

sub rollback {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    if ($this->{input}{id}) {
	my $c = new ExSite::Content(page=>$this->{input}{id},
				    name=>"body",
				    revision=>"newest");
	$c->rollback;
    }

    my $message = $ml->h3($msg{"Rollback completed"}).
	$ml->p($msg{"The page may need to be republished to make this change public."}).
	$ml->p($ml->a($msg{"Close Window"},{href=>"javascript:window.close()"}));
    if ($share{Page}) {
	$share{Page}->errorpage($ml->html($ml->body($ml->center($message))));
    }
    else {
	return $ml->div($message,{align=>"center"});
    }
}

sub template {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    my $site = $this->mysite;
    my $s = new ExSite::Section(section=>$site);
    if (! $s->allow_update("section",$site->{section_id})) {
	return $this->Error($msg{"Permission denied - please contact your system administrator."}); 
    }
    if ($ENV{REQUEST_METHOD} eq "POST") {
	my $tid = $this->{input}{template};
	if ($tid) {
	    # 1) set site's default template
	    my $site = $this->mysite;
	    $db->update("section",{template_id=>$tid},{section_id=>$site->{section_id}});
	    # 2) reset template id of all pages in this site
	    $db->update("page",{template_id=>$tid},{section_id=>$site->{section_id}, type=>"page"});
	    $out .= $ml->h3($msg{"Done!"});
	    $out .= $ml->p($msg{"Published pages will continue to show the old template until you republish your site."});
	}
	else {
	    $out .= $this->Error($msg{"No template selected."});
	}
	my $url = $this->link(ecmd=>undef);
	$out .= $ml->p($ml->a($msg{"Back to My Site"},{href=>$url}));
    }
    else {
	# no form data - publish a form
	$out = $ml->h1($msg{"Change Template"});
	$out .= $ml->form(
			  $this->list_template($site->{template_id}).
			  $ml->p(
				 $ml->input(undef,{type=>"submit",value=>$msg{Submit}}).
				 $ml->input(undef,{type=>"button",value=>$msg{Cancel},OnClick=>"javascript:history.back()"})
				 ),
			  {method=>"POST"}
			  );
    }
    return $out;
}

# simplified version of CMS::list_template

sub list_template {
    my ($this,$id) = @_;
    my ($out,@ptempl,@options);
    my $site = $this->mysite()->{section_id};
    $out .= $ml->p($msg{"Select a template:"});
    $out .= $ml->script("<!--
function NewWindow(target){
	var NewWin = window.open(target,'','width=800,height=500,scrollbars,resizable')
	}
-->\n",
			{language=>"javascript"});

    my $db = $share{DB};
    my $radiobox;
    # fetch public templates
    @ptempl = $db->fetch_match("page",{status=>"active",type=>"template",access=>"public"});
    # ... plus the member-only templates from the current section
    push @ptempl, $db->fetch_match("page",{status=>"active",type=>"template",access=>"members",section_id=>$site},"title");
    my $sec = $db->fetch("section",$site);
    if ($sec && $sec->{parent_id}) {
	# ... plus the member-only templates from the parent section
	push @ptempl, $db->fetch_match("page",{status=>"active",type=>"template",access=>"members",section_id=>$sec->{parent_id}},"title");
    }
    # ...plus the private templates from this section
    push @ptempl, $db->fetch_match("page",{status=>"active",type=>"template",access=>"administrators",section_id=>$site},"title");

    if (@ptempl > 0) {
#	foreach my $pt (sort { lc($a->{title}) cmp lc($b->{title}) } @ptempl) {
	foreach my $pt (reverse @ptempl) {
	    my $radio = { 
		type=>"radio",
		name=>"template",
		value=>$pt->{page_id},
	    };
	    if ($pt->{page_id} == $id) { $radio->{checked} = undef; }
	    $radiobox .= $ml->p(
				$ml->dl(
					$ml->dt(
						$ml->input(undef,$radio).
						$ml->strong($pt->{title})
						).
					$ml->dd(
						$pt->{description}."  [".
						$ml->a($msg{preview},{href=>"javascript:NewWindow('$config{server}{CGIpath}/page.cgi?_id=$pt->{page_id}')"})
						)
					)
				);
	}
    }
    else {
	$radiobox .= $ml->p($msg{"Sorry, there don't seem to be any templates to choose from."});
    }
    $out .= $ml->blockquote($radiobox);
    return $out;
}

sub config {
    my $this = shift;
    my $site = $this->mysite;
    my $s = new ExSite::Section(section=>$site);
    if (! $s->allow_update("section",$site->{section_id})) {
	return $this->Error($msg{"Permission denied - please contact your system administrator."}); 
    }
    my $out;
    if ($this->{input}{title}) {
	# form input present - process the input

	# sanity checks ??

	$share{DB}->update("section",{section_id=>$site->{section_id},
				      title=>$this->{input}{title},
				      description=>$this->{input}{subtitle}});
	my $url = $this->link(ecmd=>undef);
	$out = $ml->h1($msg{"Website Reconfigured"}).
	    $ml->p($msg{"Changes to titles and descriptions will not be publicly visible until you re-publish the site."}).
	    $ml->p($ml->a($msg{"Back to control panel"},{href=>$url}));
    }
    else {
	$out = $ml->h1($msg{"My Website Configuration"});
	my $url = $this->link();

	##### escape quotes etc. in field values?

	my $form = new ExSite::FormBuilder(method=>"post",action=>$url);
	$form->input(type=>"text",name=>"title",prompt=>$msg{"Website Title"},size=>60,required=>1,value=>$site->{title});
	$form->input(type=>"text",name=>"description",prompt=>$msg{"Subtitle/Description"},size=>60,required=>0,value=>$site->{description});
	$out .= $form->make;
    }
    return $out;
}

sub passwd {
    my $this = shift;
    my $out;
    if ($this->{input}{passwd1}) {
	# form input present - process the input
	my $stat = $share{DB}->change_password($this->{input}{passwd0},
					       $this->{input}{passwd1},
					       $this->{input}{passwd2});
	if ($stat) { return $stat; }  # error
	my $url = $this->link(ecmd=>undef,lib=>undef,image=>undef);
	$out .= $ml->h1($msg{"Password Changed Successfully"}).
	    $ml->p($ml->a($msg{"Back to website control panel"},{href=>$url}).".");
    }
    else {
	$out = $ml->h1($msg{"Change My Password"});
	my $url = $this->link();
	my $form = new ExSite::FormBuilder(method=>"post",action=>$url);
	$form->input(type=>"password",name=>"passwd0",prompt=>"Current Password:",required=>1);
	$form->input(type=>"password",name=>"passwd1",prompt=>"New Password:",required=>1);
	$form->input(type=>"password",name=>"passwd2",prompt=>"(enter it again, to confirm):",required=>1);
	$out .= $form->make;
    }
    return $out;
}

sub publish {
    my ($this,$page) = @_;
    my $db = $share{DB};
    my $site = $this->mysite;
    my $out;
    if ($page) {
	my $p = $db->fetch("page",$page);
	if ($p->{section_id} == $site->{section_id}) {
	    # spawn the publish program, so it runs setuid
	    $ENV{QUERY_STRING} = "page=$page";
	    $out .= `$config{CGIroot}/publish.cgi`;
	}
	else {
	    $out .= $this->Error($msg{"Permission denied: invalid page."});
	}
    }
    else {
	# spawn the publish program, so it runs setuid
	$ENV{QUERY_STRING} = "section=$site->{section_id}";
	$out .= `$config{server}{CGIroot}/publish.cgi`;
    }
    return $out;
}

# find the default library for the site

sub get_library {
    my ($this) = @_;
    my $lib = [];
    my $db = $share{DB};
    my $site = $this->mysite();
    if ($this->{input}{lib}) {
	$lib = $db->fetch_match("library",{section_id=>$site->{section_id},
					   page_id=>$this->{input}{lib}});
    }
    if (@$lib == 0) {
	$lib = $db->fetch_match("library",{section_id=>$site->{section_id},
					   filename=>"images"});
    }
    return $lib->[0];
}

# display the library contents

sub library {
    my ($this) = @_;
    my $lib = $this->get_library();
    if (! $lib) {
	return $this->Error($msg{"No library found."});
	# if we want to be nice, we should include an option to create one
    }

    my @contents = $share{DB}->fetch_child("content","page",$lib->{page_id});

    my $out .= $ml->h1($msg{"Library Manager"}).
	$ml->p($msg{"Click on image thumbnails to preview full-sized images."});
    $out .= $this->mysite_css;

    my $libout;
    my $nimg = 0;
    foreach my $c (@contents) {
	# foreach :
	# show : name, size
	# options : view, update, delete
	$nimg++;
	my $content = new ExSite::Content(content=>$c);
	$content->set_context("child");
	my (undef,$size,undef) = $content->get_fileinfo();
	if ($size) { $size .= " K"; }
	my $cdata = $content->{context}{content_data};
	my $curl = $this->show_thumb($c);
	my $delurl = $this->link(ecmd=>"delimg",image=>$c->{content_id});
	my $updurl = $this->link(ecmd=>"updateimg",image=>$c->{content_id});
	my $tools = $ml->a(
			   $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_img.gif",height=>20,width=>20,border=>0,title=>$msg{"Update Image"},alt=>$msg{"Update Image"}}),
			   {href=>$updurl}
			   );
	$tools .= $ml->a(
			$ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_del.gif",height=>20,width=>20,border=>0,title=>$msg{"Delete Image"},alt=>$msg{"Delete Image"}}),
			{href=>$delurl}
			);
	$libout .= $ml->tr(
			   $ml->td($curl).$ml->td($size).$ml->td($tools),
			   {class=>($nimg % 2 ? "A" : "B")}
			   );
    }
    if ($libout) {
	$out .= $ml->table(
			   $ml->tr(
				   $ml->th($msg{Image}).
				   $ml->th($msg{Size}).
				   $ml->th($msg{Options})
				   ).
			   $libout,
			   { class=>"Report" }
			   );
    }
    else {
	$out .= $ml->p($msg{"No images have been uploaded yet."});
    }
    
    # add new image

    my $url;
    if (! $config{MySite}{$this->site_type}{allow_addimg}) {
	$url = "javascript:alert('".$msg{"You must upgrade your site to upload images."}."')";
    }
    else {
	my $imglimit = $this->image_limit;
	if ($imglimit == 0 || (scalar @contents < $imglimit)) {
	    $url = $this->link(ecmd=>"addimg",image=>undef);
	}
	else {
	    $url = "javascript:alert('".$msg{"You are only allowed $imglimit uploaded images."}."')";
	}
    }
    $out .= $ml->p(
		   $ml->a(
			  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/MySite/ico_newimg.gif",border=>0,alt=>$msg{"Add Image"},align=>"middle"}).
			  $msg{"Add New Image/File"},
			  { href=>$url }
			  )
		   );
    $url = $this->link(ecmd=>undef,lib=>undef,image=>undef);
    $out .= $ml->p("&laquo; ".
		   $ml->a($msg{"Back to website control panel"},{href=>$url}).
		   ".");
    return $out;
}

sub show_thumb {
    my ($this,$img) = @_;
    my $c = new ExSite::Content(content=>$img);
    $c->set_context("child");
    if ($c->is_image) {
	my $url = $c->get_url;
	my $thumb = $c->get_thumb_html;
	my $attr = $c->get("content_data","attributes");
	$attr =~ /width=\"?(\d+)\"?/i;
	my $width = $1+20;
	$attr =~ /height=\"?(\d+)\"?/i;
	my $height = $1+20;
	if ($width == 20 || $height == 20) {
	    # bad dimensions; extract dimensions from the original image
	    my $image = new ExSite::Image($c->get("content_data","fdata"));
	    ($width,$height) = $image->dim;
	    $height += 20;
	    $width += 20;
	}
	return "<a href=\"javascript:popup_custom($width,$height,'$url')\">$thumb</a>";
    }
    else {
	return $c->get_html;
    }
    return undef;
}

sub image_add {
    my $this = shift;
    my $out;
    my $lib = $this->get_library();
    if (! $lib) {
	return $this->Error($msg{"No image library found."});
	# if we want to be nice, we should include an option to create one
    }
    if (! $config{MySite}{$this->site_type}{allow_addimg}) {
	$out .= $this->Error($msg{"You must upgrade your site to upload images."});
    }
    else {
	my @contents = $share{DB}->fetch_child("content","page",$lib->{page_id});
	my $imglimit = $this->image_limit;
	if ($imglimit == 0 || (scalar @contents < $imglimit)) {
	    $out .= $this->image_upload;
	}
	elsif ($imglimit < 0) {
	    $out .= $this->Error($msg{"You are not allowed any uploaded images.  Please upgrade your website to enable this feature."});
	}
	else {
	    $out .= $this->Error($msg{"You are only allowed $imglimit uploaded images."});
	}
    }
    return $out;
}

sub image_update {
    my $this = shift;
    return $this->image_upload($this->{input}{image});
}

sub image_upload {
    my ($this,$image_id) = @_;
    my ($img,$content_id,$in,$data,$err,$first,$out,$lib);
    $in = new ExSite::Input;
    $data = $in->post();
    $first = 1;

    if ($data->{upload}) {
	# user is uploading a file

	# check for image id
	my $img = $data->{image};
	if ($img) {
	    # looks like an update
	    my $content = $share{DB}->fetch("content",$img);
	    $lib = $this->get_library;
	    if ($content->{page_id} == $lib->{page_id}) {
		$content_id = $content->{content_id};
	    }
	    else {
		$err = &substitute($msg{"Invalid image [[image]] [[img]] specified [[content]] != [[lib]]"},{ image=>$data->{image}, img=>$img, content=>$content->{page_id}, lib=>$lib->{page_id} }); ;
	    }
	}

	# perform the update
	if (! $err) {
	    $first = 0;

	    # read file contents
	    my $rawdata = undef;
	    my $size = 0;
	    my $mimedata = "";
	    my $file = $data->{upload};
	    my $continue = 1;
	    while ($continue) {
		$continue = read($file,$rawdata,1024,$size);
		$size += $continue;
	    }

	    # check for a valid file
	    if ($file !~ /\.(png|gif|jpe?g)$/i) {
		# non-image file type
		$out .= $ml->p(
			       $ml->strong($msg{Warning}).
			       $msg{"you have uploaded a non-image file type.  This file will 
not be displayed in your web pages, although ExSite can provide a link
to allow the viewer to download the file and view it using some other
program."}
			       );
	    }
	    if ($size == 0) { 
		# no file data
		$err = $msg{"No file data was received."};
	    }
	    elsif ($config{MySite}{max_upload_size} &&
		   $size > $config{MySite}{max_upload_size}) { 
		# too big - will cause out-of-memory errors
		$err = $msg{"File is too large.  Please limit your uploaded files to less than $config{MySite}{max_upload_size} bytes."};
	    }
	    else {
		# we seem to have a valid file
		$file = &clean_filename($file);
		my $img = new ExSite::Image($file,$rawdata);
		$img->shrink(size=>$config{MySite}{max_image_size});
		my $fdata = $img->encode;
		my $fname = $img->name;
		my $opt = "attribute,thumb";
		if ($content_id) {
		    # make a new revision
		    my $c = new ExSite::ContentData;
                    my $cdid = $c->make({content_id=>$content_id,
					 mime_type=>&MimeType($fname),
					 fdata=>$fdata,},
					$opt,
					);
		}
		else {
		    # make a new content object
		    if (! $lib) { $lib = $this->get_library; }
		    my $c = new ExSite::Content;
		    $content_id = $c->make({page_id=>$lib->{page_id},
					    name=>$fname,
					    type=>"editorial",},
					   {mime_type=>&MimeType($fname),
					    fdata=>$fdata,},
					   $opt
					   );
		    if (! $content_id) {
			$out .= $c->show_diagnostics("error","html");
		    }
		}
	    }
	}
    }
    if ($err) {
	$out = $this->Error($err);
    }
    if ($first || $err) {
	# show an upload form
	$out .= $ml->h1($msg{"Upload a new file"});
	my $form = new ExSite::FormBuilder(method=>"post");
	$form->multipart;
	$form->input(name=>"image",type=>"hidden",value=>$image_id);
	$form->input(name=>"upload",prompt=>$msg{"Select file:"},type=>"file",required=>1);
	$out .= $form->make;
	$out .= $ml->h3($msg{"Tips"}).
	    $ml->ul(
		    $ml->li($msg{"If you are uploading an image, make sure that it is GIF, JPEG, or PNG format.  Any other files will be treated as documents."}).
		    $ml->li($msg{"Note that JPEG images are best for photographs.  GIF or PNG may work better for line art.  If your image is is another format, you will need to convert it to one of these formats using a graphic editing 
program."}).
		    $ml->li($msg{"If you require your image to have precise dimensions, you will need to resize the image first, using a graphic editing program. Otherwise it will be automatically resized to be no larger than 500 pixels across and 64K in size."})
		    );
    }
    else {
	# successful upload - back to library view
	$out .= $this->library;
    }
    return $out;
}

sub image_remove {
    my $this = shift;
    my $db = $share{DB};
    $this->{input}{image} =~ /^(\d+)$/;
    my $img = $1;
    my $out;
    if ($img) {
	my $content = $db->fetch("content",$img);
	if ($content->{page_id} == $this->get_library()->{page_id}) {
	    &run_publisher("content",$img,1);
	    $db->trash_r("content",$img,1);
	}
	else { 
	    $out = $this->Error($msg{"Invalid image specified."});
	}
    }
    else { 
	$out = $this->Error($msg{"No such image."});
    }
    $out .= $this->library;
    return $out;
}

# site_type : basic, regular, advanced

sub site_type {
    my $this = shift;
    my $note = $this->mysite()->{note};
    return "regular" if ($note =~ /regular/);
    return "advanced" if ($note =~ /advanced/);
    return "basic";
}

sub page_limit {
    my $this = shift;
    return $config{MySite}{$this->site_type}{page_limit};
}

sub image_limit {
    my $this = shift;
    return $config{MySite}{$this->site_type}{image_limit};
}

# capital-E error, so we don't overload the base class error handler

sub Error {
    my ($this) = shift;
    my $out = $ml->h3($msg{"Problem:"});
    $out .= $this->error(@_);
    return $out;
}

1;
