package Modules::PrintFriendly;
#-----------------------------------------------------------------------
#
#   PrintFriendly : re-render the page in a printer-friendly format.
#
#   This module hacks the context for the page to change its template
#   to a bare-bones template that only includes a body object.
#
#-----------------------------------------------------------------------
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Page;
use ExSite::Content;
use ExSite::Input;
use ExSite::Misc;
use ExSite::ML;
use ExSite::URI;
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$opt) = @_;
    my $p = $share{Page};
    my $db = $share{DB};
    my $in = $this->{input};
    my %opt = &DecodeString($opt);
    $this->{opt} = \%opt;
    my $ml = new ExSite::ML;
    my $out;
    if ($in->{printer_friendly}) {
	if (! $this->{printer_friendly}) {
	    my $p = $share{Page};
	    my $site = $p->get("section","title");
	    my $title = $p->get("page","title");
	    #my $url = $share{Page}->get_url;
	    my $uri = new ExSite::URI;
	    $uri->query(printer_friendly=>undef);
	    my $url = $uri->write_full();
#	    my $url = "http://".$ENV{SERVER_NAME}.$ENV{REQUEST_URI};

	    # hack the layout of the current page, to eliminate the template

	    $ml->Doc(
		     $ml->table(
				$ml->tr(
					$ml->td(
						$ml->a(
						       $ml->img(undef,
								{ src=>"$config{server}{HTMLpath}/_Modules/PrintFriendly/print.gif",
								  height=>16,
								  width=>16,
								  border=>0,
								  align=>"middle",
							      }),
						       {href=>"javascript:print()"}
						       ).
						" print".
						$ml->br().
						$ml->a(
						       $ml->img(undef,
								{ src=>"$config{server}{HTMLpath}/_Modules/PrintFriendly/close.gif",
								  height=>16,
								  width=>16,
								  border=>0,
								  align=>"middle",
							      } ),
						       {href=>"javascript:window.close()"}
						       ).
						" close",
						{ style=>"font-size:9pt;" }
						)
					),
				{align=>"right"}
				)
		     );
	    unless ($this->{opt}{url} eq "false") {
		$ml->_p("URL: ".$ml->a($url,{href=>$url}));
	    }
	    $ml->_hr();
	    $ml->Append("<!--content(body)-->\n");

	    $ml->__body();
	    $ml->Prepend(
			 $ml->head(
				   $ml->title($title).
				   $ml->style("\n* { font-family:Arial, Helvetica; }\n")
				   )
			 );
	    $ml->__html();

	    $share{Page}{html} = $ml->Write();
	    $this->{printer_friendly} = 1;   # only rebuild the page once
	    $share{Page}->redo();   # rebuild the page
	}
    }
    else {
	my $io = new ExSite::Input;
	return undef if ($io->is_post());
	my $url = $this->link(printer_friendly=>1);
	$out = "<a href=\"$url\" target=\"print\" title=\"Printer-friendly version\">";
	my $label = "Printer-friendly&nbsp;version";
	my $alt = "Printer-friendly version";
	my $icon = "$config{server}{HTMLpath}/_Modules/PrintFriendly/print.gif";
	if ($this->{opt}{fmt} eq "icon-on-top") {
	    $out = $ml->img(undef,{src=>$icon, height=>16, width=>16, border=>0, alt=>$alt}).
		$ml->br().
		$ml->span($label,{style=>"font-size:8pt;"});
	}
	elsif ($this->{opt}{fmt} eq "icon-on-right") {
	    $out = $ml->span($label,{style=>"font-size:8pt;"})."&nbsp;".
		$ml->img(undef,{src=>$icon, height=>16, width=>16, border=>0, alt=>$alt});
	}
	elsif ($this->{opt}{fmt} eq "noicon") {
	    $out = $ml->span($label,{style=>"font-size:8pt;"});
	}
	elsif ($this->{opt}{fmt} eq "icon-only") {
	    $out = $ml->img(undef,{src=>$icon,height=>16,width=>16,border=>0,alt=>$alt});
	}
	else {   # icon-on-left
	    $out = $ml->img(undef,{src=>$icon,height=>16,width=>16,border=>0,alt=>$alt}).
		"&nbsp;".
		$ml->span($label,{style=>"font-size:8pt;"});
	}
	$out = $ml->a($out,{href=>$url,target=>"print",title=>$alt});

	# wrap the widget
	if ($this->{opt}{align} eq "right") {
	    $out = $ml->table(
			      $ml->tr( $ml->td($out,{align=>"center"}) ),
			      { class=>"PrintFriendly",align=>"right" }
			      );
	}
	elsif ($this->{opt}{fmt} eq "icon-on-top") {
	    $out = $ml->table(
			      $ml->tr( $ml->td($out,{align=>"center"}) ),
			      { class=>"PrintFriendly" }
			      );
	}
	else {
	    $out = $ml->div($out,{class=>"PrintFriendly"});
	}
    }
    return $out;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "PrintFriendly regenerates the current page, but in a printer-friendly format with no graphics and minimal colors.";
    }
    elsif (/DynContentParameter/) {
	return "<p><b>Layout:</b>
<select name=\"param\">
<option value=\"\">(default)</option>
<option value=\"fmt=icon-on-left\">Icon then text</option>
<option value=\"fmt=icon-on-right\">Text then icon</option>
<option value=\"fmt=icon-on-top\">Icon above text</option>
<option value=\"fmt=icon-only\">Icon only</option>
<option value=\"fmt=noicon\">No icon (text only)</option>
</select></p>\n";

	### FIXME: removed params with '&align=right' because the & is
	### caught by the browser and escaped, fouling up the formatting
	### of our options string.

    }
    return undef;
}

1;
