package Modules::QA;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2009 - Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#  This modules processes user-configurable forms.
#
#----------------------------------------------------------------------------

use strict;

use ExSite::Config;
use ExSite::Misc;
use ExSite::ML;
use ExSite::Util;
use ExSite::Crypt;
use ExSite::Object;
use ExSite::ObjectList;
use Modules::BaseDCD;
use Modules::QA::Form;

use vars qw(@ISA $ml $QA $maxrec);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    $ml = &get_obj("ML");
    my %opt = &DecodeString($options);
    my $in = new ExSite::Input;
    $in = $in->combine;
    my $key_id;
    if ($in->{'_qakey'}) {
	my $c = new ExSite::Crypt;
	($key_id,undef) = split /;/, $c->decrypt($in->{'_qakey'});
    }
    my $form_id = $opt{form} || $key_id;
    my $action = $this->link();
    if ($form_id) {
	my $form = new Modules::QA::Form(id=>$form_id);
	return $form->show(action=>$action);
    }
    else {
	return $this->error("No form to display!");
    }
}

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my $this = shift;
    $_ = shift;           # $_ is the ioctl request

    if (/isRestricted/) {
	return 0;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/ModuleName/) {
	return "Web Forms";
    }
    elsif (/ModuleInfo/) {
	return "This modules processes user-configurable forms.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/DynContentParam/) {
	$ml = &get_obj("ML");
	my $forms = $this->my_forms();
	if ($forms->count) {
	    my $out .= $ml->strong("Forms:");
	    my $option;
	    while (my $f = $forms->next) {
		$option .= $ml->option($f->name,{value=>$f->id});
	    }
	    $out .= $ml->select($option);
	    return $out;
	}
	return undef;  # no forms; use blank parameter field
    }
    elsif (/ToDo/) {
	return \&todo;
    }
}

# ToDo : find all untranslated questions

sub todo {
    my $this = shift;
    my @version = split /\|/, $share{DB}{map}->regexp("list:version");
    my @task;
    if (@version > 0) {
	
################################################

    }
    return @task;
}

sub my_forms {
    my $this = shift;
    my $db = $share{DB};
    my $sid = $this->get_section_id();
    my @form;
    if ($config{QA}{admin_scope} eq "global") {
	# show all forms
	push @form, $db->fetch_all("qa_form");
    }
    else {
	# local scope; show only forms for selected section
	if ($db->level > 2) { 
	    push @form, $db->fetch_match("qa_form",{section_id=>0});
	}
	if ($sid) { 
	    push @form, $db->fetch_match("qa_form",{section_id=>$sid});
	}
    }
    my $forms = new ExSite::ObjectList(type=>"qa_form",list=>\@form);
    $forms->sort("name");
    return $forms;   # ObjectList
}

# Control Panel : browse and export responses

sub ctrl_panel {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    $ml = &get_obj("ML");

    $db->handler("show_foreign_key",\&show_foreign_key);

    if ($this->{input}{response}) {
	$out .= $this->restart();
	$out .= $this->show_response($this->{input}{response});
    }
    elsif ($this->{input}{form}) {
	if ($this->{input}{qacmd} eq "export") {
	    $out .= $this->export_responses($this->{input}{form});
	}
	elsif ($this->{input}{qacmd} eq "preview") {
	    $out .= $this->restart();
	    $out .= $this->preview($this->{input}{form});
	}
	elsif ($this->{input}{qacmd} eq "responses") {
	    $out .= $this->restart();
	    $out .= $this->show_responses($this->{input}{form});
	}
	else {
	    $out .= $this->restart();
	    $out .= $this->show_form($this->{input}{form});
	}
    }
    else {
	if ($config{QA}{admin_scope} ne "global") {
	    $out .= $this->set_section_id();
	}
	my $forms = $this->my_forms();
	my $r = new ExSite::ReportBuilder(title=>"Forms");
	$r->headers(["Form","Title","Responses","Section"]);
	while (my $f = $forms->next) {
	    $r->push($f->id,
		     $ml->a($f->name,{href=>$this->link(form=>$f->id())}),
		     $f->count_responses(),
		     $f->showdata("section_id"));
	}
	$r->tools($ml->a("[+] new form",{href=>"javascript:popup_large('$config{server}{CGIpath}/$config{prog}{ctrlpanel}/ObjectBrowser/qa_form?ob=new')"}));
	$out .= $r->make();
    }
    return $out;
}

sub restart {
    my $this = shift;
    return $ml->p(
		  &ExSite::HTML::Button(type=>"button",label=>"&lt; Back",url=>"javascript:history.back()")." ".
		  &ExSite::HTML::Button(type=>"button",label=>"Restart",url=>$this->link(form=>undef,qacmd=>undef))
		  );
}

sub show_form {
    my ($this,$formid) = @_;
    my $out;
    my $db = $share{DB};
    my $f = new Modules::QA::Form(id=>$formid);

    my $rpt = new ExSite::ReportBuilder(title=>$f->name());
    $rpt->headers(["Form #","Questions","Responses","Status","Last Modified"]);
    $rpt->data(
	       $f->id(),
	       $f->count_questions,
	       $f->count_responses,
	       $f->showdata("status"),
	       $f->showdata("mtime")
	       );
    $rpt->foot(
	       &ExSite::HTML::Button(label=>"Preview",type=>"button",url=>$this->link(qacmd=>"preview"))." ".
	       $ml->button("Edit",{type=>"button",class=>"button",onclick=>"popup_large('$config{server}{CGIpath}/$config{prog}{ctrlpanel}/ObjectBrowser/qa_form/$formid')"})." ".
	       &ExSite::HTML::Button(label=>"Show Reponses",type=>"button",url=>$this->link(qacmd=>"responses"))." ".
	       &ExSite::HTML::Button(label=>"Export Responses",type=>"button",url=>$this->link(qacmd=>"export"))
	       );
    return $rpt->make();
}

sub preview {
    my ($this,$formid) = @_;
    my $f = new Modules::QA::Form(id=>$formid);
    return $f->show();
}

sub get_responses {
    my ($this,$form) = @_;
    my $responses = new ExSite::ObjectList(type=>"qa_response");
    $responses->load({qa_form_id=>$form->id},"ctime desc");
    return $responses;
}

sub get_response_report {
    my ($this,$form) = @_;
    my $responses = $this->get_responses($form);
    my $rpt = new ExSite::ReportBuilder(title=>"Responses");
    $rpt->set("dynamic",1);
    my @head = qw(Response Date Member Version);
    push @head, $form->question_labels();
    $rpt->headers(\@head);
    $rpt->tools($ml->a("export to excel",{href=>$this->link(qacmd=>"export")}));
    while (my $r = $responses->next()) {
	my @data = ( $ml->a($r->id(),{href=>$this->link(response=>$r->id)}) );
	push @data, $r->getdata("ctime"), $r->showdata("member_id"), $r->showdata("version");
	if ($r->defined()) {
	    while (my $q = $r->{questions}->next) {
		push @data, $q->answer($r->id)->show(na=>undef);
	    }
	}
	$rpt->push(@data);
    }
    return $rpt;
}

sub show_responses {
    my ($this,$formid) = @_;
    my $out;
    my $f = new Modules::QA::Form(id=>$formid);
    my $r = $this->get_response_report($f);
    return $r->make();
}

sub show_response {
    my ($this,$response_id) = @_;
    my $r = new Modules::QA::Response(id=>$response_id);
    return $r->show;
}

sub export_responses {
    my ($this,$formid) = @_;
    my $f = new Modules::QA::Form(id=>$formid);
    my $r = $this->get_response_report($f);
    $ml->cancel();
    print "Content-type: application/vnd.ms-excel\n\n";
    $r->export("excel");
    return undef;
}

# scoped into Report class
sub show_foreign_key {
    my ($this,$table,$column,$data,$id,$datatype) = @_;
    return undef if ($table !~ /qa_form|qa_response/);
    # dereference foreign keys
    my (undef,$ftable) = split /[:=]/,$datatype;
    # look up foreign row
    my %fdata = $this->fetch($ftable,$data);
    my $fkey = $this->get_key($ftable);
    my $label = $this->record_label($ftable,$fdata{$fkey},\%fdata);
    return $label;
}

1;
