#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2009 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::QA::Answer;

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Form;
use ExSite::Misc;
use ExSite::Object;
use Modules::QA::Base;
use Modules::QA::Response;
use Modules::QA::Question;

use vars qw(@ISA);
@ISA = qw(ExSite::Object);

sub my_type { return "qa_answer"; }

sub name {
    my $this = shift;
    return $this->value() || "[n/a]";
}

sub value {
    my $this = shift;
    return $this->loaded() ? $this->getdata("value") : undef;
}

sub response {
    my $this = shift;
    return $this->loaded() ? new Modules::QA::Response(id=>$this->getdata("qa_reponse_id")) : undef;
}

sub question {
    my $this = shift;
    return $this->loaded() ? new Modules::QA::Question(id=>$this->getdata("qa_question_id")) : undef;
}

#=== DISPLAY

sub show {
    my ($this,%opt) = @_;
    if ($this->ok) {
	if ($opt{version}) { $this->set("version",$opt{version}); }
	if ($this->loaded()) {
	    my $datatype = $this->question->datatype;
	    if ($datatype) {
		# use show_data for its security 
		# (eg. masking tags)
		return $this->DB()->show_data_noauth("answer",
						     "value",
						     $this->getdata("value"),
						     $this->{id},
						     $datatype);
	    }
	    else {
		return $this->value();
	    }
	}
	else {
	    return $this->error("Answer does not exist.");
	}
    }
    # not ok means no ID, ie. no answer was recorded for this question
    #return "[no answer]";
    return exists $opt{na} ? $opt{na} : "[n/a]";
}

sub copy {
    my $this = shift;
    # can't copy individual answers - try Response::copy() instead
    return undef;
}

1;


