#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2009 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

### TODO:
###
### add a "key" flag, use for reporting purposes


package Modules::QA::Question;

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Form;
use ExSite::Misc;
use ExSite::Object;
use Modules::QA::Base;

use vars qw(@ISA);
@ISA = qw(ExSite::Object);

sub my_type { return "qa_question"; }

# name : human-friendly handle for the question

sub name {
    my $this = shift;
    return $this->label();
}

sub base {
    my $this = shift;
    return $this->{base}
}

# qname : computer-friendly handle for the question (used in forms)

sub qname {
    my $this = shift;
    my $name;
    if ($this->loaded()) {
	$name = 
	    $this->getdata("name") ||          # explicit name
	    $this->{base}->getdata("name") ||  # base has a default name
	    "qa".$this->id;                    # make up a name
    }
    return $name;
}

sub label {
    my $this = shift;
    return $this->loaded() ? $this->{base}->label() : undef;
}

sub format {
    my $this = shift;
    return $this->loaded() ? $this->{base}->getdata("format") : undef;
}

sub datatype {
    my $this = shift;
    return $this->loaded() ? $this->{base}->datatype() : undef;
}

sub is_required {
    my $this = shift;
    $this->getdata("flags") =~ /required/;
}

sub is_freeform {
    my $this = shift;
    return $this->loaded() ? $this->{base}->is_freeform() : undef;
}

sub answer_regexp {
    my $this = shift;
    return $this->loaded() ? $this->{base}->answer_regexp() : undef;
}

sub answer_values {
    my $this = shift;
    return $this->loaded() ? $this->{base}->answer_values() : undef;
}

sub answer {
    my ($this,$response_id) = @_;
    my @r = $this->DB()->fetch_match("qa_answer",{qa_question_id=>$this->id,
						  qa_response_id=>$response_id});
    return (scalar @r > 0) ?
	new Modules::QA::Answer(data=>$r[-1]) :
	new Modules::QA::Answer();
}

sub validate_answer {
    my ($this,$value) = @_;
    # is $value a valid answer to this question?
    if ($this->loaded()) {
	if ($value) {
	    return $this->{base}->validate_answer($value);
	}
	elsif ($this->is_required()) {
	    return &substitute($msg{"\"[[label]]\" is required."},{label=>$this->label()});
	}
    }
    return undef;
}

sub set_version {
    my ($this,$version) = @_;
    if ($this->defined()) {
	$this->{base}->set("version",$version);
	$this->{base}->load_translations();
    }
    $this->set("version",$version);
}

#=== DISPLAY

sub show {
    my ($this,%opt) = @_;
    if ($this->ok) {
	if ($opt{version}) { $this->set("version",$opt{version}); }
	if ($this->loaded()) {
	    if ($this->{base}->exists()) {
		if ($this->getdata("flags") =~ /required/) {
		    # adds JS to validate that we have data in this field
		    $opt{required} = 1;
		}
		my $name = $this->getdata("name");
		my $preset = $this->getdata("value");
#		if ($this->getdata("flags") =~ /hint/) {
#		    # adds JS to clear the hint value on focus
#		    $opt{hint} = $preset;
#		}
#		elsif ($preset) {
		if (! $opt{name} && $name) {
		    $opt{name} = $opt{id} = $name;
		}
		if (! $opt{value} && $preset) {
		    $opt{value} = $preset;
		}
		return $this->{base}->show(%opt);
	    }
	    else {
		return $this->error($msg{"Question is undefined."});
	    }
	}
	else {
	    return $this->error($msg{"Question does not exist."});
	}
    }
    return $this->error($this->{status});
}

# load record : can pass an ID or a datahash

sub loaded {
    my $this = shift;
    if (! $this->{data} || ! $this->{base}) { $this->load(@_); }
    return ($this->{data} && $this->{base});
}

sub load {
    my ($this,$data) = @_;
    if ($this->{data} || $this->SUPER::load()) {
	# got the question; now load the base (ie. question content)
	$this->{base} = new Modules::QA::Base(id=>$this->{data}{qa_base_id});
	if ($this->{version}) {
	    $this->{base}->set("version",$this->{version});
	}
	$this->{base}->load();
    }
}

sub copy {
    my $this = shift;
    # can't copy questions within a form
    return undef;
}

#=== Answers...

1;


