package Modules::QTranslator;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# Modules::QTranslator
# translate questionnaire questions
#-----------------------------------------------------------------------

=pod

get all questionnaires in a section
get all versions in these questionnaires
select a version from this list
foreach questionnaire that supports that version
    foreach base question in that questionnaire
        show translate link if question is out of date


=cut


use strict;

use ExSite::Config;
use ExSite::Misc;
use ExSite::HTML;

use Modules::Questionnaire::QuestionBase;

# inherit basic module behaviours from BaseDCD
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this, $opt) = @_;
    my ($db,$out,$status,%got_questionnaire);
    $db = $share{DB};
    $out .= $this->selectLang;
    if ($this->{input}{version}) {
	$out .= $this->selectQuestionnaire;
	if ($this->{input}{questionnaire}) {
	    if ($this->{input}{q}) {
		if ($this->{input}{action} =~ /do_make/) {
		    if (! $this->doMake()) {
			$out .= $this->show_diagnostics("error","html");
		    }
		    $out .= $this->translateQuestion();
		}
		elsif ($this->{input}{action} =~ /make/) {
		    $out .= $this->makeQuestion();
		}
		elsif ($this->{input}{action} =~ /do_edit/) {
		    if (! $this->doEdit()) {
			$out .= $this->show_diagnostics("error","html");
		    }
		    $out .= $this->translateQuestion();
		}
		elsif ($this->{input}{action} =~ /edit/) {
		    $out .= $this->editQuestion();
		}
		elsif ($this->{input}{action} =~ /okay/) {
		    $this->approveQuestion();
		    $out .= $this->showTranslatorQuestions();
		}
		else {
		    $out .= $this->translateQuestion();
		}
	    }
	    else {
		$out .= $this->showTranslatorQuestions();
	    }
	}
    }

    return $out;
}

=pod
    my $sid = $share{Page}->id("section");
    my $db = $share{DB};
    my @Q = $db->fetch_match("q_questionnaire",{section_id=>$sid});
    foreach my $Q (@Q) {
	my @q = $db->fetch_match("q_question",{questionnaire_id=>$Q->{questionnaire_id}});
	foreach my $q (@q) {
	}
    }
=cut

sub showTranslatorQuestions {
    my $this = shift;
    my ($db,$out,$status,$version,%got_questionnaire);
    $version = $this->{input}{version};
    $db = $share{DB};
    my @questionnaire;
    if ($this->{input}{questionnaire} > 0) {
	push @questionnaire, scalar $db->fetch("q_questionnaire",$this->{input}{questionnaire});
    }
    else {
	@questionnaire = $db->fetch_match("q_questionnaire",{section_id=>$this->my_section()},"questionnaire_id");
    }
    foreach my $Q (@questionnaire) {
	my @versions = split /,\s*/,$Q->{versions};
	if (grep /^$this->{input}{version}$/,@versions) {
	    my @question = $db->fetch_match("q_question",{questionnaire_id=>$Q->{questionnaire_id}},"sort_key");
	    foreach my $q (@question) {
		my $master = $db->fetch("q_question_base",$q->{question_base_id});
		my @qbase = $db->fetch_match("q_question_base",{master_id=>$q->{question_base_id}});
		my %q_by_version = &keywise("version",\@qbase);
		if (exists $q_by_version{$version}) {
		    if (($master->{timestamp} cmp $q_by_version{$version}{timestamp}) > 0) {
			$status = "check";
		    }
		    else {
			$status = "okay";
		    }
		}
		else {
		    $status = "missing";
		}
#		if ($status ne "okay") {
		    if (! $got_questionnaire{$Q->{questionnaire_id}}) {
			$out .= "<h4>$Q->{title}</h4>\n";
			$got_questionnaire{$Q->{questionnaire_id}} = 1;
		    }
		    my $label = &html_to_plaintext($master->{label});
		    $label = substr($label,0,100);
		    if (length $label == 100) { $label .= "..."; }
		    my $url = $this->link(q=>$master->{question_base_id},action=>undef);
		    $out .= "<a href='$url'><img src='$config{server}{HTMLpath}/_Modules/Questionnaire/$status.gif' border='0'></a> $label<br>\n";
#		}
	    }
	}
    }

    return $out;
}

sub translateQuestion {
    my $this = shift;
    my $m = new Modules::Questionnaire::QuestionBase($this->{input}{q});
    my $out = &ExSite::HTML::BasicBox(title=>"Master Question",pane=>$m->show_input(1,""));
    my $t = new Modules::Questionnaire::QuestionBase($this->{input}{q},$this->{input}{version});
    my $donebutton = $this->{input}{label} ? "Done" : "Cancel";
    if ($t->version() eq $this->{input}{version}) {
	my $tout = $t->show_input(1,"");
	if ($tout) {
	    $out .= &ExSite::HTML::BasicBox(title=>"<i>$this->{input}{version}</i> Version",pane=>$tout);
	}
	my $oklink = $this->link(action=>"okay");
	my $editlink = $this->link(action=>"edit");
	$out .= &ExSite::HTML::Button(label=>"Approve",url=>$this->link(action=>"okay"));
	$out .= &ExSite::HTML::Button(label=>"Edit",url=>$this->link(action=>"edit"));
	$out .= &ExSite::HTML::Button(label=>$donebutton,url=>$this->link(action=>undef,q=>undef));
    }
    else {
	$out .= &ExSite::HTML::BasicBox(title=>"<i>$this->{input}{version}</i> Version",pane=>&ExSite::HTML::Button(label=>"Create $this->{input}{version} version",url=>$this->link(action=>"make")));
	$out .= &ExSite::HTML::Button(label=>$donebutton,url=>$this->link(action=>undef,q=>undef));
    }
    return $out;
}

sub makeQuestion {
    my $this = shift;
    my $m = new Modules::Questionnaire::QuestionBase($this->{input}{q});
    my $mlabel = $m->label;
    my $mvalue = $m->current()->{value};
    my $out;
#    $out .= &ExSite::HTML::BasicBox(title=>"Master Question",pane=>$m->show_input(1,""));
    $out .= $this->questionForm("do_make",$mlabel,$mvalue,undef,undef);
    return $out;
}

sub editQuestion {
    my $this = shift;
    my $m = new Modules::Questionnaire::QuestionBase($this->{input}{q});
    my $mlabel = $m->label;
    my $mvalue = $m->current()->{value};
    my $t = new Modules::Questionnaire::QuestionBase($this->{input}{q},$this->{input}{version});
    my $tlabel = $t->label;
    my $tvalue = $t->current()->{value};
    my $out;
#    $out .= &ExSite::HTML::BasicBox(title=>"Master Question",pane=>$m->show_input(1,""));
    $out .= $this->questionForm("do_edit",$mlabel,$mvalue,$tlabel,$tvalue);
    return $out;
}

sub questionForm {
    my ($this,$action,$mlabel,$mvalue,$label,$value) = @_;
    my $out;
    my $url = $this->link(action=>$action);
    my $out .= "<form method='post' action='$url'>
<h3>Label:</h3>\n";
    $mlabel =~ s/</&lt;/g;
    $mlabel =~ s/>/&gt;/g;
    $out .= &ExSite::HTML::FrameBox(title=>"Master Version Label",pane=>$mlabel);
    my $input = "<textarea name='label' rows='4' cols='60'>$label</textarea>";
    $out .= &ExSite::HTML::FrameBox(title=>"<i>$this->{input}{version}</i> Version Label",pane=>$input);
    if ($mvalue) {
	$out .= "<h3>Value(s):</h3>\n";
	$mvalue =~ s/</&lt;/g;
	$mvalue =~ s/>/&gt;/g;
	$mvalue =~ s/\n/<br>\n/g;
	$out .= &ExSite::HTML::FrameBox(title=>"Master Version Value(s)",pane=>$mvalue);
	$input = "<textarea name='value' rows='4' cols='60'>$value</textarea>";
	$out .= &ExSite::HTML::FrameBox(title=>"<i>$this->{input}{version}</i> Version Value(s)",pane=>$input);
    }
    $out .= "<input type='submit' value='Save'> <input type='button' value='Cancel' onclick='history.back()'>\n";
    return $out;
}

sub doMake {
    my $this = shift;
    my $m = new Modules::Questionnaire::QuestionBase($this->{input}{q});
    return $m->makeVersion(label=>$this->{input}{label},
			   value=>$this->{input}{value},
			   version=>$this->{input}{version}
			   );
}

sub doEdit {
    my $this = shift;
    my $m = new Modules::Questionnaire::QuestionBase($this->{input}{q},
						     $this->{input}{version});
    my $current = $m->current();
    $current->{label} = $this->{input}{label};
    $current->{value} = $this->{input}{value};
    return $share{DB}->update("q_question_base",$current);
}

sub approveQuestion {
    my $this = shift;
    my $q = new Modules::Questionnaire::QuestionBase($this->{input}{q},
						     $this->{input}{version});
    return $q->approve();
}

sub selectLang {
    my $this = shift;

    my @Q = $share{DB}->fetch_match("q_questionnaire",{section_id=>$this->my_section()});

    my %v;
    foreach my $Q (@Q) { 
	my @v = split(/,\s*/,$Q->{versions});
	foreach my $v (@v) { $v{$v} = 1; }
    }

    my @versions = keys %v;

    my $url = $this->link();
    my $out = "<form method='GET'>
<input type='hidden' name='section_id' value='".$this->my_section()."'>
<input type='hidden' name='_id' value='$this->{input}{_id}'>
<select name='version' onchange='form.submit()'>
<option>== select your language ==</option>\n";
    foreach my $v (@versions) {
	next if ($v =~ /^(master|english)$/i);
	my $selected = ($v eq $this->{input}{version}) ? " selected" : "";
	$out .= "<option$selected>$v</option>\n";
    }
    $out .= "</select>\n</form>\n";
}

sub selectQuestionnaire {
    my $this = shift;
    my @questionnaire = $share{DB}->fetch_match("q_questionnaire",{section_id=>$this->my_section()},"questionnaire_id");
    my $out = "<form method='GET'>
<input type='hidden' name='section_id' value='".$this->my_section()."'>
<input type='hidden' name='_id' value='$this->{input}{_id}'>
<input type='hidden' name='version' value='$this->{input}{version}'>
<select name='questionnaire' onchange='form.submit()'>
<option>== choose questionnaire ==</option>
<option value='-1'>ALL QUESTIONNAIRES</option>\n";
    foreach my $Q (@questionnaire) {
	$out .= "<option value='$Q->{questionnaire_id}'";
	if ($Q->{questionnaire_id} == $this->{input}{questionnaire}) {
	    $out .= " selected";
	}
	$out .= ">$Q->{title}</option>\n";
    }
    $out .= "</select>\n</form>\n";
    if ($this->{input}{q}) {
	my $url = $this->link(q=>undef);
	$out .= "<p><a href='$url'>Back to questionnaire</a></p>\n";
    }
    return $out;
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleInfo/) {
	return "QTranslator is a tool for translating Questionnaires.";
    }
    elsif (/ModuleName/) {
	return "Translator";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    return undef;
}

sub ctrl_panel {
    my $this = shift;
    my $out = $this->set_section_id();
    $out .= $this->write();
    return $out;
}

sub my_section {
    my $this = shift;
    if ($share{Page}) {
	return $share{Page}->id("section");
    }
    return $this->get_section_id() || 35;
}



1;

