package Modules::RSSReader;

use strict;
use XML::RSS;
use LWP::Simple;

use ExSite::Config;
use ExSite::Misc;
use ExSite::Mail;
use ExSite::ReportBuilder;
use ExSite::Util;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

my $debug = 0;
if ($debug) {
print "Content-type: text/html\n\n";
}

# todo: 
# support shared rss feeds?
# security is an issue because who can add a shared feed to all sites?

sub write {
    my ($this,$options) = @_;
    my $in = $this->{input};  # set by BaseDCD::read
    my %opt = &DecodeString($options);
	%opt = (%opt,%{$in});
    my $action = $opt{action};

	# get the feed url
	if ($opt{feed}) {
		my $rss = $this->process_feed($opt{feed});
		return $this->print_html($rss,\%opt);
	} elsif ($action eq "do_form") {
		my $rss = $this->do_form;
		return $this->print_html($rss,\%opt);
	} elsif ($action eq "input") {
		return $this->input;
	} else {
		return $this->frontpage;
	}
	
}

# return a XML::RSS object from a feed url
sub process_feed {
	my ($this,$url) = @_;

	my ($content,$file);

	# create new instance of XML::RSS
	my $rss = new XML::RSS;

	# argument is a URL
	if ($url =~ /http:/i) {
		$content = get($url);
		return "<p>Error: Could not retrieve $url</p>" unless $content;
	    # parse the RSS content
	    eval { $rss->parse($content); }; 
	
	# argument is a file
	} else {
		$content = get("http://" . $url);
		return "<p>Error: Could not retrieve $url</p>" unless $content;
	    # parse the RSS content
	    eval { $rss->parse($content); }; 		
	} 
=pod	
	else {
	    $file = $url;
	    return "<p>Error: File $file doesn't exist.</p>" unless -e $file;
	    # parse the RSS file
	    $rss->parsefile($file);
	}
=cut
	return $rss;
}

sub ioctl {
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/) {
		return "RSS Reader";
	} elsif (/ControlPanel/) {
		return \&ctrl_panel;
	} elsif (/ModuleInfo/){
		return "RSS Reader aggregates RSS feeds into your website";
	} elsif (/DynContentParam/) {
		return $this->getDynContentParameter();
	} elsif (/isService/) {
		return 1;
    }
}

sub ctrl_panel {
    my $this = shift;
    my ($selector,$out,$rcmd);

    $out = $this->site_selector;
    $rcmd = $this->{input}{rcmd};
    
	if ($rcmd =~ /add|delete|do_add|edit|do_rank/) {
	 	$out .= $this->$rcmd;
	}	
	else {
		$out .= $this->manage;
	}
    return $out;
}

sub getOption {
	my ($this,$url,$icon) = @_;
	return "<a href=\"$url\">$icon</a>";
}

sub getIcon {
	my ($this,$name,$label,$attr) = @_;
	my (%icons,%default_attribute,$img);
	%default_attribute = (
	height=>16,
	width=>16,
	border=>0,
	align=>"middle",
	hspace=>3,
	);
	%icons = (
	new=>{img=>"new16.gif",label=>"Add New"},
	edit=>{img=>"edit16.gif",label=>"Edit"},
	report=>{img=>"report16.gif",label=>"Report"},
	delete=>{img=>"del16.gif",label=>"Delete"},
	);
	$label = $label || $icons{$name}{label};
	$img = "$config{server}{HTMLpath}/_Modules/RSSReader/".$icons{$name}{img};
	return "[$label]" if (!exists($icons{$name}));
	my (@attribute,$out);
	foreach my $def_attr(keys(%default_attribute)){
		next if $attr->{$def_attr};
		push(@attribute,"$def_attr=\"$default_attribute{$def_attr}\"");
	}
	foreach my $atr(keys(%$attr)){
		push(@attribute,"$atr=\"$attr->{$atr}\"");
	}	
	$out = "<img src=\"$img\" alt=\"$label\" title=\"$label\"";
	$out .= @attribute > 0 ? " ".join(" ",@attribute).">" : "";
	return $out;
}

sub manage {
    my ($this) = @_;
    my $db = $share{DB};

	my @head = (
	{label=>"Rank",column=>"rank"},
	{label=>"Label",column=>"label"},
	{label=>"Title",column=>"title"},
	{label=>"URL",column=>"url"},
	{label=>"Options",column=>"options"}	
	);	
	my $newurl = $this->link(rcmd=>"add");
	my $newicon = $this->getIcon("new","add new");
	my $newoption = $this->getOption($newurl,$newicon);
	
	my $user = $db->my_user_record;
	my $section_id = $this->{input}{section_id} || $this->get_section_id || $user->{section_id};

	if (!$section_id) { return "<p>Please select a website.</p>"; }
	my $site = $db->fetch("section",$section_id);
	my $out .= "<h3>$site->{title}</h3>\n";
	my @feeds = $db->fetch_match("rss_feed",{section_id=>$section_id});
	my ($header,$data) = $this->getReportBuilderData(\@head,\@feeds,"rss_feed");
    
	$out .= "<form method=\"post\">";
	$out .= "<input type=\"hidden\" name=\"rcmd\" value=\"do_rank\">";
	my $rpt = new ExSite::ReportBuilder(
					 title=>"RSS Feeds",
					 headers=>$header,
					 data=>$data,
					 tools=>"<a href=\"$newurl\">$newoption</a>"
				);
	$out .= $rpt->make;
	$out .= "<p><input type=\"submit\" value=\"Save Rank\"></p>";
	$out .= "</form>";
	return $out;
}

sub input_rank {
	my ($this,$table,$col,$rec) = @_;
	my $input_name = "rank#${table}#$rec->{rss_id}";
	my $value = $rec->{$col} || 0;
	return <<END;
<input type="text" name="$input_name" value="$value" size="3">
END
}

sub getReportBuilderData {
	my ($this,$header,$records,$table) = @_;
	my (@header,@data);
	my @data;
	my @recs = sort {$a->{rank} <=> $b->{rank}} @$records;
	foreach my $head (@$header) {	
		my $label = $head->{label};
		push(@header, $label);
	}

	foreach my $rec (@recs) {
		my @data_row;
		foreach my $head (@$header) {
			if ($head->{column} eq "options") {
				my $editurl = $this->link(rcmd=>"edit",id=>$rec->{rss_id});
				my $editicon = $this->getIcon("edit","edit");
				my $editoption = $this->getOption($editurl,$editicon);				
					
				my $delurl = $this->link(rcmd=>"delete",tab=>"rss_feed",id=>$rec->{rss_id});	
				my $delicon = $this->getIcon("delete","delete");
				my $deloption = $this->getOption($delurl,$delicon);

				my $options = "$editoption&nbsp;$deloption";
				push(@data_row, $options);				
			} elsif ($head->{column} eq "rank") {
				my $rank_input = $this->input_rank("rss_feed","rank",$rec);
				push(@data_row,$rank_input);
			} else {		
				push(@data_row,$rec->{$head->{column}});
			}
		}
		push(@data,\@data_row);
	}
	
	return (\@header,\@data);
}	

sub add {
	my ($this) = @_;
    my $db = $share{DB};
    my $url = $this->link(rcmd=>"do_add");
    my $out = "<h3>Add New RSS Feed</h3>\n";
	my $hide = ["title","logo","description"];
    $out .= $db->make(table=>"rss_feed",action=>$url,hide=>$hide,data=>{section_id=>$this->{input}{section_id}},extra=>{rcmd=>"do_add"});
	return $out;
}

sub edit {
	my ($this) = @_;
    my $db = $share{DB};
    my $url = $this->link(rcmd=>"do_add");
    my $out = "<h3>Edit RSS Feed</h3>\n";
	my $hide = ["title","logo","description"];
    $out .= $db->make(table=>"rss_feed",record=>$this->{input}{id},action=>$url,hide=>$hide,extra=>{rcmd=>"do_add"});
	return $out;
}

sub do_add {
    my ($this) = @_;
    my $db = $share{DB};
	$db->get_data;

    my @keys = keys %{$db->{form}{rss_feed}};
    my $feed_id = $keys[0];

	my $url = $db->{form}->{rss_feed}{$feed_id}{url};
	my $rss = $this->process_feed($url);	
	if ($rss =~ /Error/) {
		return "Sorry, the system could not retrieve your RSS feed.
 There may be several reasons for this.  For example it may be an Atom Feed(unsupported) or the feed may be out of service 
 at the moment in which case you can try again at a later time.<br><br>Click <a href=\"javascript:back()\">here</a> to try another feed." . $this->manage; 
	}

	$db->{form}{rss_feed}{$feed_id}{title} = $rss->{channel}{title};
	$db->{form}{rss_feed}{$feed_id}{description} = $rss->{channel}{description};
	$db->{form}{rss_feed}{$feed_id}{logo} = $rss->{image}{url};
    my $out = $db->do;
    if ($out =~ /Done\!/) {
	# kludge reply screen
	$out = $this->manage;
    }
    return $out;
}

sub delete {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    if ($in->{tab} && $in->{id}) {
	$share{DB}->trash_r($in->{tab},$in->{id});	
	my $p = $share{Page} || $share{PageBuf};
	my $url = $this->link(rcmd=>undef,tab=>undef,id=>undef);
	if ($p) {
	    $p->redirect($url);
	    return "";
	}
	else {
	    return "<h1>Deletion Complete</h1>\n<p><a href=$url>Back to control panel</a></p>\n";
	}
    }
    else {
	return "<h1>Problem</h1>\n<p class=error>Permission denied: invalid website.</p>\n";
    }
    return $out;
}
		

sub site_selector {
    my $this = shift;
    my $out;
	my $section_id;	
    if (!$share{Page}) {
		$out .= $this->set_section_id;
	}
    return $out;
}


sub getDynContentParameter{
	my ($this) = @_;
	my $section_id = $this->get_section_id;
	my $db = $share{DB};
	my $out;
	my $sql = "select * from rss_feed where section_id = ? or section_id = ?";
	my @params = (0,$section_id);
	my @feeds = $db->custom_query($sql,@params);
	$out .= "\n<table>";
	$out .= "\n<tr><td>";
	$out .= "<b>Select an option:</b>";
	$out .= "</td><td>";
	$out .= "<select name=\"param\">";
	$out .= "<option value=\"\">Default - allow user to select from list of all feeds</option>";
	$out .= "<option value=\"action=input\">Form - allow user to display any rss feed</option>";	
	foreach my $feed (@feeds) {
		$out .= "<option value=\"feed=$feed->{url}\">Show feed: $feed->{label}</option>";
	}
	$out .= "</select>";
	$out .= "</td></tr>";
	$out .= "\n</table>";
	return $out;
}


sub print_html {
    my ($this,$rss,$opt) = @_;
	my $in = $this->{input};
	if ($rss =~ /Error/i || !$rss->{channel}->{title}) { return "Sorry, the system could not retrieve your RSS feed.
 There may be several reasons for this.  For example it may be an Atom Feed(unsupported) or the feed may be out of service 
 at the moment in which case you can try again at a later time.<br><br>Click <a href=\"javascript:back()\">here</a> to try another feed."; }
    my $topurl = $this->link(feed=>undef);
	my $top;
	my $rss_title = $opt->{label} || $rss->{channel}->{title};
	if ($in->{menu}) { $top = "<a href=$topurl>Top</a>&nbsp&gt;&nbsp"; }
	my $out = <<HTML;
<table class="rss" cellspacing="1" cellpadding="4" border="0" width="100%">
<tr>
<td valign="middle" align="center" class="rss_header">
<b>$top<a href="$rss->{channel}->{link}" target="_blank">$rss->{channel}->{title}</a></b>
<p>$rss->{channel}{description}</p></td>
</tr>
<tr><td>
HTML
	
    # display channel image
    if ($rss->{image}->{url}) {
	$out .= <<HTML;
<center>
<p><a href="$rss->{image}->{link}" target="_blank"><img src="$rss->{image}->{url}" alt="$rss->{image}->{title}" border="0"
HTML
	$out .= " width=\"$rss->{image}->{width}\"" if $rss->{image}->{width};
	$out .= " height=\"$rss->{image}->{height}\"" if $rss->{image}->{height};
	$out .= "></a></center><br>\n";
    }
    
    # display the channel items
    my $item_count = 0;
    my $max_items = $opt->{max_items} || $config{RSSReader}{max_items} || 5;
    my $max_desc = $opt->{max_desc} || $config{RSSReader}{max_desc} || 250;
    foreach my $item (@{$rss->{items}}) {
	last if ($item_count >= $max_items);
	my $itemlink = $item->{link} || $item->{guid};
	next unless defined($item->{title}) && defined($itemlink);
	
	my $item_title = $this->clean($item->{title});
	$out .= "<dl><dt><a href=\"$itemlink\" target=\"_blank\">$item_title</a></dt>\n";
	my $item_desc = $this->clean($item->{description});
	if (!$in->{html}) { $item_desc = &html_to_plaintext($item_desc); }
	$item_desc = substr $item_desc, 0, $max_desc;
	$item_desc .= " ...";
	$out .= "<dd>$item_desc</dd></dl>\n";
	$item_count++;
    }

    # if there's a textinput element
    if ($rss->{textinput}->{title}) {
	$out .= <<HTML;
<form method="get" action="$rss->{textinput}->{link}">
$rss->{textinput}->{description}<br> 
<input type="text" name="$rss->{textinput}->{name}"><br>
<input type="submit" value="$rss->{textinput}->{title}">
</form>
HTML
    }

    # if there's a copyright element
    if ($rss->{channel}->{copyright}) {
	$out .= <<HTML;
<p><sub>$rss->{channel}->{copyright}</sub></p>
HTML
    }

    $out .= <<HTML;
</td>
</tr>
</table>

HTML
	return $out;
}

sub input {
	my ($this) = @_;
	my $db = $share{DB};
	my $out;
	my $url = $this->link(action=>"do_form");
	$out .= "<h1>ExSite Reader</h1>";
    $out .= "<form METHOD=POST ENCTYPE=\"multipart/form-data\" ACTION=\"$url\">\n<table CLASS=\"form\">\n";
   	$out .= "<tr><td><span class=\"label\">Paste a feed url</span>:</td><td>\n";
	$out .= $db->input_html(name=>"rss##feed",type=>"text",size=>40);
	$out .= "</td></tr>\n";
	$out .= "<tr><td colspan=\"2\">e.g<br>http://rss.news.yahoo.com/rss/topstories<br>Does not currently support atom links</td></tr></table>";
    $out .= $db->make_form_buttons;	
	$out .= "</form>\n";
	return $out; 
}


# css for frontpage contained within a regular body
sub frontpageCSS {
	my ($this) = @_;
	my $out .= <<END;
<style>
.rss_top
{
border: 1px solid #ccc;
padding-top: 9px;
padding-right: 0px;
padding-bottom: 9px;
padding-left: 9px;
min-height: 75px;
margin-bottom: 5px;
}

.rss_top p { padding-left: 200px; }

.rss_top .image
{
float: left;
padding: 10px;
text-align: center;
width: 200px;
}
.rss_top .image img {
max-width: 150px;
}
</style>	
END
	return $out;	
}

sub templateFrontPageFeed {
	my ($this) = @_;
	my $out = "
<div class=rss_top><div class=image>
<img src=\"<!--[[feed_logo_src]]-->\"></div>
<p><a href=\"<!--[[feed_url]]-->\"><!--[[feed_title]]--></a></p></div>";
	return $out;
}
	
sub frontpage {
	my ($this) = @_;
	my $db = $share{DB};
	my $out;
	my $p = $share{Page};
	my $section_id = $share{DB}->this_site()->{section_id};
	my $sql = "select * from rss_feed where section_id = ? or section_id = ?";
	my @params = (0,$section_id);
	my @feeds = $db->custom_query($sql,@params);
	
# new templating stuff
	
	my $template = $this->templateFrontPageFeed;
	
	$out .= $this->frontpageCSS if $config{RSSReader}{css} == 1;
	@feeds = sort {$a->{rank} <=> $b->{rank}} @feeds;
	foreach my $feed (@feeds) {
		my $html_opt;
		if ($feed->{enable_html} eq "Y") { $html_opt = 1 };
		my $url = $this->link(feed=>$feed->{url},menu=>1,html=>$html_opt,label=>$feed->{label});
		my %data;
		$data{feed_logo_src} = $feed->{logo} ||	"$config{server}{HTMLpath}/_Modules/RSSReader/nophoto.gif";
		$data{feed_url} = $url;
		$data{feed_title} = $feed->{label} || $feed->{title};
		$out .= $this->merge($this->templateFrontPageFeed,\%data);
	}
	return $out;
}
	
sub do_form {
	my ($this) = @_;
	my $db = $share{DB};
    $db->get_data;

	my $feed = $db->{form}->{rss}->{_}->{feed};
	return $this->process_feed($feed);
}

# template data merge
sub merge{
	my ($this,$view,$data) = @_;
	return if !$view || !$data;
	$view =~ s/\<\!\-\-\[\[((\w+(\_| )?)+\w*)\]\]\-\-\>/$data->{$1}/g;
	return $view;
}

sub clean {
	my ($this,$text) = @_;
	$text =~ s/[^[:ascii:]]+//g;
	return $text;
}

sub do_rank {
	my ($this) = @_;
	my $input = $this->{input};
	my $db = $share{DB};
	foreach my $key(keys(%$input)) {
		if ($key =~ /^rank#/i){
			my @fields = split("#",$key,3);	
			my $value = $input->{$key};
			my $table = $fields[1];
			my $id = $fields[2];
			$db->update($table,{rank=>$value},{rss_id=>$id});
		}
	}
	return $this->manage;
}

1;
