package Modules::SysMsg;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2008 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Form;
use ExSite::Misc;
use ExSite::Util;
use ExSite::ML;
use ExSite::ReportBuilder;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# write : dummy method;  this is an administrator tool only

sub write {
    my ($this,$options) = @_;
    return "";
}

# ioctl : this is a public tool

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "System Messages";
    }
    elsif (/ModuleInfo/) {
	return "System Messages are coded in English by default.  This plug-in 
allows you to manage the translations of your system messages into 
alternate languages.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ToDo/) {
	return \&todo;
    }
    elsif (/Category/) {
	return "System";
    }
    return undef;
}

sub ctrl_panel {
    my $this = shift;
    my $db = $share{DB};
    my $version = $this->my_version;
    my $out = $this->select_version($version);
    if (! $version || $version =~ /^English$/i) {
	my $ml = &get_obj("ML");
	$out .= $ml->p("Select another language to manage its translations.");
    }
    elsif ($this->{input}{mid}) {
	$out .= $this->update_message($this->{input}{mid},$this->{input}{tid});
	
#	$out .= $this->{input}{action} eq "update" ?
#	    $this->update_message($this->{input}{mid},$this->{input}{tid}) : 
#	    $this->review_message($this->{input}{mid},$this->{input}{tid});
    }
    elsif ($version && $version !~ /English/) {
	$out .= $this->search_form();
	if ($this->{input}{search}) { 
	    $out .= $this->do_search(); 
	}
	else {
	    my @msg = $share{DB}->fetch_all("sysmsg");
	    my @altmsg = $share{DB}->fetch_match("sysmsg_alt",{version=>$version});
	    $out .= $this->version_report($version,\@msg,\@altmsg,0);
	}
    }
    return $out;
}

sub todo {
    my $this = shift;
    my @task;
    my $db = $share{DB};
    my @msg = $db->custom_query("select * from sysmsg s,sysmsg_alt a where a.sysmsg_id=s.sysmsg_id and length(a.message_alt)=0");
    if (@msg > 0) {
	my $link = "$config{server}{server}$config{server}{CGIpath}/$config{prog}{ctrlpanel}/SysMsg";
	my $count = scalar @msg;
	push @task, {link=>$link,
		     title=>"System messages awaiting translation",
		     description=>"$count messages are missing their translations",
		     };
    }
    return @task;
}

sub review_message {
    my ($this,$mid,$tid) = @_;
    my $version = $this->my_version;
    my $omsg = $share{DB}->fetch("sysmsg",$mid);
    if (! $omsg) {
	return $this->error("no such message");
    }
    my $amsg;
    if ($tid && $version) {
	$amsg = $share{DB}->fetch("sysmsg_alt",$tid);
	if ($amsg->{sysmsg_id} != $omsg->{sysmsg_id}) {
	    return $this->error("mismatched messages");
	}
    }
    my $status = "okay";
    if (! $amsg || ! $amsg->{message_alt}) { 
	$status = "missing"; 
    }
    elsif (($amsg->{ctime} cmp $omsg->{ctime}) < 0) { 
	$status = "check"; 
    }
    my $ml = &get_obj("ML");
    my $statusimg = $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/SysMsg/$status.gif",border=>0});

    my $link = $this->link(action=>"update");

    my $r = new ExSite::ReportBuilder(title=>$msg{Message},
				      headers=>[$msg{English},
						$version,
						$msg{Status}],
				      data=>[$omsg->{message},
					     $amsg ? $amsg->{message_alt} : "",
					     $statusimg],
				      tools=>$ml->a($msg{update},
						    {href=>$link}),
				      );
    my $out .= $ml->p(
		      $ml->a("&lt; $msg{back}",{href=>$this->link(action=>undef,
								  mid=>undef,
								  tid=>undef),
						class=>"button"})
		      );
    $out .= $r->make;
    return $out;
}

sub update_message {
    my ($this,$mid,$tid) = @_;
    my $version = $this->my_version;
    my $omsg = $share{DB}->fetch("sysmsg",$mid);
    if (! $omsg) {
	return $this->error($msg{"no such message"});
    }
    my $amsg;
    if ($tid && $version) {
	$amsg = $share{DB}->fetch("sysmsg_alt",$tid);
	if ($amsg->{sysmsg_id} != $omsg->{sysmsg_id}) {
	    return $this->error($msg{"mismatched messages"});
	}
    }
    my $input = new ExSite::Input;
    my $pin = $input->post;
    my $ml = &get_obj("ML");
    if ($pin->{message}) {
	# ExSite::Message self-manages translation updates
	my %m;
	tie %m, "ExSite::Message", $version, 1, \%share, \%store;
	$m{$omsg->{message}} = $pin->{message};
	if ($share{DB}->errorcheck()) {
	    return $share{DB}->show_diagnostics("error","html");
	}
	&redirect($this->link(__plaintext=>1,action=>undef,mid=>undef,tid=>undef));
    }
    else {
	my $f = new ExSite::FormBuilder(method=>"post");
	$f->input(name=>"message_orig",
		  type=>"textarea",
		  cols=>80,
		  rows=>4,
		  prompt=>$msg{English},
		  value=> $omsg->{message},
		  disabled=>1,
		  #readonly=>1,
		  );
	$f->input(name=>"message",
		  type=>"textarea",
		  cols=>80,
		  rows=>4,
		  prompt=>$version,
		  value=> $amsg ? $amsg->{message_alt} : "",
		  required=>1,
		  );
	return $f->make;
    }
}

sub version_report {
    my ($this,$version,$messages,$alt_messages,$show_okay) = @_;
    my @msg = @$messages;
    my @altmsg = @$alt_messages;
    my %altmsg = &keywise("sysmsg_id",\@altmsg);
    my $r = new ExSite::ReportBuilder(title=>"$msg{Messages} ($version)");
    $r->headers([$msg{Message},$msg{"Translated Message"},$msg{Status}]);
    my $okaycount = 0;
    foreach my $m (sort { lc $a->{message} cmp lc $b->{message} } @msg) { 
	my $am = $altmsg{$m->{sysmsg_id}};
	if (! $show_okay && $am && $am->{message_alt} && (($am->{ctime} cmp $m->{ctime}) > 0)) {
	    $okaycount++;
	    next;
	}
	if (! $am) { $am = {}; }
	my $status = "okay";
	if (! $am->{message_alt}) { 
	    $status = "missing"; 
	}
	elsif (($am->{ctime} cmp $m->{ctime}) < 0) { 
	    $status = "check"; 
	}
	my $ml = &get_obj("ML");
	my $statuslink = $ml->a(
				$ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/SysMsg/$status.gif",border=>0}),
				{href=>$this->link(mid=>$m->{sysmsg_id},tid=>$am->{sysmsg_alt_id})}
				);
	$r->push($m->{message},$am->{message_alt},$statuslink);
    }
    if (! $show_okay && $okaycount) {
	$r->foot("$okaycount ".$msg{"up-to-date messages also found."});
    }
    return $r->make;
}

sub do_search {
    my $this = shift;
    my $version = $this->my_version;
    my $term = $this->{input}{search};
    if (! $term) { 
	return $this->warn("nothing to search for");
    }
    my @msg = $share{DB}->fetch_all("sysmsg");
    my @altmsg = $share{DB}->fetch_match("sysmsg_alt",{version=>$version});
    my %altmsg = &keywise("sysmsg_id",\@altmsg);
    my (@search_msg, @search_alt);
    foreach my $m (sort { lc $a->{message} cmp lc $b->{message} } @msg) { 
	my $am = $altmsg{$m->{sysmsg_id}};
	if ($m->{message} =~ /$term/ || $am->{message_alt} =~ /$term/) {
	    push @search_msg, $m;
	    if ($am) { push @search_alt, $am; }
	}
    }
    return $this->version_report($version,\@search_msg,\@search_alt,1);
}

sub select_version {
    my $this = shift;
    my $version = shift || $this->my_version;
    my $datatype = $share{DB}->{map}->get_datatype("list:version");
    return $this->error($msg{"no versions/languages found"}) if (! $datatype);

    my @opt = split /\|/,$datatype->{regexp};

    my $f = new ExSite::FormBuilder(method=>"get");
    $f->name("VersionSelector");
    $f->template("[[version:prompt]]: [[version:input]]");
    $f->input(name=>"version",
	      type=>"select",
	      prompt=>$msg{"Select language"},
	      options=>\@opt,
	      value=>$version,#$this->my_version(),
	      nullvalue=>"== $msg{select} ==",
	      onchange=>"document.forms['VersionSelector'].submit()",
	      );
    $f->set("buttons"," ");
    return $f->make;
}

sub search_form {
    my $this = shift;
    my $f = new ExSite::FormBuilder(method=>"post");
    $f->name("Search");
    $f->template("[[search:prompt]]: [[search:input]] <input type='submit' value='Search'>");
    $f->input(name=>"search",
	      type=>"text",
	      prompt=>$msg{"Search"},
	      value=>$this->{input}{search},
	      );
    $f->set("buttons"," ");
    return $f->make;
}

sub my_version {
    my $this = shift;
    my $version = $this->{input}{version} || $config{default_version};
    if (! $version || $version =~ /^English$/i) {
	my $datatype = $share{DB}->{map}->get_datatype("list:version");
	my @version = split /\|/, $datatype->{regexp};
	if (@version == 1) {
	    # if there is only one version, select that by default
	    $version = $version[0];
	}
    }
    return $version;
}

1;

