package Modules::Tags;
#-----------------------------------------------------------------------
#
#   Tags : tag selected content with keywords for more efficient
#   browsing, indexing, and searching.
#
#-----------------------------------------------------------------------
#   Copyright 2001-2008 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Object;
use ExSite::Misc;
use ExSite::HTML;
use ExSite::Module qw(&service_page);
use Modules::BaseDCD;
use Modules::Zine::Article;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# public functions

sub write {
    my ($this,$opt) = @_;
    my %opt = $opt ? &DecodeString($opt) : %{$this->{input}};
    my $keyword = $opt{keyword};
    my $sid = $share{Page}->id("section");
    my $out;
    if ($opt eq "list") {
	# use this option for sidebar indexes
	my $zp = &service_page("Zine",$sid);
	my $zpid = $zp ? $zp->id() : 0;
	my $index = $this->get_index_data($sid,$zpid);
	$out = $this->index_list_keywords($index);
    }
    elsif ($keyword) {
	# generate a list of resources under a keyword
	$out = $this->index_keyword($keyword,$sid);
    }
    elsif ($opt{tags}) {
	# generate a linked list of tags under an item
	$out = $this->my_tags($opt{tags},$opt{id});
    }
    else {
	# generate a full index for current section
	my $indextype = $opt{indextype} || $config{Tags}{indextype} || "list";
	$out = $this->index($sid,$indextype);
    }
    return "<div class=\"Tags\">$out</div>\n";
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Keyword Tags";
    }
    elsif (/ModuleInfo/) {
	return "This module allows the webmaster to tag selected content
with arbitrary key words and phrases for more efficient browsing, indexing,
and searching.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/DynContentParameter/) {
	return "<p><strong>Index Style:</strong>
<select name=\"param\">
<option value=\"\">(default)</option>
<option value=\"indextype=list\">Keyword listing</option>
<option value=\"indextype=alpha\">Alphabetical index</option>
<option value=\"indextype=tree\">Tree view</option>
</select></p>\n";
    }
    elsif (/Search/) {
	return \&search_tags;
    }
    elsif (/Category/) {
	return "CMS";
    }
    return undef;
}

# admin view

sub ctrl_panel {
    my $this = shift;
    $this->{admin} = 1;

    my ($out,$tab,$cmd);
    $out .= "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_Modules/Tags/Tags.css\">\n";

    $cmd = $this->{input}{tagcmd};

    if ($cmd =~ /tags/) {
	if ($cmd eq "addtags") {
	    return $this->add_tag();
	}
	$out .= $this->list_taggable_content();
	$tab = 2;
    }
    elsif ($cmd =~ /keyw/) {
	if ($this->{input}{term}) {
	    $out .= $this->show_keyw($this->{input}{term},$this->{input}{section_id});
	}
	elsif ($cmd eq "newkeyw") {
	    $out .= $this->add_keyw();
	}
	elsif ($this->{input}{kwid}) {
	    if ($cmd eq "disablekeyw") {
		$this->disable_keyw($this->{input}{kwid});
		$out .= $this->list_keyw();
	    }
	    elsif ($cmd eq "enablekeyw") {
		$this->enable_keyw($this->{input}{kwid});
		$out .= $this->list_keyw();
	    }
	    elsif ($cmd eq "delkeyw") {
		$this->delete_keyw($this->{input}{kwid});
		$out .= $this->list_keyw();
	    }
	    elsif ($cmd eq "editkeyw") {
		$out .= $this->edit_keyw($this->{input}{kwid});
	    }
	    else {
		$out .= $this->show_keyw($this->{input}{kwid});
	    }
	}
	else {
	    $out .= $this->list_keyw();
	}
	$tab = 1;
    }
    else {
	$out .= "<form method=\"GET\">\n";
	$out .= "<table width='100%'><tr><td>\n";
	$out .= $this->set_section_id();
	$out .= "</td><td>\n";
	$out .= $this->index_selector();
	$out .= "</td></tr></table>\n";
	$out .= "</form>\n";
	    
	my $sid = $this->get_section_id();
	if ($sid) {
	    my $indextype = $this->{input}{indextype} || "alpha";
	    $out .= $this->index($sid,$indextype);
	}
	$tab = 0;
    }

    my %tab = ( tabs => [ 
			  { label=>"Indexes", url=>$this->link(tagcmd=>"index",kwid=>undef,tag=>undef,section_id=>undef) },
			  { label=>"Keywords", url=>$this->link(tagcmd=>"keyw",kwid=>undef,tag=>undef,section_id=>undef) },
			  { label=>"Tags", url=>$this->link(tagcmd=>"tags",kwid=>undef,tag=>undef,section_id=>undef) },
		      ],
		thistab => $tab,
#		fill_tabbar => 1,
		pane => $out,
		);
    return &ExSite::HTML::TabBox(%tab);
}

# selectors for index previews

sub set_section_id {
    my $this = shift;
    my $out;
    if (! $share{Page}) {
	# only output a site selector on control panels
	my $sid = $this->{input}{section_id} ? $this->{input}{section_id} : 0;
	my @sites = $share{DB}->my_real_sites;
	$out .= "Select website: <select name=\"section_id\" onchange=\"document.forms[0].submit()\">\n<option value=\"0\">== choose a site ==</option>\n";
	foreach my $s (@sites) {
	    $out .= "<option value=\"$s->{section_id}\"";
	    $out .= " selected" if ($s->{section_id} == $sid);
	    $out .= ">$s->{title}</option>\n";
	}
	$out .= "</select>\n";
    }
    return $out;
}

sub index_selector {
    my ($this) = @_;
    my $out;
    if (! $share{Page}) { 
	my $type = $this->{input}{indextype};
	my @types = (
		     { value=>"list",text=>"list" },
		     { value=>"alpha",text=>"alphabetical" },
		     { value=>"tree",text=>"tree view" },
		     );
	$out = "Index style: <select name=\"indextype\" onchange=\"document.forms[0].submit()\">
<option value=\"\">== choose a style ==</option>
\n";
	foreach my $it (@types) {
	    $out .= "<option value=\"$it->{value}\"";
	    if ($it->{value} eq $this->{input}{indextype}) {
		$out .= " selected";
	    }
	    $out .= ">$it->{text}</option>\n";
	}
	$out .= "</select>\n";
    }
    return $out;
}

# display an admin view of a single keyword

sub show_keyw {
    my ($this,$id) = @_;
    my $keyword = $share{DB}->fetch("keyword",$id);
    my $out;
    my $url = $this->link(tagcmd=>"keyw",kwid=>undef);
    $out .= "<p>".&ExSite::HTML::Button(url=>$url,label=>"&lt; Back")."</p>\n";
    my $kw = new ExSite::Object(type=>"keyword",
				id=>$keyword->{keyword_id},
				data=>$keyword);
    $out .= $kw->show(title=>"Tag '$keyword->{keyword}'");
    $url = $this->link(tagcmd=>"editkeyw");
    $out .= "<p>".&ExSite::HTML::Button(url=>$url,label=>"Edit")."</p>\n";

    my @keyw = $share{DB}->fetch_match("keyword_tag",{keyword_id=>$id});
    if (@keyw > 0) {
	$out .= "<h3>This keyword is tagging:</h3>\n";
	foreach my $k (@keyw) {
	    $out .= "<div class=\"TagsContent\">".$this->link_to_tagged_content($k)."</div>\n";
	}
    }
    else {
	$out .= "<p>Nothing is tagged with this keyword.</p>\n";
    }
    return $out;
}

# generate links to admin previews of tagged content items

sub link_to_tagged_content {
    my ($this,$tag) = @_;
    my ($type,$id);
    if ($tag->{section_id}) { 
	$type = "section";
	$id = $tag->{section_id};
    }
    elsif ($tag->{page_id}) { 
	$type = "page";
	$id = $tag->{page_id};
    }
    elsif ($tag->{content_id}) { 
	$type = "content";
	$id = $tag->{content_id};
    }
    elsif ($tag->{article_id}) { 
	$type = "article";
	$id = $tag->{article_id};
    }
    my ($label,$url) = $this->describe_content($type,$id);
    my $icon = $this->content_icon($type,$tag);
    return "<a href='javascript:popup_medium(\"$url\")'>$icon $label</a>";
}

# choose an appropriate icon to mark taggable content with

sub content_icon {
    my ($this,$type,$tag) = @_;
    my $imgdir = "$config{server}{HTMLpath}/_Modules/Tags";
    my $img;
    if ($type eq "section" || $tag->{section_id}) { 
	$img ="sitemap.png";
    }
    elsif ($type eq "page" || $tag->{page_id}) { 
	$img ="page_link.png";
    }
    elsif ($type eq "content" || $tag->{content_id}) { 
	$img = "picture.png";
    }
    elsif ($type eq "article" || $tag->{article_id}) { 
	$img = "page_white_text.png";
    }
    return "<img src=\"$imgdir/$img\" border=0>";
}

# label a piece of taggable content

sub describe_content {
    my ($this,$type,$id) = @_;
    my ($label,$url);
    $_ = $type;
    if (/section/) { 
	my $s = new ExSite::Section(id=>$id);
	$url = $s->get_url();
	my $section = $s->get();
	$section->{name} = $section->{url} || $section->{publish_to} || "base section";
	$label = &substitute("[[title]] ([[name]])",$section);
    }
    elsif (/page/) { 
	my $p = new ExSite::Page(id=>$id);
	$url = $p->get_url_canonical();
	$p->set_context("parent");
	my $page = $p->get();
	my $section = $p->get("section");
	$page->{name} = $section->{title}.": ".$p->{label};
	my $s = new ExSite::Section(id=>$p->id("section"));
	$label = &substitute("[[name]] ([[filename]])",$page);
    }
    elsif (/content/) { 
	my $c = new ExSite::Content(id=>$id);
	$url = $c->get_url();
	$c->set_context("parent");
	my $content = $c->get();
	my $section = $c->get("section");
	$content->{name} = $section->{title}.": ".$content->{name};
	$label = &substitute("\"[[name]]\" [[description]]",$content);
    }
    elsif (/article/) { 
	$a = $share{DB}->fetch("article",$id);
	$url = "$config{server}{CGIpath}/$config{prog}{ctrlpanel}/Zine?aid=$id";
	$label = &substitute("\"[[title]]\"",$a);
    }
    return ($label,$url);
}

# generate tree-view of all taggable content

sub list_taggable_content {
    my $this = shift;
    my $db = $share{DB};
    my @keyw_items = $this->get_tags();
    my @slist;
    my @tags;
    my $tagurl;

    foreach my $section ($db->my_real_sites) {
	my @plist;
	my $s = new ExSite::Section(section=>$section);
	foreach my $p ($s->get_all_pages()) {
	    next if (! $p->is_active || $p->is_template());
	    my @clist;
	    foreach my $c ($p->fetch_all_content_objs()) {
		next if ($c->authtype() ne "editorial");
		$c->set_context("child");
		if ($c->is_publishable()) {
		    @tags = $this->get_tags("content",$c->id());
		    my $label = $this->content_icon("content") . " " .
			$c->get_my("name") . $this->show_keywords(@tags);
		    $tagurl = $this->link(tagcmd=>"addtags",
					  section_id=>undef,
					  page_id=>undef,
					  content_id=>$c->id(),
					  article_id=>undef,
					  _bare=>1);
		    push @clist, { label => $label,
				   url => "javascript:popup_small('$tagurl')",
				   open => 0,
			  };
		}
	    }
	    @tags = $this->get_tags("page",$p->id());
	    my $label = $this->content_icon("page") . " " .
		($p->get_my("label") || $p->get_my("title")) . 
		" (".$p->get_my("filename").") " .
		$this->show_keywords(@tags);
	    next if (! $p->is_page() && scalar @clist == 0);
	    $tagurl = $this->link(tagcmd=>"addtags",
				  section_id=>undef,
				  page_id=>$p->id(),
				  content_id=>undef,
				  article_id=>undef,
				  _bare=>1);
	    push @plist, { label => $label,
			   url => $p->is_page() ? 
			       "javascript:popup_small('$tagurl')" :
			       undef,
			   open => 0,
			   sublist => scalar @clist > 0 ? \@clist : undef,
		       };
	}
	@tags = $this->get_tags("section",$s->id());
	my $label = $this->content_icon("section") . " " .
	    $s->get_my("title") . $this->show_keywords(@tags);
	$tagurl = $this->link(tagcmd=>"addtags",
			      section_id=>$section->{section_id},
			      page_id=>undef,
			      content_id=>undef,
			      article_id=>undef,
			      _bare=>1);
	push @slist, { label => $label,
		       open => ($this->{input}{section_id} == $section->{section_id}),
		       url => "javascript:popup_small('$tagurl')",
		       sublist => scalar @plist > 0 ? \@plist : undef,
		   };
    }

    my $out = "<h1>Website Sections</h1>\n";
    $out .= scalar @slist > 0 ?
	&ExSite::HTML::DynList(@slist) :
	"<p>No sections found.</p>\n";

    my @zlist;
    # add zines to section list
    my $z = new Modules::Zine::Article;
    my $admin = $db->is_admin();
    my @z = $z->all_zines();
    foreach my $zine (@z) {
	$z->setup($zine);
	next if (! $admin && ! $z->owns());
	my @alist;
	my $atree = $z->fetch_tree($z->{article_id});
	$z->save();
	foreach my $a ($atree->collapse()) {
	    next if ($a->{status} ne "active");
	    next if ($a->{type} ne "article");
	    $z->set($a);
	    @tags = $this->get_tags("article",$a->{article_id});
	    my $label = $this->content_icon("article") . " " .
		$z->description() . $this->show_keywords(@tags);
	    $tagurl = $this->link(tagcmd=>"addtags",
				  section_id=>undef,
				  page_id=>undef,
				  content_id=>undef,
				  article_id=>$a->{article_id},
				  _bare=>1);
	    push @alist, { label => $label,
			   url => "javascript:popup_small('$tagurl')",
			   open => 0,
		       };
	}

	$z->restore();
	@tags = $this->get_tags("article",$z->{article_id});
	# add zines to section list
	my $label = $this->content_icon("article") . " " .
	    $z->description() . $this->show_keywords(@tags);
	$tagurl = $this->link(tagcmd=>"addtags",
			      section_id=>undef,
			      page_id=>undef,
			      content_id=>undef,
			      article_id=>$zine->{article_id},
			      _bare=>1);
	push @zlist, { label => $label,
		       open => 0,
		       url => "javascript:popup_small('$tagurl')",
		       sublist => scalar @alist > 0 ? \@alist : undef,
		       };
    }
    
    $out .= "<h1>e-Zines</h1>\n";
    $out .= scalar @zlist > 0 ?
	&ExSite::HTML::DynList(@zlist) :
	"<p>No e-zines found.</p>\n";

    return $out;
}

# display the keywords used to tag a particular item

sub show_keywords { 
    my ($this,@tags) = @_;
    return scalar @tags > 0 ?
	" <span class='TagsKeywords'>Tagged as: ".
	join(", ",map { "<em>".$_->{keyword}."</em>" } @tags).
	"</span>\n"
	: undef;
}

# report all keywords in the system

sub list_keyw {
    my $this = shift;
    my $out;

    # list available keywords
    my $url = $this->link();
    # this doesn't give any usage indicators
    #my @keywords = $share{DB}->fetch_all("keyword","keyword");
    # this one doesn't show unused keywords
    #my @keywords = $share{DB}->custom_query("select keyword.*,count(keyword_tag.keyword_tag_id) as count from keyword,keyword_tag where keyword.keyword_id=keyword_tag.keyword_id group by keyword.keyword");
    my @keywords = $share{DB}->custom_query("select keyword.*,count(keyword_tag.keyword_tag_id) as count from keyword left join keyword_tag on keyword.keyword_id=keyword_tag.keyword_id group by keyword.keyword");
    my ($url,@links);
    my $r = new ExSite::ReportBuilder(
				      headers=>["Keyword","Links","Options"],
				      width=>"100%",
				      );
    my $imgpath = "$config{server}{HTMLpath}/_Modules/Tags";
    foreach my $kw (@keywords) {
	my $word = $kw->{keyword};
	my $able_cmd = $kw->{status} eq "active" ? "disable" : "enable";
	my $able_img = $kw->{status} eq "active" ? "minus" : "plus";
	my $eurl = $this->link(kwid=>$kw->{keyword_id},tagcmd=>$able_cmd."keyw");
	my $vurl = $this->link(kwid=>$kw->{keyword_id},tagcmd=>"viewkeyw");
	my $durl = "javascript:confirm_custom(\"Delete this keyword and lose all of its links?\",\"".$this->link(kwid=>$kw->{keyword_id},tagcmd=>"delkeyw")."\")";
	$r->push($word,$kw->{count},"<a href='$vurl'><img src='$imgpath/view12.gif' border=0 height=13 width=12 alt='view' title='view'></a> <a href='$eurl'><img src='$imgpath/${able_img}12.gif' border=0 height=13 width=12 alt='$able_cmd' title='$able_cmd'></a> <a href='$durl'><img src='$imgpath/rm12.gif' border=0 height=13 width=12 alt='delete' title='delete'></a>");
    }
    $out .= &ExSite::HTML::ScrollBox(title=>"Keywords",
				     pane=>$r->make(),
				     width=>"300px",
				     height=>"300px",
				     padding=>"0px",
				     );
    $out .= &ExSite::HTML::Button(label=>"+ new keyword",
				  url=>$this->link(tagcmd=>"newkeyw"));
    $out .= "<p>";
    $out .= &ExSite::HTML::HelpPopup(label=>"click for help",
				     message=>"This tab is used to add, remove,
enable, and disable the available tagging keywords in your system.  
Use the 'Tags' tab to actually tag specific content with these keywords.");
    $out .= "</p>";
    return $out;
}

# generate browseable keyword indexes for a section

sub index {
    my ($this,$section_id,$type) = @_;
    my $db = $share{DB};

    # validate section
    if (! $section_id) {
	return $this->error("Need a section to display a keyword index.");
    }
    my $s = new ExSite::Section(id=>$section_id);

    # can we index articles in this section?
    my $zp = &service_page("Zine",$section_id);
    my $zpid = $zp ? $zp->id() : 0;

    my $index = $this->get_index_data($section_id,$zpid);
    if ($type =~ /alpha/) {
	return $this->index_alpha($index);
    }
    elsif ($type =~ /tree/) {
	return $this->index_treeview($index);
    }
    else { 
	# simple keyword list
	return $this->index_list($index);
    }
}

# compile an index structure, containing all index data for a site

sub get_index_data {
    my ($this,$section_id,$zpid) = @_;
    my $db = $share{DB};

    my $index = {};
    my @keyw_items = $this->get_tags();

    # compile a list of tagged items
    foreach my $tag (@keyw_items) {
	my $item = {};
	if ($tag->{section_id}) {
	    next if ($tag->{section_id} != $section_id);
	    my $s = new ExSite::Section(id=>$tag->{section_id});
	    $item = { title=>$s->get_my("title"),
		      url=>$s->get_url(),
		      descr=>$s->get_my("description"),
		      type=>"Website",
		      date=>undef };
	}
	elsif ($tag->{page_id}) {
	    my $p = new ExSite::Page(id=>$tag->{page_id});
	    $p->set_context("parent");
	    next if ($p->id("section") != $section_id);
	    $item = { title=>$p->get_my("title"),
		      url=>$p->get_url(),
		      descr=>$p->get_my("description"),
		      type=>"Web Page",
		      date=>undef };
	}
	elsif ($tag->{content_id}) {
	    my $c = new ExSite::Content(id=>$tag->{page_id});
	    $c->set_context();
	    next if ($c->id("section") != $section_id);	
	    $item = { title=>$c->get_my("name"),
		      url=>$c->get_url(),
		      descr=>$c->get_my("description"),
		      type=>undef,
		      date=>undef };
	}
	elsif ($zpid && $tag->{article_id}) {
	    my $a = new Modules::Zine::Article($tag->{article_id});
	    my $z = $a->my_zine();
	    next if ($z->{section_id} && $z->{section_id} != $section_id);	
	    my $zpage = new ExSite::Page(id=>$zpid);
	    my $url = $zpage->link(zine=>"show",aid=>$tag->{article_id});
	    $item = { title=>$a->title(),
		      url=>$url,
		      descr=>$a->abstract(),
		      type=>"Article",
		      date=>$a->timestamp() };
	}

	my $link = "<div class=\"TagsContent\"><a href='$item->{url}'>$item->{title}</a> ($item->{type})";
	if ($item->{descr} || $item->{date}) {
	    $link .= "<br>";
	    if ($item->{descr}) {
		$link .= $item->{descr}."  ";
	    }
	    if ($item->{date}) {
		$link .= "<em>(posted $item->{date})</em>";
	    }
	}
	$link .= "</div>";
	if (! exists $index->{$tag->{keyword}}) {
	    $index->{$tag->{keyword}} = [];
	}
	push @{$index->{$tag->{keyword}}}, $link;
    }

    return $index;
}

# simple index format, display a list of keywords

sub index_list {
    my ($this,$index) = @_;

    my ($out,$url);
    if ($this->{input}{keyword}) { 
	$out .= $this->index_keyword($this->{input}{keyword},undef,$index);
	$url = $this->link(keyword=>undef);
	$out .= "<p><a href=\"$url\">&lt; Back to index</a></p>\n";
    }
    else {
	$out .= $this->index_list_keywords($index);
    }
    return $out;
}

sub index_list_keywords {
    my ($this,$index) = @_;
    my ($out,$url);
    my @sorted_tags = sort { uc($a) cmp uc($b) } keys %$index;
    foreach my $tag (@sorted_tags) {
	$url = $this->link(keyword=>$tag);
	my $count = scalar (@{$index->{$tag}});
	$out .= "<div class=\"TagsIndexKeyword\"><a href=\"$url\">$tag</a> ($count)</div>\n";
    }
    return $out;
}

# alphabetized index, broken out by letter

sub index_alpha {
    my ($this,$index) = @_;

    my ($out,$url);
    if ($this->{input}{keyword}) { 
	$out .= $this->index_keyword($this->{input}{keyword},undef,$index);
	$url = $this->link(keyword=>undef);
	$out .= "<p><a href=\"$url\">&lt; Back to index</a></p>\n";
    }
    else {
	my $current_letter = "\0";
	my @used_letters;
	my @sorted_tags = sort { uc($a) cmp uc($b) } keys %$index;
	foreach my $tag (@sorted_tags) {
	    my $letter = uc substr($tag,0,1);
	    if ($letter ne $current_letter) {
		### add links back to top?
		$out .= "<a name='$letter'></a><h2>$letter</h2>\n";
		push @used_letters, "<a href='#$letter'>$letter</a>";
	    }
	    $current_letter = $letter;
	    $url = $this->link(keyword=>$tag);
	    my $count = scalar (@{$index->{$tag}});
	    $out .= "<div class=\"TagsIndexKeyword\"><a href=\"$url\">$tag</a> ($count)</div>\n";
	}
	my $letter_index = "<p class=\"TagsIndexLetter\">\n".
	    join (" - ",@used_letters).
	    "</p>";
	$out = $letter_index.$out;
    }
    return $out;
}

# treeview index

sub index_treeview {
    my ($this,$index) = @_;

    my (%letter,%tags,%links);
    my @sorted_tags = sort { uc($a) cmp uc($b) } keys %$index;
    my ($letter,$last_tag);

    my @index;
    my @letter = qw(A B C D E F G H I J K L M N O P Q R S T U V W X Y Z);
    foreach my $letter (@letter) {
	my ($n,@letterlist);
	foreach my $tag (@sorted_tags) {
	    my $letter1 = uc substr($tag,0,1);
	    next if ($letter1 lt $letter);
	    last if ($letter1 gt $letter);
	    $n = scalar @{$index->{$tag}};
	    my $listitem = {
		text => join("\n",@{$index->{$tag}}),
		label => $n > 0 ? "$tag ($n)" : $tag,
		open => 0,
	    };
	    push @letterlist, $listitem;
	}
	$n = scalar @letterlist;
	push @index, {
	    label=> $n > 0 ? "$letter ($n)" : $letter,
	    open=>0,
	    sublist => @letterlist > 0 ? \@letterlist : undef,
	}
    }

    return &ExSite::HTML::DynList(@index);
}

# sub-index, showing all linked resources under a particular keyword

sub index_keyword {
    my ($this,$keyword,$section_id,$index) = @_;
    my ($out,$items); 
    if (! $index) {
	my $zp = &service_page("Zine",$section_id);
	my $zpid = $zp ? $zp->id() : 0;
	$index = $this->get_index_data($section_id,$zpid);
    }
    $items = $index->{$keyword};
    my $title = ucfirst($keyword);
    $out .= "<h2>$title</h2>\n";
    foreach my $item (@$items) {
	$out .= $item;
    }
    return $out;
}

# display keyword tags for a particular item, linked back to the index

sub my_tags {
    my ($this,$class,$id) = @_;
    my $tags = $this->get_tags($class,$id);
    my @taglinks;
    foreach my $tag (@$tags) {
	my $url = $this->link(keyword=>$tag->{keyword});
	push @taglinks, "<a href=\"$url\">$tag->{keyword}</a>";
    }
    return @taglinks > 0 ?
	"<div class=\"TagsKeywords\">Filed under: ".join(", ",@taglinks)."</div>\n" :
	undef;
}

# get all tags linking to a particular item

sub get_tags {
    my ($this,$class,$id) = @_;
    if (! exists $this->{tags}) {
	my @items = $share{DB}->custom_query("select keyword.keyword,keyword_tag.* from keyword,keyword_tag where keyword.keyword_id=keyword_tag.keyword_id");
	$this->{tags} = \@items;
    }
    my $tags;
    my %class_col = ( section => "section_id",
		      page => "page_id",
		      content => "content_id",
		      article => "article_id" );
    if ($class && $id) {
	$tags = [];
	foreach my $tag (@{$this->{tags}}) {
	    if ($tag->{$class_col{$class}} == $id) { push @$tags, $tag; }
	}
    }
    else {
	$tags = $this->{tags};
    }
    return wantarray ? @$tags : $tags;
}

# set a keyword inactive

sub disable_keyw { 
    my ($this,$id) = @_;
    return if (! $id);
    $share{DB}->update("keyword",{keyword_id=>$id,status=>"inactive"});
}

# set a keyword active

sub enable_keyw { 
    my ($this,$id) = @_;
    return if (! $id);
    $share{DB}->update("keyword",{keyword_id=>$id,status=>"active"});
}

# move a keyword to the trash

sub delete_keyw { 
    my ($this,$id) = @_;
    return if (! $id);
    $share{DB}->trash_r("keyword",$id);
}

# add a new keyword

sub add_keyw {
    my $this = shift;
    if ($this->{input}{keyword}) {
	my $kw = new ExSite::Object(type=>"keyword");
	$kw->do_make();
	&redirect($this->link(tagcmd=>"keyw"));
    }
    else {
	my $out = "<h1>Add new keyword</h1>
<p>A keyword can be any word or phrase up to 255 characters in length.
Inactive keywords will not appear in public keyword indexes.</p>\n";
	my $kw = new ExSite::Object(type=>"keyword");
	$out .= $kw->make(data=>{status=>"active"},extra=>{reply=>$this->link(__plaintext=>1,tagcmd=>"keyw")});
	return $out;
    }
}

# modify an existing keyword

sub edit_keyw {
    my ($this,$id) = @_;
    if ($this->{input}{keyword}) {
	my $kw = new ExSite::Object(type=>"keyword");
	$kw->do_edit();
	&redirect($this->link(tagcmd=>"keyw"));
    }
    else {
	my $out = "<h1>Edit keyword</h1>\n";
	my $kw = new ExSite::Object(type=>"keyword",id=>$id);
	$out .= $kw->edit(extra=>{reply=>$this->link(__plaintext=>1,tagcmd=>"keyw")});
	return $out;
    }
}

# add or remove tags

sub add_tag {
    my ($this) = @_;
    my $in = $this->{input};
    if ($in->{add}) {
	my %data = ( keyword_id => $in->{add} );
	my @cols = qw(section_id page_id content_id article_id);
	foreach my $col (@cols) { 
	    if ($in->{$col}) { $data{$col} = $in->{$col}; }
	}
	$share{DB}->insert("keyword_tag",\%data);
    }
    if ($in->{rm}) {
	$share{DB}->trash_key("keyword_tag",$in->{rm});
    }

    my ($descr,$url,%data,$col,@tags);
    if ($in->{article_id}) {
	$col = "article_id";
	@tags = $this->get_tags("article",$in->{article_id});
	($descr,$url) = $this->describe_content("article",$in->{article_id});
	$data{article_id} = $in->{article_id}; 
    }
    elsif ($in->{content_id}) { 
	$col = "content_id";
	@tags = $this->get_tags("content",$in->{content_id});
	($descr,$url) = $this->describe_content("content",$in->{content_id});
	$data{content_id} = $in->{content_id}; 
    }
    elsif ($in->{page_id}) { 
	$col = "page_id";
	@tags = $this->get_tags("page",$in->{page_id});
	($descr,$url) = $this->describe_content("page",$in->{page_id});
	$data{page_id} = $in->{page_id}; 
    }
    elsif ($in->{section_id}) { 
	$col = "section_id";
	@tags = $this->get_tags("section",$in->{section_id});
	($descr,$url) = $this->describe_content("section",$in->{section_id});
	$data{section_id} = $in->{section_id}; 
    }
    
    my @allkw = $share{DB}->fetch_all("keyword","keyword");
    my $tagre = join("|",map { $_->{keyword} } @tags);
    my @unusedkw;
    foreach my $kw (@allkw) {
	if ($kw->{keyword} !~ /^($tagre)$/) {
	    push @unusedkw,$kw;
	}
    }
    
    my $out = "<h3>Update tags for $descr</h3>\n";
    my $f = new ExSite::FormBuilder();

    my @addopt = map { { value=>$_->{keyword_id}, text=>$_->{keyword} } } @unusedkw;
    my @rmopt = map { { value=>$_->{keyword_tag_id}, text=>$_->{keyword} } } @tags;
    $f->input(prompt=>"Available tags (select to add): ",
	      type=>"select",
	      name=>"add",
	      input=> @addopt > 0 ? undef : "(no tags available)",
	      nullvalue=>"== add ==",
	      options=>\@addopt);
    $f->input(prompt=>"Current tags (select to remove): ",
	      type=>"select",
	      name=>"rm",
	      input=> @rmopt > 0 ? undef : "(no tags)",
	      nullvalue=>"== remove ==",
	      options=>\@rmopt);
    $f->input(type=>"hidden",
	      name=>$col,
	      value=>$in->{$col});
    $f->submit();
    $out .= $f->make;
    return $out;
}

# Search system hook

sub search_tags {
    my ($this,$search,$site,$page) = @_;
    my $uri = $page->get_uri_dynamic();
    
    # can we index articles in this section?
    my $sid = $site->id();
    my $zp = &service_page("Zine",$sid);
    my $zpid = $zp ? $zp->id() : 0;

    # get our index data (we only use this to get the used keywords)
    my $index = $this->get_index_data($sid,$zpid);
    my @tags = keys %$index;

    ### FIXME: should we add just the tag to the search index (as below),
    ### or should go through each tagged item, locate it in the index, and
    ### add the tag to the searchterms, or increase the weight of the tag?

    # add each keyword to the search index
    foreach my $tag (@tags) {
	$uri->parameter("keyword",$tag);
	my $count = scalar @{$index->{$tag}};

	# add this new URL to the index
	# Give it a special title and description.
	my $url = $uri->write();
	$search->add_url($url,
			 $site->id(),
			 $tag,
			 "$count item(s)",
			 "public",
			 );

	# index the tag name with a moderately high weight
	$search->index($url,5,$tag);
    }

    # return our status messages to the search system
    return "Indexed ".scalar @tags." keyword tags...<br>\n";
}

1;

=pod



=head2 CSS Notes

=over 4

=item div.Tags

Wraps all public output.

=item div.TagsContent

Wraps links to tagged content.

=item (span|div).TagsKeywords

Wraps the list of keywords that have been applied to an item.

=item div.TagsIndexKeyword

Wraps a keyword link in an index.

=item p.TagsIndexLetter

Wraps the letter shortcut menu in alphabetical indexes.

=back

=cut
