package Modules::TellAFriend;

# Send an email message about the current page

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Mail;
use ExSite::Form;
use ExSite::Captcha;

use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    my $in = $this->{input};
	
	my %opt = &DecodeString($options);
	
    if ($in->{from} && $in->{to}) {
	# we have some form input
	return $this->send;
    }
    elsif ($in->{tellafriend}) {
	return $this->mailform();
    }
    else {
	my $url = $this->link(tellafriend=>1);
	return "<a href=\"$url\">$config{TellAFriend}{anchor}</a>";
    }
    return;
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/isService/) {
	return 1;
    }
}

sub mailform {
    my ($this) = @_;
    my $formurl = $this->link;
    my $db = $share{DB};
    my ($subject,$message,$from,$to);
    my %data;

    if ($db) {
	# see if we can find an appropriate from address
	if ($share{Page} && ! $share{Page}->is_publishing) {
	    $db->authorize;
	    my %user = $db->my_user_record();
	    $from = $user{email};
	}
    }
    if (!$subject) {
	# set default subject to the website name
	#$subject = "Re: ".$config{site}{name};
	$subject = $config{TellAFriend}{subject};
	$subject =~ s/\"/\&quot\;/;
    }
    $message = $this->get_message();
    $message =~ s/\n/<br>\n/g;
    my $c = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
    my $captcha = $c->make();
    my $input_text_size = $config{TellAFriend}{input_text_size} || 40;
    my $greeting_input = $config{TellAFriend}{editable_greeting} ?
	"<tr><td>Add your own greeting:</td><td><textarea name=\"message\" rows=\"3\" cols=\"50\"></textarea></td></tr>" : "";
    my $subject_input = $config{TellAFriend}{editable_subject} ?
	"<input type=\"text\" size=\"40\" name=\"subject\" value=\"$subject\">" : $subject;

    my $label_from = $config{TellAFriend}{label_from} || "Your&nbsp;email&nbsp;address:";
    my $label_to = $config{TellAFriend}{label_to} || "Your&nbsp;friend's&nbsp;email&nbsp;address:";
    my $label_subject = $config{TellAFriend}{label_subject} || "Subject:";
    return "<div class=\"TellAFriend\">
$config{TellAFriend}{heading}
<form action=\"$formurl\" method=\"POST\">
<table>
<tr><td>${label_from}</td><td><input type=\"text\" name=\"from\" size=\"${input_text_size}\" value=\"$from\"></td></tr>
<tr><td>${label_to}</td><td><input type=\"text\" name=\"to\" size=\"${input_text_size}\" value=\"$to\"></td></tr>
<tr><td>${label_subject}</td><td>$subject_input</td></tr>
$greeting_input
<tr><td>Message:</td><td>
$message
</td></tr>
<tr><td colspan=2>
<div style='border:1px dotted blue; padding:10px;'>
$captcha
</div>
</td></tr>
<tr><td colspan=2><input type=\"submit\" value=\"Send\"></td></tr>
</table>
</form>
</div>\n";
}

sub get_message {
    my ($this,$info) = @_;
    my $message = $this->run_handler("TellAFriendGetMessage");
	return $message if $message;
    if ($config{TellAFriend}{editable_greeting} && $this->{input}{message}) {
	$message = $this->{input}{message}."\n\n";
    }
    $message .= $config{TellAFriend}{message};
    my $sectionid = $share{Page}->id("section");
    require ExSite::Section;
    my $s = new ExSite::Section(id=>$sectionid);
    $message .= "\n\n".$s->get_url();
    return $message;
    my $url;
    if ($ENV{HTTP_REFERER}) {
	$url = $ENV{HTTP_REFERER};
    }
    else {
	$url = $config{server}{server}.$ENV{REQUEST_URI};
    }
    $message .= "\n\n$url\n";
    return $message;
}

sub send {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    my $addr = $share{DB}->owner_email();
    my @err;
    foreach (split /,/,$addr) {
	if (!/^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	    push @err, "$_: invalid email address";
	}
    }
    if ($in->{from} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	push @err, "invalid FROM address";
    }
    if ($in->{to} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	push @err, "invalid TO address";
    }
    if (! $in->{message} && $config{TellAFriend}{editable_message}) {
	push @err, "no message to send";
    }
    if (! $in->{subject} && $config{TellAFriend}{editable_subject}) { 
	push @err, "no subject";
    }
    if (! $in->{captcha}) {
	push @err, "missing captcha";
    }
    else {
	# evaluate the user's response
	my $c = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
	if (! $c->pass()) {
	    push @err, "Sorry, that's not the correct anti-spam answer.";
	}
    }
    if (@err > 0) {
	$out = "<p class=\"error\">".join("<br>\n",@err)."</p>\n";
	$out .= "<p><a href=\"javascript:history.back()\">&lt;&nbsp;Back</a></p>\n";
    }
    else {
	if (require ExSite::Mail) {
	    my $subject = $config{TellAFriend}{editable_subject} ? $in->{subject} : $config{TellAFriend}{subject};
	    my $message = $this->get_message();
	    my $stat = &SendMail($in->{to},$in->{from},$subject,$message);
	    if ($stat) {
		$out .= "<p class=\"error\">Problem sending message: error code $stat.</p>\n";
	    }
	    else {
		$out .= "<h3>Message sent successfully - thank you!</h3>\n";
	    }
	}
	if ($this->{options}{reply}) {
	    $out .= "<p><a href=\"$this->{options}{reply}\">Back</a></p>\n";
	}
    }
    return $out;
}

1;

