#!/usr/bin/perl
#-----------------------------------------------------------------------
# webdb.cgi - general-purpose db admin tool
#
# This script acts as a front end to Modules::WebDB.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Module;
use ExSite::Form;
use ExSite::PageBuf;
use Modules::WebDB;

&exsite_init;
my $db = new ExSite::Form;
my $wdb = new Modules::WebDB;

# undo default WebDB link configuration
$config{form}{mode} = $config{report}{linkmode} = "cgi";

#info
my $name = "WebDB";
my $info = "The Database Manager provides functions for searching, browsing, 
editing, and adding records to any of the tables in your website's database. 
It is an expert-level tool that gives you the ability to \"hack\" your 
website's data.";
my $icon = &ExSite::Module::icon("WebDB");

# build control panel
my $page = new ExSite::PageBuf(title=>"WebDB - $config{site}{name}",
			    header=>0, footer=>0);
$page->printTo(2,"<html>
<head>
<meta http-equiv=\"Content-Type\" content=\"text/html; charset=$config{charset}\">
<title>$name</title>
<link rel=\"stylesheet\" href=\"$config{server}{HTMLpath}/_ExSite/css/ExSite.css\" type=\"text/css\">
<script src=\"$config{server}{HTMLpath}/_ExSite/js/misc.js\" type=\"text/javascript\"></script>
</head>
<body leftmargin=\"0\" rightmargin=\"0\" topmargin=\"0\" class=\"ctrlpanel\" marginwidth=\"0\" marginheight=\"0\">
<div class=\"topbar\">
<table class=\"titlebar\"><tr><td><img width=\"32\" alt=\"WebDB\" src=\"$config{server}{HTMLpath}/_Modules/WebDB/icon.gif\" height=\"32\" border=\"0\"></td>
<td class=\"titlebar\"><span class=\"title\">WebDB</span>
</td>
<td align=\"right\"><a href=\"$config{server}{CGIpath}/webdb.cgi\"><img width=\"30\" alt=\"Home\" src=\"$config{server}{HTMLpath}/_ExSite/images/button-home.gif\" title=\"Home\" height=\"30\" border=\"0\"></a>
</td>
<td align=\"right\"><a href=\"javascript:popup_medium('$config{help_url}')\"><img width=\"30\" alt=\"Help\" src=\"$config{server}{HTMLpath}/_ExSite/images/button-help.gif\" title=\"Help\" height=\"30\" border=\"0\"></a>
</td>
</tr>
</table>
</div>
<div class=\"ctrlpanel\">\n");

$page->printTo(3,"</body></html>");

if ($db->authorize < $config{auth}{root_auth_level} - 1) {
    $page->print("<h3 class=error>Permission denied</h3><p>You do not have sufficient system priveleges to use this feature.</p>\n");
}
else {
    $page->print($wdb->write);
}

$page->show;

&exsite_close();

