package Modules::WebDir;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Input;
use ExSite::Util;
use ExSite::Mail;

use Modules::BaseDCD;
use base ("Modules::BaseDCD");

my $ml = ExSite::Config::get_obj("ML");

#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	$obj->initialize_object();
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "Web Directory";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "The Web Directory module manages and serves a directory of web resources organized in hierarchical topics.";
	}elsif (/Publish/){
		return \&publish;
	}
	return;
}
sub write{
	my ($this,$options) = @_;
	my %opt = &DecodeString($options);
	my $action = $this->{input}{action} || $opt{action};
	$this->section_id($opt{section_id});
	my $out = "";
	if ($action eq "link_report" && $this->{input}{resource_id}){
		$out = $this->link_report($this->{input}{resource_id});
	}elsif ($action eq "process_link_report"){
		$out = $this->process_link_report($this->{input});
	# Does the directory search
	}elsif ($action eq "dir_search" && $this->{input}{search_field}){
		$out = $this->directorySearch($this->{input}{search_field});
	# Add a new link
	}elsif ($action eq "new_link"){
		$out = $this->resourceNew();
	# Process new/edit link data
	}elsif ($action eq "post_link"){
		$out = $this->processPost();
		if ($this->admin_mode()){
			$this->republish();
		}
	# Show web directory
	}else{
		$out = $this->webDirectory();
	}
	return $out;
}
#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	$this->section_id($this->{input}{section_id});
	my $action = $this->{input}{action};
	
	if ($action eq "republish"){
		$this->republish();
		$action = "";
	}
	
	$this->admin_mode(1);

	if ($this->is_owner() && $action =~ /resource_(disable|reactivate|approve)/i && $this->{input}{resource_id}){
		$this->$action($this->{input}{resource_id});
		$this->admin_mode(0);
		$this->redirect($this->link(action=>undef,resource_id=>undef,__plaintext=>1));
		return "done";
	}
	my $out;
	# Edit a resource
	if ($this->is_owner() && $action eq "resource_edit" && $this->{input}{resource_id}){
		$out = $this->resourceEdit($this->{input}{resource_id});
	# Edit a topic
	}elsif ($this->is_owner() && $action eq "topic_edit" && $this->{input}{topic_id}){
		$out = $this->topicEdit($this->{input}{topic_id});
	# Process post using do
	}elsif ($action eq "process_doform"){
		$out = $this->processDoForm();
	# Delete a topic
	}elsif ($this->is_owner() && $action eq "topic_delete" && $this->{input}{topic_id}){
		$out = $this->topicDelete($this->{input}{topic_id});
		$this->republish();
	# Add a new topic
	}elsif ($action eq "topic_new"){
		$out = $this->topicNew();
		$this->republish();
	# Delete a topic
	}elsif ($action eq "topic_delete" && $this->{input}{topic_id}){
		$out = $this->topicDelete($this->{input}{topic_id});
		$this->republish();
	# Delete a resource
	}elsif ($this->is_owner() && $action eq "resource_delete" && $this->{input}{resource_id}){
		$out = $this->resourceDelete($this->{input}{resource_id});
		$this->republish();
	}elsif (!$this->section_id()){
		$out = $this->set_section_id();
	}else{
		$out = $this->write();
	}
	$this->admin_mode(0);
	return $out;
}
#----------------------------------------------------------------------
sub admin_mode{
	my ($this,$mode) = @_;
	if (defined $mode){
		$this->{admin_mode} = $mode || 0;
	}
	return $this->{admin_mode} || 0;
}
sub Captcha{
	my ($this) = @_;
	if (ref($this->{captcha}) !~ /captcha/i){
		require ExSite::Captcha;
		$this->{captcha} = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
	}
	return $this->{captcha};
}
sub gen_captcha{
	my ($this) = @_;
	my $captcha = $this->Captcha()->make();
	if ($this->admin_mode()){
		$captcha = "";
	}
	return $captcha;
}
sub valid_captcha{
	my ($this) = @_;
	if ($this->admin_mode()){
		return 1;
	}
	if ($this->Captcha()->pass()){
		return 1;
	}
	return 0;
}
sub resolve_link{
	my ($this,$in) = @_;
	# $1: path
	# $2: section id
	# $3: action
	# $4: resource id
	$in =~ s/\[\[(.+?)\|(.+?)\|(.+?)\|(.+?)\]\]/$this->link(path=>($1 eq "undef" ? undef : $1),section_id=>($2 eq "undef" ? undef : $2),action=>($3 eq "undef" ? undef : $3),resource_id=>($4 eq "undef" ? undef : $4))/ieg;
	return $in;
}
sub link{
	my ($this,%opt) = @_;
	if ($this->{publishing}){
		my $path = $opt{path} || "undef";
		my $sectionid = $opt{section_id} || "undef";
		my $action = $opt{action} || "undef";
		my $resourceid = $opt{resource_id} || "undef";
		return "[[$path|$sectionid|$action|$resourceid]]";
	}else{
		return $this->SUPER::link(%opt);
	}
	return;
}
# Set/get section id
sub section_id{
	my ($this,$id) = @_;
	if (defined $id){
		$this->{section_id} = $id;
	}
	if (!$this->{section_id}){
		if (exists($share{Page})){
			$this->{section_id} = $share{Page}->id("section");
		}elsif ($this->{input}{section_id}){
			$this->{section_id} = $this->{input}{section_id};
		}
	}
	return $this->{section_id} || 0;
}
# Modified sub do from Form.pm
# - added republish() calls
sub processDoForm{
	my ($this) = @_;
	my $reply;
	my $out;
	my $redirect = 1;
	my $action;
	if (!$this->{db}->{data}) {
		if (! $this->{db}->get_data()) {
			$this->{db}->error("do form: Nothing to do!");
		}
	}
	if (!$action){
		$action = $this->{db}->get_action();
	}
	$this->{db}->{action} = $action;

	# fetch the destination URL

	if (exists $this->{db}->{data}->{reply}){
		$reply = $this->{db}->{data}->{reply};
		delete $this->{db}->{data}->{reply};
		if ($reply =~ /doform\.cgi/){
			# recursion is perilous
			$reply = "";
		}
	}

	# cancel, if requested

	if ($this->{db}->{submit_action} eq "Cancel"){
		# bail
		if ($this->{db}->{handler}->{cancel_form}) {
			my $out = &{$this->{db}->{handler}->{cancel_form}}($this);
			if ($out){
				return $out;
			}
		}
		if ($reply){
			$this->redirect($reply);
			return;
		}else{
			return $ml->h3("Operation cancelled.");
		}
	}

	# do everything that touches the database

	$this->{db}->do_db($action);
	if ($this->{db}->errorcheck){
		# uh-oh!
		$out = $ml->h3("Problem:");
		$out .= $ml->p($this->{db}->show_diagnostics("error","html"));
		$out .= $ml->p($ml->a("Go back to form.",{href=>"javascript:history.back()"}));
	}elsif ($reply){
		# republish the whole web directory
		$this->republish();
		# success; return the user to the original page
		$this->redirect($reply);
	}elsif ($this->{db}->{output}){
		# success; return the status report to the user
		return $this->{output};
	}else{
		# success; generic confirmation message
		$out = $ml->H3("Done!");
	}
	return $out;
}
#----------------------------------------------------------------------
sub getModuleDiskPath{
	my ($this) = @_;
	return "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/WebDir/top";
}
# Get disck path
sub getDiskPath{
	my ($this) = @_;
	my $sectionid = $this->section_id();	
	my $path = $this->getModuleDiskPath();
	if ($sectionid){
		$path .= "/${sectionid}";
	}
	return $path;
}
# Publish web Directory
sub publish{
	my ($this,$opt) = @_;
	my @sections;
	if ($this->{input}{section_id}){
		push(@sections,$this->{input}{section_id});
	}else{
		my @sites = $this->{db}->my_real_sites();
		foreach my $site(@sites){
			push(@sections,$site->{section_id});
		}
	}

	my $out = "";
	if (scalar(@sections) > 0){
		$out .= $this->mkdir($this->getModuleDiskPath());
		foreach my $sid(@sections){
			$this->section_id($sid);
			# Publish mode begins
			$this->{publishing} = 1;
			my $filepath = $this->getDiskPath;
			# Create file directory
			$out .= $this->mkdir($filepath);
			# Write top level web directory
			$out .= $this->writeToDisk($filepath,"index.html",$this->webDirectory());
			my @topics = $this->fetchTopicBySection();
			foreach my $topic(@topics){
				my $directorypath = $filepath.$this->getPath($topic->{webdir_topic_id});
				my $filename = "index.html";
				# Create file directory based on the directory path
				$out .= $this->mkdir($directorypath);
				# Readjust links
				my $content = $this->webDirectory($topic->{webdir_topic_id});
				# Write sub leve web directory
				$out .= $this->writeToDisk($directorypath,$filename,$content);
			}
			# Publish mode ends
			$this->{publishing} = 0;
		}
	}else{
		$out = $ml->p("Nothing to publish!");
	}
	return "" if $opt->{silence};
	return $out;
}

# Background publishing
sub republish{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	require ExSite::Util;
	ExSite::Util::run_publisher("module","WebDir",0,{section_id=>$sectionid,silent=>1});
	return;
}
# Published file check 
# Display the published web directory contents for public only
sub checkPublishFile{
	my ($this,$dirpath) = @_;
	# We are publishing the pages
	return 0 if $this->{publishing};
	# The user is an admin
	return 0 if $this->admin_mode();
	my $dir = $this->getDiskPath.$dirpath;
	my $file = $dir."/index.html";
	# File exists
	return 1 if (-e $file);
	return 0;
}
# Published file check
sub checkPublishTopLevel{
	my ($this) = @_;
	return $this->checkPublishFile();
}
# Fix sublevel file path
sub fixSubLevelPath{
	my ($this,$dirpath) = @_;
	if ($dirpath =~ /\//){
		my @paths = split(/\//,$dirpath);
		$dirpath = $paths[-1];
	}
	return $this->getPath($dirpath);
}
# Published file check
sub checkPublishSubLevel{
	my ($this,$dirpath) = @_;
	$dirpath = $this->fixSubLevelPath($dirpath);
	return $this->checkPublishFile($dirpath);
}
# Get published file contents
sub getFilePublished{
	my ($this,$dirpath) = @_;
	# filename
	my $filename = $this->getDiskPath.$dirpath."/index.html";
	my $out;
	if (open(IFILE,"<$filename")){
		my @lines = <IFILE>;
		close(IFILE);
		# Output file contents
		$out .= join("",@lines);
		$out = $this->resolve_link($out);
	}else{
		$this->{db}->error("WebDir:: Failed to load $filename : $!");
		$out .= $ml->span("Error: $1",{class=>"error"});
	}	
	return $out;
}
# Get published file contents
sub topLevelPublishFile{
	my ($this) = @_;
	return $this->getFilePublished();
}
# Get published file contents
sub subLevelPublishFile{
	my ($this,$dirpath) = @_;
	$dirpath = $this->fixSubLevelPath($dirpath);
	return $this->getFilePublished($dirpath);
}
sub directoryTaintedCheck{
        my ($this,$directory) = @_;
        $directory =~ /^((\/{1}[a-zA-Z0-9_-]+)+)$/;
        my $checked = $1;
        return $checked;
}
sub fileTaintedCheck{
        my ($this,$filename) = @_; 
        $filename =~ /^([a-zA-Z0-9_-]+\.[a-zA-Z0-9_-]+)$/;
        my $checked = $1;
        return $checked;
}

# Create a directory
sub mkdir{
	my ($this,$dirty_directory) = @_;
	# Tainted check
	my $clean_directory = $this->directoryTaintedCheck($dirty_directory);		
	if (!$clean_directory){
		$this->error("mkdir $clean_directory: failed: $dirty_directory - Invalid Directory");
		return $ml->span("FAILED: $dirty_directory - Invalid Directory",{class=>"error"});
	}
	my $directory = $clean_directory;
	# Directory existed.
	return if (-e $directory);
	my $out = "Create directory $directory ...";
	if (mkdir($directory)){
		$out .= "succeed.";
	}else{
		$this->error("mkdir $directory: failed: $!");
		$out .= $ml->span("FAILED: $!",{class=>"error"});
	}
	return $ml->p($out);
}
# Write files to the disk
sub writeToDisk{
	# $file: filename
	# $content: file content
	my ($this,$dirty_directory,$dirty_file,$content) = @_;
	# Tainted check
	my $clean_directory = $this->directoryTaintedCheck($dirty_directory);
	my $clean_filename = $this->fileTaintedCheck($dirty_file);
	
	if (!$clean_directory){
		$this->{db}->error("Catalog::publish: $dirty_directory - Invalid Directory");
		return $ml->span("FAILED: $dirty_directory - Invalid Directory",{class=>"error"});
	}
	if (!$clean_filename){
		$this->{db}->error("Catalog::publish: $dirty_file - Invalid Filename");
		return $ml->span("FAILED: $dirty_file - Invalid Filename",{class=>"error"});
	}
	
	my $file = "${clean_directory}/${clean_filename}";

	my $out = "Writing $file ...";
	if (open(OFILE,">$file")){
		# Write files
		print OFILE $content;
		close(OFILE);
		$out .= "succeed.";
	}else{
		# Error
		$this->{db}->error("WebDir::publish: publish $file failed: $!");
		$out .= $ml->span("FAILED: $!",{class=>"error"});
	}
	return $ml->p($out);
}
# Edit a resource
sub resourceEdit{
	# $resource_id: resource id
	my ($this,$resource_id) = @_;
	# Fetch the resource
	my %res = $this->{db}->fetch("webdir_resource",$resource_id);
	my %input;
	# Gather resource data
	foreach my $key(keys(%res)){
		my $hashkey = "resource_".$key;
		$input{$hashkey} = $res{$key};
	}
	# The resource form uses the merge field 'resource_id'(not 'webdir_resource_id') to indicate the primary key of the record.
	$input{resource_id} = $input{resource_webdir_resource_id};
	delete $input{resource_webdir_resource_id};
	# Get the categories that a resource is attached to
	my @category = $this->{db}->fetch_match("webdir_join",{webdir_resource_id=>$resource_id});
	# Get category infor for the drop down list
	$input{category_list_1} = $this->getCategoryList($category[0]{webdir_topic_id});
	$input{category_list_2} = $this->getCategoryList($category[1]{webdir_topic_id});
	$input{category_list_3} = $this->getCategoryList($category[2]{webdir_topic_id});
	my $form = $this->html_resourceForm();
	# Merge source data into the form
	return $this->merge($form,\%input);
}
# Process post data for adding/editing a resource
sub processPost{
	my ($this) = @_;
	# Post query
	my %input = %{$this->{input}};
	# Category info
	my @category;
	my %category;
	# Store resource data
	my %record;
	# Reply url
	my $reply = $input{reply};
	foreach my $key(keys(%input)){
		# All resource related data are store in the resource_.... keys
		next if $key !~ /^resource/i;
		# Category information
		if ($key =~ /^resource_category_(\d+)/i){
			my $cat_id = $1;
			next if !$input{$key};
			# To prevent the system from inserting duplicate links into the directory 
			# when the user selects duplicate categories
			$category{$input{$key}} = 1;
		# Resource information
		}elsif ($key =~ /resource_(.+)/i){
			my $col = $1;
			# Post data use 'resource_id' to store the record id.
			# The resource table uses 'webdir_resource_id' to store the record id.
			# Adjustment needs to be made for record id, but everthing else should remain the same.
			$col = $col eq "id" ? "webdir_resource_id" : $col;
			$record{$col} = $input{$key};
		}
	}
	my @validation_error;
	# Put the selected categories into an array
	@category = keys(%category);
	if (@category < 1){
		push(@validation_error,"You must select a category for the link.");
	}
	if (!$this->valid_captcha()){
		push(@validation_error,"You have entered an incorrect answer for the image.");
	}
	# The user didn't select a category on the form
	if (scalar(@validation_error) > 0){
		# Error message
		$input{custom_message} = $ml->p("Error: ".$ml->br().join($ml->br(),@validation_error));
		# Gather merge field info
		$input{category_list_1} = $this->getCategoryList($input{resource_category_1});
		$input{category_list_2} = $this->getCategoryList($input{resource_category_2});
		$input{category_list_3} = $this->getCategoryList($input{resource_category_3});
		my $form = $this->html_resourceForm();
		# return the user back to the form
		return $this->merge($form,\%input);
	}
	# record id
	my $res_id;
	# insert a new record
	if (!$record{webdir_resource_id}){
		# For admin, the status is set to 'active'
		# For eveyone else, the status is set to 'suggested
		$record{status} = $this->admin_mode() ? "active" : "suggested";
		# Remove the primary key
		delete $record{webdir_resource_id};
		# Insert the record
		$res_id = $this->{db}->insert("webdir_resource",\%record);
		$record{webdir_resource_id} = $res_id;
		# Send email notification to the administrator
		$this->sendNotification(\%record,\@category);
	# update a record
	}else{
		$res_id = $record{webdir_resource_id};
		# Update the record
		$this->{db}->update("webdir_resource",\%record,{webdir_resource_id=>$res_id});	
		# Find all categories that a resource is linked to and delete them
		my @join_records = $this->{db}->fetch_match("webdir_join",{webdir_resource_id=>$res_id});
		foreach my $join(@join_records){
			$this->delete("webdir_join",$join->{webdir_join_id});
		}
	}
	my $admin_email = $this->adminEmail();
	my $error = "Error: The system failed to add a new link to the web directory.";
	$error .= $ml->br();
	$error .= "Please contact the administrator(";
	$error .= $ml->a($admin_email,{href=>"mailto:$admin_email"});
	$error .= ")";
	$error .= $ml->br();
	$error .= $ml->input(undef,{type=>"button",value=>"&lt; Back",onclick=>"history.back()"});
	# Return an error message if update/insert fails
	return $ml->p($error) if !$res_id;
	# Link a resource to topics
	foreach my $cat_id(@category){
		my %join = (
			webdir_resource_id=>$res_id,
			webdir_topic_id=>$cat_id,
		);
		$this->{db}->insert("webdir_join",\%join);
	}
	# Republish web directory
	$this->republish();
	if ($this->admin_mode()){
		$this->redirect($reply);
		return;
	}else{
		return $ml->p("Thank you for submitting your URL".$ml->br().$ml->a("Click here to continue",{href=>"$reply"}));
	}
	return "";
}
# Get adminstrator's email address
sub adminEmail{
	my ($this) = @_;
	my $stat = $this->run_handler("WebDir_adminEmail");
	return $stat if (defined $stat);
	return $this->{db}->owner_email();
}

sub link_report{
	my ($this,$id) = @_;
	if ($id !~ /\d+/){
		return $ml->p("Access denied!");
	}
	my @options = (
		{value=>"bad_link",label=>"Incorrectly categorized"},
		{value=>"dead_link",label=>"Dead link"},
		{value=>"inappropriate_link",label=>"Inappropriate content"}
	);
	my $inputs;
	foreach my $op(@options){
		my $label = $op->{label};
		$label =~ s/_/ /;
		$label = ucfirst($label);
		my %l_option = (
			type=>"radio",
			name=>"reason",
			value=>$op->{value}
		);
		if ($op->{value} =~ /dead/i){
			$l_option{checked} = undef;
		}
		$inputs .= $ml->input(undef,\%l_option).$label.$ml->br();
	}
	$inputs .= $ml->input(undef,{type=>"hidden",name=>"action",value=>"process_link_report"});
	$inputs .= $ml->input(undef,{type=>"hidden",name=>"resource_id",value=>$id});
	$inputs .= $ml->br();
	$inputs .= $this->gen_captcha();
	$inputs .= $ml->input(undef,{type=>"submit",value=>"Submit"});
	$inputs .= " ";
	$inputs .= $ml->input(undef,{type=>"reset",value=>"Reset"});
	my %link = $this->{db}->fetch("webdir_resource",$id);
	my $link_info = $link{anchor} || "n/a";
	$link_info .= " - ".$link{url};
	$link_info .= $link{description} ? $ml->br().$link{description} : "";
	my $p_url = $this->link(action=>undef,resource_id=>undef);
	return $ml->form($ml->h3("Report a link").$ml->p($link_info).$ml->p("Reason:").$inputs,{method=>"post",action=>$p_url});
}
sub process_link_report{
	my ($this,$in) = @_;
	my @error;
	if (!$this->valid_captcha()){
		push(@error,"You have entered an incorrect answer for the image.");
	}
	my $id = $in->{resource_id};
	if (!$id || $id !~ /\d+/){
		push(@error,"Attempt to report an invalid link.");
	}
	my $reason = $in->{reason};
	if (!$reason || $reason !~ /^(dead|bad|inappropriate)_/i){
		push(@error,"Attempt to report a link without a reason.");
	}
	if (scalar(@error) > 0){
		return $ml->h3("Error").$ml->p(join($ml->br(),@error));
	}
	$reason =~ s/^(.+?)_.*$/$1/;
	my $sub = "resource".ucfirst($reason);
	$this->$sub($id);
	return $ml->p("Your request has been submitted.".$ml->br().$ml->a("click here to continue",{href=>$this->link(action=>undef,resource_id=>undef)}));
}
# Set a resource's status to 'defective-dead'
sub resourceDead{
	# $resource_id: resource id
	my ($this,$resource_id) = @_;
	return $this->setResourceStatus($resource_id,"defective-dead");
}
# Set a resource's status to 'defective-bad-topic'
sub resourceBad{
	# $resource_id: resource id
	my ($this,$resource_id) = @_;
	return $this->setResourceStatus($resource_id,"defective-bad-topic");
}
# Set a resource's status to 'defective-inappropriate'
sub resourceInappropriate{
	# $resource_id: resource id
	my ($this,$resource_id,$reply) = @_;
	return $this->setResourceStatus($resource_id,"defective-inappropriate");
}
# Search function
sub directorySearch{
	# $phrase: key phrase entered on the search field
	my ($this,$phrase) = @_;
	my $sectionid = $this->section_id();
	# escape quotes
	$phrase =~ s/"/\\"/g;
	my $sql = <<END;
select 
	a.*, 
	b.webdir_topic_id as topic_id, 
	c.topic as topic_name,
	c.section_id as section_id
from 
	webdir_resource as a 
	left join webdir_join as b on a.webdir_resource_id = b.webdir_resource_id 
	left join webdir_topic as c on b.webdir_topic_id = c.webdir_topic_id 
where 
	(c.section_id = ? or c.section_id = 0 or c.section_id is null or c.section_id = '')
	and (a.anchor like ? or a.description like ?) 
	and (a.status like "active" or a.status like "defective%"
END
	$sql .= $this->admin_mode() ? " or status like \"disabled\"" : "";
	$sql .= ") 
	order by a.webdir_resource_id";
	my @results = $this->{db}->custom_query($sql,$sectionid,"%$phrase%","%$phrase%");
	my %result;
	# Reorganize the sql query results so that they are listed by the resource id
	foreach my $res(@results){
		# Extra topic info into a hash
		my %topic = (name=>$res->{topic_name},id=>$res->{topic_id});
		# Remove topic info
		delete $res->{topic_name};
		delete $res->{topic_id};
		my $id = $res->{webdir_resource_id};
		# Index the result by resource id
		$result{$id} = $res if !exists($result{$id});
		# Attach topic info
		push(@{$result{$id}{topics}},\%topic);
	}
	my $noresult = $ml->p("Your search - ".$ml->Element("b",$phrase)." - did not match any webpages.");
	# Gather merge field info
	my %data;
	# Search form
	$data{search} = $this->html_searchForm();
	# Search results
	$data{result} = @results > 0 ? $this->searchResult(\%result) : $noresult;
	# Add new link
	$data{new_link} = $this->html_newResource();
	# Admin reports
	if ($this->admin_mode()){
		$data{report} = $this->adminReport();
	}
	return $this->html_resultListTemplate(\%data);
}
# Send an email notification to the administrator whenever a new link is added to the directory
sub sendNotification{
	# $record: resource record(hash reference)
	# $category: category list(array reference)
	my ($this,$record,$category) = @_;
	# Don't send email notifications when:
	# 1. The link status isn't 'suggested'
	# 2. The record doesn't have an id
	return if (!$record->{webdir_resource_id} || $record->{status} !~ /suggested/i);
	# Email notification has been turned off
	return if !$config{$this->{class_name}}{enable_notification};
	# Gather data for the merge fields
	my %data = (		
		resource_description=>$record->{description},
		resource_url=>$this->html_resourceUrl($record),
		resource_link=>$record->{anchor},
	);
	my @categories;
	if (scalar(@$category) > 0){
		# Find category info
		my $sql = "select * from webdir_topic where (webdir_topic_id = ".join(" or webdir_topic_id = ",@$category).")";
		@categories = $this->{db}->custom_query($sql);
	}
	my @category_names;
	foreach my $cat(@categories){
		push(@category_names,$cat->{topic});
	}
	$data{resource_category} = join(", ",@category_names);
	my $template = $this->html_searchResourceTemplate();
	# Gather data for sending the notification email
	my $to = $this->adminEmail();
	my $from = $this->adminEmail();
	my $subject = "A link has been added into the web directory";
	my $body = $this->merge($template,\%data);
	# Strip html code
	$body =~ s/<(\/)?(.+?)>//g;
	if ($to){
		&SendMail($to,$from,$subject,$body);
	}	
	return;
	
}
# Format search Results
sub formatResult{
	# $result: search result(hash reference)
	my ($this,$result) = @_;
	my @newResult;
	foreach my $key(keys(%$result)){
		# Gather merge field info
		my %res = %{$result->{$key}};		
		my @topics;
		foreach my $topic(@{$res{topics}}){
			push(@topics,$ml->a("$topic->{name}",{href=>$this->link(path=>$this->getPath($topic->{id}),action=>undef,search_field=>undef)}));
		}
		$res{resource_category} = join(", ",@topics);
		
		my %res_data = $this->getResourceDisplayData(\%res);
		%res = (%res,%res_data);
		
		my $template = $this->html_searchResourceTemplate();
		# Store formatted results into an array
		push(@newResult,$this->merge($template,\%res));
	}
	return \@newResult;
}
sub getResourceDisplayData{
	my ($this,$res) = @_;
	my %new_res_data = (
		resource_link=>$this->html_resourceLink($res),
		resource_url=>$this->html_resourceUrl($res),
		resource_description=>$res->{description},
		resource_option=>$this->resourceOption($res),
	);
	# Only access level 3 admins get to modify links under section_id 0 topics
	if (!$res->{section_id}){
		if (!$this->{db}->is_admin()){
			$new_res_data{resource_admin_option} = "";
		}
	}else{
		$new_res_data{resource_admin_option} = $this->resourceAdminOptionOther($res);
	}
	return wantarray ? %new_res_data : \%new_res_data;
}

sub resourceOption{
	my ($this,$res) = @_;
	if ($config{WebDir}{disable_link_report}){
		return "";
	}
	my $url = $this->link(action=>"link_report",resource_id=>$res->{webdir_resource_id},path=>$this->{input}{path});
	return $ml->a("Report this link",{href=>$url,class=>"webdir_report"});
}
# Search result
sub searchResult{
	# $result: search result(hash reference)
	my ($this,$result) = @_;
	my @result_data = @{$this->formatResult($result)};
	return join("",@result_data);
}
sub fetchTopicBySection{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	my $sql = <<END;
select * 
from webdir_topic 
where (section_id = ? or section_id is null or section_id = '' or section_id = 0)
order by topic asc
END
	my @recs = $this->{db}->custom_query($sql,$sectionid);
	return wantarray ? @recs : \@recs;
}
# Fetch topics
sub fetchTopic{
	my ($this,$topic) = @_;
	my $sectionid = $this->section_id();
	my $where = !$topic ? "(parent is null or parent = 0 or parent = '')" : "parent = $topic";
	my $sql = <<END;
select * 
from webdir_topic 
where (section_id = ? or section_id is null or section_id = '' or section_id = 0) and $where 
order by topic asc
END
	my @recs = $this->{db}->custom_query($sql,$sectionid);
	return wantarray ? @recs : \@recs;
}
# Fix topic path
sub topicList{
	# $path: path from the query e.g. '/1/2/3/4/5/6'
	my ($this,$path) = @_;
	my @oldlist = split(/\//,$path);
	my @newlist;
	# remove the first element from array
	for(my $i = 1; $i < @oldlist; $i++){
		push(@newlist,$oldlist[$i]);
	}
	return \@newlist;
}
# Display web directory
sub webDirectory{
	# $dirpath: web directory path(option - mainly used for publishing web directories to the disk)
	my ($this,$dirpath) = @_;
	if ($dirpath){
		# make sure that value of $dirpath begins with a '/'.. e.g. '/12323'
		$dirpath = $dirpath !~ /^\// ? "/$dirpath" : $dirpath;
	}
	# Get path info from the query
	my $path = $dirpath || $this->{input}{path};
	# Fix topic path
	my @topiclist = @{$this->topicList($path)};
	# The selected topic locates on the last element of the array
	my $topicid = $topiclist[-1];
	my $out;
	if (!$topicid){
		# Top level listing
		$out = $this->toplevel($topicid);
	}else{
		# Sub level listing
		$out = $this->sublevel($topicid,$path);
	}
	return $out;
}
sub getPath{
	my ($this,$id) = @_;
	my @paths = $this->pathSearch($id);
	my $out;
	foreach my $path(@paths){
		$out .= "/$path->{webdir_topic_id}";
	}
	return $out;
}
# Search for paths
sub pathSearch{
	# $id: topic id
	my ($this,$id) = @_;
	my %rec = $this->{db}->fetch("webdir_topic",$id);
	# Recursively searching for parent topics of a topic
	if ($rec{parent}){
		return ($this->pathSearch($rec{parent}),\%rec);
	}else{
		return (\%rec);
	}
	return;
}
# Show path info 
sub sublevelPath{
	# $listdata: topic list(array reference)
	my ($this,$listdata) = @_;	
	# Selected category
	my $currentdir = $listdata->[-1];
	# Doing recursive search on parent topics if there is only record in the array, @listdata
	# (This happens when the user clicks on the category link from the search results)
	if (@$listdata == 1){
		my @checklist = $this->pathSearch($currentdir->{webdir_topic_id});
		# Reset the topic list
		$listdata = \@checklist;
	}
	# Remove the selected topic.  Parent topics are used to create clickable link topics.
	# The selected topic isn't required in this case.
	my %current_topic = %{pop(@$listdata)};
	# 'Top' refers to the toplevel(root) directory
	my @pathlist = ($ml->a("Top",{href=>$this->link(path=>undef)}));
	# $toppath contains the path info.  It grows after each foreach loop.
	my $toppath;
	foreach my $data(@$listdata){
		# Update path info
		$toppath .= "/$data->{webdir_topic_id}";
		# Create clickable link topics
		my $link = $ml->a("$data->{topic}",{href=>$this->link(path=>$toppath)});
		push(@pathlist,$link);
	}
	# Add the selected topic at the end
	push(@pathlist,$currentdir->{topic});
	my $out = $ml->p(join(" &gt; ",@pathlist));
	$out .= $this->html_directoryTitle({title=>$currentdir->{topic},admin_option=>$this->adminTopicOption(\%current_topic),description=>$currentdir->{description}});
	return $out;
}
# Sub level web directory
sub sublevel{
	# $topicid: selected topic
	# $dirpath: web directory path
	my ($this,$topicid,$dirpath) = @_;
	# Show published files
	if ($this->checkPublishSubLevel($dirpath)){
		return $this->subLevelPublishFile($dirpath);
	}

	my $sql = <<END;
select
	webdir_resource.*,
	webdir_topic.section_id as section_id
from
	webdir_resource
	left join webdir_join on webdir_join.webdir_resource_id = webdir_resource.webdir_resource_id
	left join webdir_topic on webdir_topic.webdir_topic_id = webdir_join.webdir_topic_id
where
	webdir_join.webdir_topic_id = ? and 
	(webdir_resource.status like "active" or webdir_resource.status like "defective%"
END
	if ($this->admin_mode()){
		$sql .= " or webdir_resource.status like \"disabled\"";
	}
	$sql .= ") order by webdir_resource.anchor asc";
	my @resources = $this->{db}->custom_query($sql,$topicid);
	
	
	# Get topic path info
	my @topicList = @{$this->topicList($dirpath)};
	my @topicListData;
	# Fetch topic info
	foreach my $id(@topicList){
		my %rec = $this->{db}->fetch("webdir_topic",$id);
		push(@topicListData,\%rec);
	}
	# Fetch selected topic's child topics
	my @subtopics = $this->fetchTopic($topicid);
	# Gather merge field data
	my %data;
	# Search form
	$data{search} = $this->html_searchForm();
	# Topic path
	$data{path} = $this->sublevelPath(\@topicListData);
	# Child topics
	$data{topic} = $this->html_sublevelDirectory(\@subtopics,$dirpath);
	# Resources
	$data{resource} = $this->html_sublevelResource(\@resources);
	# Suggest a link
	$data{new_link} = $this->html_newResource();
	# Admin reports
	if ($this->admin_mode()){
		$data{report} = $this->adminReport();
	}
	return $this->html_sublevelTemplate(\%data);
}
# Root web directory
sub toplevel{
	my ($this,$topicid) = @_;
	# Show published files
	if ($this->checkPublishTopLevel()){
		return $this->topLevelPublishFile();
	}
	# Fetch all root topics
	my @topics = @{$this->fetchTopic($topicid)};
	my @topicid;
	# Compile a list of root directory id's
	foreach my $topic(@topics){
		push(@topicid,$topic->{webdir_topic_id});
	}
	my @subtopics;
	if (scalar(@topicid) > 0){
		# Fetch all child topics that are linked to the root topics
		my $sql = "select * from webdir_topic where (parent = ".join(" or parent = ",@topicid).")";
		@subtopics = $this->{db}->custom_query($sql);
	}
	my %subtopics;
	# In	dex child topics by parent topoic id
	foreach my $sub(@subtopics){
		$subtopics{$sub->{parent}}{$sub->{topic}} = $sub;
	}
	# Gather merge field data
	my %data;
	# Search form
	$data{search} = $this->html_searchForm();
	# Root web directory
	$data{topic} = $this->html_toplevelDirectory(\@topics,\%subtopics);
	# Heading
	$data{heading} = $config{$this->{class_name}}{toplevel}{heading} || "The web organized by topic into categories.";
	# Admin reports
	if ($this->admin_mode()){
		$data{report} = $this->adminReport();
		$data{new_link} = $this->adminTopLevelNewLink();
	}
	return $this->html_toplevelTemplate(\%data);
}
sub adminTopLevelNewLink{
	my ($this) = @_;
	return $ml->Element("center",$ml->a("Add a new topic",{href=>$this->link(action=>"topic_new")}));
}
# Find the number of descendants on a topic
sub findTopicResource{
	# $topicid: topoic id
	my ($this,$topicid) = @_;
	my %descendants = %{$this->{db}->find_descendants("webdir_topic",$topicid)};
	my @join_ids = exists($descendants{webdir_join}) ? @{$descendants{webdir_join}} : ();
	my $sql = "select * from webdir_join where webdir_join_id = ".join(" or webdir_join_id = ",@join_ids);
	my @joins = $this->{db}->custom_query($sql);
	my @ids;
	foreach my $join(@joins){
		push(@ids,$join->{webdir_resource_id});
	}
	$sql = "select * from webdir_resource where (webdir_resource_id = ".join(" or webdir_resource_id = ",@ids).")";
	$sql .= " and (status like 'active' or status like 'defective%'";
	$sql .= $this->admin_mode() ? " or status like 'disabled'" : "";
	$sql .= ")";
	my $count = @{$this->{db}->custom_query($sql)};
	return $count;
}
# Reorganize the data for outputting html
sub displayDirectory{
	# $topics: list of topics(array reference
	# $column: number of columns in the table
	my ($this,$topics,$column) = @_;
	require POSIX;
	# Figure out the number of rows in the table
	my $row = POSIX::ceil(@$topics/$column);
	# Record position in the array, $topics
	my $rec_pos = 0;
	# Organize the data column by column
	my %coldata;
	# Loop throught each column
	for(my $i = 0; $i < $column; $i++){
		# Use modulus to figure out the number of records per column
		my $looprow = (((@$topics % $column) - 1) >= $i) || (@$topics % $column == 0) ? $row : $row - 1;
		# Populate each column with data
		for(my $j = 0; $j < $looprow; $j++){
			# Column number
			my $hashindex = $j + 1;
			# Merge field
			my $dataindex = "row".($i + 1);
			my $topic = $topics->[$rec_pos];
			# Merge field data
			$coldata{$hashindex}{$dataindex} = $topic->{display_data};
			# Increment the index
			$rec_pos++;
		}		
	}
	# Once we have the data formatted column by column, we need to convert
	# the data from a hash to an array.  We merge data by row not by column.
	# Each array represents a row in the table.
	my @rowdata;
	# Numericly sort the hash keys
	foreach my $hindex(sort {$a <=> $b} keys(%coldata)){
		my %hdata = %{$coldata{$hindex}};
		push(@rowdata,\%hdata);
	}
	return \@rowdata;
}
# Input exsite handler
sub handler_input_exsite{
	my ($this,%opt) = @_;
	if ($opt{table} eq "webdir_topic" && $opt{column} eq "parent" && $config{$share{webdir}->{class_name}}{edit}{categorylist}{treemode}){
		my $select = $opt{value};
		my $record = $opt{record};
		my $out = $ml->select($share{webdir}->getCategoryList($select),{name=>"webdir_topic#$record#parent#key"});
		$opt{input} = $out;
		$this->input_html(%opt);
		return $out;
	}
	return;
}
# Create a new topic
sub topicNew{
	my ($this,$section_id) = @_;
	my $path = $this->{input}{path};
	my $pid = 0;
	if ($path){
		my @paths = split(/\//,$path);
		$pid = $paths[-1];
	}
	$share{webdir} = $this;
	$this->{db}->handler("input_exsite",\&handler_input_exsite);
	my $reply = $this->link(action=>undef,__plaintext=>1);
	my %data = (
		parent=>$pid,
		section_id=>$this->section_id(),
	);
	my @hide;
	if (!$this->{db}->is_admin()){
		@hide = ("section_id");
	}
	return $this->{db}->make(table=>"webdir_topic",data=>\%data,hide=>\@hide,extra=>{reply=>$reply});
}
# Edit a topic
sub topicEdit{
	my ($this,$topic_id) = @_;
	$share{webdir} = $this;
	$this->{db}->handler("input_exsite",\&handler_input_exsite);
	my $reply = $this->link(action=>undef,topic_id=>undef,__plaintext=>1);
	my @hide;
	if (!$this->{db}->is_admin()){
		@hide = ("section_id");
	}
	return $this->{db}->make(table=>"webdir_topic",record=>$topic_id,hide=>\@hide,action=>$this->link(action=>undef,topic_id=>undef),extra=>{action=>"process_doform",reply=>$reply});
}
# Delete a topic
sub topicDelete{
	my ($this,$topic_id) = @_;
	$this->delete("webdir_topic",$topic_id);
	return;
}
# Delete a resource
sub resourceDelete{
	my ($this,$resource_id) = @_;
	$this->delete("webdir_resource",$resource_id);
	return;
}
# Create a new resource
sub resourceNew{
	my ($this) = @_;
	my $reply = $this->link(action=>undef,__plaintext=>1);
	my $form = $this->html_resourceForm();
	# Gather merge field data
	my $newCategoryList = $this->getCategoryList();
	my %data = (
		reply=>$this->link(action=>undef),
		category_list_1=>$newCategoryList,
		category_list_2=>$newCategoryList,
		category_list_3=>$newCategoryList,
	);
	return $this->merge($form,\%data);
}

# Set a resource's status to 'disabled'
sub resource_disable{
	# $resource_id: resource record id
	my ($this,$resource_id) = @_;
	return $this->setResourceStatus($resource_id,"disabled");
}
# Set a resource's status to 'active'
sub resource_reactivate{
	# $resource_id: resource record id
	my ($this,$resource_id) = @_;
	return $this->setResourceStatus($resource_id,"active");
}
# Set a resource's status to 'active'
sub resource_approve{
	# $resource_id: resource record id
	my ($this,$resource_id) = @_;
	return $this->resource_reactivate($resource_id);
}
# Set a resource's status
sub setResourceStatus{
	# $resource_id: resource record id
	# $status: status
	# $reply: reply url
	my ($this,$resource_id,$status) = @_;
	# No resource is provided
	return if !$resource_id;
	$this->{db}->update("webdir_resource",{status=>$status},{webdir_resource_id=>$resource_id});

	# Don't republish the web directory is the status is set to suggested or defective
	if ($status !~ /suggested|defect/i){
		$this->republish();
	}
	return;
}
# Report option
sub reportOptions{
	# $options: report options(array reference)
	my ($this,$options) = @_;
	# Output html for report options
	my $out;
	foreach my $op (@$options){
		$out .= $ml->a($this->getIcon($op->{id},$op->{label}),{href=>"$op->{url}"});
	}
	return $out;
}
# Fetch directory resources by section_id if one is found
sub fetchTopicResource{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	my @recs;
	if ($sectionid){
		my $sql = <<END;
select
	webdir_resource.*
from
	webdir_resource
	left join webdir_join on webdir_join.webdir_resource_id = webdir_resource.webdir_resource_id
	left join webdir_topic on webdir_topic.webdir_topic_id = webdir_join.webdir_topic_id
where
	webdir_topic.section_id = ?
END
		if ($this->admin_mode()){
			if ($this->{db}->is_admin()){
				$sql .= " or webdir_topic.section_id = 0 or webdir_topic.section_id = '' or webdir_topic.section_id is null";
			}
		}
		@recs = $this->{db}->custom_query($sql,$sectionid);
	}else{
		@recs = $this->{db}->fetch_all("webdir_resource");
	}
	return wantarray ? @recs : \@recs;
}
# Admin Reports
sub adminReport{
	my ($this) = @_;
	if (!$this->is_owner()){
		return;
	}
	# Fetch all resources
	my @resources = $this->fetchTopicResource();
	# Resource report headers
	my @link_header = ("Options","URL","Anchor Text","Description");
	# Defective resources
	my @defect;
	# Suggested resources
	my @suggest;
	# Group resources by status
	foreach my $resource(@resources){
		my $url = $this->{db}->show_data("webdir_resource","url",$resource->{url},$resource->{webdir_resource_id});
		# Defective resources
		if ($resource->{status} =~ /^defect/i){
			# report options	
			my $ops = $this->resourceAdminOptionDefect($resource);
			my $status = $resource->{status};
			$status =~ s/defective-//gi;
			push(@defect,[$ops,$url,$resource->{anchor},$resource->{description},$status]);
		# Suggested resources
		}elsif ($resource->{status} =~ /^suggest/i){
			# report options
			my $ops = $this->resourceAdminOptionSuggest($resource);
			push(@suggest,[$ops,$url,$resource->{anchor},$resource->{description}]);		
		}
	}
	# Create reports
	my $suggest = $ml->p($this->ReportBuilder({title=>"Suggested Resources",headers=>\@link_header,data=>\@suggest}));
	push(@link_header,"Status");
	my $defect = $ml->p($this->ReportBuilder({title=>"Defective Resources",headers=>\@link_header,data=>\@defect}));
	my $out = $this->javascript();
	$out .= $suggest.$defect;
	return $out;	
}
# Admin topic options
sub adminTopicOption{
	my ($this,$topic) = @_;
	# Report options
	my @ops = (
		{id=>"new",url=>$this->link(action=>"topic_new",resource_id=>undef),label=>"New Topic"},
		{id=>"edit",url=>$this->link(action=>"topic_edit",topic_id=>$topic->{webdir_topic_id},resource_id=>undef),label=>"Edit '$topic->{topic}'"},
	);
	# Can this directory be deleted?
	my $removable = $this->is_webdir_removable($topic->{webdir_topic_id});
	if ($removable){
		push(@ops,{id=>"delete",url=>"javascript:deleteTopic($topic->{webdir_topic_id})",label=>"Delete '$topic->{topic}'"});
	}
	# Only access level 3 users get to update global directories
	if (!$this->{db}->is_admin() && !$topic->{section_id}){
		@ops = ();
	}
	return $this->reportOptions(\@ops);
}
# Find child topics and resources
sub findChildTopicResource{
	# $topic_id: topic id
	my ($this,$topic_id) = @_;
	# Find all descendants of a tpoic
	my %descendants = %{$this->{db}->find_descendants("webdir_topic",$topic_id)};
	# Child topics
	my $topic = scalar(@{$descendants{webdir_topic}}) == 1 ?  0 : scalar(@{$descendants{webdir_topic}}) - 1;
	# Child resources
	my $resource = exists($descendants{webdir_join}) ? scalar(@{$descendants{webdir_join}}) : 0;
	return ($topic,$resource);
}
# Resource admin options(active/disabled resources)
sub resourceAdminOptionOther{
	# $resource: hash reference
	my ($this,$resource) = @_;
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"resource_edit",resource_id=>$resource->{webdir_resource_id}),label=>"Edit '$resource->{anchor}'"},
		{id=>"delete",url=>"javascript:deleteResource($resource->{webdir_resource_id})",label=>"Delete '$resource->{anchor}'"},
	);
	if ($resource->{status} =~ /^disabled/i){
		push(@ops,{id=>"reactivate",url=>$this->link(action=>"resource_reactivate",resource_id=>$resource->{webdir_resource_id}),label=>"Reactivate '$resource->{anchor}'"});
	}else{
		push(@ops,{id=>"disable",url=>$this->link(action=>"resource_disable",resource_id=>$resource->{webdir_resource_id}),label=>"Disable '$resource->{anchor}'"});
	}
	return $this->reportOptions(\@ops);
}
# Resource admin options(defective resources)
sub resourceAdminOptionDefect{
	# $resource: hash reference
	my ($this,$resource) = @_;
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"resource_edit",resource_id=>$resource->{webdir_resource_id}),label=>"Edit '$resource->{anchor}'"},
		{id=>"disable",url=>$this->link(action=>"resource_disable",resource_id=>$resource->{webdir_resource_id}),label=>"Disable '$resource->{anchor}'"},
		{id=>"reactivate",url=>$this->link(action=>"resource_reactivate",resource_id=>$resource->{webdir_resource_id}),label=>"Reactivate '$resource->{anchor}'"},
		{id=>"delete",url=>"javascript:deleteResource($resource->{webdir_resource_id})",label=>"Delete '$resource->{anchor}'"},
	);
	return $this->reportOptions(\@ops);
}
# Resource admin options(suggested resources)
sub resourceAdminOptionSuggest{
	# $resource: hash reference
	my ($this,$resource) = @_;
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"resource_edit",resource_id=>$resource->{webdir_resource_id}),label=>"Edit '$resource->{anchor}'"},
		{id=>"approve",url=>$this->link(action=>"resource_approve",resource_id=>$resource->{webdir_resource_id}),label=>"Approve '$resource->{anchor}'"},
		{id=>"delete",url=>"javascript:deleteResource($resource->{webdir_resource_id})",label=>"Delete '$resource->{anchor}'"},
	);
	return $this->reportOptions(\@ops);
}
# Get icons
sub getIcon{
	my ($this,$id,$label) = @_;
	my %icon = (
		edit=>{label=>"Edit",image=>"edit12.gif"},
		approve=>{label=>"Approve",image=>"approve12.gif"},
		disable=>{label=>"Disable",image=>"disable12.gif"},
		reactivate=>{label=>"Reactivate",image=>"approve12.gif"},
		delete=>{label=>"Delete",image=>"rm12.gif"},
		new=>{label=>"New",image=>"new12.gif"},
	);
	$label = $label || $icon{$id}{label};
	my %data = (
		image_url=>"$config{server}{server}$config{server}{HTMLpath}/_Modules/$this->{class_name}/$icon{$id}{image}",
		image_label=>$label,
		image_border=>0,
		image_hspace=>2,
		image_align=>"middle",
	);
	return $this->html_icon(\%data);
}
#------------------------------------------------------------------------------------------------
# Icon template
sub html_icon{
	my ($this,$data) = @_;
	my $template = $ml->img(undef,{src=>"<!--[[image_url]]-->",border=>"<!--[[image_border]]-->" ,hspace=>"<!--[[image_hspace]]-->",align=>"<!--[[image_align]]-->",title=>"<!--[[image_label]]-->", alt=>"<!--[[image_label]]-->"});
	return $this->merge($template,$data);
}
# Resource link
sub html_resourceLink{
	my ($this,$rec) = @_;
	my $url = $this->html_resourceUrl($rec);
	my %a_option = (
		href=>"$url",
	);
	if ($config{$this->{class_name}}{show_link_openwindow}){
		$a_option{target} = "_blank";
	}
	return $ml->a($rec->{anchor},\%a_option);
}
# Resource url
sub html_resourceUrl{
	my ($this,$rec) = @_;
	my $out = $rec->{url};
	$out = $out =~ /^http:\/\// ? $out : "http://$out";
	return $out;
}
# Template for showing a single resource
sub html_sublevelResourceTemplate{
	my ($this) = @_;
	return $this->html_sublevelResourceTemplateAdmin() if $this->admin_mode();
	return $this->html_sublevelResourceTemplatePublic();
}
# Template for showing a single resource - admin
sub html_sublevelResourceTemplateAdmin{
	my ($this) = @_;
	return $ml->span("<!--[[resource_link]]-->",{class=>"WebDirName"})." - ".$ml->span("<!--[[resource_url]]-->",{class=>"WebDirUrl"})." <!--[[resource_admin_option]]-->".$ml->br().$ml->span("<!--[[resource_description]]-->",{class=>"WebDirDescription"}).$ml->br().$ml->span("<!--[[resource_option]]-->",{class=>"WebDirNotify"});
}
# Template for showing a single resource - public
sub html_sublevelResourceTemplatePublic{
	my ($this) = @_;
	return $ml->span("<!--[[resource_link]]-->",{class=>"WebDirName"})." - ".$ml->span("<!--[[resource_url]]-->",{class=>"WebDirUrl"}).$ml->br().$ml->span("<!--[[resource_description]]-->",{class=>"WebDirDescription"}).$ml->br().$ml->span("<!--[[resource_option]]-->",{class=>"WebDirNotify"});
}
# Template for showing a single resource - public
sub html_searchResourceTemplatePublic{
	my ($this) = @_;
	return $ml->p($ml->span("<!--[[resource_link]]-->",{class=>"WebDirName"}).$ml->br()."Category: <!--[[resource_category]]-->".$ml->br().$ml->span("<!--[[resource_description]]-->",{class=>"WebDirDescription"}).$ml->br().$ml->span("<!--[[resource_url]]-->",{class=>"WebDirUrl"}).$ml->br().$ml->span("<!--[[resource_option]]-->",{class=>"WebDirNotify"}));
}
# Template for showing a single resource - admin
sub html_searchResourceTemplateAdmin{
	my ($this) = @_;
	return $ml->p($ml->span("<!--[[resource_link]]-->",{class=>"WebDirName"})." <!--[[resource_admin_option]]-->".$ml->br()."Category: <!--[[resource_category]]-->".$ml->br().$ml->span("<!--[[resource_description]]-->",{class=>"WebDirDescription"}).$ml->br().$ml->span("<!--[[resource_url]]-->",{class=>"WebDirUrl"}).$ml->br().$ml->span("<!--[[resource_option]]-->",{class=>"WebDirNotify"}));
}
# Template for showing a single resource
sub html_searchResourceTemplate{
	my ($this) = @_;
	return $this->html_searchResourceTemplateAdmin() if $this->admin_mode();
	return $this->html_searchResourceTemplatePublic();
}
# Template for showing a list of results
sub html_resultListTemplate{
	my ($this,$data) = @_;
	my $template = $this->admin_mode() ? $this->html_resultListTemplateAdmin() : $this->html_resultListTemplatePublic();
	return $this->merge($template,$data);
}
# 'Suggest a link' link
sub html_newResource{
	my ($this) = @_;
	if ($config{WebDir}{disable_link_suggestion}){
		return "";
	}
	return $ml->p($ml->Element("center",$ml->a("Missing link? Suggest a link.",{href=>$this->link(action=>"new_link")})));
}
sub html_directoryTitleAdmin{
	my ($this) = @_;
	return $ml->h1("<!--[[title]]--> <!--[[admin_option]]-->").$ml->p("<!--[[description]]-->");
}
sub html_directoryTitlePublic{
	my ($this) = @_;
	return $ml->h1("<!--[[title]]-->").$ml->p("<!--[[description]]-->");
}
# Directory title
sub html_directoryTitle{
	my ($this,$data) = @_;
	my $template = $this->admin_mode() ? $this->html_directoryTitleAdmin() : $this->html_directoryTitlePublic();
	return $this->merge($template,$data);
}
# List resources
sub html_sublevelResource{
	# $resources: list of resources(array reference)
	my ($this,$resources) = @_;
	my $out = $ml->h3("Web pages");
	# No resources found
	return $out.$ml->p("No web pages found.") if @$resources < 1;
	foreach my $res(@$resources){
		# Gather data for merge fields
		my %data = $this->getResourceDisplayData($res);
		my $template = $this->html_sublevelResourceTemplate();
		# Populate the output html
		$out .= $ml->p($this->merge($template,\%data));
	}
	return $out;
}

# Template for root web directory
sub html_toplevelTemplate{
	my ($this,$data,$opt) = @_;
	my $template = $this->admin_mode() ? $this->html_toplevelTemplateAdmin() : $this->html_toplevelTemplatePublic();
	return $this->merge($template,$data);
}

# Template for root web directory
sub html_sublevelTemplate{
	my ($this,$data,$opt) = @_;
	my $template = $this->admin_mode() ? $this->html_sublevelTemplateAdmin() : $this->html_sublevelTemplatePublic();
	return $this->merge($template,$data);
}
# Web directory search form
sub html_searchForm{
	my ($this) = @_;
	my $form_out = $ml->input(undef,{type=>"hidden",name=>"action",value=>"dir_search"});
	$form_out .= $ml->table($ml->tr($ml->td($ml->input(undef,{type=>"text",size=>30,name=>"search_field",value=>""})).$ml->td($ml->input(undef,{type=>"submit",value=>" Search "}))));
	return $ml->Element("center",$ml->form($form_out,{method=>"post",action=>$this->link(action=>undef,path=>undef)}));
}
# Template for table rows
sub html_sublevelRowTemplate{
	my ($this,$cols) = @_;
	$cols = !$cols ? 3 : $cols;
	my $out;
	for(my $i = 0; $i < $cols; $i++){
		my $merge_key = "row".($i + 1);
		$out .= $ml->td("<!--[[${merge_key}]]-->",{valign=>"top"});
	}
	return $out;
}
# Root webdirectory topic link - public
sub html_directoryCellTitleLinkPublic{
	my ($this) = @_;
	return $ml->a("<!--[[topic]]-->",{href=>"<!--[[url]]-->"});
}
# Root webdirectory topic link - admin
sub html_directoryCellTitleLinkAdmin{
	my ($this) = @_;
	return $ml->a("<!--[[topic]]-->",{href=>"<!--[[url]]-->"})." <!--[[admin_option]]-->";
}
# Root webdirectory topic link
sub html_directoryCellTitleLink{
	my ($this,$data) = @_;
	my $template = $this->admin_mode() ? $this->html_directoryCellTitleLinkAdmin() : $this->html_directoryCellTitleLinkPublic();
	return $this->merge($template,$data);
}
# Create clickable link topic
sub html_sublevelCell{
	my ($this,$topic,$dirpath) = @_;
	my $path = "$dirpath/$topic->{webdir_topic_id}";
	my $out = $this->html_directoryCellTitleLink({url=>$this->link(path=>"/$topic->{webdir_topic_id}"),topic=>$topic->{topic},admin_option=>$this->adminTopicOption($topic)});
	return $out if !$config{$this->{class_name}}{show_resource_count};
	my $resources = $this->findTopicResource($topic->{webdir_topic_id});
	$out .= " ($resources)";
	return $out;
}
# Show the child topics
sub html_sublevelDirectory{
	# $topics: list of topics(array reference)
	my ($this,$topics,$dirpath) = @_;
	# No topics found
	return if @$topics < 1;
	# Create clickable link for every topics
	for(my $i = 0; $i < @$topics; $i++){
		$topics->[$i]{display_data} = $this->html_sublevelCell($topics->[$i],$dirpath);
	}
	# The number of columns the table is going to show
	my $column = $config{$this->{class_name}}{sublevel}{display_column} || 3;
	# Convert topic data into merge field data
	my @rowdata = @{$this->displayDirectory($topics,$column)};
	# Get template for table rows
	my $rowtemplate = $this->html_sublevelRowTemplate($column);
	# Output the page
	my $out;
	foreach my $rowdat(@rowdata){
		$out .= $ml->tr($this->merge($rowtemplate,$rowdat));
	}
	return $ml->h3("Categories").$ml->table($out,{width=>"100%",class=>"WebDirCategories"});
}
# Show the root topics
sub html_toplevelDirectory{
	# $topics: list of root topics(array reference)
	# $subtopics: list of child topics(hash reference)
	my ($this,$topics,$subtopics) = @_;
	# No root topics found
	return $ml->p("No categories found") if @$topics < 1;
	# The number of child topics to be shown under a root topic
	my $subtopicPerCell = $config{$this->{class_name}}{toplevel}{childtopic_display} || 2;
	# Create clickable link topics
	for(my $i = 0; $i < @$topics; $i++){
		$topics->[$i]{display_data} = $this->html_toplevelCell($subtopicPerCell,$topics->[$i],$subtopics->{$topics->[$i]{webdir_topic_id}});
	}
	# The number of columns the table is going to show
	my $column = $config{$this->{class_name}}{toplevel}{display_column} || 3;
	# Convert topic data into merge field data
	my @rowdata = @{$this->displayDirectory($topics,$column)};
	# Get template for table rows
	my $rowtemplate = $this->html_toplevelRowTemplate($column);
	# Output the page
	my $out;
	foreach my $rowdat(@rowdata){
		$out .= $ml->tr($this->merge($rowtemplate,$rowdat));
	}
	return $ml->table($out,{class=>"WebDirectory",cellspacing=>10,cellpadding=>5});
}
# Display for each cell in the root web directory page
sub html_toplevelCell{
	# $limit: number of child topics to show
	# $topic: current topic(hash reference)
	# $subtopics: list of child topics(hash reference)
	my ($this,$limit,$topic,$subtopics) = @_;
	# Root topic clickable link
	my $topiclink = $this->html_directoryCellTitleLink({url=>$this->link(path=>"/$topic->{webdir_topic_id}"),topic=>$topic->{topic},admin_option=>$this->adminTopicOption($topic)});
	# Convert child topics from hashes to arrays
	my @stopics = sort keys(%$subtopics);
	# Store child topic html code in an array
	my @sub_topics;
	for(my $i = 0; $i < $limit; $i++){
		# Exit the loop when the number of child records is less than the $limit
		last if (@stopics < ($i + 1));
		# The child topic that we are walking on
		my $currtopic = $subtopics->{$stopics[$i]};
		# Create clickable link
		my $subtopiclink = $ml->a($currtopic->{topic},{href=>$this->link(path=>"/$topic->{webdir_topic_id}/$currtopic->{webdir_topic_id}")});
		push(@sub_topics,$subtopiclink);
	}
	# Output the page
	my $out = $ml->Element("b",$topiclink);
	$out .= $ml->br() if keys(%$subtopics) > 0;
	if (@sub_topics > 0){
		$out .= join(", ",@sub_topics)."...";
	}
	return $out;
}
# Template for table rows(root web directory)
sub html_toplevelRowTemplate{
	my ($this,$cols) = @_;
	$cols = !$cols ? 3 : $cols;
	my $out;
	for(my $i = 0; $i < $cols; $i++){
		my $merge_key = "row".($i + 1);
		$out .= $ml->td("<!--[[${merge_key}]]-->",{valign=>"top"});
	}
	return $out;
}
#----------------------------------------------------------------------
# Admin resource style
sub html_adminResourceStyle{
	my ($this) = @_;
	my $out = <<END;
.WebDirUrl{
	color: #6f6f6f;
	font-size: 12px;
}
.WebDirDescription{
	font-size: 12px;
}
.WebDirNotifySelect{
	font-size: 8px;
	color: #6f6f6f;
}
END
	return $ml->style($out);
}
# Template for showing a list of results - public
sub html_resultListTemplatePublic{
	my ($this) = @_;
	return $ml->div("<!--[[search]]-->".$ml->h3("Directory").$ml->table($ml->tr($ml->td("<!--[[result]]-->")))."<!--[[new_link]]-->",{class=>"WebDir"});;
}
# Template for showing a list of results - admin
sub html_resultListTemplateAdmin{
	my ($this) = @_;
	my $out = $this->html_adminResourceStyle();
	$out .= $ml->div("<!--[[report]]-->".$ml->hr()."<!--[[search]]-->".$ml->h3("Directory").$ml->table($ml->tr($ml->td("<!--[[result]]-->")))."<!--[[new_link]]-->",{class=>"WebDir"});
	return $out;
}
# Template for root web directory - admin
sub html_toplevelTemplateAdmin{
	my ($this) = @_;
	return $ml->div("<!--[[report]]-->".$ml->hr()."<!--[[search]]-->".$ml->h3("<!--[[heading]]-->",{align=>"center"}).$ml->hr().$ml->Element("center","<!--[[topic]]-->").$ml->hr()."<!--[[new_link]]-->",{class=>"WebDir"});
}
# Template for root web directory - public
sub html_toplevelTemplatePublic{
	my ($this) = @_;
	return $ml->div("<!--[[search]]-->".$ml->h3("<!--[[heading]]-->",{align=>"center"}).$ml->hr().$ml->Element("center","<!--[[topic]]-->").$ml->hr(),{class=>"WebDir"});
}
# Sublevel web directory page - public
sub html_sublevelTemplateAdmin{
	my ($this) = @_;
	my $out = $this->html_adminResourceStyle();
	my $html_css = <<END;
.WebDirUrl{
	color: #6f6f6f;
	font-size: 12px;
}
.WebDirDescription{
	font-size: 12px;
}
.WebDirNotifySelect{
	font-size: 8px;
	color: #6f6f6f;
}
END
	$out .= $ml->style($html_css);
	$out .= $ml->div("<!--[[report]]-->".$ml->hr()."<!--[[search]]--><!--[[path]]--><!--[[topic]]--><!--[[resource]]--><!--[[new_link]]-->",{class=>"WebDir"});
	return $out;
}
# Sublevel web directory page - public
sub html_sublevelTemplatePublic{
	my ($this) = @_;
	return $ml->div("<!--[[search]]--><!--[[path]]--><!--[[topic]]--><!--[[resource]]--><!--[[new_link]]-->",{class=>"WebDir"});
}
# Form for adding/editing a resource
sub html_resourceForm{
	my ($this) = @_;
	my $captcha = $this->gen_captcha();
	my $html_form;
	foreach my $hd({name=>"reply",value=>"<!--[[reply]]-->"},{name=>"action",value=>"post_link"},{name=>"resource_id",value=>"<!--[[resource_id]]-->"}){
		$html_form .= $ml->input(undef,{type=>"hidden",name=>"$hd->{name}",value=>"$hd->{value}"});
	}
	foreach my $cat({label=>"Category 1",name=>"resource_category_1",content=>"<!--[[category_list_1]]-->"},{label=>"Category 2",name=>"resource_category_2",content=>"<!--[[category_list_2]]-->"},{label=>"Category 3",name=>"resource_category_3",content=>"<!--[[category_list_3]]-->"}){
		$html_form .= $ml->p($ml->span("$cat->{label}",{class=>"label"}).$ml->br().$ml->select("$cat->{content}",{name=>"$cat->{name}"}));
	}
	$html_form .= $ml->p($ml->span("URL",{class=>"label"}).$ml->br().$ml->input(undef,{type=>"text",size=>80,name=>"resource_url",value=>"<!--[[resource_url]]-->"}));
	$html_form .= $ml->p($ml->span("Anchor Text",{class=>"label"}).$ml->br().$ml->input(undef,{type=>"text",size=>80,name=>"resource_anchor",value=>"<!--[[resource_anchor]]-->"}));
	$html_form .= $ml->p($ml->span("Description",{class=>"label"}).$ml->br().$ml->textarea("<!--[[resource_description]]-->",{name=>"resource_description",wrap=>"virtual",rows=>4,cols=>80}));
	$html_form .= $ml->p($captcha);
	$html_form .= $ml->p($ml->input(undef,{type=>"submit",value=>"Submit"})."&nbsp;&nbsp;".$ml->input(undef,{type=>"reset",value=>"Reset"}));
	return $ml->div($ml->h2("Suggest a link")."<!--[[custom_message]]-->".$ml->form($html_form,{method=>"post",action=>$this->link(action=>undef)}),{class=>"WebDir"});
}
#--------------------------------------------------------------------
# Build data tree
sub buildDataTree{
	# $rec: record
	# $data: a list containing records indexed by their parent record
	# $primary_key: name of the primary key column
	my ($this,$rec,$data,$primary_key) = @_;
	# Null record
	return undef if !exists($rec->{$primary_key});
	my @data;
	# Look for descendant records
	if (exists($data->{$rec->{$primary_key}})){
		@data = @{$data->{$rec->{$primary_key}}};
	}
	# Build sub tree
	foreach my $dat(@data){
		push(@{$rec->{child_record}},$this->buildDataTree($dat,$data,$primary_key));
	}
	return $rec;
}
# Get all categories in the drop down list format - tree
sub html_getCategoryList_tree{
	# $label_col: label column
	# $data: list of data
	# $primary_key: name of the primary key
	# $foreign_key: name of the parent key
	# $select: selected value 
	my ($this,$data,$label_col,$primary_key,$foreign_key,$select) = @_;
	my %data_index;
	# Index records by its foreign key
	foreach my $dat(@$data){
		my $index = $dat->{$foreign_key} || 0;
		push(@{$data_index{$index}},$dat);
	}
	my %result = %{$this->buildDataTree({$primary_key=>0,$label_col=>"== select =="},\%data_index,$primary_key)};
	return $this->html_displayTreeStyleList(\%result,$primary_key,$label_col,$select);
}
sub html_displayTreeStyleList{
	# $data: data to be shown(hash reference)
	# $primary_key: name of the primary key column
	# $label_col: name of the label column
	# $select: selected value
	# $sep: a string used to show different tree levels
	my ($this,$data,$primary_key,$label_col,$select,$sep) = @_;
	my %t_option = (
		value=>"$data->{$primary_key}",
	);
	if ($select == $data->{$primary_key}){
		$t_option{selected} = undef;
	}
	my $out = $ml->option("$sep$data->{$label_col}",\%t_option);
	# Display descendant records recursively
	foreach my $dat(@{$data->{child_record}}){
		my $separator .= $sep.$config{$this->{class_name}}{edit}{categorylist}{treemode_separator};
		$out .= $this->html_displayTreeStyleList($dat,$primary_key,$label_col,$select,$separator);
	}
	return $out;
}
# Get all categories in the drop down list format - list
sub html_getCategoryList_list{
	# $data: list of records
	# $select: selected value
	my ($this,$data,$select) = @_;
	my @options;
	push(@options,$ml->option("== Select == ",{value=>""}));
	# Aphabetically sorted list
	foreach my $cat (@$data){
		my $id = $cat->{webdir_topic_id};
		my %link_option = (
			value=>$id,
		);
		if($select == $id){
			$link_option{selected} = undef;
		}
		push(@options,$ml->option($cat->{topic},\%link_option));
	}
	return join("",@options);
}
# Get all categories in the drop down list format
sub getCategoryList{
	# $select: selected value
	my ($this,$select) = @_;
	my @categories = $this->fetchTopicBySection();
	# Check and see if the treemode is on
	if ($config{$this->{class_name}}{edit}{categorylist}{treemode}){
		return $this->html_getCategoryList_tree(\@categories,"topic","webdir_topic_id","parent",$select);
	}else{		
		return $this->html_getCategoryList_list(\@categories,$select);
	}
	return;
}
#----------------------------------------------------------------------

# Delete confirmation popup
sub javascript{
	my ($this) = @_;
	my $url_del_resource = $this->link(action=>"resource_delete");
	my $url_del_topic = $this->link(action=>"topic_delete");
	my $out = <<END;
	function deleteResource(id){
		if (confirm('Are you sure want to delete this resource?')){
			location.href = '${url_del_resource}&resource_id=' + id;
		}
	}
	function deleteTopic(id){
		if (confirm('Are you sure you want to delete this topic?')){
			location.href = '${url_del_topic}&topic_id=' + id;
		}
	}
END
	return $ml->script($out);
}
# Build reports using ExSite::ReportBuilder
sub ReportBuilder{
	my ($this,$opt) = @_;
	my $rb = ExSite::Config::get_obj("ReportBuilder");;
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
 	return $rb->make();
}
# Page redirection
sub redirect{
	my ($this,$reply) = @_;
	$reply = $reply || $this->link(action=>undef,__plaintext=>1);
	if ($reply !~ /^http:\/\//){
		$reply = "$config{server}{server}$reply";
	}
	return ExSite::Util::redirect($reply);
}
# Data merge
sub merge{
	my ($this,$view,$data) = @_;
	return if !$view || !$data;
	$view =~ s/\<\!\-\-\[\[((\w+(\_| )?)+\w*)\]\]\-\-\>/$data->{$1}/g;
	return $view;
}
# Check to see if there are any links or sub directories linked to a directory
sub is_webdir_removable{
	my ($this,$id) = @_;
	my @child = $this->{db}->fetch_match("webdir_topic",{parent=>$id});
	my @links = $this->{db}->fetch_match("webdir_join",{webdir_topic_id=>$id});
	if (scalar(@child) < 1 && scalar(@links) < 1){
		return 1;
	}
	return 0;
}
# Delete a record
sub delete{
	my ($this,$table,$id) = @_;
	return if !$id || !$table;
	my @tobedeleted;
	if ($table eq "webdir_topic"){
		# Only allow directory deletion if a directory has no links or sub directories.
		if ($this->is_webdir_removable($id)){
			push(@tobedeleted,{table=>"webdir_topic",id=>$id});
		}
	}elsif ($table eq "webdir_resource"){
		# Delete all join records
		my @joins = $this->{db}->fetch_match("webdir_join",{webdir_resource_id=>$id});
		foreach my $j (@joins){
			push(@tobedeleted,{table=>"webdir_join",id=>$j->{webdir_join_id}});
		}
		# Delete the main record
		push(@tobedeleted,{table=>"webdir_resource",id=>$id});
	}elsif ($table eq "webdir_join"){
		push(@tobedeleted,{table=>"webdir_join",id=>$id});
	}
	foreach my $t(@tobedeleted){
		$this->{db}->trash_key($t->{table},$t->{id});
	}
	$this->redirect($this->link(action=>undef,resource_id=>undef,topic_id=>undef,__plaintext=>1));
	return;
}
sub is_owner{
	my ($this) = @_;
	return 1 if $this->{db}->is_admin();
	my $sectionid = $this->section_id();
	my %site = $this->{db}->fetch("section",$sectionid);
	return $this->{db}->is_manager(\%site);
}
#----------------------------------------------------------------------
1;