#!/usr/bin/perl

package Modules::Zine::Form;

#==========================================================================
#==========================================================================

use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::HTML;
use ExSite::Crypt;
use ExSite::Image;
use ExSite::Time;
use ExSite::RSS;
use ExSite::Tree;
use ExSite::URI;
use ExSite::Captcha;
#use ExSite::FormBuilder;
use Modules::Zine::Base;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Base);

#==========================================================================
# posting methods

# newmsg: wrapper for compose

sub newmsg {
    my $this = shift;
    my $subject;
    if ($this->{config}{default_subject}) {
	$subject = $this->title;
	if ($subject !~ /^Re:/i) {
	    $subject = "Re: $subject";
	}
    }
    my $out = $this->makeform({type=>"comment",title=>$subject},$this->{article});
    if ($out =~ /<form/i) {
	my $helpfile = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Zine/help_comment.html";
	if (-e $helpfile) {
	    $out .= &get_file($helpfile);
	}
    }
    return $out;
}

# reply: like newmsg, but inserts content from parent message

sub reply {
    my $this = shift;
    my $art = $this->{article};
    my %comment;
    $comment{type} = "comment";
    # add "Re:" to subject
    $comment{title} = $this->title;
    if ($comment{title} !~ /^Re:/i) {
	$comment{title} = "Re: $comment{title}";
    }
    # quote parent article
    my $quote = "&gt;".$this->{article}{body};
    $quote =~ s/\n/\n\&gt\;/g;
    $quote = $this->author." wrote:\n".$quote;
    my $quote_js = $quote;
    $quote_js =~ s/\r?\n/\\n\&gt\;/g;
    if ($this->{config}{quote_parent}) {
	$comment{body} = $quote;
    }
    # quote option allows for dynamic quoting
    $comment{quote} = "\n$quote_js\n";
    my $out = $this->makeform(\%comment,$this->{article});
    if ($out =~ /<form/i) {
	my $helpfile = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Zine/help_reply.html";
	if (-e $helpfile) {
	    $out .= &get_file($helpfile);
	}
	else {
	    $helpfile = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Zine/help_comment.html";
	    if (-e $helpfile) {
		$out .= &get_file($helpfile);
	    }
	}
    }
    return $out;
}

#==========================================================================
# we have our own form processing tools for Zine

sub makeform {
    my ($this,$data,$parent) = @_;
    my $db = $share{DB};
    if (! $data->{type}) { $data->{type} = "article"; }
    if (! $parent) { $parent = $this->fetch($data->{parent_id}); }
    if (! $data->{parent_id}) { $data->{parent_id} = $parent->{article_id}; }
    if (! $data->{reference}) { 
	if ($data->{type} eq "comment" && $this->type ne "comment") {
	    # reset the reference article if this is a top-level comment
	    $data->{reference} = $parent->{article_id};
	}
	elsif ($this->class($parent) eq "zine") {
	    # we should reference the nearest Zine to get good indexing behaviour
	    
	    $data->{reference} = $parent->{article_id};
	}
	else {
	    # maintain the same reference article
	    $data->{reference} = $parent->{reference} || $parent->{article_id}; 
	}
    }
    if (! $data->{thread} && $data->{type} eq $parent->{type}) { 
	$data->{thread} = $parent->{thread} || $parent->{article_id};
    }

    # security checks
    my $stat = $this->allow_post($data);
    return $stat if ($stat);

    $this->save;
    $this->set($data);
    my $class = $this->class;
    my $refclass = $this->refclass;
    my $ztype = $this->zinetype;

    my $type = $data->{type};
    if ($type eq "article" && $parent->{type} eq "blog") {
	$type .= "_blog";
    }
    elsif ($type eq "article" && $parent->{type} eq "photoblog") {
	$type .= "_photoblog";
    }
    elsif ($type eq "article" && $parent->{type} eq "wiki") {
	$type .= "_wiki";
    }

    my $columns = $class eq "zine" ? 
	$this->confrule("fields",undef,$data->{type}) :
	$this->confrule("fields",$data->{type},$parent->{type});
    my @hide;
    if ($this->{admin}) {
	$data->{owner} = $db->my_uid;
	if (! $data->{section_id}) {
	    $data->{section_id} = $db->this_site()->{section_id};
	}
	if ($db->is_admin) {
	    push @$columns, "owner";
	    push @$columns, "section_id";
	}
	else {
	    @hide = ("owner", "section_id");
	}
    }

    my $out;
    my $ml = &get_obj("ML");
    my $article_name = $class eq "zine" ? 
	$this->confrule("whatami",undef,$data->{type}) :
	$this->confrule("whatami",$data->{type},$parent->{type});
    if ($data->{article_id}) {
	$out = $ml->h3($msg{"Edit $article_name"});
    }
    else {
	$out = $ml->h3($msg{"New $article_name"});
    }
    # some JS
    $out .= $share{Page} ? $ml->script(undef,{type=>'text/javascript',src=>'$config{server}{HTMLpath}/_ExSite/js/misc.js'}) : undef;

    my $map = $db->{map}->get_map("article");

    my $url = $this->link(zine=>"submit");
    my $form = $db->form(method=>"POST",
			 formid=>"exsiteform",
			 formname=>"ZineForm",
			 action=>$url,);
    my $required_cols = $this->confrule("required",$class,$ztype);
    foreach my $col (@$columns) {
	if ($col eq "content_type" && $db->{htmleditor}) {
	    # don't prompt for content type if we're using the HTMLeditor
	    next;
	}
	if ($col eq "author" && ! $data->{author} && $db->authorize()) {
	    $data->{author} = $db->my_name();
	}
        my %column = $db->{map}->get_column("article",$col);
	my $label = $this->label(\%column,$type);
	my $reqd = scalar grep(/^$column{column}$/,@$required_cols);
	my $warntext = $msg{"WARNING: all changes will be lost.  Continue?"};
	if ($col eq "body" && 
	    $type =~ /article/ &&
	    $this->{input}{input_method} ne "text" && 
	    ($db->use_wysiwyg || ($this->{input}{input_method} eq "html"))) {
	    # try to use the HTML editor for normal articles
	    my $page = $this->{input}{_id};
	    my $section = $this->{reference}{section_id} || $this->{article}{section_id};
	    my $switchurl = $this->link(input_method=>"text");
	    my $tail = "&rarr; ".$ml->a($msg{"plain text editor"},{href=>"javascript:confirm_custom('$warntext','$switchurl')"});
	    my %opt = ( name=>$col,
			editor_height=>350,
			editor_width=>550,
			table=>"article",
			column=>"body",
			record=>$data->{article_id},
			page=>$page,
			section=>$section,
			links=>1,
			editor=>{
			    op_image=>($db->owns),
			    op_link=>1,
			    op_table=>1,
			    op_dyncontent=>0,
			},
			tail=>$tail,
			required=>$reqd,
			prompt=>$label,
			);
	    $db->input_htmleditor(%opt);
	    my $edtype = $db->use_wysiwyg ? "HTML" : "mixed";
	    $db->input_html(name=>"content_type",
			    type=>"hidden",
			    value=>$edtype,
			    );
	}
	else {
	    # author name field is optionally readonly on member forums
	    my $readonly = ($col eq "author" && 
			    ! $this->{config}{editable_author} && 
			    $db->level > 0);
	    my $tail;
	    if ($col eq "body") {
		if ($data->{quote}) {
		    $tail = $this->quote_button($col,$data->{quote});
		}
		if ($type=~ /article/) {
		    my $switchurl = $this->link(input_method=>"html");
		    my $warntext = $msg{""};
		    $tail = $ml->br."&rarr; ".
			$ml->a($msg{"rich text editor"},{href=>"javascript:confirm_custom('$warntext','$switchurl')"});
		}
	    }
	    $db->input_exsite(name=>$col,
			      prompt=>$label,
			      size=>$column{size},
			      datatype=>$column{datatype},
			      value=>$data->{$col},
			      table=>"article",
			      column=>$col,
			      record=>$data->{article_id},
			      required=>$reqd,
			      readonly=>$readonly,
			      tail=>$tail,
			      );
	}
    }

    # attachments
    my $natt = $this->rule("attachments",$this->class($data));
    if ($natt > 0) {
	for (my $iatt=1; $iatt <= $natt; $iatt++) {
	    my $i = $natt ? "#$iatt" : "";
	    my $att_in = $form->inputtag(type=>"file",name=>"attachment_${iatt}");
	    my $cap_in = $form->inputtag(type=>"text",name=>"caption_${iatt}");
	    $form->input(type=>"preformatted",
			 name=>"Attach_$i",
			 prompt=>" ",
			 input=>"$msg{Attachment} $i ${att_in}&nbsp;$msg{Caption}:&nbsp;${cap_in}".$ml->br);
	}
	# we're using preformatted inputs, so FormBuilder doesn't know to
	# automatically switch to multipart encoding
	$form->multipart;
    }

    # hidden fields
    foreach my $col (@hide) {
        my %column = $db->{map}->get_column("article",$col);
	my $label;
	if ($type eq "comment") {
	    $label = $this->{config}{comment_label}{$col} || $column{label} || $column{column};
	}
	else {
	    $label = $column{label} || $column{column};
	}
	$db->input_html(name=>$col,
			type=>"hidden",
			value=>$data->{$col},
			);
    }

    # catpcha for unauthenticated contributors
    if (! $db->level && $this->{config}{use_captcha}) {
	my $c = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
	$form->input(name=>"captcha",
		     prompt=>" ",
		     input=>$c->make());
    }

    # add (protected) control info 
    my ($ctrlinfo,$ctrl_crypt);
    $ctrlinfo = "$data->{type};$data->{article_id};$data->{parent_id};$data->{reference};$data->{thread}";
    $ctrl_crypt = $this->{config}{secure_forms} ?
	ExSite::Crypt->new()->encrypt($ctrlinfo) :
	$ctrlinfo;
    $db->input_html(name=>"ctrlinfo",type=>"hidden",value=>$ctrl_crypt);
    #### reply screen?

    $form->buttons;
    $out .= $form->make;

    $this->restore;

    return $out;
}

# quote_button creates a link to insert $data into the field $name
sub quote_button {
    my ($this,$name,$data) = @_;
    $data =~ s/\"/\\\"/g;
    $data =~ s/\n/\\n/g;
    $data =~ s/\&gt;/>/g;
    my $ml = &get_obj("ML");
    my $out = $ml->script("<!--
function enter_quote() {
    document.ZineForm.body.value = document.ZineForm.body.value + \"$data\";
}
-->\n",{language=>"JavaScript"});
    (tied %msg)->nohighlight;
    $out .= $ml->input(undef,{type=>"button",value=>$msg{"quote"},onclick=>"enter_quote()"});
    (tied %msg)->restore;
    return $out;
}

#==========================================================================
# full configuration forms

sub newzine {
    my ($this) = @_;
    my $ml = &get_obj("ML");
    my $out = $ml->h1($msg{"New e-Zine"});
    $out .= $this->adminform;
    return $out;
}

sub confform {
    my ($this) = @_;
    my $type = ucfirst($this->type);
    my $ml = &get_obj("ML");
    my $out = $ml->h1($msg{"Configure $type"});
    $out .= $this->adminform;
    return $out;
}

sub adminform {
    my ($this) = @_;
    my $db = $share{DB};
    my $data = $this->{article};

    # security checks
    my $stat = $this->allow_post($data);
    return $stat if ($stat);
    my $authlevel = $db->level;

    my $url = $this->link(zine=>"submit");
    my $form = $db->form(method=>"POST",action=>$url,);

    my ($write, %column);
    my $ncols = $db->{map}->get_ncol("article");
    
    for (my $icolumn = 0; $icolumn < $ncols; $icolumn++) {
        %column = $db->{map}->get_column("article",$icolumn);
	if (! $column{label}) { $column{label} = $column{column}; }

	if ($column{write} <= $authlevel) {
	    # emphasize required fields
	    #### use $config{Zine}{required} instead of dbmap
	    my $reqd = ($column{validate} =~ /soft|hard/);
	    my $datatype = $column{datatype} =~ /^key[:=]article$/ ?
		"int" : $column{datatype};
	    $db->input_column(
			      name=>$column{column},
			      prompt=>$column{label},
			      table=>"article", 
			      column=>$column{column},
			      size=>$column{size},
			      record=>$data->{article_id},
			      datatype=>$datatype,
			      value=>$data->{$column{column}},
			      required=>$reqd,
			      );
    
	}
    }

    # add protected control info 
    my $ctrlinfo = "$data->{type};$data->{article_id};$data->{parent_id};$data->{reference};$data->{thread}";
    my $ctrl_crypt = $this->{config}{secure_forms} ?
	ExSite::Crypt->new()->encrypt($ctrlinfo) :
	$ctrlinfo;
    $db->input_html(name=>"ctrlinfo",type=>"hidden",value=>$ctrl_crypt);
    #### reply screen?

    $form->buttons;
    return $form->make;
}

sub doform {
    my $this = shift;
    my $in = new ExSite::Input;
    my $data = $in->post;
    my $db = $share{DB};
    my ($error,@err);

    my $submit = $data->{submit_action};
    if ($submit eq "Cancel") { return (); } 

    # process the catpcha for unauthenticated contributors
    if (! $db->level && $this->{config}{use_captcha}) {
	if ($data->{captcha}) {
	    my $c = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
	    if (! $c->pass) {
		$error = $msg{"You did not answer the anti-spam question correctly."};
	    }
	}
	else {
	    $error = $msg{"Anti-spam question was missing."};
	}
	if ($error) {
	    $this->error($error);
	    return ($error);
	}
    }

    # decode control info
    my $ctrlinfo = $this->{config}{secure_forms} ?
	ExSite::Crypt->new()->decrypt($data->{ctrlinfo}) :
	$data->{ctrlinfo};
    if ($ctrlinfo !~ /^[-\w]*;\w*;\w*;\w*;\w*$/) {
	# tampering detected - abort
	#$error = "bad control data: $ctrlinfo";
	$error = $msg{"bad control data"};
	$this->error($error);
	return ($error);
    }
    my @ctrlinfo = split(/;/,$ctrlinfo);
    my $ctype = shift @ctrlinfo;
    if (! $data->{type}) { $data->{type} = $ctype; }
    $data->{article_id} = shift @ctrlinfo;
    if (! exists $data->{parent_id} || $db->level < 2) {
	($data->{parent_id},$data->{reference},$data->{thread}) = @ctrlinfo;
    }

    my $type = $data->{type};

    # clean up the keys
    my $sc = $config{form}{sepchar};
    if (exists $data->{"body${sc}html"}) {
	$data->{body} = $data->{"body${sc}html"};
    }
    if (exists $data->{"owner${sc}key"}) {
	$data->{owner} = $data->{"owner${sc}key"};
    }
    if (exists $data->{"section_id${sc}key"}) {
	$data->{section_id} = $data->{"section_id${sc}key"};
    }

    # encode picture, if any
    my $max_size = $this->{config}{max_picture_size} || 48;
    if ($data->{"picture${sc}clear"}) {
	# remove picture
	$data->{picture} = "";
	$data->{thumb} = "";
	$data->{caption} = "";
    }
    else {
	if ($data->{"picture${sc}filename"}) {
	    $data->{picture} = $this->get_upload("picture${sc}filename",$data);
	}
	if ($data->{"thumb${sc}filename"}) { 
	    $data->{thumb} = $this->get_upload("thumb${sc}filename",$data);
	}
    }
    return @err if (@err > 0);

    # extract attachments from the data
    my @attachment;
    my $natt = 1;
    while (exists $data->{"attachment_$natt"}) { 
	if ($data->{"attachment_$natt"}) {
	    my $afile = $this->get_upload("attachment_$natt",$data);
	    $afile =~ /^(.+)\#/;
	    my $aname = $1;
	    push @attachment, {
		type => "attachment",
		name => $aname,
		title => $aname,
		picture => $afile,
		caption => $data->{"caption_${natt}"},
	    };
	}
	$natt++;
    }

    # $data should now contain the complete article data as submitted
    # Now we prep it for insertion into the DB

    $error = $this->prepare($data);
    if ($error) {
	# prep failed
	return ($error);
    }

    # $data now contains a fully normalized article record, ready for
    # insertion. 

    my $null = "NULL";
    if ($data->{article_id}) {
	$error = $db->update("article",$data);
	if (! $error) { 
	    # update our internal copy of the article
	    foreach my $key (keys %$data) { 
		$this->{article}{$key} = $data->{$key};
	    }
	    # get the status for moderation logic, below
	    $data->{status} or $data->{status} = $this->{article}{status};
	}
    }
    else {
	$data->{ctime} = \$null;  # forces auto-setting of MySQL timestamp
	# set a default status
	$data->{status} = $this->{config}{$type}{status} || "active";
	my $aid = $db->insert("article",$data);
	if (!$aid) { 
	    $error = $db->{query_error};
	}
	else {
	    $data->{article_id} = $aid;
	}
    }
    if ($error) {
	return ($error) if ($error);
    }

    # save the attachments
    
    foreach my $att (@attachment) {
	$att->{ctime} = \$null;
	$att->{parent_id} = $data->{article_id};
	$att->{thread} = $data->{article_id};
	$att->{reference} = $data->{reference};
	$att->{status} = "active";
	$error = $this->prepare($att);
	if ($error) {
	    push @err, $error;
	}
	else {
	    my $aid = $db->insert("article",$att);
	    $att->{article_id} = $aid;
	    if (!$aid) { 
		push @err, $db->{query_error};
	    }
	    else {
		$this->publish_unprivileged($att);
	    }
	}
    }

    $this->{moderation_requested} = 0;
    if (@err > 0) {
	$this->error(@err);
	return (@err);
    }
    elsif ($data->{type} eq "comment") {
	my $nattach = scalar @attachment;
	if ($data->{status} eq "active") {
	    if (! exists $data->{article_id}) {
		# only send notifications on inserts, not on edits
		$this->notify($data,$nattach);
	    }
	}
	elsif ($data->{status} !~ /active|archived/) {
	    $this->notify_moderator($data,$nattach);
	    $this->{moderation_requested} = 1;
	}
    }
    
    $this->publish_unprivileged($data);
 
    # republish the parent zine's rss feed
    my $ref = $data->{reference} ? 
	$this->fetch($data->{reference}) : $this->{article};
    if ($this->class($ref) eq "zine" && $data->{status} eq "active") {
	$this->publish_unprivileged($data,{rss=>1});
    }
    return ();
}

# prepare : prepare an article record for insertion into the database.
# Returns an error message if there is an unfixable problem with the data, 
# undef if not.  The data itself is passed as a hashref.
#
# Preparation involves the following:
# - validates existing data
# - detects missing data
# - remove data that does not correspond to database columns
# - sets the owner and reference fields, if necessary
# - sets and/or repairs the name field
# - ensures that text fields conform to HTML display rules
# - scales the picture, if any

sub prepare {
    my ($this,$data) = @_;
    my $db = $share{DB};

    # check type 
    if (! $data->{type}) {
	# no type - better abort
	return $this->error($msg{"undefined article type"});
    }
    my $type = $data->{type};

    # check references
    foreach my $key (qw(parent_id reference thread)) {
	if (! defined $data->{$key}) { $data->{$key} = 0; }
    }
    if (! exists $data->{article_id}) {
	# this is an insert - check for meaningful references 
	if ($this->class($data) ne "zine") {
	    if (! $data->{parent_id} || ! $data->{reference}) {
		return $this->error($msg{"submission contains invalid references"});
	    }
	}
    }

    # set a status
    if (! $data->{status}) {
	if (($this->{article_id} != $data->{article_id}) || 
	    ! $this->{article}{status}) { 
	    # article doesn't appear to have a status - set a default
	    $this->save();
	    $this->set($data);
	    my $zinetype = $this->zinetype();
	    $data->{status} = $zinetype ? 
		$this->confrule("status",$type,$this->zinetype) :
		"active";
	    $this->restore;
	}
    }

    # set owner on inserts
    if (! $data->{owner} && ! $data->{article_id}) {
	my $uid = $db->my_uid;
	if ($uid) { $data->{owner} = $uid; }
    }

    # set section_id on zine inserts for non-sysadmins
    if ($this->class($data) eq "zine" && 
	! $db->is_admin()) {
	if ($data->{section_id}) {
	    # validate section id
	    if (! $db->owns($data->{section_id})) {
		return $this->error($msg{"you cannot create zines in that site or section"});
	    }
	}
	else {
	    # force section ID to manager's home section
	    $data->{section_id} = $db->my_user_record()->{section_id};
	}
    }

    # fix text fields
    if (exists $data->{body}) {
    	# Unescape %7B to { ...etc
    	require ExSite::Util;
	$data->{body} = &restore_cms_tags($data->{body});
	# Full HTML allowed in body for normal articles, but not comments ...
	$data->{body} = $this->clean_text($data->{body},
					  $type eq "comment" ? 1 : 2);
	if ($this->{config}{hyperlink_urls}) {
	    $data->{body} = &hyperlink_urls($data->{body});
	}
    }
    my $allow_html = ($type =~ /comment/) ?
	$this->{config}{allow_html_in_comments} : undef;
    # undef means we defer to basic ExSite rules for whether or not
    # to allow html;  ie. config setting report.text.honor_html
    foreach my $f (qw(author title subtitle summary about caption)) {
	if (exists $data->{$f}) {
	    $data->{$f} = $this->clean_text($data->{$f},$allow_html);
	    if ($this->{config}{hyperlink_urls}) {
		$data->{$f} = &hyperlink_urls($data->{$f});
	    }
	}
    }

    # set the article name
    if ($type !~ /comment|attachment/) {
	if (! $data->{name}) {
	    $data->{name} = $data->{title};
	}
	$data->{name} =~ s/[^\w-]/_/g;
	$data->{name} =~ s/_{2,}/_/g;
    }

    # scale picture, if any
    my ($img,$thumb,$error);
    if ($data->{thumb}) { 
	($img,undef,$error) = $this->resize($data->{thumb},0,0);
	return $this->error($error) if ($error);
	$data->{thumb} = $img;
    }
    if ($data->{picture}) { 
	($img,$thumb,$error) = $this->resize($data->{picture},
					     $this->{config}{allow_documents},
					     1);
	return $this->error($error) if ($error);
	$data->{picture} = $img;
	if ($thumb && ! $data->{thumb}) {
	    $data->{thumb} = $thumb;
	}
    }

    # check for required fields
    my @missing;
    my $required = $this->rule("required",$data);
    if (ref $required eq "ARRAY") {
	foreach my $req (@$required) {
	    if (exists $data->{$req}) {  # field is present
		if (! $data->{$req}) {   # data is missing
		    push @missing, $req;
		}
	    }
	}
    }
    if (@missing > 0) {
	return $this->error($msg{"missing field(s)"}.": ".join(", ",@missing));
    }

    # content filter
    if ($config{wordfilter}{mode} eq "auto") {
	my @err;
	foreach my $key (keys %$data) {
	    if ($data->{$key}) {
		my %column = $db->{map}->get_column("article",$key);
		if ($column{datatype} =~ /string|text|html/ && 
		    $column{read} <= $config{wordfilter}{threshold}) {
		    # automatic word filtering is enabled
		    my ($text,$error) = $db->validate_language($data->{$key});
		    if ($error) {
			# validation errors
			push @err, "$column{label}: $error";
		    }
		    if ($text ne $data->{$key}) {
			# update with censored text
			$data->{$key} = $text;
		    }
		}
	    }
	}
	if (@err > 0) {
	    return $this->error(join("; ",@err));
	}
    }

    # delete all hashkeys that do not correspond to db columns
    my %cols = map { $_ => 1 } $db->{map}->get_columns("article");
    foreach my $key (keys %$data) {
	if (! exists $cols{$key}) { delete $data->{$key}; }
    }

    # article passes inspection
    return undef;
}

sub get_upload {
    my ($this,$key,$data) = @_;

    my $fname = $data->{$key};
    my $fdata = undef;
    my $size = 0;
    # read file contents
    my $continue = 1;
    while ($continue) {
	$continue = read($data->{$key},$fdata,1024,$size);
	$size += $continue;
    }

    my $img = new ExSite::Image($fname,$fdata);
    return $img->encode;
}

# resize : prep an uploaded image
# $imageflag - upload must be an image, if true
# $thumbflag - also make a thumbnail, if true

sub resize {
    my ($this,$imgdata,$allow_nonimages,$make_thumb) = @_;
    my ($newimg, $newthumb, $error);
    my $img = new ExSite::Image($imgdata);
    my $size = $img->size;
    my $max_size = $this->{config}{max_picture_size} || 48;
    my $max_dim = $this->{config}{max_picture_dim} || 500;
    if ($img->{filename} =~ /\.(jpe?g|gif|png)$/i) {
	my ($img_x,$img_y) = $img->dim;
	if ($img_x > $max_dim || 
	    $img_y > $max_dim || 
	    $size/1024 > $max_size) {
	    # picture too big
	    $img->shrink(dim=>$max_dim,size=>$max_size);
	}
	$newimg = $img->encode;
	# thumbnail the pic
	if ($make_thumb && $img->thumb) {
	    $newthumb = $img->encode;
	}
    }
    else {
	if (! $allow_nonimages) {
	    $error = "$img->{filename}: ".$msg{"image must be GIF, PNG, or JPEG file type"};
	}
	elsif ($size/1024 > $max_size) {
	    $error = &substitute($msg{"[[filename]] is too large; please limit to [[size]] K"},{size=>$max_size,filename=>$img->{filename}});
	}
	else {
	    $newimg = $imgdata;
	}
    }
    return ($newimg,$newthumb,$error);
}

#deprecated?
sub post_doform {
    my $this = shift;
    # this is a dummy;  it's purpose is to be overloaded by other
    # article handling systems
    return;
}

# status : update article status only

sub status {
    my ($this,$action) = @_;
    my ($db,$stat,$error);
    $db = $share{DB};
    # security checks
    if ($this->moderator) {
	if ($action eq "approve") {
	    $error = $db->update("article",
				 {status=>"active"},
				 {article_id=>$this->{article_id}});
	    $this->{article}{status} = "active";
	    $this->notify($this->{article});
	    if ($this->refclass eq "zine") {
		# add to zine's rss feed
		$this->publish_unprivileged($this->{article},{rss=>1});
	    }
	}
	elsif ($action eq "reject") {
	    $error = $db->update("article",
				 {status=>"rejected"},
				 {article_id=>$this->{article_id}});
	    $this->{article}{status} = "rejected";
	    ### TODO: notification?
	}
	else {
	    # the status itself was passed
	    my $statuses = $db->{map}->regexp("list:article_status");
	    if ($action =~ /^($statuses)$/) {
		$error = $db->update("article",
				     {status=>$action},
				     {article_id=>$this->{article_id}});
		$this->{article}{status} = $action;
		if ($action eq "active") {
		    $this->notify($this->{article});
		}
	    }
	    else {
		$error = $db->error($msg{"article: invalid status:"}." $action");
	    }
	}
	return $db->show_diagnostics("error","html") if ($error);
	return $this->info(ucfirst $action . " " . $this->description() . " -- DONE.");
    }
    else {
	return $this->error($msg{"Moderation rejected - you do not have moderation privileges here"});
    }
    return undef;
}

# notify an author/owner when their article has been commented on

sub notify {
    my ($this,$data,$nattach) = @_;
    my $stat = $this->run_handler("ZineNotify",$data);
    return if (defined $stat);
    return if (! $this->{config}{notify});
    return if (! $data->{parent_id});
    $this->load($data->{parent_id});
    my $email = $this->author_email;
    return if ($email !~ /@.+\.\w+$/);  # cheap-and-dirty email regexp
    my $u = new ExSite::URI;
    $u->plaintext;
    $u->query(_auth=>undef,aid=>$data->{article_id},zine=>"show");
    my $url = $u->write_full;
    $u->query(aid=>$data->{parent_id});
    my $purl = $u->write_full;
    $u->query(aid=>$data->{reference});
    my $zurl = $u->write_full;
    (tied %msg)->nohighlight();
    my $attachments = $nattach ? "\n\n[$nattach ".$msg{"attachment(s)"}."]" : "";
    my $msg = &substitute($msg{"
[[author]] has commented on your [[type]] '[[title]]' at
[[server]]

---------------------------------------------------------------------
From: [[author]]
Subject: [[subject]]

[[body]][[attachments]]
---------------------------------------------------------------------

This message alone may be viewed (and replied to) at:
[[url]]

Your original message (including replies) may be viewed at:
[[purl]]\n\n"},{author=>$data->{author},type=>$this->type,title=>$this->title,subject=>$data->{title},server=>$config{server}{server},body=>$data->{body},attachments=>$attachments,url=>$url,purl=>$purl});
	  
    if ($zurl ne $purl) {
	$msg .= &substitute($msg{"The whole discussion may be viewed at:\n[[zurl]]\n"},{zurl=>$zurl});
    }
    (tied %msg)->restore();

#    return &SendMail($email,
#		     $config{site}{administrator},
#		     "[$config{site}{name}] $data->{title}",
#		     $msg);

    return &ExSite::Mail::send(
        to => $email,
    	from => $this->{config}{notification_from} || $config{site}{administrator},
    	subject => "[$config{site}{name}] $data->{title}",
    	body => $msg
    );
}

sub notify_moderator {
    my ($this,$data,$nattach) = @_;
    my $ref = $this->my_zine;
    $this->save;
    $this->set($ref);
    my $moderator_email = $this->author_email;
    my $moderator_login = $this->author_login;
    $this->restore;
    return if (! $moderator_email);

    my ($url,$purl,$turl,$url_auth,$purl_auth,$turl_auth,$authtoken);
    my $u = new ExSite::URI;
    $u->plaintext;
    $u->query(aid=>$data->{article_id},zine=>"show");
    $u->authorize($moderator_login,0);
    $url_auth = $u->write_full;
    if ($data->{parent_id}) {
	$u->query(_auth=>undef,aid=>$data->{parent_id});
	$u->authorize($moderator_login,0);
	$purl_auth = $u->write_full;
    }
    if ($data->{thread}) {
	$u->query(_auth=>undef,aid=>$data->{thread});
	$u->authorize($moderator_login,0);
	$turl_auth = $u->write_full;
    }
    my $author = $data->{author} || "(anonymous)";
    my $attachments = $nattach ? "\n\n[$nattach ".$msg{"attachment(s)"}."]" : "";
    my $msg = &substitute($msg{"[[author]] has commented on your [[type]] '[[title]]' at
[[server]]

---------------------------------------------------------------------
From: [[author]]
Subject: [[subject]]

[[body]][[attachments]]
---------------------------------------------------------------------

The comment can also be viewed at:
[[url]]\n\n"},{author=>$author,type=>$this->type,title=>$this->title,subject=>$data->{title},server=>$config{server}{server},body=>$data->{body},attachments=>$attachments,url=>$url_auth});
    if ($purl) {
	$msg .= $msg{"It may be viewed in the context of the parent message at:"}."\n$purl_auth\n\n";
    }
    if ($turl && $turl ne $purl) { 
	$msg .= $msg{"The whole thread may be viewed at:"}."\n$turl_auth\n\n";
    }

    $u->query(_auth=>undef,aid=>$data->{article_id},zine=>"approve");
    $u->authorize($moderator_login,0);
    my $approve = $u->write_full;

    $u->query(_auth=>undef,aid=>$data->{article_id},zine=>"reject");
    $u->authorize($moderator_login,0);
    my $reject = $u->write_full;

    $msg .= "\n=== ".$msg{"MODERATION LINKS"}." ===\n\n".$msg{"APPROVE this comment"}.": $approve\n\n".$msg{"REJECT this comment"}.":  $reject\n";
	
    return &ExSite::Mail::send(
        to => $moderator_email,
    	from => $config{site}{administrator},
    	subject => "[$config{site}{name}] $ref->{title} - ".$msg{"MODERATION REQUESTED"},
    	body => $msg
    );
}

1;
