package Modules::Zine::IndexPaged;

=pod

=cut

use strict;
use ExSite::Config;
use Modules::Zine::Index;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Index);

sub index {
    my ($this,%opt) = @_;

    # get our current position in the index
    $opt{pos} = $this->{input}{goto} || 0;

    # get all the articles in the index
    my @articles = $this->fetch_index(%opt);
    $this->{nart} = scalar @articles;

    # format the index
    return $this->index_paged(\%opt,@articles);
}

sub fetch_index {
    my ($this,%opt) = @_;

    my $mode = $opt{mode} || "article";
    my $class = $this->class;

    if ($class eq $mode) {
	$this->save;
	$this->load($this->{reference});
    }
    my @art = $this->fetch_child_articles($mode);
    if ($class eq $mode) {
	$this->restore;
    }

    $this->{nart} = scalar @art;
    $this->{narch} = 0;
    
    return wantarray ? @art : \@art;
}

sub index_paged {
    my ($this,$opt,@art) = @_;
    my $index;
    my $ml = &get_obj("ML");

    my $pos = $opt->{pos};
    if ($pos > $#art) { $pos = $#art; }
    elsif ($pos < 0) { $pos = 0; }

    # Show the current article.
    if ($pos >= 0) {
	$this->save;
	$this->set($art[$pos]);
	$index .= $this->show_article("full");
	$this->restore;
    }

    my $next = $pos < $#art ? $pos + 1 : -1;
    my $prev = $pos > 0 ? $pos - 1 : -1;

    my ($url,@indexlink);
    if ($prev >= 0) {
	$url = $this->link(aid=>$this->{article_id},
			   goto=>$prev,
			   zine=>"show");
	push @indexlink, $ml->a("&lt;&nbsp;".$msg{Prev}, { href=>$url });
    }
    my $n = 0;
    foreach my $a (@art) {
	my %opt = (href=>$this->link(aid=>$this->{article_id},goto=>$n,zine=>"show"));
	if ($n == $pos) { $opt{class} = "ZinePagedIndexCurrent"; }
	$n++;
	push @indexlink, $ml->a($n,\%opt);
    }
    if ($next >= 0) {
	$url = $this->link(aid=>$this->{article_id},
			   goto=>$next,
			   zine=>"show");
	push @indexlink, $ml->a($msg{Next}."&nbsp;&gt;", { href=>$url });
    }

    if (@indexlink > 0) {
	$index .= $ml->div($msg{More}.": ".join("&nbsp;",@indexlink),{class=>"ZinePagedIndex"});
    }
    return $index;
}

sub index_paged_old {
    my ($this,$opt,@art) = @_;
    my $index;
    my @art2;

    # get articles of correct type
    foreach my $a (@art) {
	next if ($this->class($a) ne $opt->{mode});
	unshift @art2,$a;  # this reverses the order of @art
    }
    return undef if (@art2 == 0);

    # figure out where we are in the index
    my $id = $this->{article_id};
    my $pos = 0;
    foreach my $a (@art2) { 
	last if ($a->{article_id} == $id);
	$pos++;
    }

    if ($pos > $#art2) {
	# we do not appear to be in the index yet.
	# Show the first article.
	$this->save;
	$this->set($art2[0]);
	$index .= $this->show_article();
	### do we want to show the sub-index of this article?
	$this->restore;
	$pos = 0;
    }

    my $next = $pos < $#art2 ? $pos + 1 : -1;
    my $prev = $pos > 0 ? $pos - 1 : -1;

    my ($url,@indexlink);
    if ($prev >= 0) {
	$url = $this->link(aid=>$art2[$prev]{article_id},zine=>"show");
	push @indexlink, "<a href=\"$url\">&lt;&nbsp;Prev</a>";
    }
    my $n = 0;
    foreach my $a (@art2) {
	$n++;
	$url = $this->link(aid=>$a->{article_id},zine=>"show");
	my $class = ($n - 1) == $pos ? " class=\"ZinePagedIndexCurrent\"" : undef;
	push @indexlink, "<a href=\"$url\"$class>$n</a>";
    }
    if ($next >= 0) {
	$url = $this->link(aid=>$art2[$next]{article_id},zine=>"show");
	push @indexlink, "<a href=\"$url\">Next&nbsp;&gt;</a>";
    }

    if (@indexlink > 0) {
	$index .= "<div class=\"ZinePagedIndex\">More: ".join("&nbsp;",@indexlink)."</div>\n";
    }
    return $index;
}

sub archive_count {
    my $this = shift;
    ## no archives in paged indexes??
    return 0;
}

1;

