#!/usr/bin/perl

package Modules::Zine::IndexTable;

=pod

=head1 Modules::Zine::IndexTable

This is a plug-in indexing engine to display article indexes in a
tabular format.

It uses the same article selection rules as IndexBasic.  It ignores
the indexformat.

=cut

use strict;
use ExSite::Config;
use Modules::Zine::Index;  # inherit fetch_index, archive;

use vars qw(@ISA $ml);
@ISA = qw(Modules::Zine::Index);

sub index {
    my ($this,%opt) = @_;
    $ml = &get_obj("ML");

    # save current article
    $this->save;

    my $cols = $opt{columns} || ["title","author","ctime"];
    my $index = $this->header(@$cols);

    my @articles = $this->fetch_index(%opt);
    foreach my $art (@articles) {
	$this->set($art);
	$index .= $this->show_item(@$cols);
    }

    # reset article
    $this->restore;

    return $ml->table($index,{class=>"ZineIndex"});
}

sub show_item {
    my ($this,@col) = @_;
    my @colout;
    foreach my $c (@col) {
	push @colout, $c eq "title" ? 
	    $this->show_link() :
	    $this->show_data($c);
    }
    return $ml->tr( join("", map { $ml->td($_) } @colout), { class=>"ZineItem" } );
}

sub header {
    my ($this,@col) = @_;
    my @colout;
    foreach my $c (@col) {
	my %cmap = $share{DB}->{map}->get_column("article",$c);
	push @colout, $this->label(\%cmap);
    }
    return $ml->tr( join("", map { $ml->th($_) } @colout) );
}

1;

